@@ -4,49 +4,19 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/Literals.h"
 #include "mozilla/mozalloc.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/Unused.h"
 #include "mozilla/Vector.h"
+#include "mozilla/gtest/MozHelpers.h"
 #include "mozmemory.h"
 #include "nsCOMPtr.h"
-#include "nsICrashReporter.h"
-#include "nsServiceManagerUtils.h"
 #include "Utils.h"
 
 #include "gtest/gtest.h"
 
 #ifdef MOZ_PHC
-#  include "replace_malloc_bridge.h"
-#endif
-
-#if defined(DEBUG) && !defined(XP_WIN) && !defined(ANDROID)
-#  define HAS_GDB_SLEEP_DURATION 1
-extern unsigned int _gdb_sleep_duration;
-#endif
-
-// Death tests are too slow on OSX because of the system crash reporter.
-#ifndef XP_DARWIN
-static void DisableCrashReporter() {
-  nsCOMPtr<nsICrashReporter> crashreporter =
-      do_GetService("@mozilla.org/toolkit/crash-reporter;1");
-  if (crashreporter) {
-    crashreporter->SetEnabled(false);
-  }
-}
-
-// Wrap ASSERT_DEATH_IF_SUPPORTED to disable the crash reporter
-// when entering the subprocess, so that the expected crashes don't
-// create a minidump that the gtest harness will interpret as an error.
-#  define ASSERT_DEATH_WRAP(a, b) \
-    ASSERT_DEATH_IF_SUPPORTED(    \
-        {                         \
-          DisableCrashReporter(); \
-          a;                      \
-        },                        \
-        b)
-#else
-#  define ASSERT_DEATH_WRAP(a, b)
+#  include "PHC.h"
 #endif
 
 using namespace mozilla;
@@ -55,13 +25,13 @@ class AutoDisablePHCOnCurrentThread {
  public:
   AutoDisablePHCOnCurrentThread() {
 #ifdef MOZ_PHC
-    ReplaceMalloc::DisablePHCOnCurrentThread();
+    mozilla::phc::DisablePHCOnCurrentThread();
 #endif
   }
 
   ~AutoDisablePHCOnCurrentThread() {
 #ifdef MOZ_PHC
-    ReplaceMalloc::ReenablePHCOnCurrentThread();
+    mozilla::phc::ReenablePHCOnCurrentThread();
 #endif
   }
 };
@@ -239,7 +209,7 @@ TEST(Jemalloc, PtrInfo)
   ASSERT_TRUE(InfoEq(info, TagUnknown, nullptr, 0U, 0U));
 
   // Stack memory.
-  int stackVar;
+  int stackVar = 0;
   jemalloc_ptr_info(&stackVar, &info);
   ASSERT_TRUE(InfoEq(info, TagUnknown, nullptr, 0U, 0U));
 
@@ -295,11 +265,8 @@ TEST(Jemalloc, Arenas)
   free(ptr);
   moz_dispose_arena(arena);
 
-#ifdef HAS_GDB_SLEEP_DURATION
   // Avoid death tests adding some unnecessary (long) delays.
-  unsigned int old_gdb_sleep_duration = _gdb_sleep_duration;
-  _gdb_sleep_duration = 0;
-#endif
+  SAVE_GDB_SLEEP_LOCAL();
 
   // Can't use an arena after it's disposed.
   // ASSERT_DEATH_WRAP(moz_arena_malloc(arena, 80), "");
@@ -333,9 +300,7 @@ TEST(Jemalloc, Arenas)
   moz_dispose_arena(arena2);
   moz_dispose_arena(arena);
 
-#ifdef HAS_GDB_SLEEP_DURATION
-  _gdb_sleep_duration = old_gdb_sleep_duration;
-#endif
+  RESTORE_GDB_SLEEP_LOCAL();
 }
 
 // Check that a buffer aPtr is entirely filled with a given character from
@@ -383,9 +348,9 @@ class SizeClassesBetween {
 };
 
 #define ALIGNMENT_CEILING(s, alignment) \
-  (((s) + ((alignment)-1)) & (~((alignment)-1)))
+  (((s) + ((alignment) - 1)) & (~((alignment) - 1)))
 
-#define ALIGNMENT_FLOOR(s, alignment) ((s) & (~((alignment)-1)))
+#define ALIGNMENT_FLOOR(s, alignment) ((s) & (~((alignment) - 1)))
 
 static bool IsSameRoundedHugeClass(size_t aSize1, size_t aSize2,
                                    jemalloc_stats_t& aStats) {
@@ -399,7 +364,7 @@ static bool CanReallocInPlace(size_t aFromSize, size_t aToSize,
   // PHC allocations must be disabled because PHC reallocs differently to
   // mozjemalloc.
 #ifdef MOZ_PHC
-  MOZ_RELEASE_ASSERT(!ReplaceMalloc::IsPHCEnabledOnCurrentThread());
+  MOZ_RELEASE_ASSERT(!mozilla::phc::IsPHCEnabledOnCurrentThread());
 #endif
 
   if (aFromSize == malloc_good_size(aToSize)) {
@@ -464,11 +429,8 @@ TEST(Jemalloc, JunkPoison)
   jemalloc_stats_t stats;
   jemalloc_stats(&stats);
 
-#  ifdef HAS_GDB_SLEEP_DURATION
   // Avoid death tests adding some unnecessary (long) delays.
-  unsigned int old_gdb_sleep_duration = _gdb_sleep_duration;
-  _gdb_sleep_duration = 0;
-#  endif
+  SAVE_GDB_SLEEP_LOCAL();
 
   // Create buffers in a separate arena, for faster comparisons with
   // bulk_compare.
@@ -497,6 +459,9 @@ TEST(Jemalloc, JunkPoison)
   params.mMaxDirty = size_t(-1);
   arena_id_t arena = moz_create_arena_with_params(&params);
 
+  // Mozjemalloc is configured to only poison the first four cache lines.
+  const size_t poison_check_len = 256;
+
   // Allocating should junk the buffer, and freeing should poison the buffer.
   for (size_t size : sSizes) {
     if (size <= stats.large_max) {
@@ -511,7 +476,8 @@ TEST(Jemalloc, JunkPoison)
       // We purposefully do a use-after-free here, to check that the data was
       // poisoned.
       ASSERT_NO_FATAL_FAILURE(
-          bulk_compare(buf, 0, allocated, poison_buf, stats.page_size));
+          bulk_compare(buf, 0, std::min(allocated, poison_check_len),
+                       poison_buf, stats.page_size));
     }
   }
 
@@ -527,8 +493,9 @@ TEST(Jemalloc, JunkPoison)
     ASSERT_EQ(ptr, ptr2);
     ASSERT_NO_FATAL_FAILURE(
         bulk_compare(ptr, 0, prev + 1, fill_buf, stats.page_size));
-    ASSERT_NO_FATAL_FAILURE(
-        bulk_compare(ptr, prev + 1, size, poison_buf, stats.page_size));
+    ASSERT_NO_FATAL_FAILURE(bulk_compare(ptr, prev + 1,
+                                         std::min(size, poison_check_len),
+                                         poison_buf, stats.page_size));
     moz_arena_free(arena, ptr);
     prev = size;
   }
@@ -552,12 +519,14 @@ TEST(Jemalloc, JunkPoison)
           // beyond the valid range.
           if (to_size > stats.large_max) {
             size_t page_limit = ALIGNMENT_CEILING(to_size, stats.page_size);
-            ASSERT_NO_FATAL_FAILURE(bulk_compare(ptr, to_size, page_limit,
-                                                 poison_buf, stats.page_size));
+            ASSERT_NO_FATAL_FAILURE(bulk_compare(
+                ptr, to_size, std::min(page_limit, poison_check_len),
+                poison_buf, stats.page_size));
             ASSERT_DEATH_WRAP(ptr[page_limit] = 0, "");
           } else {
-            ASSERT_NO_FATAL_FAILURE(bulk_compare(ptr, to_size, from_size,
-                                                 poison_buf, stats.page_size));
+            ASSERT_NO_FATAL_FAILURE(bulk_compare(
+                ptr, to_size, std::min(from_size, poison_check_len), poison_buf,
+                stats.page_size));
           }
         } else {
           // Enlarging allocation
@@ -601,7 +570,8 @@ TEST(Jemalloc, JunkPoison)
         ASSERT_NE(ptr, ptr2);
         if (from_size <= stats.large_max) {
           ASSERT_NO_FATAL_FAILURE(
-              bulk_compare(ptr, 0, from_size, poison_buf, stats.page_size));
+              bulk_compare(ptr, 0, std::min(from_size, poison_check_len),
+                           poison_buf, stats.page_size));
         }
         ASSERT_NO_FATAL_FAILURE(
             bulk_compare(ptr2, 0, from_size, fill_buf, stats.page_size));
@@ -632,7 +602,8 @@ TEST(Jemalloc, JunkPoison)
         ASSERT_NE(ptr, ptr2);
         if (from_size <= stats.large_max) {
           ASSERT_NO_FATAL_FAILURE(
-              bulk_compare(ptr, 0, from_size, poison_buf, stats.page_size));
+              bulk_compare(ptr, 0, std::min(from_size, poison_check_len),
+                           poison_buf, stats.page_size));
         }
         ASSERT_NO_FATAL_FAILURE(
             bulk_compare(ptr2, 0, to_size, fill_buf, stats.page_size));
@@ -654,9 +625,7 @@ TEST(Jemalloc, JunkPoison)
   moz_arena_free(buf_arena, fill_buf);
   moz_dispose_arena(buf_arena);
 
-#  ifdef HAS_GDB_SLEEP_DURATION
-  _gdb_sleep_duration = old_gdb_sleep_duration;
-#  endif
+  RESTORE_GDB_SLEEP_LOCAL();
 }
 #endif  // !defined(XP_WIN) || !defined(MOZ_CODE_COVERAGE)
 
@@ -669,11 +638,8 @@ TEST(Jemalloc, TrailingGuard)
   jemalloc_stats_t stats;
   jemalloc_stats(&stats);
 
-#ifdef HAS_GDB_SLEEP_DURATION
   // Avoid death tests adding some unnecessary (long) delays.
-  unsigned int old_gdb_sleep_duration = _gdb_sleep_duration;
-  _gdb_sleep_duration = 0;
-#endif
+  SAVE_GDB_SLEEP_LOCAL();
 
   arena_id_t arena = moz_create_arena();
   ASSERT_TRUE(arena != 0);
@@ -705,9 +671,7 @@ TEST(Jemalloc, TrailingGuard)
 
   moz_dispose_arena(arena);
 
-#ifdef HAS_GDB_SLEEP_DURATION
-  _gdb_sleep_duration = old_gdb_sleep_duration;
-#endif
+  RESTORE_GDB_SLEEP_LOCAL();
 }
 
 TEST(Jemalloc, LeadingGuard)
@@ -719,11 +683,8 @@ TEST(Jemalloc, LeadingGuard)
   jemalloc_stats_t stats;
   jemalloc_stats(&stats);
 
-#ifdef HAS_GDB_SLEEP_DURATION
   // Avoid death tests adding some unnecessary (long) delays.
-  unsigned int old_gdb_sleep_duration = _gdb_sleep_duration;
-  _gdb_sleep_duration = 0;
-#endif
+  SAVE_GDB_SLEEP_LOCAL();
 
   arena_id_t arena = moz_create_arena();
   ASSERT_TRUE(arena != 0);
@@ -755,9 +716,7 @@ TEST(Jemalloc, LeadingGuard)
   moz_arena_free(arena, ptr);
   moz_dispose_arena(arena);
 
-#ifdef HAS_GDB_SLEEP_DURATION
-  _gdb_sleep_duration = old_gdb_sleep_duration;
-#endif
+  RESTORE_GDB_SLEEP_LOCAL();
 }
 
 TEST(Jemalloc, DisposeArena)
@@ -765,11 +724,8 @@ TEST(Jemalloc, DisposeArena)
   jemalloc_stats_t stats;
   jemalloc_stats(&stats);
 
-#ifdef HAS_GDB_SLEEP_DURATION
   // Avoid death tests adding some unnecessary (long) delays.
-  unsigned int old_gdb_sleep_duration = _gdb_sleep_duration;
-  _gdb_sleep_duration = 0;
-#endif
+  SAVE_GDB_SLEEP_LOCAL();
 
   arena_id_t arena = moz_create_arena();
   void* ptr = moz_arena_malloc(arena, 42);
@@ -805,7 +761,107 @@ TEST(Jemalloc, DisposeArena)
   // Using the arena after it's been disposed of is MOZ_CRASH-worthy.
   ASSERT_DEATH_WRAP(moz_arena_malloc(arena, 42), "");
 
-#ifdef HAS_GDB_SLEEP_DURATION
-  _gdb_sleep_duration = old_gdb_sleep_duration;
-#endif
+  RESTORE_GDB_SLEEP_LOCAL();
+}
+
+static void CheckPtr(void* ptr, size_t size) {
+  EXPECT_TRUE(ptr);
+  jemalloc_ptr_info_t info;
+  jemalloc_ptr_info(ptr, &info);
+  EXPECT_EQ(info.tag, TagLiveAlloc);
+  EXPECT_EQ(info.size, malloc_good_size(size));
+}
+
+static void CheckStats(const char* operation, unsigned iteration,
+                       jemalloc_stats_lite_t& baseline,
+                       jemalloc_stats_lite_t& stats, size_t num_ops,
+                       ptrdiff_t bytes_diff) {
+  if ((baseline.allocated_bytes + bytes_diff != stats.allocated_bytes
+
+       || baseline.num_operations + num_ops != stats.num_operations)) {
+    // All the tests that check stats, perform some operation, then check stats
+    // again can race with other threads.  But the test can't be made thread
+    // safe without a sagnificant amount of work.  However this IS a problem
+    // when stepping through the test using a debugger, since other threads are
+    // likely to run while the current thread is paused.  Instead of neading a
+    // debugger our printf here can help understand a failing test.
+    fprintf(stderr, "Check stats failed after iteration %u operation %s\n",
+            iteration, operation);
+
+    EXPECT_EQ(baseline.allocated_bytes + bytes_diff, stats.allocated_bytes);
+    EXPECT_EQ(baseline.num_operations + num_ops, stats.num_operations);
+  }
+}
+
+TEST(Jemalloc, StatsLite)
+{
+  // Disable PHC allocations for this test, because even a single PHC
+  // allocation occurring can throw it off.
+  AutoDisablePHCOnCurrentThread disable;
+
+  // Use this data to make an allocation, resize it twice, then free it.  Some
+  // The data uses a few size classes and does a combination of in-place and
+  // moving reallocations.
+  struct {
+    // The initial allocation size.
+    size_t initial;
+    // The first reallocation size and number of operations of the reallocation.
+    size_t next;
+    size_t next_ops;
+    // The final reallocation size and number of operations of the reallocation.
+    size_t last;
+    size_t last_ops;
+  } TestData[] = {
+      /* clang-format off */
+      { 16,      15,     0, 256,     2},
+      {128_KiB,  64_KiB, 1,  68_KiB, 1},
+      {  4_MiB,  16_MiB, 2,   3_MiB, 2},
+      { 16_KiB, 512,     2,  32_MiB, 2},
+      /* clang-format on */
+  };
+
+  arena_id_t my_arena = moz_create_arena();
+
+  unsigned i = 0;
+  for (auto data : TestData) {
+    // Assert that the API returns /something/ a bit sensible.
+    jemalloc_stats_lite_t baseline;
+    jemalloc_stats_lite(&baseline);
+
+    // Allocate an object.
+    void* ptr = moz_arena_malloc(my_arena, data.initial);
+    CheckPtr(ptr, data.initial);
+
+    jemalloc_stats_lite_t stats1;
+    jemalloc_stats_lite(&stats1);
+    CheckStats("malloc()", i, baseline, stats1, 1,
+               malloc_good_size(data.initial));
+
+    // realloc the item in-place.
+    ptr = moz_arena_realloc(my_arena, ptr, data.next);
+    CheckPtr(ptr, data.next);
+
+    jemalloc_stats_lite_t stats2;
+    jemalloc_stats_lite(&stats2);
+    CheckStats("realloc() 1", i, stats1, stats2, data.next_ops,
+               malloc_good_size(data.next) - malloc_good_size(data.initial));
+
+    // realloc so it has to move to a different location
+    ptr = moz_arena_realloc(my_arena, ptr, data.last);
+    CheckPtr(ptr, data.last);
+
+    jemalloc_stats_lite_t stats3;
+    jemalloc_stats_lite(&stats3);
+    CheckStats("realloc() 2", i, stats2, stats3, data.last_ops,
+               malloc_good_size(data.last) - malloc_good_size(data.next));
+
+    moz_arena_free(my_arena, ptr);
+    jemalloc_stats_lite_t stats4;
+    jemalloc_stats_lite(&stats4);
+    CheckStats("free()", i, stats3, stats4, 1, -malloc_good_size(data.last));
+
+    i++;
+  }
+
+  moz_dispose_arena(my_arena);
 }
