@@ -8,10 +8,14 @@ import android.content.Context
 import android.os.Parcelable
 import android.util.AttributeSet
 import android.util.JsonReader
+import androidx.annotation.OptIn
 import androidx.annotation.VisibleForTesting
+import mozilla.components.ExperimentalAndroidComponentsApi
 import mozilla.components.browser.engine.fission.GeckoWebContentIsolationMapper.intoWebContentIsolationStrategy
 import mozilla.components.browser.engine.gecko.activity.GeckoActivityDelegate
 import mozilla.components.browser.engine.gecko.activity.GeckoScreenOrientationDelegate
+import mozilla.components.browser.engine.gecko.autofill.DefaultRuntimeAddressStructureAccessor
+import mozilla.components.browser.engine.gecko.autofill.RuntimeAddressStructureAccessor
 import mozilla.components.browser.engine.gecko.ext.getAntiTrackingPolicy
 import mozilla.components.browser.engine.gecko.ext.getEtpCategory
 import mozilla.components.browser.engine.gecko.ext.getEtpLevel
@@ -19,9 +23,16 @@ import mozilla.components.browser.engine.gecko.ext.getStrictSocialTrackingProtec
 import mozilla.components.browser.engine.gecko.integration.LocaleSettingUpdater
 import mozilla.components.browser.engine.gecko.mediaquery.from
 import mozilla.components.browser.engine.gecko.mediaquery.toGeckoValue
+import mozilla.components.browser.engine.gecko.preferences.DefaultGeckoPreferenceAccessor
+import mozilla.components.browser.engine.gecko.preferences.GeckoPreferenceAccessor
+import mozilla.components.browser.engine.gecko.preferences.GeckoPreferenceObserverDelegate
+import mozilla.components.browser.engine.gecko.preferences.GeckoPreferencesUtils.intoBrowserPreference
+import mozilla.components.browser.engine.gecko.preferences.GeckoPreferencesUtils.intoGeckoBranch
+import mozilla.components.browser.engine.gecko.preferences.GeckoPreferencesUtils.intoSetGeckoPreference
 import mozilla.components.browser.engine.gecko.profiler.Profiler
 import mozilla.components.browser.engine.gecko.serviceworker.GeckoServiceWorkerDelegate
-import mozilla.components.browser.engine.gecko.translate.GeckoTranslationUtils.intoTranslationError
+import mozilla.components.browser.engine.gecko.translate.DefaultRuntimeTranslationAccessor
+import mozilla.components.browser.engine.gecko.translate.RuntimeTranslationAccessor
 import mozilla.components.browser.engine.gecko.util.SpeculativeSessionFactory
 import mozilla.components.browser.engine.gecko.webextension.GeckoWebExtension
 import mozilla.components.browser.engine.gecko.webextension.GeckoWebExtensionException
@@ -40,18 +51,21 @@ import mozilla.components.concept.engine.EngineView
 import mozilla.components.concept.engine.Settings
 import mozilla.components.concept.engine.activity.ActivityDelegate
 import mozilla.components.concept.engine.activity.OrientationDelegate
+import mozilla.components.concept.engine.autofill.AddressStructure
 import mozilla.components.concept.engine.content.blocking.TrackerLog
 import mozilla.components.concept.engine.content.blocking.TrackingProtectionExceptionStorage
 import mozilla.components.concept.engine.fission.WebContentIsolationStrategy
 import mozilla.components.concept.engine.history.HistoryTrackingDelegate
 import mozilla.components.concept.engine.mediaquery.PreferredColorScheme
+import mozilla.components.concept.engine.preferences.Branch
+import mozilla.components.concept.engine.preferences.BrowserPrefObserverDelegate
+import mozilla.components.concept.engine.preferences.BrowserPreference
+import mozilla.components.concept.engine.preferences.BrowserPreferencesRuntime
+import mozilla.components.concept.engine.preferences.SetBrowserPreference
 import mozilla.components.concept.engine.serviceworker.ServiceWorkerDelegate
-import mozilla.components.concept.engine.translate.Language
 import mozilla.components.concept.engine.translate.LanguageModel
 import mozilla.components.concept.engine.translate.LanguageSetting
 import mozilla.components.concept.engine.translate.ModelManagementOptions
-import mozilla.components.concept.engine.translate.ModelState
-import mozilla.components.concept.engine.translate.TranslationError
 import mozilla.components.concept.engine.translate.TranslationSupport
 import mozilla.components.concept.engine.translate.TranslationsRuntime
 import mozilla.components.concept.engine.utils.EngineVersion
@@ -74,12 +88,12 @@ import org.mozilla.geckoview.AllowOrDeny
 import org.mozilla.geckoview.ContentBlocking
 import org.mozilla.geckoview.ContentBlockingController
 import org.mozilla.geckoview.ContentBlockingController.Event
+import org.mozilla.geckoview.ExperimentalGeckoViewApi
 import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoRuntime
 import org.mozilla.geckoview.GeckoRuntimeSettings
 import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoWebExecutor
-import org.mozilla.geckoview.TranslationsController
 import org.mozilla.geckoview.WebExtensionController
 import org.mozilla.geckoview.WebNotification
 import java.lang.ref.WeakReference
@@ -97,7 +111,10 @@ class GeckoEngine(
     executorProvider: () -> GeckoWebExecutor = { GeckoWebExecutor(runtime) },
     override val trackingProtectionExceptionStore: TrackingProtectionExceptionStorage =
         GeckoTrackingProtectionExceptionStorage(runtime),
-) : Engine, WebExtensionRuntime, TranslationsRuntime {
+    private val geckoPreferenceAccessor: GeckoPreferenceAccessor = DefaultGeckoPreferenceAccessor(),
+    private val runtimeTranslationAccessor: RuntimeTranslationAccessor = DefaultRuntimeTranslationAccessor(),
+    private val addressStructureAccessor: RuntimeAddressStructureAccessor = DefaultRuntimeAddressStructureAccessor(),
+) : Engine, WebExtensionRuntime, TranslationsRuntime, BrowserPreferencesRuntime {
     private val executor by lazy { executorProvider.invoke() }
     private val localeUpdater = LocaleSettingUpdater(context, runtime)
 
@@ -143,6 +160,16 @@ class GeckoEngine(
         }
     }
 
+    /**
+     * Notifies Gecko observers of "mobile-telemetry-pref-changed".
+     * The data is "enabled" or "disabled" based on [enabled]
+     *
+     * @param enabled Whether the user enabled or disabled telemetry
+     */
+    override fun notifyTelemetryPrefChanged(enabled: Boolean) {
+        runtime.notifyTelemetryPrefChanged(enabled)
+    }
+
     /**
      * Fetch a list of trackers logged for a given [session] .
      *
@@ -340,7 +367,6 @@ class GeckoEngine(
     /**
      * See [Engine.registerWebExtensionDelegate].
      */
-    @Suppress("Deprecation")
     override fun registerWebExtensionDelegate(
         webExtensionDelegate: WebExtensionDelegate,
     ) {
@@ -352,6 +378,7 @@ class GeckoEngine(
                 ext: org.mozilla.geckoview.WebExtension,
                 permissions: Array<out String>,
                 origins: Array<out String>,
+                dataCollectionPermissions: Array<out String>,
             ): GeckoResult<NativePermissionPromptResponse>? {
                 val result = GeckoResult<NativePermissionPromptResponse>()
 
@@ -359,11 +386,13 @@ class GeckoEngine(
                     GeckoWebExtension(ext, runtime),
                     permissions.toList(),
                     origins.toList(),
+                    dataCollectionPermissions.toList(),
                 ) { data ->
                     result.complete(
                         NativePermissionPromptResponse(
                             data.isPermissionsGranted,
                             data.isPrivateModeGranted,
+                            data.isTechnicalAndInteractionDataGranted,
                         ),
                     )
                 }
@@ -372,18 +401,17 @@ class GeckoEngine(
             }
 
             override fun onUpdatePrompt(
-                current: org.mozilla.geckoview.WebExtension,
-                updated: org.mozilla.geckoview.WebExtension,
+                extension: org.mozilla.geckoview.WebExtension,
                 newPermissions: Array<out String>,
                 newOrigins: Array<out String>,
+                newDataCollectionPermissions: Array<out String>,
             ): GeckoResult<AllowOrDeny>? {
                 val result = GeckoResult<AllowOrDeny>()
                 webExtensionDelegate.onUpdatePermissionRequest(
-                    GeckoWebExtension(current, runtime),
-                    GeckoWebExtension(updated, runtime),
-                    // We pass both permissions and origins as a single list of
-                    // permissions to be shown to the user.
-                    newPermissions.toList() + newOrigins.toList(),
+                    GeckoWebExtension(extension, runtime),
+                    newPermissions.toList(),
+                    newOrigins.toList(),
+                    newDataCollectionPermissions.toList(),
                 ) { allow ->
                     if (allow) result.complete(AllowOrDeny.ALLOW) else result.complete(AllowOrDeny.DENY)
                 }
@@ -394,12 +422,14 @@ class GeckoEngine(
                 extension: org.mozilla.geckoview.WebExtension,
                 permissions: Array<out String>,
                 origins: Array<out String>,
+                dataCollectionPermissions: Array<out String>,
             ): GeckoResult<AllowOrDeny>? {
                 val result = GeckoResult<AllowOrDeny>()
                 webExtensionDelegate.onOptionalPermissionsRequest(
                     GeckoWebExtension(extension, runtime),
                     permissions.toList(),
                     origins.toList(),
+                    dataCollectionPermissions.toList(),
                 ) { allow ->
                     if (allow) result.complete(AllowOrDeny.ALLOW) else result.complete(AllowOrDeny.DENY)
                 }
@@ -472,8 +502,7 @@ class GeckoEngine(
     override fun listInstalledWebExtensions(onSuccess: (List<WebExtension>) -> Unit, onError: (Throwable) -> Unit) {
         runtime.webExtensionController.list().then(
             {
-                val extensions = it?.map {
-                        extension ->
+                val extensions = it?.map { extension ->
                     GeckoWebExtension(extension, runtime)
                 } ?: emptyList()
 
@@ -521,11 +550,16 @@ class GeckoEngine(
         extensionId: String,
         permissions: List<String>,
         origins: List<String>,
+        dataCollectionPermissions: List<String>,
         onSuccess: (WebExtension) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        if (permissions.isEmpty() && origins.isEmpty()) {
-            onError(IllegalStateException("Either permissions or origins must not be empty"))
+        if (permissions.isEmpty() && origins.isEmpty() && dataCollectionPermissions.isEmpty()) {
+            onError(
+                IllegalStateException(
+                    "At least one of permissions, origins or dataCollectionPermissions must not be empty",
+                ),
+            )
             return
         }
 
@@ -533,6 +567,7 @@ class GeckoEngine(
             extensionId,
             permissions.toTypedArray(),
             origins.toTypedArray(),
+            dataCollectionPermissions.toTypedArray(),
         ).then(
             {
                 val enabledExtension = GeckoWebExtension(it!!, runtime)
@@ -553,11 +588,16 @@ class GeckoEngine(
         extensionId: String,
         permissions: List<String>,
         origins: List<String>,
+        dataCollectionPermissions: List<String>,
         onSuccess: (WebExtension) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        if (permissions.isEmpty() && origins.isEmpty()) {
-            onError(IllegalStateException("Either permissions or origins must not be empty"))
+        if (permissions.isEmpty() && origins.isEmpty() && dataCollectionPermissions.isEmpty()) {
+            onError(
+                IllegalStateException(
+                    "At least one of permissions, origins or dataCollectionPermissions must not be empty",
+                ),
+            )
             return
         }
 
@@ -565,6 +605,7 @@ class GeckoEngine(
             extensionId,
             permissions.toTypedArray(),
             origins.toTypedArray(),
+            dataCollectionPermissions.toTypedArray(),
         ).then(
             {
                 val enabledExtension = GeckoWebExtension(it!!, runtime)
@@ -717,6 +758,24 @@ class GeckoEngine(
         runtime.orientationController.delegate = null
     }
 
+    /**
+     * See [BrowserPreferencesRuntime.registerPreferenceObserverDelegate].
+     */
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun registerPrefObserverDelegate(prefObserverDelegate: BrowserPrefObserverDelegate) {
+    runtime.preferencesObserverDelegate = GeckoPreferenceObserverDelegate(
+        delegate = prefObserverDelegate,
+    )
+}
+
+    /**
+     * See [BrowserPreferencesRuntime.unregisterPreferenceObserverDelegate].
+     */
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun unregisterPrefObserverDelegate() {
+        runtime.preferencesObserverDelegate = null
+    }
+
     override fun registerServiceWorkerDelegate(serviceWorkerDelegate: ServiceWorkerDelegate) {
         runtime.serviceWorkerDelegate = GeckoServiceWorkerDelegate(
             delegate = serviceWorkerDelegate,
@@ -729,8 +788,13 @@ class GeckoEngine(
         runtime.serviceWorkerDelegate = null
     }
 
-    override fun handleWebNotificationClick(webNotification: Parcelable) {
-        (webNotification as? WebNotification)?.click()
+    override fun handleWebNotificationClick(webNotification: Parcelable, action: String?) {
+        val notification = (webNotification as? WebNotification)
+        if (action != null) {
+            notification?.click(action)
+        } else {
+            notification?.click()
+        }
     }
 
     /**
@@ -752,8 +816,7 @@ class GeckoEngine(
                 onSuccess()
                 GeckoResult<Void>()
             },
-            {
-                    throwable ->
+            { throwable ->
                 onError(throwable)
                 GeckoResult<Void>()
             },
@@ -767,19 +830,9 @@ class GeckoEngine(
         onSuccess: (Boolean) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.isTranslationsEngineSupported().then(
-            {
-                if (it != null) {
-                    onSuccess(it)
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.isTranslationsEngineSupported(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -792,19 +845,11 @@ class GeckoEngine(
         onSuccess: (Long) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.checkPairDownloadSize(fromLanguage, toLanguage).then(
-            {
-                if (it != null) {
-                    onSuccess(it)
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getTranslationsPairDownloadSize(
+            fromLanguage = fromLanguage,
+            toLanguage = toLanguage,
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -815,29 +860,9 @@ class GeckoEngine(
         onSuccess: (List<LanguageModel>) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.listModelDownloadStates().then(
-            {
-                if (it != null) {
-                    val listOfModels = mutableListOf<LanguageModel>()
-                    for (each in it) {
-                        val language = each.language?.let {
-                                language ->
-                            Language(language.code, each.language?.localizedDisplayName)
-                        }
-                        val status = if (each.isDownloaded) ModelState.DOWNLOADED else ModelState.NOT_DOWNLOADED
-                        val model = LanguageModel(language, status, each.size)
-                        listOfModels.add(model)
-                    }
-                    onSuccess(listOfModels)
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getTranslationsModelDownloadStates(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -848,34 +873,9 @@ class GeckoEngine(
         onSuccess: (TranslationSupport) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.listSupportedLanguages().then(
-            {
-                if (it != null) {
-                    val listOfFromLanguages = mutableListOf<Language>()
-                    val listOfToLanguages = mutableListOf<Language>()
-
-                    if (it.fromLanguages != null) {
-                        for (each in it.fromLanguages!!) {
-                            listOfFromLanguages.add(Language(each.code, each.localizedDisplayName))
-                        }
-                    }
-
-                    if (it.toLanguages != null) {
-                        for (each in it.toLanguages!!) {
-                            listOfToLanguages.add(Language(each.code, each.localizedDisplayName))
-                        }
-                    }
-
-                    onSuccess(TranslationSupport(listOfFromLanguages, listOfToLanguages))
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getSupportedTranslationLanguages(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -887,22 +887,10 @@ class GeckoEngine(
         onSuccess: () -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        val geckoOptions =
-            TranslationsController.RuntimeTranslation.ModelManagementOptions.Builder()
-                .operation(options.operation.toString())
-                .operationLevel(options.operationLevel.toString())
-
-        options.languageToManage?.let { geckoOptions.languageToManage(it) }
-
-        TranslationsController.RuntimeTranslation.manageLanguageModel(geckoOptions.build()).then(
-            {
-                onSuccess()
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.manageTranslationsLanguageModel(
+            options = options,
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -913,20 +901,9 @@ class GeckoEngine(
         onSuccess: (List<String>) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.preferredLanguages().then(
-            {
-                if (it != null) {
-                    onSuccess(it)
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getUserPreferredLanguages(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -952,24 +929,10 @@ class GeckoEngine(
         onSuccess: (LanguageSetting) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.getLanguageSetting(languageCode).then(
-            {
-                if (it != null) {
-                    try {
-                        onSuccess(LanguageSetting.fromValue(it))
-                    } catch (e: IllegalArgumentException) {
-                        onError(e.intoTranslationError())
-                    }
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getLanguageSetting(
+            languageCode = languageCode,
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -982,15 +945,11 @@ class GeckoEngine(
         onSuccess: () -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.setLanguageSettings(languageCode, languageSetting.toString()).then(
-            {
-                onSuccess()
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.setLanguageSetting(
+            languageCode = languageCode,
+            languageSetting = languageSetting,
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -1001,27 +960,9 @@ class GeckoEngine(
         onSuccess: (Map<String, LanguageSetting>) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.getLanguageSettings().then(
-            {
-                if (it != null) {
-                    try {
-                        val result = mutableMapOf<String, LanguageSetting>()
-                        it.forEach { item ->
-                            result[item.key] = LanguageSetting.fromValue(item.value)
-                        }
-                        onSuccess(result)
-                    } catch (e: IllegalArgumentException) {
-                        onError(e.intoTranslationError())
-                    }
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getLanguageSettings(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -1032,23 +973,9 @@ class GeckoEngine(
         onSuccess: (List<String>) -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.getNeverTranslateSiteList().then(
-            {
-                if (it != null) {
-                    try {
-                        onSuccess(it)
-                    } catch (e: IllegalArgumentException) {
-                        onError(e.intoTranslationError())
-                    }
-                } else {
-                    onError(TranslationError.UnexpectedNull())
-                }
-                GeckoResult<Void>()
-            },
-            { throwable ->
-                onError(throwable.intoTranslationError())
-                GeckoResult<Void>()
-            },
+        runtimeTranslationAccessor.getNeverTranslateSiteList(
+            onSuccess = onSuccess,
+            onError = onError,
         )
     }
 
@@ -1061,22 +988,248 @@ class GeckoEngine(
         onSuccess: () -> Unit,
         onError: (Throwable) -> Unit,
     ) {
-        TranslationsController.RuntimeTranslation.setNeverTranslateSpecifiedSite(setting, origin).then(
+        runtimeTranslationAccessor.setNeverTranslateSpecifiedSite(
+            origin = origin,
+            neverTranslate = setting,
+            onSuccess = onSuccess,
+            onError = onError,
+        )
+    }
+
+    /**
+     * See [BrowserPreferencesRuntime.registerPrefForObservation].
+     */
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun registerPrefForObservation(
+        pref: String,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.registerGeckoPrefForObservation(pref).then(
             {
                 onSuccess()
                 GeckoResult<Void>()
             },
             { throwable ->
-                onError(throwable.intoTranslationError())
+                onError(throwable)
                 GeckoResult<Void>()
             },
         )
     }
 
+    /**
+     * See [BrowserPreferencesRuntime.unregisterPrefForObservation].
+     */
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun unregisterPrefForObservation(
+        pref: String,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.unregisterGeckoPrefForObservation(pref).then(
+            {
+                onSuccess()
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.getBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun getBrowserPref(
+        pref: String,
+        onSuccess: (BrowserPreference<*>) -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.getGeckoPref(pref).then(
+            { geckoPrefResult ->
+                if (geckoPrefResult != null) {
+                    onSuccess(
+                        geckoPrefResult.intoBrowserPreference(),
+                    )
+                } else {
+                    onError(Throwable("Browser preference was unexpectedly null!"))
+                }
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.setBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    @OptIn(ExperimentalGeckoViewApi::class)
+    override fun getBrowserPrefs(
+        prefs: List<String>,
+        onSuccess: (List<BrowserPreference<*>>) -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.getGeckoPrefs(prefs).then(
+            { geckoPrefsResult ->
+                if (geckoPrefsResult != null) {
+                    onSuccess(
+                        geckoPrefsResult.map { pref -> pref.intoBrowserPreference() },
+                    )
+                } else {
+                    onError(Throwable("The browser preferences list was unexpectedly null!"))
+                }
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.setBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    override fun setBrowserPref(
+        pref: String,
+        value: String,
+        branch: Branch,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.setGeckoPref(pref, value, branch.intoGeckoBranch()).then(
+            {
+                onSuccess()
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.setBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    override fun setBrowserPref(
+        pref: String,
+        value: Int,
+        branch: Branch,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.setGeckoPref(pref, value, branch.intoGeckoBranch()).then(
+            {
+                onSuccess()
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.setBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    override fun setBrowserPref(
+        pref: String,
+        value: Boolean,
+        branch: Branch,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.setGeckoPref(pref, value, branch.intoGeckoBranch()).then(
+            {
+                onSuccess()
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                GeckoResult<Void>()
+            },
+        )
+    }
+
+    /**
+     * See [Engine.setBrowserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    @OptIn(ExperimentalGeckoViewApi::class)
+    @Suppress("TooGenericExceptionCaught")
+    override fun setBrowserPrefs(
+        prefs: List<SetBrowserPreference<*>>,
+        onSuccess: (Map<String, Boolean>) -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        try {
+            val geckoPrefs = prefs.map { pref -> pref.intoSetGeckoPreference() }
+            geckoPreferenceAccessor.setGeckoPrefs(geckoPrefs).then(
+                { geckoPrefsResult ->
+                    if (geckoPrefsResult != null) {
+                        onSuccess(
+                            geckoPrefsResult,
+                        )
+                    } else {
+                        onError(Throwable("The browser preferences map was unexpectedly null!"))
+                    }
+                    GeckoResult<Void>()
+                },
+                { throwable ->
+                    onError(throwable)
+                    GeckoResult<Void>()
+                },
+            )
+        } catch (throwable: Throwable) {
+            onError(throwable)
+        }
+    }
+
+    /**
+     * See [Engine.clearBrowserUserPref].
+     */
+    @ExperimentalAndroidComponentsApi
+    override fun clearBrowserUserPref(
+        pref: String,
+        onSuccess: () -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        geckoPreferenceAccessor.clearGeckoUserPref(pref).then(
+            {
+                onSuccess()
+                GeckoResult<Void>()
+            },
+            { throwable ->
+                onError(throwable)
+                   GeckoResult<Void>()
+            },
+        )
+    }
+
+    override fun getAddressStructure(
+        countryCode: String,
+        onSuccess: (AddressStructure) -> Unit,
+        onError: (Throwable) -> Unit,
+    ) {
+        addressStructureAccessor.getAddressStructure(countryCode, onSuccess, onError)
+    }
+
     /**
      * See [Engine.profiler].
      */
-    override val profiler: Profiler? = Profiler(runtime)
+    override val profiler: Profiler = Profiler()
 
     override fun name(): String = "Gecko"
 
@@ -1147,6 +1300,18 @@ class GeckoEngine(
                         if (cookiePurging != value.cookiePurging) {
                             setCookiePurging(value.cookiePurging)
                         }
+
+                        if (getBounceTrackingProtectionMode() != policy.bounceTrackingProtectionMode.mode) {
+                            setBounceTrackingProtectionMode(policy.bounceTrackingProtectionMode.mode)
+                        }
+
+                        if (allowListBaselineTrackingProtection != value.allowListBaselineTrackingProtection) {
+                            setAllowListBaselineTrackingProtection(value.allowListBaselineTrackingProtection)
+                        }
+
+                        if (allowListConvenienceTrackingProtection != value.allowListConvenienceTrackingProtection) {
+                            setAllowListConvenienceTrackingProtection(value.allowListConvenienceTrackingProtection)
+                        }
                     }
 
                     defaultSettings?.trackingProtectionPolicy = value
@@ -1449,6 +1614,18 @@ class GeckoEngine(
             get() = runtime.settings.userCharacteristicPingCurrentVersion
             set(value) { runtime.settings.setUserCharacteristicPingCurrentVersion(value) }
 
+        override var baselineFingerprintingProtection: Boolean?
+            get() = runtime.settings.baselineFingerprintingProtection
+            set(value) {
+                value?.let { runtime.settings.setBaselineFingerprintingProtection(it) }
+            }
+
+        override var baselineFingerprintingProtectionOverrides: String?
+            get() = runtime.settings.baselineFingerprintingProtectionOverrides
+            set(value) {
+                value?.let { runtime.settings.setBaselineFingerprintingProtectionOverrides(it) }
+            }
+
         override var webContentIsolationStrategy: WebContentIsolationStrategy?
             get() = runtime.settings.webContentIsolationStrategy?.intoWebContentIsolationStrategy()
             set(value) {
@@ -1471,13 +1648,33 @@ class GeckoEngine(
             get() = runtime.settings.cookieBehaviorOptInPartitioningPBM
             set(value) { runtime.settings.setCookieBehaviorOptInPartitioningPBM(value) }
 
-        override var certificateTransparencyMode: Int
-            get() = runtime.settings.certificateTransparencyMode
-            set(value) { runtime.settings.setCertificateTransparencyMode(value) }
+        override var certificateTransparencyMode: Int?
+            get() = runtime.settings.certificateTransparencyMode.or(2)
+            set(value) { value?.let { runtime.settings.setCertificateTransparencyMode(value) } }
 
-        override var postQuantumKeyExchangeEnabled: Boolean
-            get() = runtime.settings.postQuantumKeyExchangeEnabled
-            set(value) { runtime.settings.setPostQuantumKeyExchangeEnabled(value) }
+        override var postQuantumKeyExchangeEnabled: Boolean?
+            get() = runtime.settings.postQuantumKeyExchangeEnabled.or(false)
+            set(value) { value?.let { runtime.settings.setPostQuantumKeyExchangeEnabled(value) } }
+
+        override var dohAutoselectEnabled: Boolean
+            get() = runtime.settings.dohAutoselectEnabled
+            set(value) { runtime.settings.setDohAutoselectEnabled(value) }
+
+        override var bannedPorts: String
+            get() = runtime.settings.bannedPorts
+            set(value) { runtime.settings.setBannedPorts(value) }
+
+        override var lnaBlockingEnabled: Boolean
+            get() = runtime.settings.lnaBlockingEnabled
+            set(value) { runtime.settings.setLnaBlockingEnabled(value) }
+
+        override var crliteChannel: String?
+            get() = runtime.settings.crliteChannel
+            set(value) { value?.let { runtime.settings.setCrliteChannel(value) } }
+
+        override var safeBrowsingV5Enabled: Boolean?
+            get() = runtime.settings.contentBlocking.safeBrowsingV5Enabled.or(false)
+            set(value) { value?.let { runtime.settings.contentBlocking.setSafeBrowsingV5Enabled(value) } }
     }.apply {
         defaultSettings?.let {
             this.javascriptEnabled = it.javascriptEnabled
@@ -1513,6 +1710,8 @@ class GeckoEngine(
             this.fdlibmMathEnabled = it.fdlibmMathEnabled
             this.emailTrackerBlockingPrivateBrowsing = it.emailTrackerBlockingPrivateBrowsing
             this.userCharacteristicPingCurrentVersion = it.userCharacteristicPingCurrentVersion
+            this.baselineFingerprintingProtection = it.baselineFingerprintingProtection
+            this.baselineFingerprintingProtectionOverrides = it.baselineFingerprintingProtectionOverrides
             this.webContentIsolationStrategy = it.webContentIsolationStrategy
             this.fetchPriorityEnabled = it.fetchPriorityEnabled
             this.parallelMarkingEnabled = it.parallelMarkingEnabled
@@ -1520,10 +1719,14 @@ class GeckoEngine(
             this.cookieBehaviorOptInPartitioningPBM = it.cookieBehaviorOptInPartitioningPBM
             this.certificateTransparencyMode = it.certificateTransparencyMode
             this.postQuantumKeyExchangeEnabled = it.postQuantumKeyExchangeEnabled
+            this.dohAutoselectEnabled = it.dohAutoselectEnabled
+            this.bannedPorts = it.bannedPorts
+            this.lnaBlockingEnabled = it.lnaBlockingEnabled
+            this.crliteChannel = it.crliteChannel
+            this.safeBrowsingV5Enabled = it.safeBrowsingV5Enabled
         }
     }
 
-    @Suppress("ComplexMethod")
     internal fun ContentBlockingController.LogEntry.BlockingData.getLoadedCategory(): TrackingCategory {
         val socialTrackingProtectionEnabled = settings.trackingProtectionPolicy?.strictSocialTrackingProtection
             ?: false
