@@ -403,8 +403,11 @@ class GeckoWebExtension(
     }
 
     @VisibleForTesting
-    internal fun getIcon(size: Int): GeckoResult<Bitmap> {
-        return nativeExtension.metaData.icon.getBitmap(size)
+    internal fun getIcon(size: Int): GeckoResult<Bitmap?> {
+        return nativeExtension.metaData.icon.getBitmap(size).then(
+            { GeckoResult.fromValue(it) },
+            { GeckoResult.fromValue(null) },
+        )
     }
 
     /**
@@ -442,7 +445,12 @@ class GeckoPort(
 
 private fun GeckoNativeWebExtensionAction.convert(): Action {
     val loadIcon: (suspend (Int) -> Bitmap?)? = icon?.let {
-        { size -> icon?.getBitmap(size)?.await() }
+        { size ->
+            icon?.getBitmap(size)?.then(
+                { GeckoResult.fromValue(it) },
+                { GeckoResult.fromValue(null) },
+            )?.await()
+        }
     }
 
     val onClick = { click() }
