@@ -37,6 +37,7 @@ import mozilla.components.concept.engine.translate.TranslationEngineState
 import mozilla.components.concept.engine.translate.TranslationError
 import mozilla.components.concept.engine.translate.TranslationOperation
 import mozilla.components.concept.engine.window.WindowRequest
+import mozilla.components.concept.fetch.Headers.Names.E_TAG
 import mozilla.components.concept.fetch.Response
 import mozilla.components.lib.state.Store
 
@@ -241,6 +242,7 @@ internal class EngineObserver(
             skipConfirmation = skipConfirmation,
             openInApp = openInApp,
             response = response,
+            etag = response?.headers?.get(E_TAG),
         )
 
         store.dispatch(
