@@ -19,6 +19,7 @@ import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHig
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.NotificationChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.PersistentStorageChangedAction
 import mozilla.components.browser.state.action.ContentAction.UpdatePermissionHighlightsStateAction.Reset
+import mozilla.components.browser.state.reducer.BrowserStateReducer
 import mozilla.components.browser.state.selector.findCustomTab
 import mozilla.components.browser.state.state.AppIntentState
 import mozilla.components.browser.state.state.BrowserState
@@ -31,7 +32,6 @@ import mozilla.components.browser.state.state.content.HistoryState
 import mozilla.components.browser.state.state.content.PermissionHighlightsState
 import mozilla.components.browser.state.state.createCustomTab
 import mozilla.components.browser.state.state.createTab
-import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.HitResult
 import mozilla.components.concept.engine.history.HistoryItem
 import mozilla.components.concept.engine.manifest.WebAppManifest
@@ -40,8 +40,6 @@ import mozilla.components.concept.engine.permission.Permission.ContentGeoLocatio
 import mozilla.components.concept.engine.permission.PermissionRequest
 import mozilla.components.concept.engine.prompt.PromptRequest
 import mozilla.components.concept.engine.window.WindowRequest
-import mozilla.components.support.test.ext.joinBlocking
-import mozilla.components.support.test.libstate.ext.waitUntilIdle
 import mozilla.components.support.test.mock
 import mozilla.components.support.test.whenever
 import org.junit.Assert.assertEquals
@@ -58,19 +56,20 @@ import org.mockito.Mockito.verify
 
 @RunWith(AndroidJUnit4::class)
 class ContentActionTest {
-    private lateinit var store: BrowserStore
+    private lateinit var state: BrowserState
+
     private lateinit var tabId: String
     private lateinit var otherTabId: String
 
     private val tab: TabSessionState
-        get() = store.state.tabs.find { it.id == tabId }!!
+        get() = state.tabs.find { it.id == tabId }!!
 
     private val otherTab: TabSessionState
-        get() = store.state.tabs.find { it.id == otherTabId }!!
+        get() = state.tabs.find { it.id == otherTabId }!!
 
     @Before
     fun setUp() {
-        val state = BrowserState(
+        state = BrowserState(
             tabs = listOf(
                 createTab(url = "https://www.mozilla.org").also {
                     tabId = it.id
@@ -80,8 +79,6 @@ class ContentActionTest {
                 },
             ),
         )
-
-        store = BrowserStore(state)
     }
 
     @Test
@@ -91,9 +88,7 @@ class ContentActionTest {
         assertNotEquals(newUrl, tab.content.url)
         assertNotEquals(newUrl, otherTab.content.url)
 
-        store.dispatch(
-            ContentAction.UpdateUrlAction(tab.id, newUrl),
-        ).joinBlocking()
+        state = BrowserStateReducer.reduce(state, ContentAction.UpdateUrlAction(tab.id, newUrl))
 
         assertEquals(newUrl, tab.content.url)
         assertNotEquals(newUrl, otherTab.content.url)
@@ -106,15 +101,17 @@ class ContentActionTest {
         assertNotEquals(icon, tab.content.icon)
         assertNotEquals(icon, otherTab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateIconAction(tab.id, tab.content.url, icon),
-        ).joinBlocking()
+        )
 
         assertEquals(icon, tab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateUrlAction(tab.id, "https://www.example.org"),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.icon)
     }
@@ -126,15 +123,17 @@ class ContentActionTest {
         assertNotEquals(icon, tab.content.icon)
         assertNotEquals(icon, otherTab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateIconAction(tab.id, tab.content.url, icon),
-        ).joinBlocking()
+        )
 
         assertEquals(icon, tab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateUrlAction(tab.id, "https://www.mozilla.org/firefox"),
-        ).joinBlocking()
+        )
 
         assertEquals(icon, tab.content.icon)
     }
@@ -142,21 +141,24 @@ class ContentActionTest {
     @Test
     fun `WHEN UpdateUrlAction is dispatched by user gesture THEN the search terms are cleared`() {
         val searchTerms = "Firefox"
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateSearchTermsAction(tab.id, searchTerms),
-        ).joinBlocking()
+        )
 
         assertEquals(searchTerms, tab.content.searchTerms)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateUrlAction(tab.id, "https://www.mozilla.org", false),
-        ).joinBlocking()
+        )
 
         assertEquals(searchTerms, tab.content.searchTerms)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateUrlAction(tab.id, "https://www.mozilla.org/firefox", true),
-        ).joinBlocking()
+        )
 
         assertEquals("", tab.content.searchTerms)
     }
@@ -166,27 +168,31 @@ class ContentActionTest {
         assertFalse(tab.content.loading)
         assertFalse(otherTab.content.loading)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateLoadingStateAction(tab.id, true),
-        ).joinBlocking()
+        )
 
         assertTrue(tab.content.loading)
         assertFalse(otherTab.content.loading)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateLoadingStateAction(tab.id, false),
-        ).joinBlocking()
+        )
 
         assertFalse(tab.content.loading)
         assertFalse(otherTab.content.loading)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateLoadingStateAction(tab.id, true),
-        ).joinBlocking()
+        )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateLoadingStateAction(otherTab.id, true),
-        ).joinBlocking()
+        )
 
         assertTrue(tab.content.loading)
         assertTrue(otherTab.content.loading)
@@ -197,18 +203,30 @@ class ContentActionTest {
         assertFalse(tab.content.refreshCanceled)
         assertFalse(otherTab.content.refreshCanceled)
 
-        store.dispatch(ContentAction.UpdateRefreshCanceledStateAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateRefreshCanceledStateAction(tab.id, true),
+        )
 
         assertTrue(tab.content.refreshCanceled)
         assertFalse(otherTab.content.refreshCanceled)
 
-        store.dispatch(ContentAction.UpdateRefreshCanceledStateAction(tab.id, false)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateRefreshCanceledStateAction(tab.id, false),
+        )
 
         assertFalse(tab.content.refreshCanceled)
         assertFalse(otherTab.content.refreshCanceled)
 
-        store.dispatch(ContentAction.UpdateRefreshCanceledStateAction(tab.id, true)).joinBlocking()
-        store.dispatch(ContentAction.UpdateRefreshCanceledStateAction(otherTab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateRefreshCanceledStateAction(tab.id, true),
+        )
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateRefreshCanceledStateAction(otherTab.id, true),
+        )
 
         assertTrue(tab.content.refreshCanceled)
         assertTrue(otherTab.content.refreshCanceled)
@@ -221,9 +239,10 @@ class ContentActionTest {
         assertNotEquals(newTitle, tab.content.title)
         assertNotEquals(newTitle, otherTab.content.title)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateTitleAction(tab.id, newTitle),
-        ).joinBlocking()
+        )
 
         assertEquals(newTitle, tab.content.title)
         assertNotEquals(newTitle, otherTab.content.title)
@@ -236,9 +255,10 @@ class ContentActionTest {
         assertNotEquals(newPreviewImageUrl, tab.content.previewImageUrl)
         assertNotEquals(newPreviewImageUrl, otherTab.content.previewImageUrl)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdatePreviewImageAction(tab.id, newPreviewImageUrl),
-        ).joinBlocking()
+        )
 
         assertEquals(newPreviewImageUrl, tab.content.previewImageUrl)
         assertNotEquals(newPreviewImageUrl, otherTab.content.previewImageUrl)
@@ -249,13 +269,14 @@ class ContentActionTest {
         assertEquals(0, tab.content.progress)
         assertEquals(0, otherTab.content.progress)
 
-        store.dispatch(ContentAction.UpdateProgressAction(tab.id, 75)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, ContentAction.UpdateProgressAction(tab.id, 75))
 
         assertEquals(75, tab.content.progress)
         assertEquals(0, otherTab.content.progress)
 
-        store.dispatch(ContentAction.UpdateProgressAction(otherTab.id, 25)).joinBlocking()
-        store.dispatch(ContentAction.UpdateProgressAction(tab.id, 85)).joinBlocking()
+        state =
+            BrowserStateReducer.reduce(state, ContentAction.UpdateProgressAction(otherTab.id, 25))
+        state = BrowserStateReducer.reduce(state, ContentAction.UpdateProgressAction(tab.id, 85))
 
         assertEquals(85, tab.content.progress)
         assertEquals(25, otherTab.content.progress)
@@ -268,9 +289,10 @@ class ContentActionTest {
         assertNotEquals(searchTerms, tab.content.searchTerms)
         assertNotEquals(searchTerms, otherTab.content.searchTerms)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateSearchTermsAction(tab.id, searchTerms),
-        ).joinBlocking()
+        )
 
         assertEquals(searchTerms, tab.content.searchTerms)
         assertNotEquals(searchTerms, otherTab.content.searchTerms)
@@ -283,9 +305,10 @@ class ContentActionTest {
         assertNotEquals(newSecurityInfo, tab.content.securityInfo)
         assertNotEquals(newSecurityInfo, otherTab.content.securityInfo)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateSecurityInfoAction(tab.id, newSecurityInfo),
-        ).joinBlocking()
+        )
 
         assertEquals(newSecurityInfo, tab.content.securityInfo)
         assertNotEquals(newSecurityInfo, otherTab.content.securityInfo)
@@ -302,9 +325,10 @@ class ContentActionTest {
         assertNotEquals(icon, tab.content.icon)
         assertNotEquals(icon, otherTab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateIconAction(tab.id, tab.content.url, icon),
-        ).joinBlocking()
+        )
 
         assertEquals(icon, tab.content.icon)
         assertNotEquals(icon, otherTab.content.icon)
@@ -317,9 +341,10 @@ class ContentActionTest {
         assertNotEquals(icon, tab.content.icon)
         assertNotEquals(icon, otherTab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateIconAction(tab.id, "https://different.example.org", icon),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.icon)
     }
@@ -330,15 +355,17 @@ class ContentActionTest {
 
         assertNotEquals(icon, tab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateIconAction(tab.id, tab.content.url, icon),
-        ).joinBlocking()
+        )
 
         assertEquals(icon, tab.content.icon)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.RemoveIconAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.icon)
     }
@@ -348,14 +375,24 @@ class ContentActionTest {
         val customTab = createCustomTab("https://getpocket.com")
         val otherCustomTab = createCustomTab("https://www.google.com")
 
-        store.dispatch(CustomTabListAction.AddCustomTabAction(customTab)).joinBlocking()
-        store.dispatch(CustomTabListAction.AddCustomTabAction(otherCustomTab)).joinBlocking()
+        state =
+            BrowserStateReducer.reduce(state, CustomTabListAction.AddCustomTabAction(customTab))
+        state = BrowserStateReducer.reduce(
+            state,
+            CustomTabListAction.AddCustomTabAction(otherCustomTab),
+        )
 
-        store.dispatch(ContentAction.UpdateUrlAction(customTab.id, "https://www.example.org")).joinBlocking()
-        store.dispatch(ContentAction.UpdateTitleAction(customTab.id, "I am a custom tab")).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateUrlAction(customTab.id, "https://www.example.org"),
+        )
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateTitleAction(customTab.id, "I am a custom tab"),
+        )
 
-        val updatedCustomTab = store.state.findCustomTab(customTab.id)!!
-        val updatedOtherCustomTab = store.state.findCustomTab(otherCustomTab.id)!!
+        val updatedCustomTab = state.findCustomTab(customTab.id)!!
+        val updatedOtherCustomTab = state.findCustomTab(otherCustomTab.id)!!
 
         assertEquals("https://www.example.org", updatedCustomTab.content.url)
         assertNotEquals("https://www.example.org", updatedOtherCustomTab.content.url)
@@ -377,9 +414,10 @@ class ContentActionTest {
             sessionId = tab.id,
         )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateDownloadAction(tab.id, download1),
-        ).joinBlocking()
+        )
 
         assertEquals(download1.url, tab.content.download?.url)
         assertEquals(download1.sessionId, tab.content.download?.sessionId)
@@ -389,9 +427,10 @@ class ContentActionTest {
             sessionId = tab.id,
         )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateDownloadAction(tab.id, download2),
-        ).joinBlocking()
+        )
 
         assertEquals(download2.url, tab.content.download?.url)
         assertEquals(download2.sessionId, tab.content.download?.sessionId)
@@ -405,15 +444,17 @@ class ContentActionTest {
             sessionId = tab.id,
         )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateDownloadAction(tab.id, download),
-        ).joinBlocking()
+        )
 
         assertEquals(download, tab.content.download)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumeDownloadAction(tab.id, downloadId = "1337"),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.download)
     }
@@ -426,15 +467,17 @@ class ContentActionTest {
             sessionId = tab.id,
         )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateDownloadAction(tab.id, download),
-        ).joinBlocking()
+        )
 
         assertEquals(download, tab.content.download)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.CancelDownloadAction(tab.id, downloadId = "1337"),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.download)
     }
@@ -447,15 +490,17 @@ class ContentActionTest {
             sessionId = tab.id,
         )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateDownloadAction(tab.id, download),
-        ).joinBlocking()
+        )
 
         assertEquals(download, tab.content.download)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumeDownloadAction(tab.id, downloadId = "4223"),
-        ).joinBlocking()
+        )
 
         assertNotNull(tab.content.download)
     }
@@ -466,17 +511,19 @@ class ContentActionTest {
 
         val hitResult1: HitResult = HitResult.UNKNOWN("file://foo")
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateHitResultAction(tab.id, hitResult1),
-        ).joinBlocking()
+        )
 
         assertEquals(hitResult1, tab.content.hitResult)
 
         val hitResult2: HitResult = HitResult.UNKNOWN("file://bar")
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateHitResultAction(tab.id, hitResult2),
-        ).joinBlocking()
+        )
 
         assertEquals(hitResult2, tab.content.hitResult)
     }
@@ -485,15 +532,17 @@ class ContentActionTest {
     fun `ConsumeHitResultAction removes hit result`() {
         val hitResult: HitResult = HitResult.UNKNOWN("file://foo")
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateHitResultAction(tab.id, hitResult),
-        ).joinBlocking()
+        )
 
         assertEquals(hitResult, tab.content.hitResult)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumeHitResultAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.hitResult)
     }
@@ -504,18 +553,20 @@ class ContentActionTest {
 
         val promptRequest1: PromptRequest = mock<PromptRequest.SingleChoice>()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdatePromptRequestAction(tab.id, promptRequest1),
-        ).joinBlocking()
+        )
 
         assertEquals(1, tab.content.promptRequests.size)
         assertEquals(promptRequest1, tab.content.promptRequests[0])
 
         val promptRequest2: PromptRequest = mock<PromptRequest.MultipleChoice>()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdatePromptRequestAction(tab.id, promptRequest2),
-        ).joinBlocking()
+        )
 
         assertEquals(2, tab.content.promptRequests.size)
         assertEquals(promptRequest1, tab.content.promptRequests[0])
@@ -526,16 +577,18 @@ class ContentActionTest {
     fun `ConsumePromptRequestAction removes request`() {
         val promptRequest: PromptRequest = mock<PromptRequest.SingleChoice>()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdatePromptRequestAction(tab.id, promptRequest),
-        ).joinBlocking()
+        )
 
         assertEquals(1, tab.content.promptRequests.size)
         assertEquals(promptRequest, tab.content.promptRequests[0])
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumePromptRequestAction(tab.id, promptRequest),
-        ).joinBlocking()
+        )
 
         assertTrue(tab.content.promptRequests.isEmpty())
     }
@@ -545,17 +598,19 @@ class ContentActionTest {
         assertTrue(tab.content.findResults.isEmpty())
 
         val result: FindResultState = mock()
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.AddFindResultAction(tab.id, result),
-        ).joinBlocking()
+        )
 
         assertEquals(1, tab.content.findResults.size)
         assertEquals(result, tab.content.findResults.last())
 
         val result2: FindResultState = mock()
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.AddFindResultAction(tab.id, result2),
-        ).joinBlocking()
+        )
 
         assertEquals(2, tab.content.findResults.size)
         assertEquals(result2, tab.content.findResults.last())
@@ -563,19 +618,22 @@ class ContentActionTest {
 
     @Test
     fun `ClearFindResultsAction removes all results`() {
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.AddFindResultAction(tab.id, mock()),
-        ).joinBlocking()
+        )
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.AddFindResultAction(tab.id, mock()),
-        ).joinBlocking()
+        )
 
         assertEquals(2, tab.content.findResults.size)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ClearFindResultsAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertTrue(tab.content.findResults.isEmpty())
     }
@@ -586,17 +644,19 @@ class ContentActionTest {
 
         val windowRequest1: WindowRequest = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateWindowRequestAction(tab.id, windowRequest1),
-        ).joinBlocking()
+        )
 
         assertEquals(windowRequest1, tab.content.windowRequest)
 
         val windowRequest2: WindowRequest = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateWindowRequestAction(tab.id, windowRequest2),
-        ).joinBlocking()
+        )
 
         assertEquals(windowRequest2, tab.content.windowRequest)
     }
@@ -605,15 +665,17 @@ class ContentActionTest {
     fun `ConsumeWindowRequestAction removes request`() {
         val windowRequest: WindowRequest = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateWindowRequestAction(tab.id, windowRequest),
-        ).joinBlocking()
+        )
 
         assertEquals(windowRequest, tab.content.windowRequest)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumeWindowRequestAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.windowRequest)
     }
@@ -623,12 +685,18 @@ class ContentActionTest {
         assertFalse(tab.content.canGoBack)
         assertFalse(otherTab.content.canGoBack)
 
-        store.dispatch(ContentAction.UpdateBackNavigationStateAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateBackNavigationStateAction(tab.id, true),
+        )
 
         assertTrue(tab.content.canGoBack)
         assertFalse(otherTab.content.canGoBack)
 
-        store.dispatch(ContentAction.UpdateBackNavigationStateAction(tab.id, false)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateBackNavigationStateAction(tab.id, false),
+        )
 
         assertFalse(tab.content.canGoBack)
         assertFalse(otherTab.content.canGoBack)
@@ -639,12 +707,18 @@ class ContentActionTest {
         assertFalse(tab.content.canGoForward)
         assertFalse(otherTab.content.canGoForward)
 
-        store.dispatch(ContentAction.UpdateForwardNavigationStateAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateForwardNavigationStateAction(tab.id, true),
+        )
 
         assertTrue(tab.content.canGoForward)
         assertFalse(otherTab.content.canGoForward)
 
-        store.dispatch(ContentAction.UpdateForwardNavigationStateAction(tab.id, false)).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateForwardNavigationStateAction(tab.id, false),
+        )
 
         assertFalse(tab.content.canGoForward)
         assertFalse(otherTab.content.canGoForward)
@@ -660,9 +734,10 @@ class ContentActionTest {
         assertNotEquals(manifest, tab.content.webAppManifest)
         assertNotEquals(manifest, otherTab.content.webAppManifest)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateWebAppManifestAction(tab.id, manifest),
-        ).joinBlocking()
+        )
 
         assertEquals(manifest, tab.content.webAppManifest)
         assertNotEquals(manifest, otherTab.content.webAppManifest)
@@ -677,15 +752,17 @@ class ContentActionTest {
 
         assertNotEquals(manifest, tab.content.webAppManifest)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateWebAppManifestAction(tab.id, manifest),
-        ).joinBlocking()
+        )
 
         assertEquals(manifest, tab.content.webAppManifest)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.RemoveWebAppManifestAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.webAppManifest)
     }
@@ -703,9 +780,14 @@ class ContentActionTest {
         assertNotEquals(historyState, tab.content.history)
         assertNotEquals(historyState, otherTab.content.history)
 
-        store.dispatch(
-            ContentAction.UpdateHistoryStateAction(tab.id, historyState.items, historyState.currentIndex),
-        ).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateHistoryStateAction(
+                tab.id,
+                historyState.items,
+                historyState.currentIndex,
+            ),
+        )
 
         assertEquals(historyState, tab.content.history)
         assertNotEquals(historyState, otherTab.content.history)
@@ -715,9 +797,13 @@ class ContentActionTest {
     fun `UpdateLoadRequestAction updates load request state`() {
         val loadRequestUrl = "https://mozilla.org"
 
-        store.dispatch(
-            ContentAction.UpdateLoadRequestAction(tab.id, LoadRequestState(loadRequestUrl, true, false)),
-        ).joinBlocking()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateLoadRequestAction(
+                tab.id,
+                LoadRequestState(loadRequestUrl, true, false),
+            ),
+        )
 
         assertNotNull(tab.content.loadRequest)
         assertEquals(loadRequestUrl, tab.content.loadRequest!!.url)
@@ -730,12 +816,13 @@ class ContentActionTest {
         assertFalse(tab.content.desktopMode)
         assertFalse(otherTab.content.desktopMode)
 
-        store.dispatch(ContentAction.UpdateTabDesktopMode(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, ContentAction.UpdateTabDesktopMode(tab.id, true))
 
         assertTrue(tab.content.desktopMode)
         assertFalse(otherTab.content.desktopMode)
 
-        store.dispatch(ContentAction.UpdateTabDesktopMode(tab.id, false)).joinBlocking()
+        state =
+            BrowserStateReducer.reduce(state, ContentAction.UpdateTabDesktopMode(tab.id, false))
 
         assertFalse(tab.content.desktopMode)
         assertFalse(otherTab.content.desktopMode)
@@ -745,7 +832,7 @@ class ContentActionTest {
     fun `WHEN dispatching NotificationChangedAction THEN notificationChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.notificationChanged)
 
-        store.dispatch(NotificationChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, NotificationChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.notificationChanged)
     }
@@ -754,7 +841,7 @@ class ContentActionTest {
     fun `WHEN dispatching CameraChangedAction THEN cameraChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.cameraChanged)
 
-        store.dispatch(CameraChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, CameraChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.cameraChanged)
     }
@@ -763,7 +850,7 @@ class ContentActionTest {
     fun `WHEN dispatching LocationChangedAction THEN locationChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.locationChanged)
 
-        store.dispatch(LocationChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, LocationChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.locationChanged)
     }
@@ -772,7 +859,7 @@ class ContentActionTest {
     fun `WHEN dispatching MicrophoneChangedAction THEN locationChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.microphoneChanged)
 
-        store.dispatch(MicrophoneChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, MicrophoneChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.microphoneChanged)
     }
@@ -781,7 +868,7 @@ class ContentActionTest {
     fun `WHEN dispatching PersistentStorageChangedAction THEN persistentStorageChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.persistentStorageChanged)
 
-        store.dispatch(PersistentStorageChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, PersistentStorageChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.persistentStorageChanged)
     }
@@ -790,7 +877,7 @@ class ContentActionTest {
     fun `WHEN dispatching MediaKeySystemAccesChangedAction THEN mediaKeySystemAccessChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.mediaKeySystemAccessChanged)
 
-        store.dispatch(MediaKeySystemAccesChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, MediaKeySystemAccesChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.mediaKeySystemAccessChanged)
     }
@@ -799,7 +886,7 @@ class ContentActionTest {
     fun `WHEN dispatching LocalDeviceAccessChangedAction THEN localDeviceAccessChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.localDeviceAccessChanged)
 
-        store.dispatch(LocalDeviceAccessChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, LocalDeviceAccessChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.localDeviceAccessChanged)
     }
@@ -808,7 +895,7 @@ class ContentActionTest {
     fun `WHEN dispatching LocalNetworkAccessChangedAction THEN localNetworkAccessChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.localNetworkAccessChanged)
 
-        store.dispatch(LocalNetworkAccessChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, LocalNetworkAccessChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.localNetworkAccessChanged)
     }
@@ -817,7 +904,7 @@ class ContentActionTest {
     fun `WHEN dispatching AutoPlayAudibleChangedAction THEN autoPlayAudibleChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.autoPlayAudibleChanged)
 
-        store.dispatch(AutoPlayAudibleChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, AutoPlayAudibleChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.autoPlayAudibleChanged)
     }
@@ -826,7 +913,7 @@ class ContentActionTest {
     fun `WHEN dispatching AutoPlayInAudibleChangedAction THEN autoPlayAudibleChanged state will be updated`() {
         assertFalse(tab.content.permissionHighlights.autoPlayInaudibleChanged)
 
-        store.dispatch(AutoPlayInAudibleChangedAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, AutoPlayInAudibleChangedAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.autoPlayInaudibleChanged)
     }
@@ -835,7 +922,7 @@ class ContentActionTest {
     fun `WHEN dispatching AutoPlayAudibleBlockingAction THEN autoPlayAudibleBlocking state will be updated`() {
         assertFalse(tab.content.permissionHighlights.autoPlayAudibleBlocking)
 
-        store.dispatch(AutoPlayAudibleBlockingAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, AutoPlayAudibleBlockingAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.autoPlayAudibleBlocking)
     }
@@ -844,21 +931,21 @@ class ContentActionTest {
     fun `WHEN dispatching AutoPlayInAudibleBlockingAction THEN autoPlayInaudibleBlocking state will be updated`() {
         assertFalse(tab.content.permissionHighlights.autoPlayInaudibleBlocking)
 
-        store.dispatch(AutoPlayInAudibleBlockingAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, AutoPlayInAudibleBlockingAction(tab.id, true))
 
         assertTrue(tab.content.permissionHighlights.autoPlayInaudibleBlocking)
     }
 
     @Test
     fun `WHEN dispatching Reset THEN permissionHighlights state will be update to its default value`() {
-        store.dispatch(AutoPlayInAudibleBlockingAction(tab.id, true)).joinBlocking()
+        state = BrowserStateReducer.reduce(state, AutoPlayInAudibleBlockingAction(tab.id, true))
 
         assertEquals(
             PermissionHighlightsState(autoPlayInaudibleBlocking = true),
             tab.content.permissionHighlights,
         )
 
-        with(store) { dispatch(Reset(tab.id)).joinBlocking() }
+        state = BrowserStateReducer.reduce(state, Reset(tab.id))
 
         assertEquals(PermissionHighlightsState(), tab.content.permissionHighlights)
     }
@@ -869,17 +956,19 @@ class ContentActionTest {
 
         val appIntent1: AppIntentState = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateAppIntentAction(tab.id, appIntent1),
-        ).joinBlocking()
+        )
 
         assertEquals(appIntent1, tab.content.appIntent)
 
         val appIntent2: AppIntentState = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateAppIntentAction(tab.id, appIntent2),
-        ).joinBlocking()
+        )
 
         assertEquals(appIntent2, tab.content.appIntent)
     }
@@ -888,15 +977,17 @@ class ContentActionTest {
     fun `ConsumeAppIntentAction removes request`() {
         val appIntent: AppIntentState = mock()
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateAppIntentAction(tab.id, appIntent),
-        ).joinBlocking()
+        )
 
         assertEquals(appIntent, tab.content.appIntent)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.ConsumeAppIntentAction(tab.id),
-        ).joinBlocking()
+        )
 
         assertNull(tab.content.appIntent)
     }
@@ -905,15 +996,17 @@ class ContentActionTest {
     fun `CheckForFormDataAction updates hasFormData`() {
         assertFalse(tab.content.hasFormData)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateHasFormDataAction(tab.id, true),
-        ).joinBlocking()
+        )
 
         assertTrue(tab.content.hasFormData)
 
-        store.dispatch(
+        state = BrowserStateReducer.reduce(
+            state,
             ContentAction.UpdateHasFormDataAction(tab.id, false),
-        ).joinBlocking()
+        )
 
         assertFalse(tab.content.hasFormData)
     }
@@ -931,9 +1024,14 @@ class ContentActionTest {
             whenever(uri).thenReturn(url)
         }
 
-        store.dispatch(ContentAction.UpdatePermissionsRequest(tab.id, request1))
-        store.dispatch(ContentAction.UpdatePermissionsRequest(tab.id, request2))
-        store.waitUntilIdle()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdatePermissionsRequest(tab.id, request1),
+        )
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdatePermissionsRequest(tab.id, request2),
+        )
 
         verify(request1).merge(request2)
     }
@@ -947,9 +1045,14 @@ class ContentActionTest {
             whenever(permissions).thenReturn(listOf(AppLocationCoarse(id = "permission")))
         }
 
-        store.dispatch(ContentAction.UpdateAppPermissionsRequest(tab.id, request1))
-        store.dispatch(ContentAction.UpdateAppPermissionsRequest(tab.id, request2))
-        store.waitUntilIdle()
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateAppPermissionsRequest(tab.id, request1),
+        )
+        state = BrowserStateReducer.reduce(
+            state,
+            ContentAction.UpdateAppPermissionsRequest(tab.id, request2),
+        )
 
         verify(request1).merge(request2)
     }
