@@ -17,8 +17,8 @@ import androidx.annotation.ColorRes
 import androidx.annotation.DrawableRes
 import androidx.annotation.VisibleForTesting
 import androidx.annotation.VisibleForTesting.Companion.PRIVATE
+import androidx.appcompat.content.res.AppCompatResources
 import androidx.coordinatorlayout.widget.CoordinatorLayout
-import androidx.core.content.ContextCompat
 import androidx.core.view.forEach
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.Dispatchers
@@ -42,7 +42,7 @@ import kotlin.coroutines.CoroutineContext
 
 internal fun ImageView.setTintResource(@ColorRes tintColorResource: Int) {
     if (tintColorResource != NO_ID) {
-        imageTintList = ContextCompat.getColorStateList(context, tintColorResource)
+        imageTintList = AppCompatResources.getColorStateList(context, tintColorResource)
     }
 }
 
@@ -76,7 +76,6 @@ class BrowserToolbar @JvmOverloads constructor(
     @VisibleForTesting
     internal var searchTerms: String = ""
     private var urlCommitListener: ((String) -> Boolean)? = null
-    var isNavBarEnabled: Boolean = false
 
     /**
      * Toolbar in "display mode".
@@ -113,17 +112,17 @@ class BrowserToolbar @JvmOverloads constructor(
         set(value) { display.title = value }
 
     override var url: CharSequence
-        get() = display.url.toString()
+        get() = display.url
         set(value) {
             // We update the display toolbar immediately. We do not do that for the edit toolbar to not
             // mess with what the user is entering. Instead we will remember the value and update the
             // edit toolbar whenever we switch to it.
-            display.url = (value as String).trimmed()
+            display.url = value.trimmed()
         }
 
-    override var siteSecure: Toolbar.SiteSecurity
-        get() = display.siteSecurity
-        set(value) { display.siteSecurity = value }
+    override var siteInfo: Toolbar.SiteInfo
+        get() = display.siteInfo
+        set(value) { display.siteInfo = value }
 
     override var highlight: Highlight = Highlight.NONE
         set(value) {
@@ -357,22 +356,6 @@ class BrowserToolbar @JvmOverloads constructor(
         display.setHorizontalPadding(horizontalPadding)
     }
 
-    /**
-     * Hides the page action separator in display/edit mode.
-     */
-    override fun hidePageActionSeparator() {
-        display.hidePageActionSeparator()
-        edit.hidePageActionSeparator()
-    }
-
-    /**
-     * Shows the page action separator in display/edit mode.
-     */
-    override fun showPageActionSeparator() {
-        display.showPageActionSeparator()
-        edit.showPageActionSeparator()
-    }
-
     /**
      * Switches to URL editing mode.
      *
@@ -572,8 +555,8 @@ class BrowserToolbar @JvmOverloads constructor(
         val secondaryImage: Drawable = primaryImage,
         val secondaryContentDescription: String = primaryContentDescription,
         val isInPrimaryState: () -> Boolean = { true },
-        @ColorRes val primaryImageTintResource: Int = NO_ID,
-        @ColorRes val secondaryImageTintResource: Int = primaryImageTintResource,
+        @param:ColorRes val primaryImageTintResource: Int = NO_ID,
+        @param:ColorRes val secondaryImageTintResource: Int = primaryImageTintResource,
         val disableInSecondaryState: Boolean = true,
         override val weight: () -> Int = { -1 },
         background: Int = 0,
