@@ -965,6 +965,18 @@ private fun AcornColors.toM3ColorScheme(
     surfaceContainerHighest = surfaceContainerHighest,
     surfaceContainerLow = surfaceContainerLow,
     surfaceContainerLowest = surfaceContainerLowest,
+    primaryFixed = PhotonColors.Violet05,
+    primaryFixedDim = primaryContainer,
+    onPrimaryFixed = PhotonColors.DarkGrey90,
+    onPrimaryFixedVariant = textInverted,
+    secondaryFixed = secondaryContainer,
+    secondaryFixedDim = secondaryContainer,
+    onSecondaryFixed = textPrimary,
+    onSecondaryFixedVariant = textInverted,
+    tertiaryFixed = tertiaryContainer,
+    tertiaryFixedDim = tertiaryContainer,
+    onTertiaryFixed = textPrimary,
+    onTertiaryFixedVariant = textInverted,
 )
 
 /**
@@ -977,7 +989,7 @@ fun acornDarkColorScheme(): ColorScheme = darkColorPalette.toM3ColorScheme(
     secondaryContainer = Color(0xFF4B3974),
     tertiaryContainer = PhotonColors.Pink80,
     surface = PhotonColors.DarkGrey60,
-    inverseSurface = PhotonColors.LightGrey10,
+    inverseSurface = PhotonColors.LightGrey40,
     errorContainer = PhotonColors.Red80,
     outline = PhotonColors.LightGrey80,
     outlineVariant = PhotonColors.DarkGrey05,
@@ -999,7 +1011,7 @@ fun acornLightColorScheme(): ColorScheme = lightColorPalette.toM3ColorScheme(
     secondaryContainer = Color(0xFFE6E0F5),
     tertiaryContainer = PhotonColors.Pink05,
     surface = PhotonColors.LightGrey10,
-    inverseSurface = PhotonColors.DarkGrey60,
+    inverseSurface = PhotonColors.DarkGrey05,
     errorContainer = PhotonColors.Red05,
     outline = PhotonColors.LightGrey90,
     outlineVariant = PhotonColors.LightGrey30,
@@ -1021,7 +1033,7 @@ fun acornPrivateColorScheme(): ColorScheme = privateColorPalette.toM3ColorScheme
     secondaryContainer = Color(0xFF4B3974),
     tertiaryContainer = PhotonColors.Pink80,
     surface = Color(0xFF342B4A),
-    inverseSurface = PhotonColors.LightGrey10,
+    inverseSurface = PhotonColors.LightGrey40,
     errorContainer = PhotonColors.Red80,
     outline = PhotonColors.LightGrey80,
     outlineVariant = PhotonColors.DarkGrey05,
@@ -1054,19 +1066,3 @@ val ColorScheme.surfaceDimVariant: Color
     @Composable
     @ReadOnlyComposable
     get() = AcornTheme.colors.surfaceDimVariant
-
-/**
- * Primary Fixed
- *
- * A fixed accent color. This maintains the same tone across all themes.
- */
-val ColorScheme.primaryFixed: Color
-    get() = Color(0xFFE7DFFF)
-
-/**
- * On Primary Fixed
- *
- * Used for text and icons against the [ColorScheme.primaryFixed] color.
- */
-val ColorScheme.onPrimaryFixed: Color
-    get() = Color(0xFF15141A)
