@@ -11,10 +11,13 @@ import androidx.compose.foundation.layout.fillMaxWidth
 import androidx.compose.foundation.layout.height
 import androidx.compose.foundation.layout.padding
 import androidx.compose.foundation.shape.RoundedCornerShape
+import androidx.compose.material3.HorizontalDivider
+import androidx.compose.material3.adaptive.currentWindowAdaptiveInfo
 import androidx.compose.runtime.Composable
 import androidx.compose.runtime.remember
 import androidx.compose.ui.Alignment
 import androidx.compose.ui.Modifier
+import androidx.compose.ui.graphics.Color
 import androidx.compose.ui.platform.testTag
 import androidx.compose.ui.semantics.semantics
 import androidx.compose.ui.semantics.testTagsAsResourceId
@@ -22,7 +25,7 @@ import androidx.compose.ui.tooling.preview.PreviewLightDark
 import androidx.compose.ui.tooling.preview.PreviewParameter
 import androidx.compose.ui.tooling.preview.PreviewParameterProvider
 import androidx.compose.ui.unit.dp
-import mozilla.components.compose.base.Divider
+import androidx.window.core.layout.WindowSizeClass
 import mozilla.components.compose.base.progressbar.AnimatedProgressBar
 import mozilla.components.compose.base.theme.AcornTheme
 import mozilla.components.compose.browser.toolbar.concept.Action
@@ -42,8 +45,7 @@ import mozilla.components.ui.icons.R as iconsR
 
 private const val NO_TOOLBAR_PADDING_DP = 0
 private const val TOOLBAR_PADDING_DP = 8
-private const val MINIMUM_PROGRESS_BAR_STATE = 1
-private const val MAXIMUM_PROGRESS_BAR_STATE = 99
+private const val LARGE_TOOLBAR_PADDING_DP = 24
 
 /**
  * Sub-component of the [BrowserToolbar] responsible for displaying the URL and related
@@ -83,18 +85,23 @@ fun BrowserDisplayToolbar(
     browserActionsEnd: List<Action> = emptyList(),
     onInteraction: (BrowserToolbarEvent) -> Unit,
 ) {
-    val isProgressBarShown = remember(progressBarConfig) {
-        progressBarConfig != null &&
-            progressBarConfig.progress in MINIMUM_PROGRESS_BAR_STATE..MAXIMUM_PROGRESS_BAR_STATE
+    val windowSizeClass = currentWindowAdaptiveInfo().windowSizeClass
+    val isSmallWidthScreen = remember(windowSizeClass) {
+        windowSizeClass.minWidthDp < WindowSizeClass.WIDTH_DP_MEDIUM_LOWER_BOUND
     }
 
     Box(
         modifier = Modifier
             .background(color = AcornTheme.colors.layer1)
             .fillMaxWidth()
+            .padding(
+                horizontal = when (isSmallWidthScreen) {
+                    true -> NO_TOOLBAR_PADDING_DP.dp
+                    else -> LARGE_TOOLBAR_PADDING_DP.dp
+                },
+            )
             .semantics { testTagsAsResourceId = true },
-
-    ) {
+        ) {
         Row(
             verticalAlignment = Alignment.CenterVertically,
         ) {
@@ -117,7 +124,10 @@ fun BrowserDisplayToolbar(
                             true -> TOOLBAR_PADDING_DP.dp
                             false -> NO_TOOLBAR_PADDING_DP.dp
                         },
-                        bottom = TOOLBAR_PADDING_DP.dp,
+                        bottom = when (gravity) {
+                            Top -> TOOLBAR_PADDING_DP
+                            Bottom -> if (browserActionsEnd.isEmpty()) NO_TOOLBAR_PADDING_DP else TOOLBAR_PADDING_DP
+                        }.dp,
                     )
                     .height(48.dp)
                     .background(
@@ -177,21 +187,20 @@ fun BrowserDisplayToolbar(
             }
         }
 
+        HorizontalDivider(
+            modifier = Modifier.align(
+                when (gravity) {
+                    Top -> Alignment.BottomCenter
+                    Bottom -> Alignment.TopCenter
+                },
+            ),
+        )
+
         if (progressBarConfig != null) {
             AnimatedProgressBar(
                 progress = progressBarConfig.progress,
                 color = progressBarConfig.color,
-                modifier = Modifier.align(
-                    when (gravity) {
-                        Top -> Alignment.BottomCenter
-                        Bottom -> Alignment.TopCenter
-                    },
-                ),
-            )
-        }
-
-        if (!isProgressBarShown) {
-            Divider(
+                trackColor = Color.Transparent,
                 modifier = Modifier.align(
                     when (gravity) {
                         Top -> Alignment.BottomCenter
