@@ -6,6 +6,7 @@ package mozilla.components.concept.engine
 
 import android.content.Intent
 import androidx.annotation.CallSuper
+import mozilla.components.concept.engine.EngineSession.BounceTrackingProtectionMode
 import mozilla.components.concept.engine.EngineSession.TrackingProtectionPolicy.CookiePolicy.ACCEPT_ALL
 import mozilla.components.concept.engine.EngineSession.TrackingProtectionPolicy.CookiePolicy.ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS
 import mozilla.components.concept.engine.content.blocking.Tracker
@@ -70,11 +71,6 @@ abstract class EngineSession(
         fun onTrackerLoaded(tracker: Tracker) = Unit
         fun onNavigateBack() = Unit
 
-        /**
-         * Event to indicate a product URL is currently open.
-         */
-        fun onProductUrlChange(isProductUrl: Boolean) = Unit
-
         /**
          * Event to indicate that a page change is occurring, which will invalidate the page's
          * translations state.
@@ -256,10 +252,14 @@ abstract class EngineSession(
          * @param url The string url that was requested.
          * @param appIntent The Android Intent that was requested.
          * web content (as opposed to via the browser chrome).
+         * @param fallbackUrl the fallback URL if launch failed or denied by user.
+         * @param appName the target application name.
          */
         fun onLaunchIntentRequest(
             url: String,
             appIntent: Intent?,
+            fallbackUrl: String?,
+            appName: String?,
         ) = Unit
 
         /**
@@ -390,7 +390,6 @@ abstract class EngineSession(
      * Represents a safe browsing policy, which is indicates with type of site should be alerted
      * to user as possible harmful.
      */
-    @Suppress("MagicNumber")
     enum class SafeBrowsingPolicy(val id: Int) {
         NONE(0),
 
@@ -434,6 +433,9 @@ abstract class EngineSession(
         val cookiePolicyPrivateMode: CookiePolicy = cookiePolicy,
         val strictSocialTrackingProtection: Boolean? = null,
         val cookiePurging: Boolean = false,
+        val bounceTrackingProtectionMode: BounceTrackingProtectionMode = BounceTrackingProtectionMode.DISABLED,
+        val allowListBaselineTrackingProtection: Boolean = true,
+        val allowListConvenienceTrackingProtection: Boolean = true,
     ) {
 
         /**
@@ -479,7 +481,6 @@ abstract class EngineSession(
             ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS(5),
         }
 
-        @Suppress("MagicNumber")
         enum class TrackingCategory(val id: Int) {
 
             NONE(0),
@@ -548,18 +549,33 @@ abstract class EngineSession(
             fun none() = TrackingProtectionPolicy(
                 trackingCategories = arrayOf(TrackingCategory.NONE),
                 cookiePolicy = ACCEPT_ALL,
+                bounceTrackingProtectionMode = BounceTrackingProtectionMode.ENABLED_STANDBY,
             )
 
             /**
              * Strict policy.
              * Combining the [TrackingCategory.STRICT] plus a cookiePolicy of [ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS].
              * This is the strictest setting and may cause issues on some web sites.
+             *
+             * The [allowListBaselineTrackingProtection] and [allowListConvenienceTrackingProtection]
+             * parameters allow users to control allowlist configurations in the strict preset,
+             * without switching to a less restrictive policy.
+             *
+             * @param allowListBaselineTrackingProtection indicates if the baseline exceptions are applied
+             * @param allowListConvenienceTrackingProtection indicates if the convenience exceptions are applied
+             *
              */
-            fun strict() = TrackingProtectionPolicyForSessionTypes(
+            fun strict(
+                allowListBaselineTrackingProtection: Boolean = true,
+                allowListConvenienceTrackingProtection: Boolean = false,
+                ) = TrackingProtectionPolicyForSessionTypes(
                 trackingCategory = arrayOf(TrackingCategory.STRICT),
                 cookiePolicy = ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS,
                 strictSocialTrackingProtection = true,
                 cookiePurging = true,
+                bounceTrackingProtectionMode = BounceTrackingProtectionMode.ENABLED,
+                allowListBaselineTrackingProtection = allowListBaselineTrackingProtection,
+                allowListConvenienceTrackingProtection = allowListConvenienceTrackingProtection,
             )
 
             /**
@@ -573,6 +589,9 @@ abstract class EngineSession(
                 cookiePolicy = ACCEPT_FIRST_PARTY_AND_ISOLATE_OTHERS,
                 strictSocialTrackingProtection = false,
                 cookiePurging = true,
+                bounceTrackingProtectionMode = BounceTrackingProtectionMode.ENABLED_STANDBY,
+                allowListBaselineTrackingProtection = true,
+                allowListConvenienceTrackingProtection = true,
             )
 
             /**
@@ -587,6 +606,11 @@ abstract class EngineSession(
              *  is set.
              *  @param cookiePurging Whether or not to automatically purge tracking cookies. This will
              *  purge cookies from tracking sites that do not have recent user interaction provided.
+             *  @param bounceTrackingProtectionMode the bounce tracking protection mode to use.
+             *  @param allowListBaselineTrackingProtection indicates if the baseline tracking
+             *  protection exceptions are applied
+             *  @param allowListConvenienceTrackingProtection indicates if the convenience tracking
+             *  protection exceptions are applied
              */
             fun select(
                 trackingCategories: Array<TrackingCategory> = arrayOf(TrackingCategory.RECOMMENDED),
@@ -594,12 +618,19 @@ abstract class EngineSession(
                 cookiePolicyPrivateMode: CookiePolicy = cookiePolicy,
                 strictSocialTrackingProtection: Boolean? = null,
                 cookiePurging: Boolean = false,
+                bounceTrackingProtectionMode: BounceTrackingProtectionMode =
+                    BounceTrackingProtectionMode.ENABLED_STANDBY,
+                allowListBaselineTrackingProtection: Boolean = true,
+                allowListConvenienceTrackingProtection: Boolean = false,
             ) = TrackingProtectionPolicyForSessionTypes(
                 trackingCategory = trackingCategories,
                 cookiePolicy = cookiePolicy,
                 cookiePolicyPrivateMode = cookiePolicyPrivateMode,
                 strictSocialTrackingProtection = strictSocialTrackingProtection,
                 cookiePurging = cookiePurging,
+                bounceTrackingProtectionMode = bounceTrackingProtectionMode,
+                allowListBaselineTrackingProtection = allowListBaselineTrackingProtection,
+                allowListConvenienceTrackingProtection = allowListConvenienceTrackingProtection,
             )
         }
 
@@ -612,6 +643,7 @@ abstract class EngineSession(
             if (cookiePurging != other.cookiePurging) return false
             if (cookiePolicyPrivateMode != other.cookiePolicyPrivateMode) return false
             if (strictSocialTrackingProtection != other.strictSocialTrackingProtection) return false
+            if (bounceTrackingProtectionMode != other.bounceTrackingProtectionMode) return false
             return true
         }
 
@@ -662,6 +694,34 @@ abstract class EngineSession(
         NO_DETECTED,
     }
 
+    /**
+     * Represents settings options for bounce tracking protection.
+     */
+    enum class BounceTrackingProtectionMode(val mode: Int) {
+        /**
+         * Fully disabled.
+         */
+        DISABLED(0),
+
+        /**
+         * Fully enabled.
+         */
+        ENABLED(1),
+
+        /**
+         * Disabled, but collects user interaction data. Use this mode as the
+         * "disabled" state when the feature can be toggled on and off, e.g. via
+         * preferences.
+         */
+        ENABLED_STANDBY(2),
+
+        /**
+         * Feature enabled, but tracker purging is only simulated. Used for
+         * testing and telemetry collection.
+         */
+        ENABLED_DRY_RUN(3),
+    }
+
     /**
      * Subtype of [TrackingProtectionPolicy] to control the type of session this policy
      * should be applied to. By default, a policy will be applied to all sessions.
@@ -682,12 +742,18 @@ abstract class EngineSession(
         cookiePolicyPrivateMode: CookiePolicy = cookiePolicy,
         strictSocialTrackingProtection: Boolean? = null,
         cookiePurging: Boolean = false,
+        bounceTrackingProtectionMode: BounceTrackingProtectionMode = BounceTrackingProtectionMode.DISABLED,
+        allowListBaselineTrackingProtection: Boolean = true,
+        allowListConvenienceTrackingProtection: Boolean = true,
     ) : TrackingProtectionPolicy(
         trackingCategories = trackingCategory,
         cookiePolicy = cookiePolicy,
         cookiePolicyPrivateMode = cookiePolicyPrivateMode,
         strictSocialTrackingProtection = strictSocialTrackingProtection,
         cookiePurging = cookiePurging,
+        bounceTrackingProtectionMode = bounceTrackingProtectionMode,
+        allowListBaselineTrackingProtection = allowListBaselineTrackingProtection,
+        allowListConvenienceTrackingProtection = allowListConvenienceTrackingProtection,
     ) {
         /**
          * Marks this policy to be used for private sessions only.
@@ -700,6 +766,9 @@ abstract class EngineSession(
             cookiePolicyPrivateMode = cookiePolicyPrivateMode,
             strictSocialTrackingProtection = false,
             cookiePurging = cookiePurging,
+            bounceTrackingProtectionMode = bounceTrackingProtectionMode,
+            allowListBaselineTrackingProtection = allowListBaselineTrackingProtection,
+            allowListConvenienceTrackingProtection = allowListConvenienceTrackingProtection,
         )
 
         /**
@@ -713,6 +782,9 @@ abstract class EngineSession(
             cookiePolicyPrivateMode = cookiePolicyPrivateMode,
             strictSocialTrackingProtection = strictSocialTrackingProtection,
             cookiePurging = cookiePurging,
+            bounceTrackingProtectionMode = bounceTrackingProtectionMode,
+            allowListBaselineTrackingProtection = allowListBaselineTrackingProtection,
+            allowListConvenienceTrackingProtection = allowListConvenienceTrackingProtection,
         )
     }
 
@@ -732,6 +804,12 @@ abstract class EngineSession(
             const val LOAD_FLAGS_BYPASS_LOAD_URI_DELEGATE: Int = 1 shl 7
             const val ALLOW_ADDITIONAL_HEADERS: Int = 1 shl 15
             const val ALLOW_JAVASCRIPT_URL: Int = 1 shl 16
+
+            const val APP_LINK_LAUNCH_TYPE_COLD: Int = 1 shl 8
+            const val APP_LINK_LAUNCH_TYPE_WARM: Int = 1 shl 9
+            const val APP_LINK_LAUNCH_TYPE_HOT: Int = 1 shl 10
+            const val APP_LINK_LAUNCH_TYPE_UNKNOWN: Int = 0
+
             internal const val ALL = BYPASS_CACHE + BYPASS_PROXY + EXTERNAL + ALLOW_POPUPS +
                 BYPASS_CLASSIFIER + LOAD_FLAGS_FORCE_ALLOW_DATA_URI + LOAD_FLAGS_REPLACE_HISTORY +
                 LOAD_FLAGS_BYPASS_LOAD_URI_DELEGATE + ALLOW_ADDITIONAL_HEADERS + ALLOW_JAVASCRIPT_URL
@@ -871,6 +949,20 @@ abstract class EngineSession(
      */
     abstract fun restoreState(state: EngineSessionState): Boolean
 
+    /**
+     * Flushes the session state of the engine session.
+     *
+     * This method triggers an asynchronous flush of the current
+     * session state. The most recent state is not returned directly
+     * by this call. Instead, the updated session state will be
+     * delivered asynchronously through the observer callbacks:
+     *
+     * [EngineSession.Observer.onStateUpdated]
+     *
+     * [EngineSession.Observer.onHistoryStateChanged]
+     */
+    abstract fun flushSessionState()
+
     /**
      * Updates the tracking protection [policy] for this engine session.
      * If you want to disable tracking protection use [TrackingProtectionPolicy.none].
