@@ -14,6 +14,7 @@ import org.mozilla.gecko.annotationProcessors.classloader.AnnotatableEntity;
 import org.mozilla.gecko.annotationProcessors.classloader.ClassWithOptions;
 import org.mozilla.gecko.annotationProcessors.utils.Utils;
 
+/** Code generator for JNI bindings from annotated classes. */
 public class CodeGenerator {
   private static final Class<?>[] EMPTY_CLASS_ARRAY = new Class<?>[0];
 
@@ -31,6 +32,11 @@ public class CodeGenerator {
 
   private final HashSet<String> takenMethodNames = new HashSet<String>();
 
+  /**
+   * Constructor for CodeGenerator.
+   *
+   * @param annotatedClass The class with options to generate code for
+   */
   public CodeGenerator(ClassWithOptions annotatedClass) {
     this.cls = annotatedClass.wrappedClass;
     this.clsName = annotatedClass.generatedName;
@@ -590,6 +596,11 @@ public class CodeGenerator {
     return String.valueOf(val);
   }
 
+  /**
+   * Generates code for an annotated field.
+   *
+   * @param annotatedField The annotated field to generate code for
+   */
   public void generateField(AnnotatableEntity annotatedField) {
     final Field field = annotatedField.getField();
     final AnnotationInfo info = annotatedField.mAnnotationInfo;
@@ -629,9 +640,9 @@ public class CodeGenerator {
       }
 
       if (val != null && type.isPrimitive()) {
-        // For static final primitive fields, we can use a "static const" declaration.
+        // For static final primitive fields, we can use a "static constexpr" declaration.
         header.append(
-            "    static const "
+            "    static constexpr "
                 + Utils.getNativeReturnType(type, info)
                 + ' '
                 + info.wrapperName
@@ -712,6 +723,11 @@ public class CodeGenerator {
             + "\n");
   }
 
+  /**
+   * Generates code for an annotated constructor.
+   *
+   * @param annotatedConstructor The annotated constructor to generate code for
+   */
   public void generateConstructor(AnnotatableEntity annotatedConstructor) {
     // Unpack the tuple and extract some useful fields from the Method..
     final Constructor<?> method = annotatedConstructor.getConstructor();
@@ -760,6 +776,11 @@ public class CodeGenerator {
             + "\n");
   }
 
+  /**
+   * Generates code for inner classes.
+   *
+   * @param classes The array of inner classes to generate code for
+   */
   public void generateClasses(final ClassWithOptions[] classes) {
     if (classes.length == 0) {
       return;
@@ -826,7 +847,7 @@ public class CodeGenerator {
             + "};\n"
             + "\n"
             + "template<class Impl>\n"
-            + "const JNINativeMethod "
+            + "MOZ_GLOBINIT const JNINativeMethod "
             + clsName
             + "::Natives<Impl>::methods[] = {"
             + nativesInits
