@@ -57,9 +57,6 @@ pref("apz.fling_stopped_threshold", "0.0");
 // feel a lot better (bug 1095727)
 pref("apz.max_velocity_inches_per_ms", "0.07");
 
-// Enable overscroll on Android (bug 1230674)
-pref("apz.overscroll.enabled", true);
-
 // Don't allow a faraway second tap to start a one-touch pinch gesture (bug 1391770)
 pref("apz.second_tap_tolerance", "0.3");
 
@@ -147,6 +144,9 @@ pref("browser.translations.enable", true);
 // Used for mocking data for GeckoView Translations tests, should use in addition with an automation check.
 pref("browser.translations.geckoview.enableAllTestMocks", false);
 
+// Used for mocking data for GeckoView WebCompat tests, should use in addition with an automation check.
+pref("browser.webcompat.geckoview.enableAllTestMocks", false);
+
 // SSL error page behaviour (bug 437372)
 pref("browser.xul.error_pages.expert_bad_cert", false);
 
@@ -158,10 +158,6 @@ pref("chrome.override_package.passwordmgr", "browser");
 // Allow Console API to log messages on stdout (bug 1480544)
 pref("devtools.console.stdout.chrome", true);
 
-// Absolute path to the devtools unix domain socket file used
-// to communicate with a usb cable via adb forward.
-pref("devtools.debugger.unix-domain-socket", "@ANDROID_PACKAGE_NAME@/firefox-debugger-socket");
-
 // Enable capture attribute for file input (bug 1553603)
 pref("dom.capture.enabled", true);
 
@@ -183,13 +179,8 @@ pref("dom.ipc.keepProcessesAlive.extension", 1);
 // Keep empty content process alive on Android (bug 1447393)
 pref("dom.ipc.keepProcessesAlive.web", 1);
 
-// This value is derived from the calculation:
-// MOZ_ANDROID_CONTENT_SERVICE_COUNT - dom.ipc.processCount
-// (dom.ipc.processCount is set in GeckoRuntimeSettings.java) (bug 1619655)
-pref("dom.ipc.processCount.webCOOP+COEP", 38);
-
-// Disable the preallocated process on Android
-pref("dom.ipc.processPrelaunch.enabled", false);
+// Enable the preallocated process on Android
+pref("dom.ipc.processPrelaunch.enabled", true);
 
 // Increase script timeouts (bug 485610)
 pref("dom.max_script_run_time", 20);
@@ -201,10 +192,6 @@ pref("dom.meta-viewport.enabled", true);
 // subscription, to avoid duplicates for unacknowledged messages (bug 1207743)
 pref("dom.push.maxRecentMessageIDsPerSubscription", 0);
 
-// Allow service workers to open windows for a longer period after a notification
-// click on mobile. This is to account for some devices being quite slow (bug 1409761)
-pref("dom.serviceWorkers.disable_open_click_delay", 5000);
-
 // Enable WebShare support (bug 1402369)
 pref("dom.webshare.enabled", true);
 
@@ -246,6 +233,10 @@ pref("extensions.strictCompatibility", false);
 pref("extensions.systemAddon.update.enabled", true);
 pref("extensions.systemAddon.update.url", "https://aus5.mozilla.org/update/3/SystemAddons/%VERSION%/%BUILD_ID%/%BUILD_TARGET%/%LOCALE%/%CHANNEL%/%OS_VERSION%/%DISTRIBUTION%/%DISTRIBUTION_VERSION%/update.xml");
 
+// Enable the EnvironmentAddonBuilder collecting the Glean
+// activeAddons/theme/GMPlugins metrics.
+pref("extensions.telemetry.EnvironmentAddonBuilder", true);
+
 pref("extensions.update.background.url", "https://versioncheck-bg.addons.mozilla.org/update/VersionCheck.php?reqVersion=%REQ_VERSION%&id=%ITEM_ID%&version=%ITEM_VERSION%&maxAppVersion=%ITEM_MAXAPPVERSION%&status=%ITEM_STATUS%&appID=%APP_ID%&appVersion=%APP_VERSION%&appOS=%APP_OS%&appABI=%APP_ABI%&locale=%APP_LOCALE%&currentAppVersion=%CURRENT_APP_VERSION%&updateType=%UPDATE_TYPE%&compatMode=%COMPATIBILITY_MODE%");
 pref("extensions.update.enabled", true);
 pref("extensions.update.interval", 86400);
@@ -259,6 +250,9 @@ pref("extensions.webextOptionalPermissionPrompts", true);
 // GeckoView can trigger requests without opening geckoview.xhtml.
 pref("extensions.webextensions.early_background_wakeup_on_request", true);
 
+// Enable data collection permissions.
+pref("extensions.dataCollectionPermissions.enabled", true);
+
 // Scroll and zoom into editable form fields (bug 834613)
 pref("formhelper.autozoom", true);
 
@@ -275,8 +269,6 @@ pref("image.cache.size", 1048576); // bytes
 // Inherit locale from the OS, used for multi-locale builds
 pref("intl.locale.requested", "");
 
-pref("keyword.enabled", true);
-
 // Always tilt the caret to match the text selection guideline (bug 1097398)
 pref("layout.accessiblecaret.always_tilt", true);
 
@@ -331,14 +323,6 @@ pref("media.navigator.permission.device", true);
 // this is to preserve battery and data (bug 1540573)
 pref("media.throttle-cellular-regardless-of-download-rate", true);
 
-// Number of video frames we buffer while decoding video.
-// On Android this is decided by a similar value which varies for
-// each OMX decoder |OMX_PARAM_PORTDEFINITIONTYPE::nBufferCountMin|. This
-// number must be less than the OMX equivalent or gecko will think it is
-// chronically starved of video frames. All decoders seen so far have a value
-// of at least 4. (bug 973408)
-pref("media.video-queue.default-size", 3);
-
 // The maximum number of queued frames to send to the compositor.
 // On Android, it needs to be throttled because SurfaceTexture contains only one
 // (the most recent) image data. (bug 1299068)
@@ -347,9 +331,6 @@ pref("media.video-queue.send-to-compositor-size", 1);
 // Increase necko buffer sizes for Android (bug 560591)
 pref("network.buffer.cache.size",  16384);
 
-// CookieBehavior setting for private browsing (bug 1695050)
-pref("network.cookie.cookieBehavior.pbmode", 4);
-
 // Set HPACK receive buffer size appropriately for Android (bug 1296280)
 pref("network.http.http2.default-hpack-buffer", 4096);
 
@@ -377,11 +358,18 @@ pref("network.protocol-handler.warn-external.vnd.youtube", false);
 // (bug 888268)
 pref("network.tickle-wifi.enabled", true);
 
-// Editing PDFs is not supported on mobile
-pref("pdfjs.annotationEditorMode", -1);
+// Controls OHTTP config URL
+pref("network.ohttp.configURL", "https://prod.ohttp-gateway.prod.webservices.mozgcp.net/ohttp-configs");
 
-// Enable the floating PDF.js toolbar on GeckoView (bug 1829366)
-pref("pdfjs.enableFloatingToolbar", true);
+// Controls OHTTP relay URL
+pref("network.ohttp.relayURL", "https://mozilla-ohttp.fastly-edge.com/");
+
+// Complete the page load progress bar at different places according to this pref.
+// See the possible values below:
+// 0 no change
+// 1 complete progressbar at DOMContentLoaded
+// 2 complete progressbar at first MozAfterPaint after DOMContentLoaded
+pref("page_load.progressbar_completion", 2);
 
 // Try to convert PDFs sent as octet-stream (bug 1754499)
 pref("pdfjs.handleOctetStream", true);
@@ -389,16 +377,11 @@ pref("pdfjs.handleOctetStream", true);
 // Disable tracking protection in PBM for GeckoView (bug 1436887)
 pref("privacy.trackingprotection.pbmode.enabled", false);
 
-pref("privacy.fingerprintingProtection.pbmode", true);
-
 // Relay integration is not supported on mobile
 pref("signon.firefoxRelay.feature", "not available");
 
 pref("signon.showAutoCompleteFooter", true);
 
-// Delegate autocomplete to GeckoView (bug 1618058)
-pref("toolkit.autocomplete.delegate", true);
-
 // Locked because any other value would break GeckoView
 pref("toolkit.defaultChromeURI", "chrome://geckoview/content/geckoview.xhtml", locked);
 
@@ -409,6 +392,9 @@ pref("toolkit.telemetry.unified", false);
 pref("urlclassifier.downloadAllowTable", "");
 pref("urlclassifier.downloadBlockTable", "");
 
+// Delay the CRC32 check for URLClassifier to improve applink performance. (bug 1956920, bug 1971949)
+pref("urlclassifier.delay_prefixes_crc32_check", true);
+
 // The Potentially Harmful Apps list replaces the malware one on Android (bug 1394017)
 pref("urlclassifier.malwareTable", "goog-harmful-proto,goog-unwanted-proto,moztest-harmful-simple,moztest-malware-simple,moztest-unwanted-simple");
 
