@@ -4,16 +4,18 @@
 
 package org.mozilla.fenix.ui
 
+import androidx.compose.ui.test.junit4.AndroidComposeTestRule
 import org.junit.Rule
 import org.junit.Test
 import org.mozilla.fenix.customannotations.SmokeTest
 import org.mozilla.fenix.helpers.HomeActivityIntentTestRule
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithResId
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithResIdContainingText
-import org.mozilla.fenix.helpers.TestAssetHelper
+import org.mozilla.fenix.helpers.TestAssetHelper.addressFormAsset
 import org.mozilla.fenix.helpers.TestHelper.exitMenu
 import org.mozilla.fenix.helpers.TestHelper.packageName
 import org.mozilla.fenix.helpers.TestSetup
+import org.mozilla.fenix.helpers.perf.DetectMemoryLeaksRule
 import org.mozilla.fenix.ui.robots.autofillScreen
 import org.mozilla.fenix.ui.robots.clickPageObject
 import org.mozilla.fenix.ui.robots.homeScreen
@@ -27,9 +29,9 @@ class AddressAutofillTest : TestSetup() {
         var name = "Mozilla Fenix Firefox"
         var streetAddress = "Harrison Street"
         var city = "San Francisco"
-        var state = "Alaska"
+        var state = "AK"
         var zipCode = "94105"
-        var country = "United States"
+        var country = "US"
         var phoneNumber = "555-5555"
         var emailAddress = "foo@bar.com"
     }
@@ -38,25 +40,30 @@ class AddressAutofillTest : TestSetup() {
         var navigateToAutofillSettings = false
         var name = "Android Test Name"
         var streetAddress = "Fort Street"
-        var city = "San Jose"
-        var state = "Arizona"
+        var city = "Alberta"
+        var state = "AZ"
         var zipCode = "95141"
-        var country = "United States"
+        var country = "CA"
         var phoneNumber = "777-7777"
         var emailAddress = "fuu@bar.org"
     }
 
     @get:Rule
-    val activityIntentTestRule = HomeActivityIntentTestRule.withDefaultSettingsOverrides()
+    val composeTestRule =
+        AndroidComposeTestRule(
+            HomeActivityIntentTestRule.withDefaultSettingsOverrides(),
+        ) { it.activity }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836845
+    @get:Rule
+    val memoryLeaksRule = DetectMemoryLeaksRule()
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836845
     @SmokeTest
     @Test
     fun verifyAddressAutofillTest() {
-        val addressFormPage =
-            TestAssetHelper.getAddressFormAsset(mockWebServer)
+        val addressFormPage = mockWebServer.addressFormAsset
 
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -87,11 +94,11 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836856
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836856
     @SmokeTest
     @Test
     fun deleteSavedAddressTest() {
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -105,8 +112,9 @@ class AddressAutofillTest : TestSetup() {
                 phoneNumber = FirstAddressAutofillDetails.phoneNumber,
                 emailAddress = FirstAddressAutofillDetails.emailAddress,
             )
+
             clickManageAddressesButton()
-            clickSavedAddress("Mozilla")
+            clickSavedAddress(composeTestRule, FirstAddressAutofillDetails.name)
             clickDeleteAddressButton()
             clickCancelDeleteAddressButton()
             clickDeleteAddressButton()
@@ -115,24 +123,24 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836840
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836840
     @Test
     fun verifyAddAddressViewTest() {
         homeScreen {
         }.openThreeDotMenu {
         }.openSettings {
-        }.openAutofillSubMenu {
+        }.openAutofillSubMenu(composeTestRule) {
             clickAddAddressButton()
             verifyAddAddressView()
-        }.goBackToAutofillSettings {
+        }.goBackToAutofillSettings(composeTestRule) {
             verifyAutofillToolbarTitle()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836841
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836841
     @Test
     fun verifyEditAddressViewTest() {
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -147,18 +155,17 @@ class AddressAutofillTest : TestSetup() {
                 emailAddress = FirstAddressAutofillDetails.emailAddress,
             )
             clickManageAddressesButton()
-            clickSavedAddress("Mozilla")
+            clickSavedAddress(composeTestRule, FirstAddressAutofillDetails.name)
             verifyEditAddressView()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836839
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836839
     @Test
     fun verifyAddressAutofillToggleTest() {
-        val addressFormPage =
-            TestAssetHelper.getAddressFormAsset(mockWebServer)
+        val addressFormPage = mockWebServer.addressFormAsset
 
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -182,7 +189,7 @@ class AddressAutofillTest : TestSetup() {
             verifySelectAddressButtonExists(true)
         }.openThreeDotMenu {
         }.openSettings {
-        }.openAutofillSubMenu {
+        }.openAutofillSubMenu(composeTestRule) {
             clickSaveAndAutofillAddressesOption()
             verifyAddressAutofillSection(false, true)
         }
@@ -196,13 +203,12 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836847
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836847
     @Test
     fun verifyManageAddressesPromptOptionTest() {
-        val addressFormPage =
-            TestAssetHelper.getAddressFormAsset(mockWebServer)
+        val addressFormPage = mockWebServer.addressFormAsset
 
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -224,20 +230,19 @@ class AddressAutofillTest : TestSetup() {
         }.enterURLAndEnterToBrowser(addressFormPage.url) {
             clickPageObject(itemWithResId("streetAddress"))
             clickSelectAddressButton()
-        }.clickManageAddressButton {
+        }.clickManageAddressButton(composeTestRule) {
             verifyAutofillToolbarTitle()
         }.goBackToBrowser {
             verifySaveLoginPromptIsNotDisplayed()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836849
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836849
     @Test
     fun verifyMultipleAddressesSelectionTest() {
-        val addressFormPage =
-            TestAssetHelper.getAddressFormAsset(mockWebServer)
+        val addressFormPage = mockWebServer.addressFormAsset
 
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -293,10 +298,10 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836850
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836850
     @Test
     fun verifySavedAddressCanBeEditedTest() {
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -311,7 +316,7 @@ class AddressAutofillTest : TestSetup() {
                 emailAddress = FirstAddressAutofillDetails.emailAddress,
             )
             clickManageAddressesButton()
-            clickSavedAddress("Mozilla")
+            clickSavedAddress(composeTestRule, FirstAddressAutofillDetails.name)
             fillAndSaveAddress(
                 navigateToAutofillSettings = SecondAddressAutofillDetails.navigateToAutofillSettings,
                 name = SecondAddressAutofillDetails.name,
@@ -327,30 +332,31 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836848
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836848
     @Test
     fun verifyStateFieldUpdatesInAccordanceWithCountryFieldTest() {
         homeScreen {
         }.openThreeDotMenu {
         }.openSettings {
-        }.openAutofillSubMenu {
+        }.openAutofillSubMenu(composeTestRule) {
             verifyAddressAutofillSection(true, false)
             clickAddAddressButton()
+            clickCountryDropdown()
+            clickCountryOption("US")
             verifyCountryOption("United States")
             verifyStateOption("Alabama")
-            verifyCountryOptions("Canada", "United States")
-            clickCountryOption("Canada")
+            clickCountryDropdown()
+            clickCountryOption("CA")
             verifyStateOption("Alberta")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836858
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836858
     @Test
     fun verifyFormFieldCanBeFilledManuallyTest() {
-        val addressFormPage =
-            TestAssetHelper.getAddressFormAsset(mockWebServer)
+        val addressFormPage = mockWebServer.addressFormAsset
 
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -384,10 +390,10 @@ class AddressAutofillTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1836838
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1836838
     @Test
     fun verifyAutofillAddressSectionTest() {
-        autofillScreen {
+        autofillScreen(composeTestRule) {
             fillAndSaveAddress(
                 navigateToAutofillSettings = FirstAddressAutofillDetails.navigateToAutofillSettings,
                 isAddressAutofillEnabled = FirstAddressAutofillDetails.isAddressAutofillEnabled,
@@ -404,16 +410,8 @@ class AddressAutofillTest : TestSetup() {
             verifyAddressAutofillSection(true, true)
             clickManageAddressesButton()
             verifyManageAddressesSection(
-                "Mozilla",
-                "Fenix",
-                "Firefox",
-                "Harrison Street",
-                "San Francisco",
-                "Alaska",
-                "94105",
-                "US",
-                "555-5555",
-                "foo@bar.com",
+                FirstAddressAutofillDetails.name,
+                "Harrison Street, San Francisco, AK, US, 94105, 555-5555, foo@bar.com",
             )
         }
     }
