@@ -4,22 +4,26 @@
 
 package org.mozilla.fenix.ui
 
+import androidx.compose.ui.test.junit4.AndroidComposeTestRule
 import androidx.core.net.toUri
 import androidx.test.espresso.Espresso.pressBack
-import org.junit.Ignore
+import mozilla.components.concept.engine.utils.EngineReleaseChannel
 import org.junit.Rule
 import org.junit.Test
 import org.mozilla.fenix.customannotations.SmokeTest
+import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.settings
 import org.mozilla.fenix.helpers.HomeActivityIntentTestRule
-import org.mozilla.fenix.helpers.TestAssetHelper.getEnhancedTrackingProtectionAsset
+import org.mozilla.fenix.helpers.TestAssetHelper.enhancedTrackingProtectionAsset
 import org.mozilla.fenix.helpers.TestAssetHelper.getGenericAsset
+import org.mozilla.fenix.helpers.TestAssetHelper.waitingTimeLong
 import org.mozilla.fenix.helpers.TestHelper.appContext
 import org.mozilla.fenix.helpers.TestHelper.exitMenu
 import org.mozilla.fenix.helpers.TestHelper.mDevice
 import org.mozilla.fenix.helpers.TestHelper.restartApp
 import org.mozilla.fenix.helpers.TestHelper.scrollToElementByText
 import org.mozilla.fenix.helpers.TestSetup
+import org.mozilla.fenix.helpers.perf.DetectMemoryLeaksRule
 import org.mozilla.fenix.ui.robots.browserScreen
 import org.mozilla.fenix.ui.robots.enhancedTrackingProtection
 import org.mozilla.fenix.ui.robots.homeScreen
@@ -40,13 +44,15 @@ import org.mozilla.fenix.ui.robots.navigationToolbar
 
 class EnhancedTrackingProtectionTest : TestSetup() {
     @get:Rule
-    val activityTestRule = HomeActivityIntentTestRule(
-        isJumpBackInCFREnabled = false,
-        isTCPCFREnabled = false,
-        isWallpaperOnboardingEnabled = false,
-    )
+    val activityTestRule =
+        AndroidComposeTestRule(
+            HomeActivityIntentTestRule.withDefaultSettingsOverrides(),
+        ) { it.activity }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/416046
+    @get:Rule
+    val memoryLeaksRule = DetectMemoryLeaksRule()
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/416046
     @Test
     fun testETPSettingsItemsAndSubMenus() {
         homeScreen {
@@ -79,14 +85,15 @@ class EnhancedTrackingProtectionTest : TestSetup() {
             openExceptionsLearnMoreLink()
         }
         browserScreen {
-            verifyUrl("support.mozilla.org/en-US/kb/enhanced-tracking-protection-firefox-android")
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
+            verifyETPLearnMoreURL()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1514599
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1514599
     @Test
     fun verifyETPStateIsReflectedInTPSheetTest() {
-        val genericPage = getGenericAsset(mockWebServer, 1)
+        val genericPage = mockWebServer.getGenericAsset(1)
 
         homeScreen {
         }.openThreeDotMenu {
@@ -118,13 +125,13 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/339712
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/339712
     // Tests adding ETP exceptions to websites and keeping that preference after restart
     @SmokeTest
     @Test
     fun disablingETPOnAWebsiteAddsItToExceptionListTest() {
-        val firstPage = getGenericAsset(mockWebServer, 1)
-        val secondPage = "example.com"
+        val firstPage = mockWebServer.getGenericAsset(1)
+        val secondPage = "https://mozilla-mobile.github.io/testapp"
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstPage.url) {}
@@ -135,7 +142,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.closeEnhancedTrackingProtectionSheet {
         }.openNavigationToolbar {
         }.enterURLAndEnterToBrowser(secondPage.toUri()) {
-            verifyPageContent("Example Domain")
+            verifyPageContent("Lets test!")
         }
         enhancedTrackingProtection {
         }.openEnhancedTrackingProtectionSheet {
@@ -143,21 +150,21 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.toggleEnhancedTrackingProtectionFromSheet {
             verifyEnhancedTrackingProtectionSheetStatus("OFF", false)
         }
-        restartApp(activityTestRule)
+        restartApp(activityTestRule.activityRule)
         enhancedTrackingProtection {
         }.openEnhancedTrackingProtectionSheet {
             verifyEnhancedTrackingProtectionSheetStatus("OFF", false)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/339714
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/339714
     @Test
     fun enablingETPOnAWebsiteRemovesItFromTheExceptionListTest() {
-        val trackingPage = getEnhancedTrackingProtectionAsset(mockWebServer)
+        val trackingPage = mockWebServer.enhancedTrackingProtectionAsset
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(trackingPage.url) {
-            waitForPageToLoad()
+            verifyUrl(trackingPage.url.toString())
         }
         enhancedTrackingProtection {
         }.openEnhancedTrackingProtectionSheet {
@@ -175,19 +182,21 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.openEnhancedTrackingProtectionSheet {
         }.toggleEnhancedTrackingProtectionFromSheet {
             verifyEnhancedTrackingProtectionSheetStatus("ON", true)
-        }.openProtectionSettings {
+        }.closeEnhancedTrackingProtectionSheet {
+        }.openThreeDotMenu {
+        }.openSettings {
+        }.openEnhancedTrackingProtectionSubMenu {
         }.openExceptions {
             verifySiteExceptionExists(trackingPage.url.host.toString(), false)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/339713
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/339713
     // Tests removing TP exceptions individually or all at once
-    @Ignore("Failing, see: https://bugzilla.mozilla.org/show_bug.cgi?id=1865781")
     @Test
     fun clearWebsitesFromTPExceptionListTest() {
-        val firstPage = getGenericAsset(mockWebServer, 1)
-        val secondPage = "example.com"
+        val firstPage = mockWebServer.getGenericAsset(1)
+        val secondPage = "https://mozilla-mobile.github.io/testapp"
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstPage.url) {}
@@ -198,7 +207,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.closeEnhancedTrackingProtectionSheet {
         }.openNavigationToolbar {
         }.enterURLAndEnterToBrowser(secondPage.toUri()) {
-            verifyPageContent("Example Domain")
+            verifyPageContent("Lets test!")
         }
         enhancedTrackingProtection {
         }.openEnhancedTrackingProtectionSheet {
@@ -209,7 +218,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.openSettings {
         }.openEnhancedTrackingProtectionSubMenu {
         }.openExceptions {
-            removeOneSiteException(secondPage)
+            removeOneSiteException(secondPage.toUri().host.toString())
         }.disableExceptions {
             verifyTPExceptionsDefaultView()
             exitMenu()
@@ -220,11 +229,11 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/417444
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/417444
     @Test
     fun verifyTrackersBlockedWithStandardTPTest() {
-        val genericPage = getGenericAsset(mockWebServer, 1)
-        val trackingProtectionTest = getEnhancedTrackingProtectionAsset(mockWebServer).url
+        val genericPage = mockWebServer.getGenericAsset(1)
+        val trackingProtectionTest = mockWebServer.enhancedTrackingProtectionAsset.url
 
         homeScreen {
         }.openThreeDotMenu {
@@ -250,8 +259,13 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.openEnhancedTrackingProtectionSheet {
             verifyEnhancedTrackingProtectionSheetStatus("ON", true)
         }.openDetails {
-            verifyCrossSiteCookiesBlocked(true)
-            navigateBackToDetails()
+            // Third-party cookie tracker blocking in Nightly was disabled: https://bugzilla.mozilla.org/show_bug.cgi?id=1935156
+            if (activityTestRule.activity.components.core.engine.version.releaseChannel == EngineReleaseChannel.BETA &&
+                activityTestRule.activity.components.core.engine.version.releaseChannel == EngineReleaseChannel.RELEASE
+            ) {
+                verifyCrossSiteCookiesBlocked(true)
+                navigateBackToDetails()
+            }
             verifyCryptominersBlocked(true)
             navigateBackToDetails()
             verifyFingerprintersBlocked(true)
@@ -260,12 +274,12 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.closeEnhancedTrackingProtectionSheet {}
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/417441
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/417441
     @Test
     fun verifyTrackersBlockedWithStrictTPTest() {
         appContext.settings().setStrictETP()
-        val genericPage = getGenericAsset(mockWebServer, 1)
-        val trackingProtectionTest = getEnhancedTrackingProtectionAsset(mockWebServer).url
+        val genericPage = mockWebServer.getGenericAsset(1)
+        val trackingProtectionTest = mockWebServer.enhancedTrackingProtectionAsset.url
 
         homeScreen {
         }.openThreeDotMenu {
@@ -278,7 +292,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         // browsing a generic page to allow GV to load on a fresh run
         navigationToolbar {
         }.enterURLAndEnterToBrowser(genericPage.url) {
-        }.openTabDrawer {
+        }.openTabDrawer(activityTestRule) {
             closeTab()
         }
 
@@ -305,12 +319,12 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/561637
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/561637
     @SmokeTest
     @Test
     fun verifyTrackersBlockedWithCustomTPTest() {
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
-        val trackingPage = getEnhancedTrackingProtectionAsset(mockWebServer)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
+        val trackingPage = mockWebServer.enhancedTrackingProtectionAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -346,15 +360,15 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/562710
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/562710
     // Tests the trackers blocked with the following Custom TP set up:
     // - Cookies set to "All cookies"
     // - Tracking content option OFF
     // - Fingerprinters, cryptominers and redirect trackers checked
     @Test
     fun customizedTrackingProtectionOptionsTest() {
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
-        val trackingPage = getEnhancedTrackingProtectionAsset(mockWebServer)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
+        val trackingPage = mockWebServer.enhancedTrackingProtectionAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -390,11 +404,11 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/562709
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/562709
     @Test
     fun verifyTrackersBlockedWithCustomTPOptionsDisabledTest() {
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
-        val trackingPage = getEnhancedTrackingProtectionAsset(mockWebServer)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
+        val trackingPage = mockWebServer.enhancedTrackingProtectionAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -405,7 +419,8 @@ class EnhancedTrackingProtectionTest : TestSetup() {
             selectTrackingProtectionOption("Cookies")
             selectTrackingProtectionOption("Tracking content")
             selectTrackingProtectionOption("Cryptominers")
-            selectTrackingProtectionOption("Fingerprinters")
+            selectTrackingProtectionOption("Known Fingerprinters")
+            selectTrackingProtectionOption("Suspected Fingerprinters")
             selectTrackingProtectionOption("Redirect Trackers")
         }.goBackToHomeScreen {
             mDevice.waitForIdle()
@@ -423,11 +438,11 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2106997
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2106997
     @Test
     fun verifyTrackingContentBlockedOnlyInPrivateTabsTest() {
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
-        val trackingPage = getEnhancedTrackingProtectionAsset(mockWebServer)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
+        val trackingPage = mockWebServer.enhancedTrackingProtectionAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -450,7 +465,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
             verifyTrackingProtectionWebContent("analytics not blocked")
             verifyTrackingProtectionWebContent("Fingerprinting blocked")
             verifyTrackingProtectionWebContent("Cryptomining blocked")
-        }.goToHomescreen {
+        }.goToHomescreen(activityTestRule) {
         }.togglePrivateBrowsingMode()
         navigationToolbar {
         }.enterURLAndEnterToBrowser(trackingPage.url) {
@@ -463,8 +478,14 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         enhancedTrackingProtection {
         }.openEnhancedTrackingProtectionSheet {
         }.openDetails {
-            verifyCrossSiteCookiesBlocked(true)
-            navigateBackToDetails()
+            // Third-party cookie tracker blocking in Nightly was disabled: https://bugzilla.mozilla.org/show_bug.cgi?id=1935156
+            if (
+                activityTestRule.activity.components.core.engine.version.releaseChannel == EngineReleaseChannel.BETA &&
+                activityTestRule.activity.components.core.engine.version.releaseChannel == EngineReleaseChannel.RELEASE
+            ) {
+                verifyCrossSiteCookiesBlocked(true)
+                navigateBackToDetails()
+            }
             verifyCryptominersBlocked(true)
             navigateBackToDetails()
             verifyFingerprintersBlocked(true)
@@ -474,15 +495,15 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2285368
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2285368
     @SmokeTest
     @Test
     fun blockCookiesStorageAccessTest() {
         // With Standard TrackingProtection settings
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
         val testPage = mockWebServer.url("pages/cross-site-cookies.html").toString().toUri()
-        val originSite = "https://mozilla-mobile.github.io"
-        val currentSite = "http://localhost:${mockWebServer.port}"
+        val originHost = "mozilla-mobile.github.io"
+        val currentHost = "localhost"
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(genericWebPage.url) {
@@ -491,21 +512,21 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.enterURLAndEnterToBrowser(testPage) {
             waitForPageToLoad()
         }.clickRequestStorageAccessButton {
-            verifyCrossOriginCookiesPermissionPrompt(originSite, currentSite)
+            verifyCrossOriginCookiesPermissionPrompt(originHost, currentHost)
         }.clickPagePermissionButton(allow = false) {
             verifyPageContent("access denied")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2285369
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2285369
     @SmokeTest
     @Test
     fun allowCookiesStorageAccessTest() {
         // With Standard TrackingProtection settings
-        val genericWebPage = getGenericAsset(mockWebServer, 1)
+        val genericWebPage = mockWebServer.getGenericAsset(1)
         val testPage = mockWebServer.url("pages/cross-site-cookies.html").toString().toUri()
-        val originSite = "https://mozilla-mobile.github.io"
-        val currentSite = "http://localhost:${mockWebServer.port}"
+        val originHost = "mozilla-mobile.github.io"
+        val currentHost = "localhost"
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(genericWebPage.url) {
@@ -514,7 +535,7 @@ class EnhancedTrackingProtectionTest : TestSetup() {
         }.enterURLAndEnterToBrowser(testPage) {
             waitForPageToLoad()
         }.clickRequestStorageAccessButton {
-            verifyCrossOriginCookiesPermissionPrompt(originSite, currentSite)
+            verifyCrossOriginCookiesPermissionPrompt(originHost, currentHost)
         }.clickPagePermissionButton(allow = true) {
             verifyPageContent("access granted")
         }
