@@ -6,28 +6,30 @@ package org.mozilla.fenix.ui
 
 import android.os.Build
 import android.view.autofill.AutofillManager
+import androidx.compose.ui.test.junit4.AndroidComposeTestRule
 import androidx.core.net.toUri
+import androidx.test.filters.SdkSuppress
 import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
+import org.mozilla.fenix.customannotations.SkipLeaks
 import org.mozilla.fenix.customannotations.SmokeTest
 import org.mozilla.fenix.helpers.HomeActivityIntentTestRule
-import org.mozilla.fenix.helpers.MatcherHelper
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithResId
-import org.mozilla.fenix.helpers.MatcherHelper.itemWithResIdAndText
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithText
-import org.mozilla.fenix.helpers.TestAssetHelper
+import org.mozilla.fenix.helpers.TestAssetHelper.saveLoginAsset
+import org.mozilla.fenix.helpers.TestAssetHelper.waitingTimeLong
 import org.mozilla.fenix.helpers.TestHelper
 import org.mozilla.fenix.helpers.TestHelper.exitMenu
 import org.mozilla.fenix.helpers.TestHelper.mDevice
-import org.mozilla.fenix.helpers.TestHelper.packageName
 import org.mozilla.fenix.helpers.TestHelper.restartApp
 import org.mozilla.fenix.helpers.TestHelper.scrollToElementByText
 import org.mozilla.fenix.helpers.TestHelper.verifySnackBarText
 import org.mozilla.fenix.helpers.TestHelper.waitForAppWindowToBeUpdated
+import org.mozilla.fenix.helpers.TestHelper.waitUntilSnackbarGone
 import org.mozilla.fenix.helpers.TestSetup
+import org.mozilla.fenix.helpers.perf.DetectMemoryLeaksRule
 import org.mozilla.fenix.ui.robots.browserScreen
-import org.mozilla.fenix.ui.robots.clearTextFieldItem
 import org.mozilla.fenix.ui.robots.clickPageObject
 import org.mozilla.fenix.ui.robots.homeScreen
 import org.mozilla.fenix.ui.robots.navigationToolbar
@@ -42,7 +44,12 @@ import org.mozilla.fenix.ui.robots.setPageObjectText
 class LoginsTest : TestSetup() {
     @get:Rule
     val activityTestRule =
-        HomeActivityIntentTestRule.withDefaultSettingsOverrides(skipOnboarding = true)
+        AndroidComposeTestRule(
+            HomeActivityIntentTestRule.withDefaultSettingsOverrides(),
+        ) { it.activity }
+
+    @get:Rule
+    val memoryLeaksRule = DetectMemoryLeaksRule()
 
     @Before
     override fun setUp() {
@@ -54,7 +61,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2092713
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2092713
     // Tests the Passwords menu items and default values
     @Test
     fun loginsAndPasswordsSettingsItemsTest() {
@@ -70,7 +77,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/517816
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/517816
     // Tests only for initial state without signing in.
     // For tests after signing in, see SyncIntegration test suite
     @Test
@@ -90,7 +97,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2092925
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2092925
     @Test
     fun verifySyncLoginsOptionsTest() {
         homeScreen {
@@ -105,11 +112,10 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/523839
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/523839
     @Test
     fun saveLoginFromPromptTest() {
-        val saveLoginTest =
-            TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val saveLoginTest = mockWebServer.saveLoginAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -142,7 +148,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/960412
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/960412
     @Test
     fun openLoginWebsiteInBrowserTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/loginForm"
@@ -153,7 +159,9 @@ class LoginsTest : TestSetup() {
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
             setPageObjectText(itemWithResId("username"), userName)
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), password)
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -170,10 +178,10 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/517817
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/517817
     @Test
     fun neverSaveLoginFromPromptTest() {
-        val saveLoginTest = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val saveLoginTest = mockWebServer.saveLoginAsset
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(saveLoginTest.url) {
@@ -198,12 +206,12 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1508171
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1508171
     @SmokeTest
     @Test
+    @SkipLeaks
     fun verifyUpdatedLoginIsSavedTest() {
-        val saveLoginTest =
-            TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val saveLoginTest = mockWebServer.saveLoginAsset
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(saveLoginTest.url) {
@@ -234,7 +242,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1049971
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1049971
     @SmokeTest
     @Test
     fun verifyMultipleLoginsSelectionsTest() {
@@ -260,23 +268,23 @@ class LoginsTest : TestSetup() {
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
-            clearTextFieldItem(itemWithResId("username"))
+            waitUntilSnackbarGone()
+        }
+        navigationToolbar {
+        }.enterURLAndEnterToBrowser(loginPage.toUri()) {
+            clickPageObject(itemWithResId("username"))
             clickSuggestedLoginsButton()
-            verifySuggestedUserName(firstUser)
-            verifySuggestedUserName(secondUser)
-            clickPageObject(
-                itemWithResIdAndText(
-                    "$packageName:id/username",
-                    firstUser,
-                ),
-            )
+            verifySuggestedUserName(activityTestRule, firstUser)
+            verifySuggestedUserName(activityTestRule, secondUser)
+            clickSuggestedLogin(activityTestRule, firstUser)
             clickPageObject(itemWithResId("togglePassword"))
-            verifyPrefilledLoginCredentials(firstUser, firstPass, true)
+            verifyPrefilledLoginCredentials(activityTestRule, firstUser, firstPass, true)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/875849
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/875849
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyEditLoginsViewTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/loginForm"
         val originWebsite = "mozilla-mobile.github.io"
@@ -284,7 +292,9 @@ class LoginsTest : TestSetup() {
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
             setPageObjectText(itemWithResId("username"), "mozilla")
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -295,7 +305,7 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails(originWebsite)
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickEditLoginButton()
             setNewPassword("fenix")
             saveEditedLogin()
@@ -304,7 +314,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/875851
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/875851
     @Test
     fun verifyEditedLoginsAreSavedTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
@@ -312,8 +322,11 @@ class LoginsTest : TestSetup() {
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
             setPageObjectText(itemWithResId("username"), "mozilla")
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -324,7 +337,7 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails(originWebsite)
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickEditLoginButton()
             setNewUserName("android")
             setNewPassword("fenix")
@@ -336,14 +349,15 @@ class LoginsTest : TestSetup() {
         browserScreen {
         }.openThreeDotMenu {
         }.refreshPage {
-            waitForPageToLoad()
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
             clickPageObject(itemWithResId("togglePassword"))
-            verifyPrefilledLoginCredentials("android", "fenix", true)
+            verifyPrefilledLoginCredentials(activityTestRule, "android", "fenix", true)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2266452
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2266452
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyLoginWithNoUserNameCanNotBeSavedTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/loginForm"
         val originWebsite = "mozilla-mobile.github.io"
@@ -351,7 +365,9 @@ class LoginsTest : TestSetup() {
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
             setPageObjectText(itemWithResId("username"), "mozilla")
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -362,7 +378,7 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails(originWebsite)
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickEditLoginButton()
             clickClearUserNameButton()
             verifyUserNameRequiredErrorMessage()
@@ -372,8 +388,9 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2266453
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2266453
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyLoginWithoutPasswordCanNotBeSavedTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/loginForm"
         val originWebsite = "mozilla-mobile.github.io"
@@ -381,7 +398,9 @@ class LoginsTest : TestSetup() {
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
             setPageObjectText(itemWithResId("username"), "mozilla")
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -392,7 +411,7 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails(originWebsite)
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickEditLoginButton()
             clickClearPasswordButton()
             verifyPasswordRequiredErrorMessage()
@@ -403,8 +422,9 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/876531
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/876531
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyEditModeDismissalDoesNotSaveLoginCredentialsTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/loginForm"
         val originWebsite = "mozilla-mobile.github.io"
@@ -412,7 +432,9 @@ class LoginsTest : TestSetup() {
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
             setPageObjectText(itemWithResId("username"), "mozilla")
+            waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
+            waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
@@ -423,7 +445,7 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails(originWebsite)
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickEditLoginButton()
             setNewUserName("android")
             setNewPassword("fenix")
@@ -434,10 +456,10 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/876532
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/876532
     @Test
     fun verifyDeleteLoginButtonTest() {
-        val loginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val loginPage = mockWebServer.saveLoginAsset
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.url) {
@@ -450,13 +472,13 @@ class LoginsTest : TestSetup() {
         }.openSavedLogins {
             tapSetupLater()
             viewSavedLoginDetails("test@example.com")
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickDeleteLoginButton()
             verifyLoginDeletionPrompt()
             clickCancelDeleteLogin()
             verifyLoginItemUsername("test@example.com")
             viewSavedLoginDetails("test@example.com")
-            clickThreeDotButton(activityTestRule)
+            clickThreeDotButton(activityTestRule.activityRule)
             clickDeleteLoginButton()
             verifyLoginDeletionPrompt()
             clickConfirmDeleteLogin()
@@ -465,11 +487,12 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/517818
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/517818
     @SmokeTest
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyNeverSaveLoginOptionTest() {
-        val loginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val loginPage = mockWebServer.saveLoginAsset
 
         homeScreen {
         }.openThreeDotMenu {
@@ -489,32 +512,33 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/517819
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/517819
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun verifyAutofillToggleTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
-            waitForPageToLoad()
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
             setPageObjectText(itemWithResId("username"), "mozilla")
             waitForAppWindowToBeUpdated()
             setPageObjectText(itemWithResId("password"), "firefox")
             waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
-            waitForPageToLoad()
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
             verifySaveLoginPromptIsDisplayed()
             clickPageObject(itemWithText("Save"))
-        }.openTabDrawer {
+        }.openTabDrawer(activityTestRule) {
             closeTab()
         }
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
-            waitForPageToLoad()
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
             clickPageObject(itemWithResId("togglePassword"))
-            verifyPrefilledLoginCredentials("mozilla", "firefox", true)
-        }.openTabDrawer {
+            verifyPrefilledLoginCredentials(activityTestRule, "mozilla", "firefox", true)
+        }.openTabDrawer(activityTestRule) {
             closeTab()
         }
 
@@ -532,12 +556,13 @@ class LoginsTest : TestSetup() {
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
-            verifyPrefilledLoginCredentials("mozilla", "firefox", false)
+            verifyPrefilledLoginCredentials(activityTestRule, "mozilla", "firefox", false)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/593768
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/593768
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun doNotSaveOptionWillNotUpdateALoginTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
         val originWebsite = "mozilla-mobile.github.io"
@@ -559,7 +584,7 @@ class LoginsTest : TestSetup() {
             waitForAppWindowToBeUpdated()
             clickPageObject(itemWithResId("submit"))
             verifySaveLoginPromptIsDisplayed()
-            clickPageObject(itemWithText("Don’t update"))
+            clickPageObject(itemWithText("Not now"))
         }.openThreeDotMenu {
         }.openSettings {
         }.openLoginsAndPasswordSubMenu {
@@ -571,10 +596,11 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2090455
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2090455
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun searchLoginsByUsernameTest() {
-        val firstLoginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val firstLoginPage = mockWebServer.saveLoginAsset
         val secondLoginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
         val originWebsite = "mozilla-mobile.github.io"
 
@@ -616,10 +642,11 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/608834
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/608834
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1935209"])
     fun searchLoginsByUrlTest() {
-        val firstLoginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val firstLoginPage = mockWebServer.saveLoginAsset
         val secondLoginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
         val originWebsite = "mozilla-mobile.github.io"
 
@@ -661,10 +688,10 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2266441
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2266441
     @Test
     fun verifyLastUsedLoginSortingOptionTest() {
-        val firstLoginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val firstLoginPage = mockWebServer.saveLoginAsset
         val secondLoginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
         val originWebsite = "mozilla-mobile.github.io"
 
@@ -696,7 +723,7 @@ class LoginsTest : TestSetup() {
             verifySortedLogin(1, firstLoginPage.url.authority.toString())
         }
 
-        restartApp(activityTestRule)
+        restartApp(activityTestRule.activityRule)
 
         browserScreen {
         }.openThreeDotMenu {
@@ -708,10 +735,10 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2266442
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2266442
     @Test
     fun verifyAlphabeticalLoginSortingOptionTest() {
-        val firstLoginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val firstLoginPage = mockWebServer.saveLoginAsset
         val secondLoginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
         val originWebsite = "mozilla-mobile.github.io"
 
@@ -741,7 +768,7 @@ class LoginsTest : TestSetup() {
             verifySortedLogin(1, originWebsite)
         }
 
-        restartApp(activityTestRule)
+        restartApp(activityTestRule.activityRule)
 
         browserScreen {
         }.openThreeDotMenu {
@@ -753,7 +780,7 @@ class LoginsTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1518435
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1518435
     @Test
     fun verifyAddLoginManuallyTest() {
         val loginPage = "https://mozilla-mobile.github.io/testapp/v2.0/loginForm.html"
@@ -783,19 +810,21 @@ class LoginsTest : TestSetup() {
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(loginPage.toUri()) {
-            clickPageObject(MatcherHelper.itemWithResId("username"))
+            clickPageObject(itemWithResId("username"))
             clickSuggestedLoginsButton()
-            verifySuggestedUserName("mozilla")
-            clickPageObject(itemWithResIdAndText("$packageName:id/username", "mozilla"))
+            verifySuggestedUserName(activityTestRule, "mozilla")
+            clickSuggestedLogin(activityTestRule, "mozilla")
             clickPageObject(itemWithResId("togglePassword"))
-            verifyPrefilledLoginCredentials("mozilla", "firefox", true)
+            verifyPrefilledLoginCredentials(activityTestRule, "mozilla", "firefox", true)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2068215
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2068215
+    // The snackbar is not displayed for devices running on Android <= 12
     @Test
+    @SdkSuppress(maxSdkVersion = 32)
     fun verifyCopyLoginCredentialsToClipboardTest() {
-        val firstLoginPage = TestAssetHelper.getSaveLoginAsset(mockWebServer)
+        val firstLoginPage = mockWebServer.saveLoginAsset
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstLoginPage.url) {
