@@ -6,14 +6,15 @@ package org.mozilla.fenix.ui
 
 import androidx.compose.ui.test.junit4.AndroidComposeTestRule
 import androidx.test.espresso.Espresso.pressBack
+import androidx.test.filters.SdkSuppress
 import okhttp3.mockwebserver.MockWebServer
 import org.junit.After
 import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
+import org.mozilla.fenix.customannotations.SkipLeaks
 import org.mozilla.fenix.customannotations.SmokeTest
-import org.mozilla.fenix.helpers.AppAndSystemHelper.runWithSystemLocaleChanged
-import org.mozilla.fenix.helpers.AppAndSystemHelper.setSystemLocale
+import org.mozilla.fenix.helpers.AppAndSystemHelper.runWithAppLocaleChanged
 import org.mozilla.fenix.helpers.DataGenerationHelper.setTextToClipBoard
 import org.mozilla.fenix.helpers.HomeActivityIntentTestRule
 import org.mozilla.fenix.helpers.MockBrowserDataHelper.addCustomSearchEngine
@@ -21,12 +22,13 @@ import org.mozilla.fenix.helpers.MockBrowserDataHelper.createBookmarkItem
 import org.mozilla.fenix.helpers.MockBrowserDataHelper.createHistoryItem
 import org.mozilla.fenix.helpers.SearchDispatcher
 import org.mozilla.fenix.helpers.TestAssetHelper.getGenericAsset
+import org.mozilla.fenix.helpers.TestAssetHelper.waitingTimeLong
 import org.mozilla.fenix.helpers.TestHelper.appContext
-import org.mozilla.fenix.helpers.TestHelper.clickSnackbarButton
 import org.mozilla.fenix.helpers.TestHelper.exitMenu
 import org.mozilla.fenix.helpers.TestHelper.restartApp
-import org.mozilla.fenix.helpers.TestHelper.verifySnackBarText
 import org.mozilla.fenix.helpers.TestSetup
+import org.mozilla.fenix.helpers.perf.DetectMemoryLeaksRule
+import org.mozilla.fenix.nimbus.FxNimbus
 import org.mozilla.fenix.ui.robots.EngineShortcut
 import org.mozilla.fenix.ui.robots.homeScreen
 import org.mozilla.fenix.ui.robots.navigationToolbar
@@ -47,6 +49,9 @@ class SettingsSearchTest : TestSetup() {
         HomeActivityIntentTestRule.withDefaultSettingsOverrides(),
     ) { it.activity }
 
+    @get:Rule
+    val memoryLeaksRule = DetectMemoryLeaksRule()
+
     @Before
     override fun setUp() {
         super.setUp()
@@ -62,7 +67,7 @@ class SettingsSearchTest : TestSetup() {
         searchMockServer.shutdown()
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203333
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203333
     @Test
     fun verifySearchSettingsMenuItemsTest() {
         homeScreen {
@@ -87,7 +92,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203307
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203307
     @Test
     fun verifyDefaultSearchEnginesSettingsItemsTest() {
         homeScreen {
@@ -102,7 +107,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203308
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203308
     @SmokeTest
     @Test
     fun verifyTheDefaultSearchEngineCanBeChangedTest() {
@@ -122,32 +127,91 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/233586
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/233586
     @Test
-    fun verifyUrlAutocompleteToggleTest() {
-        homeScreen {
-        }.openSearch {
-            typeSearch("mo")
-            verifyTypedToolbarText("monster.com")
-            typeSearch("moz")
-            verifyTypedToolbarText("mozilla.org")
-        }.dismissSearchBar {
-        }.openThreeDotMenu {
-        }.openSettings {
-        }.openSearchSubMenu {
-            toggleAutocomplete()
-        }.goBack {
-        }.goBack {
-        }.openSearch {
-            typeSearch("moz")
-            verifyTypedToolbarText("moz")
+    fun verifyEnabledUrlAutocompleteToggleTest() {
+        // Currently part of an experiment https://bugzilla.mozilla.org/show_bug.cgi?id=1842106
+        // Check if "Top domain" suggestions for the address bar's autocomplete are enabled
+        if (FxNimbus.features.suggestShippedDomains.value().enabled) {
+            // If true it will use the hardcoded list of "top domain" suggestions for the address bar's autocomplete suggestions
+            homeScreen {
+            }.openSearch {
+                typeSearch("mo")
+                verifyTypedToolbarText("monster.com", exists = true)
+                typeSearch("moz")
+                verifyTypedToolbarText("mozilla.org", exists = true)
+            }
+        } else {
+            // The suggestions for the address bar's autocomplete will take use of the user's local browsing history and bookmarks
+            createHistoryItem("https://github.com/mozilla-mobile/fenix")
+            createBookmarkItem("https://github.com/mozilla-mobile/focus-android", "focus-android", 1u)
+
+            homeScreen {
+            }.openSearch {
+                typeSearch("moz")
+                // "Top domain" suggestions from the address bar's autocomplete are disabled, "moz" shouldn't autocomplete to mozilla.org
+                verifyTypedToolbarText("mozilla.org", exists = false)
+                // The address bar's autocomplete should take use of the browsing history
+                // Autocomplete with the history items url
+                typeSearch("github.com/mozilla-mobile/f")
+                verifyTypedToolbarText("github.com/mozilla-mobile/fenix", exists = true)
+                // The address bar's autocomplete should also take use of the saved bookmarks
+                // Autocomplete with the bookmarked items url
+                typeSearch("github.com/mozilla-mobile/fo")
+                verifyTypedToolbarText("github.com/mozilla-mobile/focus-android", exists = true)
+                // It should not autocomplete with links that are not part of browsing history or bookmarks
+                typeSearch("github.com/mozilla-mobile/fi")
+                verifyTypedToolbarText("github.com/mozilla-mobile/firefox-android", exists = false)
+            }
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/361817
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2660692
+    @Test
+    fun verifyDisabledUrlAutocompleteToggleTest() {
+        // Currently part of an experiment https://bugzilla.mozilla.org/show_bug.cgi?id=1842106
+        // Check if "Top domain" suggestions for the address bar's autocomplete are enabled
+        if (FxNimbus.features.suggestShippedDomains.value().enabled) {
+            // If true it will use the hardcoded list of "top domain" suggestions for the address bar's autocomplete suggestions
+            homeScreen {
+            }.openThreeDotMenu {
+            }.openSettings {
+            }.openSearchSubMenu {
+                toggleAutocomplete()
+            }.goBack {
+            }.goBack {
+            }.openSearch {
+                typeSearch("moz")
+                verifyTypedToolbarText("moz", exists = true)
+                verifyTypedToolbarText("mozilla.org", exists = false)
+            }
+        } else {
+            // The suggestions for the address bar's autocomplete will take use of the user's local browsing history and bookmarks
+            createHistoryItem("https://github.com/mozilla-mobile/fenix")
+            createBookmarkItem("https://github.com/mozilla-mobile/focus-android", "focus-android", 1u)
+
+            homeScreen {
+            }.openThreeDotMenu {
+            }.openSettings {
+            }.openSearchSubMenu {
+                toggleAutocomplete()
+            }.goBack {
+            }.goBack {
+            }.openSearch {
+                // Having the setting disabled, it should not autocomplete anymore with the history items url
+                typeSearch("github.com/mozilla-mobile/f")
+                verifyTypedToolbarText("github.com/mozilla-mobile/fenix", exists = false)
+                // Having the setting disabled, it should not autocomplete anymore with the bookmarked items url
+                typeSearch("github.com/mozilla-mobile/fo")
+                verifyTypedToolbarText("github.com/mozilla-mobile/focus-android", exists = false)
+            }
+        }
+    }
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/361817
     @Test
     fun disableSearchBrowsingHistorySuggestionsToggleTest() {
-        val websiteURL = getGenericAsset(mockWebServer, 1).url.toString()
+        val websiteURL = mockWebServer.getGenericAsset(1).url.toString()
 
         createHistoryItem(websiteURL)
 
@@ -170,10 +234,10 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/412926
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/412926
     @Test
     fun disableSearchBookmarksToggleTest() {
-        val website = getGenericAsset(mockWebServer, 1)
+        val website = mockWebServer.getGenericAsset(1)
 
         createBookmarkItem(website.url.toString(), website.title, 1u)
 
@@ -196,10 +260,12 @@ class SettingsSearchTest : TestSetup() {
                 "Firefox Suggest",
                 website.title,
             )
+        }.dismissSearchBar {
+            verifyHomeComponent(activityTestRule)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203309
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203309
     // Verifies setting as default a customized search engine name and URL
     @SmokeTest
     @Test
@@ -219,7 +285,6 @@ class SettingsSearchTest : TestSetup() {
             typeCustomEngineDetails(customSearchEngine.title, customSearchEngine.url)
             verifySaveSearchEngineButtonEnabled(true)
             saveNewSearchEngine()
-            verifySnackBarText("Created ${customSearchEngine.title}")
             verifyEngineListContains(customSearchEngine.title, shouldExist = true)
             openEngineOverflowMenu(customSearchEngine.title)
             pressBack()
@@ -238,7 +303,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203335
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203335
     @Test
     fun addCustomSearchEngineToManageShortcutsListTest() {
         val customSearchEngine = object {
@@ -261,7 +326,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203343
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203343
     @Test
     fun verifyLearnMoreLinksFromAddSearchEngineSectionTest() {
         homeScreen {
@@ -271,22 +336,18 @@ class SettingsSearchTest : TestSetup() {
             openDefaultSearchEngineMenu()
             openAddSearchEngineMenu()
         }.clickCustomSearchStringLearnMoreLink {
-            verifyUrl(
-                "support.mozilla.org/en-US/kb/manage-my-default-search-engines-firefox-android?as=u&utm_source=inproduct",
-            )
+            verifyCustomSearchEngineLearnMoreURL()
         }.openThreeDotMenu {
         }.openSettings {
         }.openSearchSubMenu {
             openDefaultSearchEngineMenu()
             openAddSearchEngineMenu()
         }.clickCustomSearchSuggestionsLearnMoreLink {
-            verifyUrl(
-                "support.mozilla.org/en-US/kb/manage-my-default-search-engines-firefox-android?as=u&utm_source=inproduct",
-            )
+            verifyCustomSearchEngineLearnMoreURL()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203310
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203310
     @Test
     fun editCustomSearchEngineTest() {
         val customSearchEngine = object {
@@ -308,7 +369,6 @@ class SettingsSearchTest : TestSetup() {
             clickEdit()
             typeCustomEngineDetails(customSearchEngine.newTitle, customSearchEngine.url)
             saveEditSearchEngine()
-            verifySnackBarText("Saved ${customSearchEngine.newTitle}")
             verifyEngineListContains(customSearchEngine.newTitle, shouldExist = true)
             pressBack()
             openManageShortcutsMenu()
@@ -316,7 +376,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203312
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203312
     @Test
     fun verifyErrorMessagesForInvalidSearchEngineUrlsTest() {
         val customSearchEngine = object {
@@ -335,20 +395,17 @@ class SettingsSearchTest : TestSetup() {
             typeCustomEngineDetails(customSearchEngine.title, customSearchEngine.badTemplateUrl)
             saveNewSearchEngine()
             verifyInvalidTemplateSearchStringFormatError()
-            typeCustomEngineDetails(customSearchEngine.title, customSearchEngine.typoUrl)
-            saveNewSearchEngine()
-            verifyErrorConnectingToSearchString(customSearchEngine.title)
             typeCustomEngineDetails(customSearchEngine.title, customSearchEngine.goodUrl)
             typeSearchEngineSuggestionString(customSearchEngine.badTemplateUrl)
             saveNewSearchEngine()
             verifyInvalidTemplateSearchStringFormatError()
-            typeSearchEngineSuggestionString(customSearchEngine.typoUrl)
+            typeCustomEngineDetails(customSearchEngine.title, customSearchEngine.typoUrl)
             saveNewSearchEngine()
-            verifyErrorConnectingToSearchString(customSearchEngine.title)
+            verifyEngineListContains(customSearchEngine.title, shouldExist = true)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203313
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203313
     @Test
     fun deleteCustomSearchEngineTest() {
         val customSearchEngineTitle = "TestSearchEngine"
@@ -364,12 +421,6 @@ class SettingsSearchTest : TestSetup() {
             verifyEngineListContains(customSearchEngineTitle, shouldExist = true)
             openEngineOverflowMenu(customSearchEngineTitle)
             clickDeleteSearchEngine()
-            verifySnackBarText("Deleted $customSearchEngineTitle")
-            clickSnackbarButton("UNDO")
-            verifyEngineListContains(customSearchEngineTitle, shouldExist = true)
-            changeDefaultSearchEngine(customSearchEngineTitle)
-            openEngineOverflowMenu(customSearchEngineTitle)
-            clickDeleteSearchEngine()
             verifyEngineListContains(customSearchEngineTitle, shouldExist = false)
             verifyDefaultSearchEngineSelected("Google")
             pressBack()
@@ -383,7 +434,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203339
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203339
     @Test
     fun deleteCustomSearchShortcutTest() {
         val customSearchEngineTitle = "TestSearchEngine"
@@ -411,7 +462,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/233588
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/233588
     // Test running on beta/release builds in CI:
     // caution when making changes to it, so they don't block the builds
     // Goes through the settings and changes the search suggestion toggle, then verifies it changes.
@@ -425,11 +476,7 @@ class SettingsSearchTest : TestSetup() {
             clickSearchSelectorButton()
             selectTemporarySearchMethod("DuckDuckGo")
             typeSearch("mozilla ")
-            verifySearchEngineSuggestionResults(
-                activityTestRule,
-                "mozilla firefox",
-                searchTerm = "mozilla ",
-            )
+            verifySearchSuggestionsAreDisplayed(activityTestRule, "mozilla firefox")
         }.dismissSearchBar {
         }.openThreeDotMenu {
         }.openSettings {
@@ -447,12 +494,12 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/464420
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/464420
     // Tests the "Don't allow" option from private mode search suggestions onboarding dialog
     @Test
     fun doNotAllowSearchSuggestionsInPrivateBrowsingTest() {
         homeScreen {
-            togglePrivateBrowsingModeOnOff()
+            togglePrivateBrowsingModeOnOff(composeTestRule = activityTestRule)
         }.openSearch {
             typeSearch("mozilla")
             verifyAllowSuggestionsInPrivateModeDialog()
@@ -461,21 +508,17 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1957063
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1957063
     // Tests the "Allow" option from private mode search suggestions onboarding dialog
     @Test
     fun allowSearchSuggestionsInPrivateBrowsingTest() {
         homeScreen {
-            togglePrivateBrowsingModeOnOff()
+            togglePrivateBrowsingModeOnOff(composeTestRule = activityTestRule)
         }.openSearch {
             typeSearch("mozilla")
             verifyAllowSuggestionsInPrivateModeDialog()
             allowSuggestionsInPrivateMode()
-            verifySearchEngineSuggestionResults(
-                activityTestRule,
-                "mozilla firefox",
-                searchTerm = "mozilla",
-            )
+            verifySearchSuggestionsAreDisplayed(activityTestRule, "mozilla firefox")
         }.dismissSearchBar {
         }.openThreeDotMenu {
         }.openSettings {
@@ -489,7 +532,8 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/888673
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/888673
+    @SdkSuppress(minSdkVersion = 34)
     @Test
     fun verifyShowVoiceSearchToggleTest() {
         homeScreen {
@@ -509,7 +553,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/412927
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/412927
     @Test
     fun verifyShowClipboardSuggestionsToggleTest() {
         val link = "https://www.mozilla.org/en-US/"
@@ -519,8 +563,8 @@ class SettingsSearchTest : TestSetup() {
         }.openNavigationToolbar {
             verifyClipboardSuggestionsAreDisplayed(link, true)
         }.visitLinkFromClipboard {
-            waitForPageToLoad()
-        }.openTabDrawer {
+            waitForPageToLoad(pageLoadWaitingTime = waitingTimeLong)
+        }.openTabDrawer(activityTestRule) {
         }.openNewTab {
         }
         navigationToolbar {
@@ -528,7 +572,7 @@ class SettingsSearchTest : TestSetup() {
             verifyClipboardSuggestionsAreDisplayed(shouldBeDisplayed = false)
         }.goBackToHomeScreen {
             setTextToClipBoard(appContext, link)
-        }.openTabDrawer {
+        }.openTabDrawer(activityTestRule) {
         }.openNewTab {
         }
         navigationToolbar {
@@ -543,7 +587,7 @@ class SettingsSearchTest : TestSetup() {
             exitMenu()
         }
         homeScreen {
-        }.openTabDrawer {
+        }.openTabDrawer(activityTestRule) {
         }.openNewTab {
         }
         navigationToolbar {
@@ -551,10 +595,11 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2233337
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2233337
     @Test
+    @SkipLeaks
     fun verifyTheSearchEnginesListsRespectTheLocaleTest() {
-        runWithSystemLocaleChanged(Locale.CHINA, activityTestRule.activityRule) {
+        runWithAppLocaleChanged(Locale.CHINA, activityTestRule.activityRule) {
             // Checking search engines for CH locale
             homeScreen {
             }.openSearch {
@@ -566,9 +611,10 @@ class SettingsSearchTest : TestSetup() {
                     "DuckDuckGo",
                 )
             }.dismissSearchBar {}
+        }
 
+        runWithAppLocaleChanged(Locale.FRENCH, activityTestRule.activityRule) {
             // Checking search engines for FR locale
-            setSystemLocale(Locale.FRENCH)
             homeScreen {
             }.openSearch {
                 clickSearchSelectorButton()
@@ -583,7 +629,7 @@ class SettingsSearchTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203334
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203334
     @Test
     fun verifyManageSearchShortcutsSettingsItemsTest() {
         homeScreen {
@@ -598,15 +644,14 @@ class SettingsSearchTest : TestSetup() {
                 EngineShortcut(name = "Google", checkboxIndex = 1, isChecked = true),
                 EngineShortcut(name = "Bing", checkboxIndex = 4, isChecked = true),
                 EngineShortcut(name = "DuckDuckGo", checkboxIndex = 7, isChecked = true),
-                EngineShortcut(name = "eBay", checkboxIndex = 10, isChecked = true),
-                EngineShortcut(name = "Wikipedia", checkboxIndex = 13, isChecked = true),
-                EngineShortcut(name = "Reddit", checkboxIndex = 16, isChecked = false),
-                EngineShortcut(name = "YouTube", checkboxIndex = 19, isChecked = false),
+                EngineShortcut(name = "Wikipedia (en)", checkboxIndex = 10, isChecked = true),
+                EngineShortcut(name = "Reddit", checkboxIndex = 13, isChecked = false),
+                EngineShortcut(name = "YouTube", checkboxIndex = 16, isChecked = false),
             )
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2203340
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2203340
     @SmokeTest
     @Test
     fun verifySearchShortcutChangesAreReflectedInSearchSelectorMenuTest() {
@@ -616,8 +661,8 @@ class SettingsSearchTest : TestSetup() {
         }.openSearchSubMenu {
             openManageShortcutsMenu()
             selectSearchShortcut(EngineShortcut(name = "Google", checkboxIndex = 1))
-            selectSearchShortcut(EngineShortcut(name = "Reddit", checkboxIndex = 16))
-            selectSearchShortcut(EngineShortcut(name = "YouTube", checkboxIndex = 19))
+            selectSearchShortcut(EngineShortcut(name = "Reddit", checkboxIndex = 13))
+            selectSearchShortcut(EngineShortcut(name = "YouTube", checkboxIndex = 16))
             exitMenu()
         }
         searchScreen {
