@@ -4,23 +4,27 @@
 
 package org.mozilla.fenix.ui
 
-import com.google.android.material.bottomsheet.BottomSheetBehavior
-import mozilla.components.concept.engine.mediasession.MediaSession
+import androidx.compose.ui.test.junit4.AndroidComposeTestRule
+import org.junit.Ignore
 import org.junit.Rule
 import org.junit.Test
+import org.mozilla.fenix.customannotations.SkipLeaks
 import org.mozilla.fenix.customannotations.SmokeTest
+import org.mozilla.fenix.ext.settings
 import org.mozilla.fenix.helpers.HomeActivityIntentTestRule
-import org.mozilla.fenix.helpers.MatcherHelper
+import org.mozilla.fenix.helpers.MockBrowserDataHelper
 import org.mozilla.fenix.helpers.RetryTestRule
-import org.mozilla.fenix.helpers.TestAssetHelper
+import org.mozilla.fenix.helpers.TestAssetHelper.genericAssets
+import org.mozilla.fenix.helpers.TestAssetHelper.getGenericAsset
+import org.mozilla.fenix.helpers.TestHelper.appContext
 import org.mozilla.fenix.helpers.TestHelper.clickSnackbarButton
 import org.mozilla.fenix.helpers.TestHelper.closeApp
 import org.mozilla.fenix.helpers.TestHelper.mDevice
 import org.mozilla.fenix.helpers.TestHelper.restartApp
 import org.mozilla.fenix.helpers.TestHelper.verifySnackBarText
 import org.mozilla.fenix.helpers.TestSetup
+import org.mozilla.fenix.helpers.perf.DetectMemoryLeaksRule
 import org.mozilla.fenix.ui.robots.browserScreen
-import org.mozilla.fenix.ui.robots.clickPageObject
 import org.mozilla.fenix.ui.robots.homeScreen
 import org.mozilla.fenix.ui.robots.navigationToolbar
 import org.mozilla.fenix.ui.robots.notificationShade
@@ -43,26 +47,34 @@ import org.mozilla.fenix.ui.robots.notificationShade
  */
 
 class TabbedBrowsingTest : TestSetup() {
-    @get:Rule
-    val activityTestRule = HomeActivityIntentTestRule.withDefaultSettingsOverrides(skipOnboarding = true)
+    @get:Rule(order = 0)
+    val composeTestRule =
+        AndroidComposeTestRule(
+            HomeActivityIntentTestRule.withDefaultSettingsOverrides(
+                skipOnboarding = true,
+            ),
+        ) { it.activity }
 
-    @Rule
+    @get:Rule(order = 1)
+    val memoryLeaksRule = DetectMemoryLeaksRule()
+
+    @Rule(order = 2)
     @JvmField
     val retryTestRule = RetryTestRule(3)
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903599
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903599
+    @Ignore("disabled - https://bugzilla.mozilla.org/show_bug.cgi?id=1989405")
     @Test
     fun closeAllTabsTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(defaultWebPage.url) {
-        }.openTabDrawer {
-            verifyExistingTabList()
-        }.openTabsListThreeDotMenu {
+        }.openTabDrawer(composeTestRule) {
+            verifyNormalTabsList()
+        }.openThreeDotMenu {
             verifyCloseAllTabsButton()
-            verifyShareTabButton()
-            verifySelectTabs()
+            verifySelectTabsButton()
         }.closeAllTabs {
             verifyTabCounter("0")
         }
@@ -73,55 +85,63 @@ class TabbedBrowsingTest : TestSetup() {
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(defaultWebPage.url) {
-        }.openTabDrawer {
-            verifyPrivateModeSelected()
-            verifyExistingTabList()
-        }.openTabsListThreeDotMenu {
+        }.openTabDrawer(composeTestRule) {
+            verifyPrivateTabsList()
+        }.openThreeDotMenu {
             verifyCloseAllTabsButton()
         }.closeAllTabs {
             verifyTabCounter("0")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2349580
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2349580
     @Test
     fun closingTabsTest() {
-        val genericURL = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val genericURL = mockWebServer.getGenericAsset(1)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(genericURL.url) {
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
             verifyExistingOpenTabs("Test_Page_1")
             closeTab()
             verifySnackBarText("Tab closed")
-            clickSnackbarButton("UNDO")
+            clickSnackbarButton(composeTestRule, "UNDO")
         }
         browserScreen {
             verifyTabCounter("1")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903604
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903604
     @Test
     fun swipeToCloseTabsTest() {
-        val genericURL = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val webPages = mockWebServer.genericAssets
+
+        MockBrowserDataHelper.createTabItem(webPages[0].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[1].url.toString())
 
-        navigationToolbar {
-        }.enterURLAndEnterToBrowser(genericURL.url) {
-            waitForPageToLoad()
-        }.openTabDrawer {
-            verifyExistingOpenTabs("Test_Page_1")
-            swipeTabRight("Test_Page_1")
-            verifySnackBarText("Tab closed")
-        }
         homeScreen {
-            verifyTabCounter("0")
-        }.openNavigationToolbar {
-        }.enterURLAndEnterToBrowser(genericURL.url) {
-            waitForPageToLoad()
-        }.openTabDrawer {
-            verifyExistingOpenTabs("Test_Page_1")
-            swipeTabLeft("Test_Page_1")
+        }.openTabDrawer(composeTestRule) {
+            verifyExistingOpenTabs(webPages[0].title)
+            verifyExistingOpenTabs(webPages[1].title)
+            swipeTabRight(webPages[0].title)
+            verifySnackBarText("Tab closed")
+            clickSnackbarButton(composeTestRule, "UNDO")
+            verifyExistingOpenTabs(webPages[0].title)
+            verifyExistingOpenTabs(webPages[1].title)
+            swipeTabRight(webPages[0].title)
+            verifySnackBarText("Tab closed")
+            verifyNoExistingOpenTabs(webPages[0].title)
+            verifyExistingOpenTabs(webPages[1].title)
+            swipeTabLeft(webPages[1].title)
+            verifySnackBarText("Tab closed")
+            clickSnackbarButton(composeTestRule, "UNDO")
+        }
+        browserScreen {
+            verifyPageContent(webPages[1].content)
+        }.openTabDrawer(composeTestRule) {
+            verifyExistingOpenTabs(webPages[1].title)
+            swipeTabLeft(webPages[1].title)
             verifySnackBarText("Tab closed")
         }
         homeScreen {
@@ -129,51 +149,31 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903591
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903591
     @Test
     fun closingPrivateTabsTest() {
-        val genericURL = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val genericURL = mockWebServer.getGenericAsset(1)
 
         homeScreen { }.togglePrivateBrowsingMode(switchPBModeOn = true)
         navigationToolbar {
         }.enterURLAndEnterToBrowser(genericURL.url) {
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
             verifyExistingOpenTabs("Test_Page_1")
-            verifyCloseTabsButton("Test_Page_1")
             closeTab()
             verifySnackBarText("Private tab closed")
-            clickSnackbarButton("UNDO")
+            clickSnackbarButton(composeTestRule, "UNDO")
         }
         browserScreen {
             verifyTabCounter("1")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903606
-    @SmokeTest
-    @Test
-    fun tabMediaControlButtonTest() {
-        val audioTestPage = TestAssetHelper.getAudioPageAsset(mockWebServer)
-
-        navigationToolbar {
-        }.enterURLAndEnterToBrowser(audioTestPage.url) {
-            mDevice.waitForIdle()
-            clickPageObject(MatcherHelper.itemWithText("Play"))
-            assertPlaybackState(browserStore, MediaSession.PlaybackState.PLAYING)
-        }.openTabDrawer {
-            verifyTabMediaControlButtonState("Pause")
-            clickTabMediaControlButton("Pause")
-            verifyTabMediaControlButtonState("Play")
-        }.openTab(audioTestPage.title) {
-            assertPlaybackState(browserStore, MediaSession.PlaybackState.PAUSED)
-        }
-    }
-
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903592
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903592
     @SmokeTest
     @Test
+    @SkipLeaks
     fun verifyCloseAllPrivateTabsNotificationTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         homeScreen {
         }.togglePrivateBrowsingMode()
@@ -190,156 +190,98 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903598
-    @SmokeTest
-    @Test
-    fun shareTabsFromTabsTrayTest() {
-        val firstWebsite = TestAssetHelper.getGenericAsset(mockWebServer, 1)
-        val secondWebsite = TestAssetHelper.getGenericAsset(mockWebServer, 2)
-        val firstWebsiteTitle = firstWebsite.title
-        val secondWebsiteTitle = secondWebsite.title
-        val sharingApp = "Gmail"
-        val sharedUrlsString = "${firstWebsite.url}\n\n${secondWebsite.url}"
-
-        homeScreen {
-        }.openNavigationToolbar {
-        }.enterURLAndEnterToBrowser(firstWebsite.url) {
-            verifyPageContent(firstWebsite.content)
-        }.openTabDrawer {
-        }.openNewTab {
-        }.submitQuery(secondWebsite.url.toString()) {
-            verifyPageContent(secondWebsite.content)
-        }.openTabDrawer {
-            verifyExistingOpenTabs("Test_Page_1")
-            verifyExistingOpenTabs("Test_Page_2")
-        }.openTabsListThreeDotMenu {
-            verifyShareAllTabsButton()
-        }.clickShareAllTabsButton {
-            verifyShareTabsOverlay(firstWebsiteTitle, secondWebsiteTitle)
-            verifySharingWithSelectedApp(
-                sharingApp,
-                sharedUrlsString,
-                "$firstWebsiteTitle, $secondWebsiteTitle",
-            )
-        }
-    }
-
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903602
-    @Test
-    fun verifyTabTrayNotShowingStateHalfExpanded() {
-        navigationToolbar {
-        }.openTabTray {
-            verifyNoOpenTabsInNormalBrowsing()
-            // With no tabs opened the state should be STATE_COLLAPSED.
-            verifyBehaviorState(BottomSheetBehavior.STATE_COLLAPSED)
-            // Need to ensure the halfExpandedRatio is very small so that when in STATE_HALF_EXPANDED
-            // the tabTray will actually have a very small height (for a very short time) akin to being hidden.
-            verifyHalfExpandedRatio()
-        }.clickTopBar {
-        }.waitForTabTrayBehaviorToIdle {
-            // Touching the topBar would normally advance the tabTray to the next state.
-            // We don't want that.
-            verifyBehaviorState(BottomSheetBehavior.STATE_COLLAPSED)
-        }.advanceToHalfExpandedState {
-        }.waitForTabTrayBehaviorToIdle {
-            // TabTray should not be displayed in STATE_HALF_EXPANDED.
-            // When advancing to this state it should immediately be hidden.
-            verifyTabTrayIsClosed()
-        }
-    }
-
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903600
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903600
     @Test
     fun verifyEmptyTabTray() {
-        navigationToolbar {
-        }.openTabTray {
-            verifyNormalBrowsingButtonIsSelected(true)
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
+            verifyNormalBrowsingButtonIsSelected()
             verifyPrivateBrowsingButtonIsSelected(false)
             verifySyncedTabsButtonIsSelected(false)
             verifyNoOpenTabsInNormalBrowsing()
-            verifyNormalBrowsingNewTabButton()
-            verifyTabTrayOverflowMenu(true)
-            verifyEmptyTabsTrayMenuButtons()
+            verifyFab()
+            verifyThreeDotButton()
+        }.openThreeDotMenu {
+            verifyTabSettingsButton()
+            verifyRecentlyClosedTabsButton()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903585
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903585
     @Test
     fun verifyEmptyPrivateTabsTrayTest() {
-        navigationToolbar {
-        }.openTabTray {
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
         }.toggleToPrivateTabs {
             verifyNormalBrowsingButtonIsSelected(false)
             verifyPrivateBrowsingButtonIsSelected(true)
             verifySyncedTabsButtonIsSelected(false)
             verifyNoOpenTabsInPrivateBrowsing()
-            verifyPrivateBrowsingNewTabButton()
-            verifyTabTrayOverflowMenu(true)
-            verifyEmptyTabsTrayMenuButtons()
+            verifyFab()
+            verifyThreeDotButton()
+        }.openThreeDotMenu {
+            verifyTabSettingsButton()
+            verifyRecentlyClosedTabsButton()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903601
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903601
     @Test
     fun verifyTabsTrayWithOpenTabTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
-        homeScreen {
-        }.openTabDrawer {
-        }.openNewTab {
-        }.submitQuery(defaultWebPage.url.toString()) {
-        }.openTabDrawer {
-            verifyNormalBrowsingButtonIsSelected(true)
-            verifyPrivateBrowsingButtonIsSelected(false)
-            verifySyncedTabsButtonIsSelected(false)
-            verifyTabTrayOverflowMenu(true)
-            verifyTabsTrayCounter()
-            verifyExistingTabList()
-            verifyNormalBrowsingNewTabButton()
-            verifyOpenedTabThumbnail()
+        navigationToolbar {
+        }.enterURLAndEnterToBrowser(defaultWebPage.url) {
+        }.openTabDrawer(composeTestRule) {
+            verifyNormalBrowsingButtonIsSelected()
+            verifyPrivateBrowsingButtonIsSelected(isSelected = false)
+            verifySyncedTabsButtonIsSelected(isSelected = false)
+            verifyThreeDotButton()
+            verifyNormalTabsList()
+            verifyFab()
+            verifyTabThumbnail()
             verifyExistingOpenTabs(defaultWebPage.title)
-            verifyCloseTabsButton(defaultWebPage.title)
+            verifyTabCloseButton()
         }.openTab(defaultWebPage.title) {
             verifyUrl(defaultWebPage.url.toString())
             verifyTabCounter("1")
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/903587
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903587
     @SmokeTest
     @Test
     fun verifyPrivateTabsTrayWithOpenTabTest() {
-        val website = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val website = mockWebServer.getGenericAsset(1)
 
         homeScreen {
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
         }.toggleToPrivateTabs {
         }.openNewTab {
         }.submitQuery(website.url.toString()) {
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
             verifyNormalBrowsingButtonIsSelected(false)
             verifyPrivateBrowsingButtonIsSelected(true)
             verifySyncedTabsButtonIsSelected(false)
-            verifyTabTrayOverflowMenu(true)
-            verifyTabsTrayCounter()
-            verifyExistingTabList()
+            verifyThreeDotButton()
+            verifyPrivateTabsList()
             verifyExistingOpenTabs(website.title)
-            verifyCloseTabsButton(website.title)
-            verifyOpenedTabThumbnail()
-            verifyPrivateBrowsingNewTabButton()
+            verifyTabCloseButton()
+            verifyTabThumbnail()
+            verifyFab()
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/927314
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/927314
     @Test
     fun tabsCounterShortcutMenuCloseTabTest() {
-        val firstWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
-        val secondWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 2)
+        val firstWebPage = mockWebServer.getGenericAsset(1)
+        val secondWebPage = mockWebServer.getGenericAsset(2)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstWebPage.url) {
             waitForPageToLoad()
-        }.goToHomescreen {
+        }.goToHomescreen(composeTestRule) {
         }.openNavigationToolbar {
         }.enterURLAndEnterToBrowser(secondWebPage.url) {
             waitForPageToLoad()
@@ -355,10 +297,16 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2343663
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2343663
     @Test
+    @SkipLeaks(
+        reasons = [
+            "https://bugzilla.mozilla.org/show_bug.cgi?id=1962065",
+            "https://bugzilla.mozilla.org/show_bug.cgi?id=1962070",
+        ],
+    )
     fun tabsCounterShortcutMenuNewPrivateTabTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(defaultWebPage.url) {}
@@ -371,10 +319,11 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2343662
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2343662
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1962065"])
     fun tabsCounterShortcutMenuNewTabTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(defaultWebPage.url) {}
@@ -387,17 +336,18 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/927315
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/927315
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1962065"])
     fun privateTabsCounterShortcutMenuCloseTabTest() {
-        val firstWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
-        val secondWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 2)
+        val firstWebPage = mockWebServer.getGenericAsset(1)
+        val secondWebPage = mockWebServer.getGenericAsset(2)
 
         homeScreen {}.togglePrivateBrowsingMode(switchPBModeOn = true)
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstWebPage.url) {
             waitForPageToLoad()
-        }.goToHomescreen {
+        }.goToHomescreen(composeTestRule) {
         }.openNavigationToolbar {
         }.enterURLAndEnterToBrowser(secondWebPage.url) {
             waitForPageToLoad()
@@ -418,10 +368,11 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2344199
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2344199
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1962065"])
     fun privateTabsCounterShortcutMenuNewPrivateTabTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         homeScreen {}.togglePrivateBrowsingMode(switchPBModeOn = true)
         navigationToolbar {
@@ -437,10 +388,11 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2344198
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2344198
     @Test
+    @SkipLeaks(reasons = ["https://bugzilla.mozilla.org/show_bug.cgi?id=1962065"])
     fun privateTabsCounterShortcutMenuNewTabTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         homeScreen {}.togglePrivateBrowsingMode(switchPBModeOn = true)
         navigationToolbar {
@@ -456,13 +408,12 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/1046683
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1046683
     @Test
     fun verifySyncedTabsWhenUserIsNotSignedInTest() {
         navigationToolbar {
-        }.openTabTray {
+        }.openTabDrawer(composeTestRule) {
             verifySyncedTabsButtonIsSelected(isSelected = false)
-            clickSyncedTabsButton()
         }.toggleToSyncedTabs {
             verifySyncedTabsButtonIsSelected(isSelected = true)
             verifySyncedTabsListWhenUserIsNotSignedIn()
@@ -471,46 +422,142 @@ class TabbedBrowsingTest : TestSetup() {
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/526244
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/903598
+    @SmokeTest
+    @Test
+    fun shareTabsFromTabsTrayTest() {
+        val firstWebsite = mockWebServer.getGenericAsset(1)
+        val secondWebsite = mockWebServer.getGenericAsset(2)
+        val firstWebsiteTitle = firstWebsite.title
+        val secondWebsiteTitle = secondWebsite.title
+        val sharingApp = "Gmail"
+        val sharedUrlsString = "${firstWebsite.url}\n\n${secondWebsite.url}"
+
+        homeScreen {
+        }.openNavigationToolbar {
+        }.enterURLAndEnterToBrowser(firstWebsite.url) {
+            verifyPageContent(firstWebsite.content)
+        }.openTabDrawer(composeTestRule) {
+        }.openNewTab {
+        }.submitQuery(secondWebsite.url.toString()) {
+            verifyPageContent(secondWebsite.content)
+        }.openTabDrawer(composeTestRule) {
+            verifyExistingOpenTabs("Test_Page_1")
+            verifyExistingOpenTabs("Test_Page_2")
+        }.openThreeDotMenu {
+            verifySelectTabsButton()
+        }.clickSelectTabsButton {
+            selectTab("Test_Page_1", 1)
+            selectTab("Test_Page_2", 2)
+        }.openThreeDotMenu {
+            verifyShareTabsButton()
+        }.clickShareTabsButton {
+            verifyShareTabsOverlay(firstWebsiteTitle, secondWebsiteTitle)
+            verifySharingWithSelectedApp(
+                sharingApp,
+                sharedUrlsString,
+                "$firstWebsiteTitle, $secondWebsiteTitle",
+            )
+        }
+    }
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/526244
     @Test
     fun privateModeStaysAsDefaultAfterRestartTest() {
-        val defaultWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
+        val defaultWebPage = mockWebServer.getGenericAsset(1)
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(defaultWebPage.url) {
-        }.goToHomescreen {
+        }.goToHomescreen(composeTestRule) {
         }.togglePrivateBrowsingMode()
-        closeApp(activityTestRule)
-        restartApp(activityTestRule)
+
+        closeApp(composeTestRule.activityRule)
+        restartApp(composeTestRule.activityRule)
+
         homeScreen {
             verifyPrivateBrowsingHomeScreenItems()
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
         }.toggleToNormalTabs {
             verifyExistingOpenTabs(defaultWebPage.title)
         }
     }
 
-    // TestRail link: https://testrail.stage.mozaws.net/index.php?/cases/view/2228470
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/2228470
     @SmokeTest
     @Test
     fun privateTabsDoNotPersistAfterClosingAppTest() {
-        val firstWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 1)
-        val secondWebPage = TestAssetHelper.getGenericAsset(mockWebServer, 2)
+        val firstWebPage = mockWebServer.getGenericAsset(1)
+        val secondWebPage = mockWebServer.getGenericAsset(2)
 
         homeScreen {
         }.togglePrivateBrowsingMode()
 
         navigationToolbar {
         }.enterURLAndEnterToBrowser(firstWebPage.url) {
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
         }.openNewTab {
-        }.submitQuery(secondWebPage.url.toString()) {}
-        closeApp(activityTestRule)
-        restartApp(activityTestRule)
+        }.submitQuery(secondWebPage.url.toString()) {
+        }
+        closeApp(composeTestRule.activityRule)
+        restartApp(composeTestRule.activityRule)
         homeScreen {
             verifyPrivateBrowsingHomeScreenItems()
-        }.openTabDrawer {
+        }.openTabDrawer(composeTestRule) {
             verifyNoOpenTabsInPrivateBrowsing()
         }
     }
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/3024942
+    @Test
+    fun verifyTabsTrayListView() {
+        appContext.settings().gridTabView = false
+
+        val webPages = mockWebServer.genericAssets
+
+        MockBrowserDataHelper.createTabItem(webPages[0].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[1].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[2].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[3].url.toString())
+
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
+            verifyNormalTabsList()
+        }.closeTabDrawer {}
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
+            verifyOpenTabsOrder(title = webPages[0].title, position = 1, isListViewEnabled = true)
+            verifyOpenTabsOrder(title = webPages[1].title, position = 2, isListViewEnabled = true)
+            verifyOpenTabsOrder(title = webPages[2].title, position = 3, isListViewEnabled = true)
+            verifyOpenTabsOrder(title = webPages[3].title, position = 4, isListViewEnabled = true)
+            swipeTabLeft(title = webPages[0].title, isListViewEnabled = true)
+            verifyOpenTabsOrder(title = webPages[1].title, position = 1, isListViewEnabled = true)
+        }
+    }
+
+    // TestRail link: https://mozilla.testrail.io/index.php?/cases/view/1126911
+    @Test
+    fun verifyTabsTrayGridView() {
+        appContext.settings().gridTabView = true
+
+        val webPages = mockWebServer.genericAssets
+
+        MockBrowserDataHelper.createTabItem(webPages[0].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[1].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[2].url.toString())
+        MockBrowserDataHelper.createTabItem(webPages[3].url.toString())
+
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
+            verifyNormalTabsList()
+        }.closeTabDrawer {}
+        homeScreen {
+        }.openTabDrawer(composeTestRule) {
+            verifyOpenTabsOrder(title = webPages[0].title, position = 1)
+            verifyOpenTabsOrder(title = webPages[1].title, position = 2)
+            verifyOpenTabsOrder(title = webPages[2].title, position = 3)
+            verifyOpenTabsOrder(title = webPages[3].title, position = 4)
+            swipeTabLeft(title = webPages[0].title)
+            verifyOpenTabsOrder(title = webPages[1].title, position = 1)
+        }
+    }
 }
