@@ -6,13 +6,21 @@ package org.mozilla.fenix.ui.robots
 
 import android.app.NotificationManager
 import android.content.Context
+import android.os.Build
+import android.os.Build.VERSION.SDK_INT
 import android.util.Log
+import androidx.test.uiautomator.UiObject
 import androidx.test.uiautomator.UiScrollable
 import androidx.test.uiautomator.UiSelector
+import org.mozilla.fenix.R
 import org.mozilla.fenix.helpers.Constants.RETRY_COUNT
 import org.mozilla.fenix.helpers.Constants.TAG
+import org.mozilla.fenix.helpers.DataGenerationHelper.getStringResource
 import org.mozilla.fenix.helpers.MatcherHelper.assertUIObjectExists
+import org.mozilla.fenix.helpers.MatcherHelper.assertUIObjectIsGone
 import org.mozilla.fenix.helpers.MatcherHelper.itemContainingText
+import org.mozilla.fenix.helpers.MatcherHelper.itemWithDescription
+import org.mozilla.fenix.helpers.MatcherHelper.itemWithResIdContainingText
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithText
 import org.mozilla.fenix.helpers.TestAssetHelper.waitingTime
 import org.mozilla.fenix.helpers.TestAssetHelper.waitingTimeShort
@@ -67,15 +75,24 @@ class NotificationRobot {
     }
 
     fun verifySystemNotificationDoesNotExist(notificationMessage: String) {
-        Log.i(TAG, "verifySystemNotificationDoesNotExist: Waiting for $waitingTime ms for notification: $notificationMessage to be gone")
-        mDevice.findObject(UiSelector().textContains(notificationMessage)).waitUntilGone(waitingTime)
-        Log.i(TAG, "verifySystemNotificationDoesNotExist: Waited for $waitingTime ms for notification: $notificationMessage to be gone")
-        assertUIObjectExists(itemContainingText(notificationMessage), exists = false)
+        mDevice.waitForWindowUpdate(packageName, waitingTime)
+        assertUIObjectIsGone(itemContainingText(notificationMessage), waitingTime = waitingTime)
     }
 
     fun verifyPrivateTabsNotification() {
-        verifySystemNotificationExists("$appName (Private)")
-        verifySystemNotificationExists("Close private tabs")
+        when (Build.VERSION.SDK_INT) {
+            // For API 34 the notification is slightly different
+            Build.VERSION_CODES.UPSIDE_DOWN_CAKE ->
+                {
+                    verifySystemNotificationExists(getStringResource(R.string.notification_erase_title_android_14))
+                    verifySystemNotificationExists(getStringResource(R.string.notification_erase_text_android_14))
+                }
+            else ->
+                {
+                    verifySystemNotificationExists("$appName (Private)")
+                    verifySystemNotificationExists("Close private tabs")
+                }
+        }
     }
 
     fun clickMediaNotificationControlButton(action: String) {
@@ -116,22 +133,31 @@ class NotificationRobot {
     fun verifyMediaSystemNotificationButtonState(action: String) =
         assertUIObjectExists(mediaSystemNotificationButton(action))
 
-    fun expandNotificationMessage() {
-        while (!notificationHeader().exists()) {
-            Log.i(TAG, "expandNotificationMessage: Waiting for $appName notification to exist")
-            scrollToEnd()
-        }
-
-        if (notificationHeader().exists()) {
-            Log.i(TAG, "expandNotificationMessage: $appName notification exists")
+    fun expandNotificationMessage(notificationItem: String = "") {
+        if (SDK_INT == Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+            Log.i(TAG, "expandNotificationMessage: Trying to expand notification: $notificationItem using a swipe down action")
+            mDevice.findObject(
+                UiSelector().resourceId("android:id/status_bar_latest_event_content")
+                    .childSelector(UiSelector().resourceId("android:id/notification_headerless_view_row"))
+                    .childSelector(UiSelector().resourceId("android:id/notification_headerless_view_column"))
+                    .childSelector(UiSelector().resourceId("android:id/notification_top_line"))
+                    .childSelector(UiSelector().textContains(notificationItem)),
+            ).swipeDown(10)
+            Log.i(TAG, "expandNotificationMessage: Expanded notification: $notificationItem using a swipe down action")
+        } else {
+            verifySystemNotificationExists(appName)
+            while (!notificationHeader().exists()) {
+                Log.i(TAG, "expandNotificationMessage: Waiting for $appName notification to exist")
+                scrollToEnd()
+            }
             // expand the notification
             Log.i(TAG, "expandNotificationMessage: Trying to click $appName notification")
             notificationHeader().click()
             Log.i(TAG, "expandNotificationMessage: Clicked $appName notification")
 
             // double check if notification actions are viewable by checking for action existence; otherwise scroll again
-            while (!mDevice.findObject(UiSelector().resourceId("android:id/action0")).exists() &&
-                !mDevice.findObject(UiSelector().resourceId("android:id/actions_container")).exists()
+            while (!mDevice.findObject(UiSelector().resourceId("android:id/action0"))
+                    .exists()
             ) {
                 Log.i(TAG, "expandNotificationMessage: App notification action buttons do not exist")
                 scrollToEnd()
@@ -139,18 +165,31 @@ class NotificationRobot {
         }
     }
 
+    fun expandMultipleDownloadNotification(notificationItem: String) {
+        Log.i(TAG, "expandMultipleDownloadNotification: Trying to expand notification: $notificationItem using a swipe down action")
+        itemWithResIdContainingText("com.android.systemui:id/notification_title", notificationItem).swipeDown(10)
+        Log.i(TAG, "expandMultipleDownloadNotification: Expanded notification: $notificationItem using a swipe down action")
+    }
+
     // Performs swipe action on download system notifications
     fun swipeDownloadNotification(
         direction: String,
         shouldDismissNotification: Boolean,
         canExpandNotification: Boolean = true,
+        notificationItem: String = "",
     ) {
         // In case it fails, retry max 3x the swipe action on download system notifications
         for (i in 1..RETRY_COUNT) {
             Log.i(TAG, "swipeDownloadNotification: Started try #$i")
             try {
+                if (SDK_INT == Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+                    expandNotificationMessage(notificationItem)
+                    assertUIObjectExists(itemContainingText(appName), waitingTime = waitingTimeShort)
+                }
+
                 var retries = 0
                 while (itemContainingText(appName).exists() && retries++ < 3) {
+                    Log.i(TAG, "swipeDownloadNotification: Started try #$retries to swipe $direction the download notification")
                     // Swipe left the download system notification
                     if (direction == "Left") {
                         itemContainingText(appName)
@@ -177,15 +216,27 @@ class NotificationRobot {
                 }
                 // Not all download related system notifications can be dismissed
                 if (shouldDismissNotification) {
-                    Log.i(TAG, "swipeDownloadNotification: $appName notification can't be dismissed: $shouldDismissNotification")
-                    assertUIObjectExists(itemContainingText(appName), exists = false)
+                    if (SDK_INT == Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+                        Log.i(TAG, "swipeDownloadNotification: $notificationItem notification can be dismissed: $shouldDismissNotification")
+                        assertUIObjectIsGone(itemContainingText(notificationItem), waitingTime = waitingTimeShort)
+                        assertUIObjectExists(itemContainingText(notificationItem), exists = false)
+                    } else {
+                        Log.i(TAG, "swipeDownloadNotification: $appName notification can be dismissed: $shouldDismissNotification")
+                        assertUIObjectExists(itemContainingText(appName), exists = false)
+                    }
                 } else {
-                    Log.i(TAG, "swipeDownloadNotification: $appName notification can be dismissed: $shouldDismissNotification")
-                    assertUIObjectExists(itemContainingText(appName))
+                    if (SDK_INT == Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+                        Log.i(TAG, "swipeDownloadNotification: $notificationItem notification can't be dismissed: $shouldDismissNotification")
+                        assertUIObjectExists(itemContainingText(appName), exists = false)
+                        assertUIObjectExists(itemContainingText(notificationItem))
+                    } else {
+                        Log.i(TAG, "swipeDownloadNotification: $appName notification can't be dismissed: $shouldDismissNotification")
+                        assertUIObjectExists(itemContainingText(appName))
+                    }
                 }
 
                 break
-            } catch (e: AssertionError) {
+            } catch (e: Throwable) {
                 Log.i(TAG, "swipeDownloadNotification: AssertionError caught, executing fallback methods")
                 if (i == RETRY_COUNT) {
                     throw e
@@ -196,9 +247,10 @@ class NotificationRobot {
                         // The download complete system notification can't be expanded
                         if (canExpandNotification) {
                             Log.i(TAG, "swipeDownloadNotification: $appName notification can be expanded: $canExpandNotification")
-                            // In all cases the download system notification title will be the app name
-                            verifySystemNotificationExists(appName)
-                            expandNotificationMessage()
+                            if (SDK_INT != Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
+                                verifySystemNotificationExists(appName)
+                                expandNotificationMessage()
+                            }
                         } else {
                             Log.i(TAG, "swipeDownloadNotification: $appName notification can't be expanded: $canExpandNotification")
                             // Using the download completed system notification summary to bring in to view an properly verify it
@@ -253,8 +305,19 @@ fun notificationShade(interact: NotificationRobot.() -> Unit): NotificationRobot
     return NotificationRobot.Transition()
 }
 
-private fun closePrivateTabsNotification() =
-    mDevice.findObject(UiSelector().text("Close private tabs"))
+private fun closePrivateTabsNotification(): UiObject {
+    lateinit var privateTabsNotification: UiObject
+
+    when (Build.VERSION.SDK_INT) {
+        // For API 34 the notification is slightly different
+        Build.VERSION_CODES.UPSIDE_DOWN_CAKE ->
+            privateTabsNotification = mDevice.findObject(UiSelector().text(getStringResource(R.string.notification_erase_title_android_14)))
+        else ->
+            privateTabsNotification = mDevice.findObject(UiSelector().text("Close private tabs"))
+    }
+
+    return privateTabsNotification
+}
 
 private fun downloadSystemNotificationButton(action: String) =
     mDevice.findObject(
@@ -263,12 +326,7 @@ private fun downloadSystemNotificationButton(action: String) =
             .textContains(action),
     )
 
-private fun mediaSystemNotificationButton(action: String) =
-    mDevice.findObject(
-        UiSelector()
-            .resourceId("com.android.systemui:id/action0")
-            .descriptionContains(action),
-    )
+private fun mediaSystemNotificationButton(action: String) = itemWithDescription(action)
 
 private fun notificationTray() = UiScrollable(
     UiSelector().resourceId("com.android.systemui:id/notification_stack_scroller"),
