@@ -7,8 +7,9 @@
 package org.mozilla.fenix.ui.robots
 
 import android.content.Intent
-import android.net.Uri
 import android.util.Log
+import androidx.compose.ui.test.junit4.ComposeTestRule
+import androidx.core.net.toUri
 import androidx.recyclerview.widget.RecyclerView
 import androidx.test.espresso.Espresso.onView
 import androidx.test.espresso.action.ViewActions
@@ -40,6 +41,7 @@ import org.hamcrest.CoreMatchers
 import org.hamcrest.CoreMatchers.endsWith
 import org.hamcrest.Matchers.allOf
 import org.mozilla.fenix.R
+import org.mozilla.fenix.helpers.AppAndSystemHelper.forceCloseApp
 import org.mozilla.fenix.helpers.AppAndSystemHelper.isPackageInstalled
 import org.mozilla.fenix.helpers.Constants.LISTS_MAXSWIPES
 import org.mozilla.fenix.helpers.Constants.PackageName.GOOGLE_PLAY_SERVICES
@@ -59,7 +61,6 @@ import org.mozilla.fenix.helpers.TestHelper.scrollToElementByText
 import org.mozilla.fenix.helpers.click
 import org.mozilla.fenix.helpers.ext.waitNotNull
 import org.mozilla.fenix.settings.SupportUtils
-import org.mozilla.fenix.ui.robots.SettingsRobot.Companion.DEFAULT_APPS_SETTINGS_ACTION
 
 /**
  * Implementation of Robot Pattern for the settings menu.
@@ -116,10 +117,10 @@ class SettingsRobot {
         onView(withText(R.string.preferences_autofill)).check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
         Log.i(TAG, "verifyAutofillButton: Verified that the \"Autofill\" button is visible")
     }
-    fun verifyLanguageButton() {
-        scrollToElementByText(getStringResource(R.string.preferences_language))
+    fun verifyLanguageButton(localizedText: String = getStringResource(R.string.preferences_language)) {
+        scrollToElementByText(localizedText)
         Log.i(TAG, "verifyLanguageButton: Trying to verify that the \"Language\" button is visible")
-        onView(withText(R.string.preferences_language)).check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
+        onView(withText(localizedText)).check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
         Log.i(TAG, "verifyLanguageButton: Verified that the \"Language\" button is visible")
     }
     fun verifyDefaultBrowserToggle(isEnabled: Boolean) {
@@ -217,12 +218,12 @@ class SettingsRobot {
             .check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
         Log.i(TAG, "verifyPrivateBrowsingButton: Verified that the \"Private browsing\" button is visible")
     }
-    fun verifySitePermissionsButton() {
-        scrollToElementByText("Site permissions")
-        Log.i(TAG, "verifySitePermissionsButton: Trying to verify that the \"Site permissions\" button is visible")
-        onView(withText("Site permissions"))
+    fun verifySiteSettingsButton() {
+        scrollToElementByText("Site settings")
+        Log.i(TAG, "verifySiteSettingsButton: Trying to verify that the \"Site permissions\" button is visible")
+        onView(withText("Site settings"))
             .check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
-        Log.i(TAG, "verifySitePermissionsButton: Verified that the \"Site permissions\" button is visible")
+        Log.i(TAG, "verifySiteSettingsButton: Verified that the \"Site permissions\" button is visible")
     }
     fun verifyDeleteBrowsingDataButton() {
         scrollToElementByText("Delete browsing data")
@@ -253,7 +254,13 @@ class SettingsRobot {
         Log.i(TAG, "verifyDataCollectionButton: Verified that the \"Data collection\" button is visible")
     }
     fun verifyOpenLinksInAppsButton() {
-        scrollToElementByText("Open links in apps")
+        Log.i(TAG, "verifyOpenLinksInAppsButton: Trying to perform scroll to the \"Open links in apps\" button")
+        onView(withId(R.id.recycler_view)).perform(
+            RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
+                hasDescendant(withText(R.string.preferences_open_links_in_apps)),
+            ),
+        )
+        Log.i(TAG, "verifyOpenLinksInAppsButton: Performed scroll to the \"Open links in apps\" button")
         Log.i(TAG, "verifyOpenLinksInAppsButton: Trying to verify that the \"Open links in apps\" button is visible")
         openLinksInAppsButton()
             .check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
@@ -341,35 +348,20 @@ class SettingsRobot {
         Log.i(TAG, "verifyAddons: Verified that the \"$extensions\" button is completely displayed")
     }
 
-    fun verifyExternalDownloadManagerButton() {
-        Log.i(TAG, "verifyExternalDownloadManagerButton: Trying to verify that the \"External download manager\" button is visible")
-        onView(
-            withText(R.string.preferences_external_download_manager),
-        ).check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
-        Log.i(TAG, "verifyExternalDownloadManagerButton: Verified that the \"External download manager\" button is visible")
-    }
-
-    fun verifyExternalDownloadManagerToggle(enabled: Boolean) {
-        Log.i(TAG, "verifyExternalDownloadManagerToggle: Trying to verify that the \"External download manager\" toggle is enabled: $enabled")
-        onView(withText(R.string.preferences_external_download_manager))
-            .check(
-                matches(
-                    hasCousin(
-                        allOf(
-                            withClassName(endsWith("Switch")),
-                            if (enabled) {
-                                isChecked()
-                            } else {
-                                isNotChecked()
-                            },
-                        ),
-                    ),
-                ),
-            )
-        Log.i(TAG, "verifyExternalDownloadManagerToggle: Verified that the \"External download manager\" toggle is enabled: $enabled")
+    fun verifyDownloadsButton() {
+        scrollToElementByText(getStringResource(R.string.preferences_downloads))
+        Log.i(TAG, "verifyExternalDownloadsButton: Trying to verify that the \"Downloads\" button is visible")
+        onView(withText(getStringResource(R.string.preferences_downloads)))
+            .check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
+        Log.i(TAG, "verifyExternalDownloadsButton: Verified that the \"Downloads\" button is visible")
     }
 
     fun verifyLeakCanaryToggle(enabled: Boolean) {
+        onView(withId(R.id.recycler_view)).perform(
+            RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
+                hasDescendant(withText(R.string.preference_leakcanary)),
+            ),
+        )
         Log.i(TAG, "verifyLeakCanaryToggle: Trying to verify that the \"LeakCanary\" toggle is enabled: $enabled")
         onView(withText(R.string.preference_leakcanary))
             .check(
@@ -390,6 +382,11 @@ class SettingsRobot {
     }
 
     fun verifyRemoteDebuggingToggle(enabled: Boolean) {
+        onView(withId(R.id.recycler_view)).perform(
+            RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
+                hasDescendant(withText(R.string.preferences_remote_debugging)),
+            ),
+        )
         Log.i(TAG, "verifyRemoteDebuggingToggle: Trying to verify that the \"Remote debugging via USB\" toggle is enabled: $enabled")
         onView(withText(R.string.preferences_remote_debugging))
             .check(
@@ -436,8 +433,16 @@ class SettingsRobot {
         Log.i(TAG, "verifyAboutHeading: Verified that the \"About\" heading is visible")
     }
 
-    fun verifyRateOnGooglePlay() = assertUIObjectExists(rateOnGooglePlayHeading())
-    fun verifyAboutFirefoxPreview() = assertUIObjectExists(aboutFirefoxHeading())
+    fun verifyRateOnGooglePlay() {
+        settingsList().scrollToEnd(LISTS_MAXSWIPES)
+        assertUIObjectExists(rateOnGooglePlayHeading())
+    }
+
+    fun verifyAboutFirefoxPreview() {
+        settingsList().scrollToEnd(LISTS_MAXSWIPES)
+        assertUIObjectExists(aboutFirefoxHeading())
+    }
+
     fun verifyGooglePlayRedirect() {
         if (isPackageInstalled(GOOGLE_PLAY_SERVICES)) {
             Log.i(TAG, "verifyGooglePlayRedirect: $GOOGLE_PLAY_SERVICES is installed")
@@ -446,13 +451,15 @@ class SettingsRobot {
                 intended(
                     allOf(
                         hasAction(Intent.ACTION_VIEW),
-                        hasData(Uri.parse(SupportUtils.RATE_APP_URL)),
+                        hasData(SupportUtils.RATE_APP_URL.toUri()),
                     ),
                 )
                 Log.i(TAG, "verifyGooglePlayRedirect: Verified intent to: $GOOGLE_PLAY_SERVICES")
             } catch (e: AssertionFailedError) {
                 Log.i(TAG, "verifyGooglePlayRedirect: AssertionFailedError caught, executing fallback methods")
                 BrowserRobot().verifyRateOnGooglePlayURL()
+            } finally {
+                forceCloseApp(GOOGLE_PLAY_SERVICES)
             }
         } else {
             BrowserRobot().verifyRateOnGooglePlayURL()
@@ -562,7 +569,7 @@ class SettingsRobot {
             return SettingsSubMenuHomepageRobot.Transition()
         }
 
-        fun openAutofillSubMenu(interact: SettingsSubMenuAutofillRobot.() -> Unit): SettingsSubMenuAutofillRobot.Transition {
+        fun openAutofillSubMenu(composeTestRule: ComposeTestRule, interact: SettingsSubMenuAutofillRobot.() -> Unit): SettingsSubMenuAutofillRobot.Transition {
             mDevice.findObject(UiSelector().textContains(getStringResource(R.string.preferences_autofill)))
                 .also {
                     Log.i(TAG, "openAutofillSubMenu: Waiting for $waitingTime ms for the \"Autofill\" button to exist")
@@ -573,8 +580,8 @@ class SettingsRobot {
                     Log.i(TAG, "openAutofillSubMenu: Clicked the \"Autofill\" button")
                 }
 
-            SettingsSubMenuAutofillRobot().interact()
-            return SettingsSubMenuAutofillRobot.Transition()
+            SettingsSubMenuAutofillRobot(composeTestRule).interact()
+            return SettingsSubMenuAutofillRobot.Transition(composeTestRule)
         }
 
         fun openAccessibilitySubMenu(interact: SettingsSubMenuAccessibilityRobot.() -> Unit): SettingsSubMenuAccessibilityRobot.Transition {
@@ -642,7 +649,7 @@ class SettingsRobot {
 
         fun openTurnOnSyncMenu(interact: SettingsTurnOnSyncRobot.() -> Unit): SettingsTurnOnSyncRobot.Transition {
             Log.i(TAG, "openTurnOnSyncMenu: Trying to click the \"Sync and save your data\" button")
-            onView(withText("Sync and save your data")).click()
+            onView(withText("Sign in")).click()
             Log.i(TAG, "openTurnOnSyncMenu: Clicked the \"Sync and save your data\" button")
 
             SettingsTurnOnSyncRobot().interact()
@@ -659,11 +666,11 @@ class SettingsRobot {
             return SettingsSubMenuPrivateBrowsingRobot.Transition()
         }
 
-        fun openSettingsSubMenuSitePermissions(interact: SettingsSubMenuSitePermissionsRobot.() -> Unit): SettingsSubMenuSitePermissionsRobot.Transition {
-            scrollToElementByText("Site permissions")
-            Log.i(TAG, "openSettingsSubMenuSitePermissions: Trying to click the \"Site permissions\" button")
-            mDevice.findObject(textContains("Site permissions")).click()
-            Log.i(TAG, "openSettingsSubMenuSitePermissions: Clicked the \"Site permissions\" button")
+        fun openSettingsSubMenuSiteSettings(interact: SettingsSubMenuSitePermissionsRobot.() -> Unit): SettingsSubMenuSitePermissionsRobot.Transition {
+            scrollToElementByText("Site settings")
+            Log.i(TAG, "openSettingsSubMenuSiteSettings: Trying to click the \"Site settings\" button")
+            mDevice.findObject(textContains("Site settings")).click()
+            Log.i(TAG, "openSettingsSubMenuSiteSettings: Clicked the \"Site settings\" button")
 
             SettingsSubMenuSitePermissionsRobot().interact()
             return SettingsSubMenuSitePermissionsRobot.Transition()
