@@ -207,11 +207,11 @@ class SettingsSubMenuHomepageRobot {
                 )
             Log.i(TAG, "verifyHomePageView: Verified that the \"Recently visited\" toggle is not checked")
         }
-        Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Thought-provoking stories\" option is visible")
+        Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Stories\" option is visible")
         pocketButton().check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
-        Log.i(TAG, "verifyHomePageView: Verified that the \"Thought-provoking stories\" option is visible")
+        Log.i(TAG, "verifyHomePageView: Verified that the \"Stories\" option is visible")
         if (pocketSwitchEnabled) {
-            Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Thought-provoking stories\" toggle is checked")
+            Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Stories\" toggle is checked")
             pocketButton()
                 .check(
                     matches(
@@ -223,9 +223,9 @@ class SettingsSubMenuHomepageRobot {
                         ),
                     ),
                 )
-            Log.i(TAG, "verifyHomePageView: Verified that the \"Thought-provoking stories\" toggle is checked")
+            Log.i(TAG, "verifyHomePageView: Verified that the \"Stories\" toggle is checked")
         } else {
-            Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Thought-provoking stories\" toggle is not checked")
+            Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Stories\" toggle is not checked")
             pocketButton()
                 .check(
                     matches(
@@ -237,7 +237,7 @@ class SettingsSubMenuHomepageRobot {
                         ),
                     ),
                 )
-            Log.i(TAG, "verifyHomePageView: Verified that the \"Thought-provoking stories\" toggle is not checked")
+            Log.i(TAG, "verifyHomePageView: Verified that the \"Stories\" toggle is not checked")
         }
         Log.i(TAG, "verifyHomePageView: Trying to verify that the \"Sponsored stories\" option is visible")
         sponsoredStoriesButton().check(matches(withEffectiveVisibility(Visibility.VISIBLE)))
@@ -337,9 +337,9 @@ class SettingsSubMenuHomepageRobot {
     }
 
     fun clickPocketButton() {
-        Log.i(TAG, "clickPocketButton: Trying to click the \"Thought-provoking stories\" option")
+        Log.i(TAG, "clickPocketButton: Trying to click the \"Stories\" option")
         pocketButton().click()
-        Log.i(TAG, "clickPocketButton: Clicked the \"Thought-provoking stories\" option")
+        Log.i(TAG, "clickPocketButton: Clicked the \"Stories\" option")
     }
 
     fun clickOpeningScreenOption(openingScreenOption: String) {
@@ -352,18 +352,6 @@ class SettingsSubMenuHomepageRobot {
         Log.i(TAG, "clickOpeningScreenOption: Clicked \"Opening screen\" option: $openingScreenOption")
     }
 
-    fun openWallpapersMenu() {
-        Log.i(TAG, "openWallpapersMenu: Trying to click the \"Wallpapers\" option")
-        wallpapersMenuButton().click()
-        Log.i(TAG, "openWallpapersMenu: Clicked the \"Wallpapers\" option")
-    }
-
-    fun selectWallpaper(wallpaperName: String) {
-        Log.i(TAG, "selectWallpaper: Trying to click wallpaper: $wallpaperName")
-        mDevice.findObject(UiSelector().description(wallpaperName)).click()
-        Log.i(TAG, "selectWallpaper: Clicked wallpaper: $wallpaperName")
-    }
-
     fun verifySponsoredShortcutsCheckBox(checked: Boolean) {
         if (checked) {
             Log.i(TAG, "verifySponsoredShortcutsCheckBox: Trying to verify that the \"Sponsored shortcuts\" check box is checked")
@@ -450,7 +438,7 @@ private fun recentlyVisitedButton() =
     onView(allOf(withText(R.string.customize_toggle_recently_visited)))
 
 private fun pocketButton() =
-    onView(allOf(withText(R.string.customize_toggle_pocket_2)))
+    onView(allOf(withText(R.string.customize_toggle_pocket_3)))
 
 private fun sponsoredStoriesButton() =
     onView(allOf(withText(R.string.customize_toggle_pocket_sponsored)))
@@ -485,5 +473,3 @@ private fun homepageAfterFourHoursButton() =
     )
 
 private fun goBackButton() = onView(allOf(withContentDescription(R.string.action_bar_up_description)))
-
-private fun wallpapersMenuButton() = onView(withText("Wallpapers"))
