@@ -43,13 +43,14 @@ import org.hamcrest.Matchers.endsWith
 import org.junit.Assert.assertFalse
 import org.junit.Assert.assertTrue
 import org.mozilla.fenix.R
+import org.mozilla.fenix.helpers.Constants.RETRY_COUNT
 import org.mozilla.fenix.helpers.Constants.TAG
 import org.mozilla.fenix.helpers.DataGenerationHelper.getAvailableSearchEngines
 import org.mozilla.fenix.helpers.DataGenerationHelper.getRegionSearchEnginesList
 import org.mozilla.fenix.helpers.DataGenerationHelper.getStringResource
 import org.mozilla.fenix.helpers.MatcherHelper.assertUIObjectExists
-import org.mozilla.fenix.helpers.MatcherHelper.itemContainingText
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithResIdAndText
+import org.mozilla.fenix.helpers.MatcherHelper.itemWithResIdContainingText
 import org.mozilla.fenix.helpers.MatcherHelper.itemWithText
 import org.mozilla.fenix.helpers.TestAssetHelper.waitingTimeShort
 import org.mozilla.fenix.helpers.TestHelper.hasCousin
@@ -58,6 +59,7 @@ import org.mozilla.fenix.helpers.TestHelper.packageName
 import org.mozilla.fenix.helpers.click
 import org.mozilla.fenix.helpers.isChecked
 import org.mozilla.fenix.helpers.isEnabled
+import androidx.preference.R as preferenceR
 
 /**
  * Implementation of Robot Pattern for the settings search sub menu.
@@ -144,7 +146,7 @@ class SettingsSubMenuSearchRobot {
         availableShortcutsEngines.forEach {
             Log.i(TAG, "verifyManageShortcutsList: Trying to verify that the ${it.name} alternative search engine is displayed")
             testRule.onNodeWithText(it.name)
-                .assert(hasAnySibling(hasContentDescription("${it.name} search engine")))
+                .assert(hasAnySibling(hasContentDescription("Search engine: ${it.name}")))
                 .assertIsDisplayed()
             Log.i(TAG, "verifyManageShortcutsList: Verify that the ${it.name} alternative search engine is displayed")
         }
@@ -264,12 +266,28 @@ class SettingsSubMenuSearchRobot {
     }
 
     fun changeDefaultSearchEngine(searchEngineName: String) {
-        Log.i(TAG, "changeDefaultSearchEngine: Trying to verify that the $searchEngineName option is visible")
-        onView(withText(searchEngineName)).check(matches(withEffectiveVisibility(ViewMatchers.Visibility.VISIBLE)))
-        Log.i(TAG, "changeDefaultSearchEngine: Verified that the $searchEngineName option is visible")
-        Log.i(TAG, "changeDefaultSearchEngine: Trying to click the $searchEngineName option")
-        onView(withText(searchEngineName)).perform(click())
-        Log.i(TAG, "changeDefaultSearchEngine: Clicked the $searchEngineName option")
+        for (i in 1..RETRY_COUNT) {
+            Log.i(TAG, "changeDefaultSearchEngine: Started try #$i")
+            try {
+                assertUIObjectExists(itemWithResIdAndText("$packageName:id/engine_text", searchEngineName))
+                Log.i(TAG, "changeDefaultSearchEngine: Trying to click the $searchEngineName option")
+                itemWithResIdAndText("$packageName:id/engine_text", searchEngineName).click()
+                Log.i(TAG, "changeDefaultSearchEngine: Clicked the $searchEngineName option")
+                verifyDefaultSearchEngineSelected(searchEngineName)
+
+                break
+            } catch (e: Exception) {
+                if (i == RETRY_COUNT) {
+                    throw e
+                } else {
+                    Log.i(TAG, "changeDefaultSearchEngine: Exception caught, executing fallback methods")
+                    settingsSubMenuSearch {
+                    }.goBackToSearchSettings {
+                        openDefaultSearchEngineMenu()
+                    }
+                }
+            }
+        }
     }
 
     fun selectSearchShortcut(shortcut: EngineShortcut) {
@@ -450,7 +468,6 @@ class SettingsSubMenuSearchRobot {
         Log.i(TAG, "saveEditSearchEngine: Trying to click the \"Save\" button")
         onView(withId(R.id.save_button)).click()
         Log.i(TAG, "saveEditSearchEngine: Clicked the \"Save\" button")
-        assertUIObjectExists(itemContainingText("Saved"))
     }
 
     fun verifyInvalidTemplateSearchStringFormatError() {
@@ -468,8 +485,7 @@ class SettingsSubMenuSearchRobot {
         closeSoftKeyboard()
         Log.i(TAG, "verifyErrorConnectingToSearchString: Performed \"Close soft keyboard\" action")
         Log.i(TAG, "verifyErrorConnectingToSearchString: Trying to verify that the \"Error connecting to $searchEngineName\" error message is displayed")
-        onView(withText(getStringResource(R.string.search_add_custom_engine_error_cannot_reach, searchEngineName)))
-            .check(matches(isDisplayed()))
+        assertUIObjectExists(itemWithResIdContainingText("$packageName:id/textinput_error", "Error connecting to “$searchEngineName”"))
         Log.i(TAG, "verifyErrorConnectingToSearchString: Verified that the \"Error connecting to $searchEngineName\" error message is displayed")
     }
 
@@ -486,6 +502,18 @@ class SettingsSubMenuSearchRobot {
             return SettingsRobot.Transition()
         }
 
+        fun goBackToSearchSettings(interact: SettingsSubMenuSearchRobot.() -> Unit): SettingsSubMenuSearchRobot.Transition {
+            Log.i(TAG, "goBackToSearchSettings: Waiting for device to be idle")
+            mDevice.waitForIdle()
+            Log.i(TAG, "goBackToSearchSettings: Waited for device to be idle")
+            Log.i(TAG, "goBackToSearchSettings: Trying to click the navigate up button")
+            goBackButton().perform(click())
+            Log.i(TAG, "goBackToSearchSettings: Clicked the navigate up button")
+
+            SettingsSubMenuSearchRobot().interact()
+            return SettingsSubMenuSearchRobot.Transition()
+        }
+
         fun clickCustomSearchStringLearnMoreLink(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
             Log.i(TAG, "clickCustomSearchStringLearnMoreLink: Trying to click the \"Search string URL\" learn more link")
             onView(withId(R.id.custom_search_engines_learn_more)).click()
@@ -506,6 +534,11 @@ class SettingsSubMenuSearchRobot {
     }
 }
 
+fun settingsSubMenuSearch(interact: SettingsSubMenuSearchRobot.() -> Unit): SettingsSubMenuSearchRobot.Transition {
+    SettingsSubMenuSearchRobot().interact()
+    return SettingsSubMenuSearchRobot.Transition()
+}
+
 /**
  * Matches search shortcut items inside the 'Manage search shortcuts' menu
  * @param name, of type String, should be the name of the search engine.
@@ -524,7 +557,7 @@ private val manageSearchShortcutsHeader = onView(withText("Manage alternative se
 
 private fun searchHistorySwitchButton(): ViewInteraction {
     Log.i(TAG, "searchHistorySwitchButton: Trying to perform scroll action to the \"Search browsing history\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText("Search browsing history")),
         ),
@@ -535,7 +568,7 @@ private fun searchHistorySwitchButton(): ViewInteraction {
 
 private fun searchBookmarksSwitchButton(): ViewInteraction {
     Log.i(TAG, "searchBookmarksSwitchButton: Trying to perform scroll action to the \"Search bookmarks\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText("Search bookmarks")),
         ),
@@ -546,7 +579,7 @@ private fun searchBookmarksSwitchButton(): ViewInteraction {
 
 private fun searchSyncedTabsSwitchButton(): ViewInteraction {
     Log.i(TAG, "searchSyncedTabsSwitchButton: Trying to perform scroll action to the \"Search synced tabs\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText("Search synced tabs")),
         ),
@@ -557,7 +590,7 @@ private fun searchSyncedTabsSwitchButton(): ViewInteraction {
 
 private fun voiceSearchSwitchButton(): ViewInteraction {
     Log.i(TAG, "voiceSearchSwitchButton: Trying to perform scroll action to the \"Show voice search\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText("Show voice search")),
         ),
@@ -568,7 +601,7 @@ private fun voiceSearchSwitchButton(): ViewInteraction {
 
 private fun autocompleteSwitchButton(): ViewInteraction {
     Log.i(TAG, "autocompleteSwitchButton: Trying to perform scroll action to the \"Autocomplete URLs\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText(getStringResource(R.string.preferences_enable_autocomplete_urls))),
         ),
@@ -579,7 +612,7 @@ private fun autocompleteSwitchButton(): ViewInteraction {
 
 private fun showSearchSuggestionSwitchButton(): ViewInteraction {
     Log.i(TAG, "showSearchSuggestionSwitchButton: Trying to perform scroll action to the \"Show search suggestions\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText("Show search suggestions")),
         ),
@@ -590,7 +623,7 @@ private fun showSearchSuggestionSwitchButton(): ViewInteraction {
 
 private fun showClipboardSuggestionSwitch(): ViewInteraction {
     Log.i(TAG, "showClipboardSuggestionSwitch: Trying to perform scroll action to the \"Show clipboard suggestions\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText(getStringResource(R.string.preferences_show_clipboard_suggestions))),
         ),
@@ -601,7 +634,7 @@ private fun showClipboardSuggestionSwitch(): ViewInteraction {
 
 private fun showSuggestionsInPrivateModeSwitch(): ViewInteraction {
     Log.i(TAG, "showSuggestionsInPrivateModeSwitch: Trying to perform scroll action to the \"Show in private sessions\" option")
-    onView(withId(androidx.preference.R.id.recycler_view)).perform(
+    onView(withId(preferenceR.id.recycler_view)).perform(
         RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
             hasDescendant(withText(getStringResource(R.string.preferences_show_search_suggestions_in_private))),
         ),
