@@ -7,6 +7,7 @@
 package org.mozilla.fenix.ui.robots
 
 import android.util.Log
+import androidx.compose.ui.test.junit4.ComposeTestRule
 import androidx.recyclerview.widget.RecyclerView
 import androidx.test.espresso.Espresso.onView
 import androidx.test.espresso.action.ViewActions.swipeDown
@@ -14,7 +15,6 @@ import androidx.test.espresso.action.ViewActions.swipeUp
 import androidx.test.espresso.assertion.ViewAssertions.matches
 import androidx.test.espresso.contrib.RecyclerViewActions
 import androidx.test.espresso.matcher.RootMatchers
-import androidx.test.espresso.matcher.ViewMatchers
 import androidx.test.espresso.matcher.ViewMatchers.Visibility
 import androidx.test.espresso.matcher.ViewMatchers.hasDescendant
 import androidx.test.espresso.matcher.ViewMatchers.isDisplayed
@@ -46,6 +46,7 @@ import org.mozilla.fenix.helpers.TestHelper.packageName
 import org.mozilla.fenix.helpers.click
 import org.mozilla.fenix.helpers.ext.waitNotNull
 import org.mozilla.fenix.nimbus.FxNimbus
+import mozilla.components.browser.menu.R as menuR
 
 /**
  * Implementation of Robot Pattern for the three dot (main) menu.
@@ -99,16 +100,20 @@ class ThreeDotMenuMainRobot {
     }
 
     fun verifyQuitButtonExists() {
-        // Need to double swipe the menu, to make this button visible.
+        expandMenuFully()
+        assertUIObjectExists(itemWithText("Quit"))
+    }
+
+    fun expandMenuFully() {
+        // Need to double swipe the menu, to make some buttons.
         // In case it reaches the end, the second swipe is no-op.
         expandMenu()
         expandMenu()
-        assertUIObjectExists(itemWithText("Quit"))
     }
 
     fun expandMenu() {
         Log.i(TAG, "expandMenu: Trying to perform swipe up action on the three dot menu")
-        onView(withId(R.id.mozac_browser_menu_menuView)).perform(swipeUp())
+        onView(withId(menuR.id.mozac_browser_menu_menuView)).perform(swipeUp())
         Log.i(TAG, "expandMenu: Performed swipe up action on the three dot menu")
     }
 
@@ -128,7 +133,7 @@ class ThreeDotMenuMainRobot {
         assertUIObjectExists(addToShortcutsButton(), exists = shouldExist)
     fun verifyRemoveFromShortcutsButton() {
         Log.i(TAG, "verifyRemoveFromShortcutsButton: Trying to perform scroll action to the \"Settings\" button")
-        onView(withId(R.id.mozac_browser_menu_recyclerView))
+        onView(withId(menuR.id.mozac_browser_menu_recyclerView))
             .perform(
                 RecyclerViewActions.scrollTo<RecyclerView.ViewHolder>(
                     hasDescendant(withText(R.string.browser_menu_settings)),
@@ -166,7 +171,7 @@ class ThreeDotMenuMainRobot {
             historyButton(),
             downloadsButton(),
             passwordsButton(),
-            addOnsButton(),
+            extensionsButton(),
             syncAndSaveDataButton(),
             findInPageButton(),
             translateButton(),
@@ -194,21 +199,17 @@ class ThreeDotMenuMainRobot {
         )
     }
 
-    fun verifyHomeThreeDotMainMenuItems(isRequestDesktopSiteEnabled: Boolean) {
+    fun verifyHomeThreeDotMainMenuItems() {
         assertUIObjectExists(
             bookmarksButton(),
             historyButton(),
             downloadsButton(),
             passwordsButton(),
-            addOnsButton(),
-            // Disabled step due to https://github.com/mozilla-mobile/fenix/issues/26788
-            // syncAndSaveDataButton,
-            desktopSiteButton(),
-            whatsNewButton(),
+            extensionsButton(),
+            syncAndSaveDataButton(),
             helpButton(),
             customizeHomeButton(),
             settingsButton(),
-            desktopSiteToggle(isRequestDesktopSiteEnabled),
         )
     }
 
@@ -240,10 +241,26 @@ class ThreeDotMenuMainRobot {
     }
 
     fun verifyTrackersBlockedByUblock() {
-        assertUIObjectExists(itemWithResId("$packageName:id/badge_text"))
-        Log.i(TAG, "verifyTrackersBlockedByUblock: Trying to verify that the count of trackers blocked is greater than 0")
-        assertTrue("$TAG: The count of trackers blocked is not greater than 0", itemWithResId("$packageName:id/badge_text").text.toInt() > 0)
-        Log.i(TAG, "verifyTrackersBlockedByUblock: Verified that the count of trackers blocked is greater than 0")
+        for (i in 1..RETRY_COUNT) {
+            Log.i(TAG, "verifyTrackersBlockedByUblock: Started try #$i")
+            try {
+                assertUIObjectExists(itemWithResId("$packageName:id/badge_text"))
+                Log.i(TAG, "verifyTrackersBlockedByUblock: Trying to verify that the count of trackers blocked is greater than 0")
+                assertTrue("$TAG: The count of trackers blocked is not greater than 0", itemWithResId("$packageName:id/badge_text").text.toInt() > 0)
+                Log.i(TAG, "verifyTrackersBlockedByUblock: Verified that the count of trackers blocked is greater than 0")
+
+                break
+            } catch (e: NumberFormatException) {
+                Log.i(TAG, "verifyTrackersBlockedByUblock: NumberFormatException caught, executing fallback methods")
+                Log.i(TAG, "verifyTrackersBlockedByUblock: Trying to click the device back button")
+                mDevice.pressBack()
+                Log.i(TAG, "verifyTrackersBlockedByUblock: Clicked the device back button")
+                browserScreen {
+                }.openThreeDotMenu {
+                    openAddonsSubList()
+                }
+            }
+        }
     }
 
     fun clickQuit() {
@@ -287,11 +304,23 @@ class ThreeDotMenuMainRobot {
             return DownloadRobot.Transition()
         }
 
+        fun openPasswords(interact: SettingsSubMenuLoginsAndPasswordsSavedLoginsRobot.() -> Unit): SettingsSubMenuLoginsAndPasswordsSavedLoginsRobot.Transition {
+            Log.i(TAG, "openPasswords: Trying to perform swipe down action on the three dot menu")
+            threeDotMenuRecyclerView().perform(swipeDown())
+            Log.i(TAG, "openPasswords: Performed swipe down action on the three dot menu")
+            Log.i(TAG, "openPasswords: Trying to click the \"Passwords\" button")
+            passwordsButton().click()
+            Log.i(TAG, "openPasswords: Clicked the \"Passwords\" button")
+
+            SettingsSubMenuLoginsAndPasswordsSavedLoginsRobot().interact()
+            return SettingsSubMenuLoginsAndPasswordsSavedLoginsRobot.Transition()
+        }
+
         fun openSyncSignIn(interact: SyncSignInRobot.() -> Unit): SyncSignInRobot.Transition {
             Log.i(TAG, "openSyncSignIn: Trying to perform swipe down action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeDown())
             Log.i(TAG, "openSyncSignIn: Performed swipe down action on the three dot menu")
-            mDevice.waitNotNull(Until.findObject(By.text("Sync and save data")), waitingTime)
+            mDevice.waitNotNull(Until.findObject(By.text("Sign in")), waitingTime)
             Log.i(TAG, "openSyncSignIn: Trying to click the \"Sync and save data\" button")
             syncAndSaveDataButton().click()
             Log.i(TAG, "openSyncSignIn: Clicked the \"Sync and save data\" button")
@@ -300,18 +329,17 @@ class ThreeDotMenuMainRobot {
             return SyncSignInRobot.Transition()
         }
 
-        fun openBookmarks(interact: BookmarksRobot.() -> Unit): BookmarksRobot.Transition {
-            Log.i(TAG, "openBookmarks: Trying to perform swipe down action on the three dot menu")
+        fun openBookmarksMenu(composeTestRule: ComposeTestRule, interact: BookmarksRobot.() -> Unit): BookmarksRobot.Transition {
+            Log.i(TAG, "openBookmarksMenu: Trying to perform swipe down action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeDown())
-            Log.i(TAG, "openBookmarks: Performed swipe down action on the three dot menu")
+            Log.i(TAG, "openBookmarksMenu: Performed swipe down action on the three dot menu")
             mDevice.waitNotNull(Until.findObject(By.text("Bookmarks")), waitingTime)
-            Log.i(TAG, "openBookmarks: Trying to click the \"Bookmarks\" button")
+            Log.i(TAG, "openBookmarksMenu: Trying to click the \"Bookmarks\" button")
             bookmarksButton().click()
-            Log.i(TAG, "openBookmarks: Clicked the \"Bookmarks\" button")
-            assertUIObjectExists(itemWithResId("$packageName:id/bookmark_list"))
+            Log.i(TAG, "openBookmarksMenu: Clicked the \"Bookmarks\" button")
 
-            BookmarksRobot().interact()
-            return BookmarksRobot.Transition()
+            BookmarksRobot(composeTestRule).interact()
+            return BookmarksRobot.Transition(composeTestRule)
         }
 
         fun clickNewTabButton(interact: SearchRobot.() -> Unit): SearchRobot.Transition {
@@ -346,14 +374,14 @@ class ThreeDotMenuMainRobot {
             return BrowserRobot.Transition()
         }
 
-        fun editBookmarkPage(interact: BookmarksRobot.() -> Unit): BookmarksRobot.Transition {
+        fun editBookmarkPage(composeTestRule: ComposeTestRule, interact: BookmarksRobot.() -> Unit): BookmarksRobot.Transition {
             mDevice.waitNotNull(Until.findObject(By.text("Bookmarks")), waitingTime)
             Log.i(TAG, "editBookmarkPage: Trying to click the \"Edit\" button")
             editBookmarkButton().click()
             Log.i(TAG, "editBookmarkPage: Clicked the \"Edit\" button")
 
-            BookmarksRobot().interact()
-            return BookmarksRobot.Transition()
+            BookmarksRobot(composeTestRule).interact()
+            return BookmarksRobot.Transition(composeTestRule)
         }
 
         fun openHelp(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
@@ -428,6 +456,13 @@ class ThreeDotMenuMainRobot {
         }
 
         fun refreshPage(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
+            if (stopLoadingButton().exists()) {
+                Log.i(TAG, "refreshPage: Trying to click the \"Stop\" button")
+                stopLoadingButton().click()
+                Log.i(TAG, "refreshPage: Clicked the \"Stop\" button")
+                browserScreen {
+                }.openThreeDotMenu {}
+            }
             refreshButton().also {
                 Log.i(TAG, "refreshPage: Waiting for $waitingTime ms for the \"Refresh\" button to exist")
                 it.waitForExists(waitingTime)
@@ -460,16 +495,16 @@ class ThreeDotMenuMainRobot {
             return HomeScreenRobot.Transition()
         }
 
-        fun openReportSiteIssue(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
-            Log.i(TAG, "openReportSiteIssue: Trying to perform swipe up action on the three dot menu")
+        fun openReportBrokenSite(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
+            Log.i(TAG, "openReportBrokenSite: Trying to perform swipe up action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeUp())
-            Log.i(TAG, "openReportSiteIssue: Performed swipe up action on the three dot menu")
-            Log.i(TAG, "openReportSiteIssue: Trying to perform swipe up action on the three dot menu")
+            Log.i(TAG, "openReportBrokenSite: Performed swipe up action on the three dot menu")
+            Log.i(TAG, "openReportBrokenSite: Trying to perform swipe up action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeUp())
-            Log.i(TAG, "openReportSiteIssue: Performed swipe up action on the three dot menu")
-            Log.i(TAG, "openReportSiteIssue: Trying to click the \"Report Site Issue\" button")
+            Log.i(TAG, "openReportBrokenSite: Performed swipe up action on the three dot menu")
+            Log.i(TAG, "openReportBrokenSite: Trying to click the \"Report broke site\" button")
             reportSiteIssueButton().click()
-            Log.i(TAG, "openReportSiteIssue: Clicked the \"Report Site Issue\" button")
+            Log.i(TAG, "openReportBrokenSite: Clicked the \"Report broke site\" button")
 
             BrowserRobot().interact()
             return BrowserRobot.Transition()
@@ -491,16 +526,6 @@ class ThreeDotMenuMainRobot {
             return FindInPageRobot.Transition()
         }
 
-        fun openWhatsNew(interact: BrowserRobot.() -> Unit): BrowserRobot.Transition {
-            mDevice.waitNotNull(Until.findObject(By.text("What’s new")), waitingTime)
-            Log.i(TAG, "openWhatsNew: Trying to click the \"What’s new\" button")
-            whatsNewButton().click()
-            Log.i(TAG, "openWhatsNew: Clicked the \"What’s new\" button")
-
-            BrowserRobot().interact()
-            return BrowserRobot.Transition()
-        }
-
         fun openReaderViewAppearance(interact: ReaderViewRobot.() -> Unit): ReaderViewRobot.Transition {
             Log.i(TAG, "openReaderViewAppearance: Trying to perform swipe up action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeUp())
@@ -568,16 +593,16 @@ class ThreeDotMenuMainRobot {
             return AddToHomeScreenRobot.Transition()
         }
 
-        fun clickInstall(interact: AddToHomeScreenRobot.() -> Unit): AddToHomeScreenRobot.Transition {
+        fun clickAddAppToHomeScreen(interact: AddToHomeScreenRobot.() -> Unit): AddToHomeScreenRobot.Transition {
             Log.i(TAG, "clickInstall: Trying to perform swipe up action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeUp())
             Log.i(TAG, "clickInstall: Performed swipe up action on the three dot menu")
             Log.i(TAG, "clickInstall: Trying to perform swipe up action on the three dot menu")
             threeDotMenuRecyclerView().perform(swipeUp())
             Log.i(TAG, "clickInstall: Performed swipe up action on the three dot menu")
-            Log.i(TAG, "clickInstall: Trying to click the \"Install\" button")
-            installPWAButton().click()
-            Log.i(TAG, "clickInstall: Clicked the \"Install\" button")
+            Log.i(TAG, "clickInstall: Trying to click the \"Add app to Home screen\" button")
+            addAppToHomeScreenButton().click()
+            Log.i(TAG, "clickInstall: Clicked the \"Add app to Home screen\" button")
 
             AddToHomeScreenRobot().interact()
             return AddToHomeScreenRobot.Transition()
@@ -670,14 +695,23 @@ class ThreeDotMenuMainRobot {
             BrowserRobot().interact()
             return BrowserRobot.Transition()
         }
+
+        fun clickTranslateButton(composeTestRule: ComposeTestRule, interact: TranslationsRobot.() -> Unit): TranslationsRobot.Transition {
+            Log.i(TAG, "clickTranslateButton: Trying to click the \"Translate page\" button")
+            translateButton().click()
+            Log.i(TAG, "clickTranslateButton: Clicked the \"Translate page\" button")
+
+            TranslationsRobot(composeTestRule).interact()
+            return TranslationsRobot.Transition(composeTestRule)
+        }
     }
 }
 private fun threeDotMenuRecyclerView() =
-    onView(withId(R.id.mozac_browser_menu_recyclerView))
+    onView(withId(menuR.id.mozac_browser_menu_recyclerView))
 
 private fun editBookmarkButton() = onView(withText("Edit"))
 
-private fun stopLoadingButton() = onView(ViewMatchers.withContentDescription("Stop"))
+private fun stopLoadingButton() = itemWithDescription("Stop")
 
 private fun closeAllTabsButton() = onView(allOf(withText("Close all tabs"))).inRoot(RootMatchers.isPlatformPopup())
 
@@ -691,8 +725,8 @@ private fun readerViewAppearanceToggle() =
 private fun removeFromShortcutsButton() =
     onView(allOf(withText(R.string.browser_menu_remove_from_shortcuts)))
 
-private fun installPWAButton() =
-    itemContainingText(getStringResource(R.string.browser_menu_add_to_homescreen))
+private fun addAppToHomeScreenButton() =
+    itemContainingText(getStringResource(R.string.browser_menu_add_app_to_homescreen))
 
 private fun openInAppButton() =
     onView(
@@ -704,10 +738,10 @@ private fun openInAppButton() =
 
 private fun clickAddonsManagerButton() {
     Log.i(TAG, "clickAddonsManagerButton: Trying to perform swipe down action on the three dot menu")
-    onView(withId(R.id.mozac_browser_menu_menuView)).perform(swipeDown())
+    onView(withId(menuR.id.mozac_browser_menu_menuView)).perform(swipeDown())
     Log.i(TAG, "clickAddonsManagerButton: Performed swipe down action on the three dot menu")
     Log.i(TAG, "clickAddonsManagerButton: Trying to click the \"Add-ons\" button")
-    addOnsButton().click()
+    extensionsButton().click()
     Log.i(TAG, "clickAddonsManagerButton: Clicked the \"Add-ons\" button")
 }
 
@@ -722,7 +756,7 @@ private fun downloadsButton() =
     itemContainingText(getStringResource(R.string.library_downloads))
 private fun passwordsButton() =
     itemContainingText(getStringResource(R.string.browser_menu_passwords))
-private fun addOnsButton() =
+private fun extensionsButton() =
     itemContainingText(getStringResource(R.string.browser_menu_extensions))
 private fun desktopSiteButton() =
     itemContainingText(getStringResource(R.string.browser_menu_desktop_site))
@@ -732,8 +766,6 @@ private fun desktopSiteToggle(state: Boolean) =
         getStringResource(R.string.browser_menu_desktop_site),
         state,
     )
-private fun whatsNewButton() =
-    itemContainingText(getStringResource(R.string.browser_menu_whats_new))
 private fun helpButton() =
     itemContainingText(getStringResource(R.string.browser_menu_help))
 private fun customizeHomeButton() =
@@ -741,7 +773,7 @@ private fun customizeHomeButton() =
 private fun settingsButton(localizedText: String = getStringResource(R.string.browser_menu_settings)) =
     itemContainingText(localizedText)
 private fun syncAndSaveDataButton() =
-    itemContainingText(getStringResource(R.string.sync_menu_sync_and_save_data))
+    itemContainingText(getStringResource(R.string.sync_menu_sign_in))
 private fun normalBrowsingNewTabButton() =
     itemContainingText(getStringResource(R.string.library_new_tab))
 private fun addBookmarkButton() =
@@ -751,7 +783,7 @@ private fun addBookmarkButton() =
     )
 private fun findInPageButton() = itemContainingText(getStringResource(R.string.browser_menu_find_in_page))
 private fun translateButton() = itemContainingText(getStringResource(R.string.browser_menu_translations))
-private fun reportSiteIssueButton() = itemContainingText("Report Site Issue")
+private fun reportSiteIssueButton() = itemContainingText("Report broken site")
 private fun addToHomeScreenButton() = itemContainingText(getStringResource(R.string.browser_menu_add_to_homescreen))
 private fun addToShortcutsButton() = itemContainingText(getStringResource(R.string.browser_menu_add_to_shortcuts))
 private fun saveToCollectionButton() = itemContainingText(getStringResource(R.string.browser_menu_save_to_collection_2))
