@@ -7,19 +7,22 @@ package org.mozilla.fenix
 import android.annotation.SuppressLint
 import android.app.ActivityManager
 import android.content.Context
-import android.net.Uri
+import android.content.Intent
 import android.os.Build
 import android.os.Build.VERSION.SDK_INT
 import android.os.StrictMode
 import android.os.SystemClock
 import android.util.Log.INFO
+import androidx.annotation.OpenForTesting
 import androidx.annotation.VisibleForTesting
 import androidx.appcompat.app.AppCompatDelegate
 import androidx.core.app.NotificationManagerCompat
 import androidx.core.content.getSystemService
+import androidx.core.net.toUri
 import androidx.lifecycle.ProcessLifecycleOwner
 import androidx.work.Configuration.Builder
 import androidx.work.Configuration.Provider
+import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.Deferred
 import kotlinx.coroutines.DelicateCoroutinesApi
 import kotlinx.coroutines.Dispatchers
@@ -27,12 +30,9 @@ import kotlinx.coroutines.Dispatchers.IO
 import kotlinx.coroutines.GlobalScope
 import kotlinx.coroutines.async
 import kotlinx.coroutines.launch
-import mozilla.appservices.Megazord
 import mozilla.appservices.autofill.AutofillApiException
 import mozilla.components.browser.state.action.SystemAction
 import mozilla.components.browser.state.selector.selectedTab
-import mozilla.components.browser.state.state.searchEngines
-import mozilla.components.browser.state.state.selectedOrDefaultSearchEngine
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.browser.storage.sync.GlobalPlacesDependencyProvider
 import mozilla.components.concept.base.crash.Breadcrumb
@@ -46,14 +46,14 @@ import mozilla.components.feature.autofill.AutofillUseCases
 import mozilla.components.feature.fxsuggest.GlobalFxSuggestDependencyProvider
 import mozilla.components.feature.search.ext.buildSearchUrl
 import mozilla.components.feature.search.ext.waitForSelectedOrDefaultSearchEngine
+import mozilla.components.feature.syncedtabs.commands.GlobalSyncedTabsCommandsProvider
 import mozilla.components.feature.top.sites.TopSitesFrecencyConfig
 import mozilla.components.feature.top.sites.TopSitesProviderConfig
+import mozilla.components.feature.webcompat.reporter.WebCompatReporterFeature
 import mozilla.components.lib.crash.CrashReporter
 import mozilla.components.service.fxa.manager.SyncEnginesStorage
-import mozilla.components.service.glean.Glean
-import mozilla.components.service.glean.config.Configuration
-import mozilla.components.service.glean.net.ConceptFetchHttpUploader
 import mozilla.components.service.sync.logins.LoginsApiException
+import mozilla.components.support.AppServicesInitializer
 import mozilla.components.support.base.ext.areNotificationsEnabledSafe
 import mozilla.components.support.base.ext.isNotificationChannelEnabled
 import mozilla.components.support.base.facts.register
@@ -64,43 +64,43 @@ import mozilla.components.support.ktx.android.arch.lifecycle.addObservers
 import mozilla.components.support.ktx.android.content.isMainProcess
 import mozilla.components.support.ktx.android.content.runOnlyInMainProcess
 import mozilla.components.support.locale.LocaleAwareApplication
-import mozilla.components.support.rusterrors.initializeRustErrors
+import mozilla.components.support.remotesettings.GlobalRemoteSettingsDependencyProvider
 import mozilla.components.support.rusthttp.RustHttpConfig
-import mozilla.components.support.rustlog.RustLog
 import mozilla.components.support.utils.BrowsersCache
 import mozilla.components.support.utils.logElapsedTime
 import mozilla.components.support.webextensions.WebExtensionSupport
+import mozilla.telemetry.glean.Glean
 import org.mozilla.fenix.GleanMetrics.Addons
 import org.mozilla.fenix.GleanMetrics.Addresses
 import org.mozilla.fenix.GleanMetrics.AndroidAutofill
 import org.mozilla.fenix.GleanMetrics.CreditCards
 import org.mozilla.fenix.GleanMetrics.CustomizeHome
 import org.mozilla.fenix.GleanMetrics.Events.marketingNotificationAllowed
-import org.mozilla.fenix.GleanMetrics.GleanBuildInfo
 import org.mozilla.fenix.GleanMetrics.Logins
 import org.mozilla.fenix.GleanMetrics.Metrics
 import org.mozilla.fenix.GleanMetrics.PerfStartup
 import org.mozilla.fenix.GleanMetrics.Preferences
 import org.mozilla.fenix.GleanMetrics.SearchDefaultEngine
-import org.mozilla.fenix.GleanMetrics.ShoppingSettings
-import org.mozilla.fenix.GleanMetrics.TabStrip
-import org.mozilla.fenix.GleanMetrics.TopSites
+import org.mozilla.fenix.GleanMetrics.TermsOfUse
 import org.mozilla.fenix.components.Components
 import org.mozilla.fenix.components.Core
 import org.mozilla.fenix.components.appstate.AppAction
-import org.mozilla.fenix.components.metrics.MetricServiceType
+import org.mozilla.fenix.components.initializeGlean
 import org.mozilla.fenix.components.metrics.MozillaProductDetector
+import org.mozilla.fenix.components.startMetricsIfEnabled
 import org.mozilla.fenix.experiments.maybeFetchExperiments
 import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.containsQueryParameters
-import org.mozilla.fenix.ext.getCustomGleanServerUrlIfAvailable
 import org.mozilla.fenix.ext.isCustomEngine
 import org.mozilla.fenix.ext.isKnownSearchDomain
-import org.mozilla.fenix.ext.setCustomEndpointIfAvailable
 import org.mozilla.fenix.ext.settings
+import org.mozilla.fenix.home.topsites.TopSitesConfigConstants.TOP_SITES_PROVIDER_LIMIT
+import org.mozilla.fenix.home.topsites.TopSitesConfigConstants.TOP_SITES_PROVIDER_MAX_THRESHOLD
 import org.mozilla.fenix.lifecycle.StoreLifecycleObserver
+import org.mozilla.fenix.lifecycle.VisibilityLifecycleObserver
 import org.mozilla.fenix.nimbus.FxNimbus
 import org.mozilla.fenix.onboarding.MARKETING_CHANNEL_ID
+import org.mozilla.fenix.perf.ApplicationExitInfoMetrics
 import org.mozilla.fenix.perf.MarkersActivityLifecycleCallbacks
 import org.mozilla.fenix.perf.ProfilerMarkerFactProcessor
 import org.mozilla.fenix.perf.StartupTimeline
@@ -108,21 +108,24 @@ import org.mozilla.fenix.perf.StorageStatsMetrics
 import org.mozilla.fenix.perf.runBlockingIncrement
 import org.mozilla.fenix.push.PushFxaIntegration
 import org.mozilla.fenix.push.WebPushEngineIntegration
-import org.mozilla.fenix.session.PerformanceActivityLifecycleCallbacks
 import org.mozilla.fenix.session.VisibilityLifecycleCallback
+import org.mozilla.fenix.settings.doh.DefaultDohSettingsProvider
+import org.mozilla.fenix.settings.doh.DohSettingsProvider
+import org.mozilla.fenix.startupCrash.StartupCrashActivity
 import org.mozilla.fenix.utils.Settings
-import org.mozilla.fenix.utils.Settings.Companion.TOP_SITES_PROVIDER_MAX_THRESHOLD
+import org.mozilla.fenix.utils.isLargeScreenSize
 import org.mozilla.fenix.wallpapers.Wallpaper
-import java.util.UUID
+import java.util.Date
 import java.util.concurrent.TimeUnit
 import kotlin.math.roundToLong
+import mozilla.components.support.AppServicesInitializer.Config as AppServicesConfig
 
 private const val RAM_THRESHOLD_MEGABYTES = 1024
 private const val BYTES_TO_MEGABYTES_CONVERSION = 1024.0 * 1024.0
 
 /**
- *The main application class for Fenix. Records data to measure initialization performance.
- *  Installs [CrashReporter], initializes [Glean]  in fenix builds and setup Megazord in the main process.
+ * The main application class for Fenix. Records data to measure initialization performance.
+ * Installs [CrashReporter], initializes [Glean] in fenix builds and setup [Megazord] in the main process.
  */
 @Suppress("Registered", "TooManyFunctions", "LargeClass")
 open class FenixApplication : LocaleAwareApplication(), Provider {
@@ -143,13 +146,6 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
     override fun onCreate() {
         super.onCreate()
-
-        if (shouldShowPrivacyNotice()) {
-            // For Mozilla Online build: Delay initialization on first run until privacy notice
-            // is accepted by the user.
-            return
-        }
-
         initialize()
     }
 
@@ -162,6 +158,11 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
         setupInAllProcesses()
 
+        // If the main process crashes before we've reached visual completeness, we consider it to
+        // be a startup crash and fork into the recovery flow. The activity that is responsible for
+        // that flow is hosted in a separate process, which means that we avoid the majority of
+        // initialization work that is done in `setupInMainProcess`
+        // Please see the README.md in the fenix/startupCrash package for more information.
         if (!isMainProcess()) {
             // If this is not the main process then do not continue with the initialization here. Everything that
             // follows only needs to be done in our app's main process and should not be done in other processes like
@@ -171,6 +172,8 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
 
         // DO NOT ADD ANYTHING ABOVE HERE.
+        // Note: That the startup crash recovery flow is hosted in a different process,
+        // so this call will be avoided in that case
         setupInMainProcessOnly()
         // DO NOT ADD ANYTHING UNDER HERE.
 
@@ -180,7 +183,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
         // We avoid blocking the main thread on startup by calling into Glean on the background thread.
         @OptIn(DelicateCoroutinesApi::class)
-        GlobalScope.launch(Dispatchers.IO) {
+        GlobalScope.launch(IO) {
             PerfStartup.applicationOnCreate.accumulateSamples(listOf(durationMillis))
         }
     }
@@ -188,48 +191,28 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
     @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
     @VisibleForTesting
     protected open fun initializeGlean() {
-        val telemetryEnabled = settings().isTelemetryEnabled
-
-        logger.debug("Initializing Glean (uploadEnabled=$telemetryEnabled})")
-
-        // for performance reasons, this is only available in Nightly or Debug builds
-        val customEndpoint = if (Config.channel.isNightlyOrDebug) {
-            // for testing, if custom glean server url is set in the secret menu, use it to initialize Glean
-            getCustomGleanServerUrlIfAvailable(this)
-        } else {
-            null
+        val settings = settings()
+        // We delay the Glean initialization until, we have user consent (After onboarding).
+        // If onboarding is disabled (when in local builds), continue to initialize Glean.
+        if (components.fenixOnboarding.userHasBeenOnboarded() || !FeatureFlags.onboardingFeatureEnabled) {
+            initializeGlean(this, logger, settings.isTelemetryEnabled, components.core.client)
         }
 
-        val configuration = Configuration(
-            channel = BuildConfig.BUILD_TYPE,
-            httpClient = ConceptFetchHttpUploader(
-                lazy(LazyThreadSafetyMode.NONE) { components.core.client },
-            ),
-            enableEventTimestamps = FxNimbus.features.glean.value().enableEventTimestamps,
-            delayPingLifetimeIo = FxNimbus.features.glean.value().delayPingLifetimeIo,
-        )
-
-        // Set the metric configuration from Nimbus.
-        Glean.applyServerKnobsConfig(FxNimbus.features.glean.value().metricsEnabled)
-
-        Glean.initialize(
-            applicationContext = this,
-            configuration = configuration.setCustomEndpointIfAvailable(customEndpoint),
-            uploadEnabled = telemetryEnabled,
-            buildInfo = GleanBuildInfo.buildInfo,
-        )
-
         // We avoid blocking the main thread on startup by setting startup metrics on the background thread.
         val store = components.core.store
-        GlobalScope.launch(Dispatchers.IO) {
-            setStartupMetrics(store, settings())
+        GlobalScope.launch(IO) {
+            setStartupMetrics(store, settings)
         }
     }
 
+    @SuppressLint("NewApi")
     @VisibleForTesting
     protected open fun setupInAllProcesses() {
-        setupCrashReporting()
-
+        // See Bug 1969818: Crash reporting requires updates to be compatible with
+        // isolated content process.
+        if (!android.os.Process.isIsolated()) {
+            setupCrashReporting()
+        }
         // We want the log messages of all builds to go to Android logcat
         Log.addSink(FenixLogSink(logsDebug = Config.channel.isDebug, AndroidLogSink()))
     }
@@ -248,11 +231,10 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
         run {
             // Make sure the engine is initialized and ready to use.
-            components.strictMode.resetAfter(StrictMode.allowThreadDiskReads()) {
+            components.strictMode.allowViolation(StrictMode::allowThreadDiskReads) {
                 components.core.engine.warmUp()
             }
 
-            // We need to always initialize Glean and do it early here.
             initializeGlean()
 
             // Attention: Do not invoke any code from a-s in this scope.
@@ -271,6 +253,10 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             // for the periodic task.
             GlobalPlacesDependencyProvider.initialize(components.core.historyStorage)
 
+            GlobalSyncedTabsCommandsProvider.initialize(lazy { components.backgroundServices.syncedTabsCommands })
+
+            initializeRemoteSettingsSupport()
+
             restoreBrowserState()
             restoreDownloads()
             restoreMessaging()
@@ -284,7 +270,16 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
 
         setupLeakCanary()
-        startMetricsIfEnabled()
+        if (components.fenixOnboarding.userHasBeenOnboarded()) {
+            startMetricsIfEnabled(
+                logger = logger,
+                analytics = components.analytics,
+                isTelemetryEnabled = settings().isTelemetryEnabled,
+                isMarketingTelemetryEnabled = settings().isMarketingTelemetryEnabled &&
+                    settings().hasMadeMarketingTelemetrySelection,
+                isDailyUsagePingEnabled = settings().isDailyUsagePingEnabled,
+            )
+        }
         setupPush()
 
         GlobalFxSuggestDependencyProvider.initialize(components.fxSuggest.storage)
@@ -295,6 +290,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
         components.appStartReasonProvider.registerInAppOnCreate(this)
         components.startupActivityLog.registerInAppOnCreate(this)
+        components.appLinkIntentLaunchTypeProvider.registerInAppOnCreate(this)
         initVisualCompletenessQueueAndQueueTasks()
 
         ProcessLifecycleOwner.get().lifecycle.addObservers(
@@ -302,11 +298,14 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                 appStore = components.appStore,
                 browserStore = components.core.store,
             ),
+            VisibilityLifecycleObserver(),
         )
 
         components.analytics.metricsStorage.tryRegisterAsUsageRecorder(this)
 
-        downloadWallpapers()
+        CoroutineScope(IO).launch {
+            components.useCases.wallpaperUseCases.fetchCurrentWallpaperUseCase.invoke()
+        }
     }
 
     @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
@@ -329,16 +328,12 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
     }
 
     private fun initVisualCompletenessQueueAndQueueTasks() {
-        val queue = components.performance.visualCompletenessQueue.queue
-
-        fun initQueue() {
-            registerActivityLifecycleCallbacks(PerformanceActivityLifecycleCallbacks(queue))
-        }
+        val queue = components.performance.visualCompletenessQueue
 
         @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
         fun queueInitStorageAndServices() {
-            components.performance.visualCompletenessQueue.queue.runIfReadyOrQueue {
-                GlobalScope.launch(Dispatchers.IO) {
+            queue.runIfReadyOrQueue {
+                GlobalScope.launch(IO) {
                     logger.info("Running post-visual completeness tasks...")
                     logElapsedTime(logger, "Storage initialization") {
                         components.core.historyStorage.warmUp()
@@ -353,14 +348,19 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                         // we can prevent with this.
                         components.core.topSitesStorage.getTopSites(
                             totalSites = components.settings.topSitesMaxLimit,
-                            frecencyConfig = TopSitesFrecencyConfig(
-                                FrecencyThresholdOption.SKIP_ONE_TIME_PAGES,
-                            ) {
-                                !Uri.parse(it.url)
-                                    .containsQueryParameters(components.settings.frecencyFilterQuery)
+                            frecencyConfig = if (FxNimbus.features.homepageHideFrecentTopSites.value().enabled) {
+                                null
+                            } else {
+                                TopSitesFrecencyConfig(
+                                    frecencyTresholdOption = FrecencyThresholdOption.SKIP_ONE_TIME_PAGES,
+                                ) {
+                                    !it.url.toUri()
+                                        .containsQueryParameters(components.settings.frecencyFilterQuery)
+                                }
                             },
                             providerConfig = TopSitesProviderConfig(
                                 showProviderTopSites = components.settings.showContileFeature,
+                                limit = TOP_SITES_PROVIDER_LIMIT,
                                 maxThreshold = TOP_SITES_PROVIDER_MAX_THRESHOLD,
                             ),
                         )
@@ -395,27 +395,27 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
 
         fun queueMetrics() {
-            if (SDK_INT >= Build.VERSION_CODES.O) { // required by StorageStatsMetrics.
-                queue.runIfReadyOrQueue {
-                    // Because it may be slow to capture the storage stats, it might be preferred to
-                    // create a WorkManager task for this metric, however, I ran out of
-                    // implementation time and WorkManager is harder to test.
-                    StorageStatsMetrics.report(this.applicationContext)
+            queue.runIfReadyOrQueue {
+                // Because it may be slow to capture the storage stats, it might be preferred to
+                // create a WorkManager task for this metric, however, I ran out of
+                // implementation time and WorkManager is harder to test.
+                StorageStatsMetrics.report(this.applicationContext)
+            }
+        }
+
+        @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
+        fun queueIncrementNumberOfAppLaunches() {
+            queue.runIfReadyOrQueue {
+                GlobalScope.launch(IO) {
+                    settings().numberOfAppLaunches += 1
                 }
             }
         }
 
-        @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
-        fun queueReviewPrompt() {
-            GlobalScope.launch(Dispatchers.IO) {
-                components.reviewPromptController.trackApplicationLaunch()
-            }
-        }
-
         @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
         fun queueRestoreLocale() {
-            components.performance.visualCompletenessQueue.queue.runIfReadyOrQueue {
-                GlobalScope.launch(Dispatchers.IO) {
+            queue.runIfReadyOrQueue {
+                GlobalScope.launch(IO) {
                     components.useCases.localeUseCases.restore()
                 }
             }
@@ -434,7 +434,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
         fun queueNimbusFetchInForeground() {
             queue.runIfReadyOrQueue {
-                GlobalScope.launch(Dispatchers.IO) {
+                GlobalScope.launch(IO) {
                     components.nimbus.sdk.maybeFetchExperiments(
                         context = this@FenixApplication,
                     )
@@ -442,26 +442,42 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             }
         }
 
-        initQueue()
+        @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
+        fun queueSuggestIngest() {
+            queue.runIfReadyOrQueue {
+                GlobalScope.launch(IO) {
+                    components.fxSuggest.storage.runStartupIngestion()
+                }
+            }
+        }
+
+        fun queueDownloadWallpapers() {
+            queue.runIfReadyOrQueue {
+                downloadWallpapers()
+            }
+        }
 
         // We init these items in the visual completeness queue to avoid them initing in the critical
         // startup path, before the UI finishes drawing (i.e. visual completeness).
         queueInitStorageAndServices()
         queueMetrics()
-        queueReviewPrompt()
+        queueIncrementNumberOfAppLaunches()
         queueRestoreLocale()
         queueStorageMaintenance()
         queueNimbusFetchInForeground()
+        queueDownloadWallpapers()
+        if (settings().enableFxSuggest) {
+            queueSuggestIngest()
+        }
+        queueCollectProcessExitInfo()
     }
 
-    private fun startMetricsIfEnabled() {
-        if (settings().isTelemetryEnabled) {
-            components.analytics.metrics.start(MetricServiceType.Data)
-            components.analytics.crashFactCollector.start()
-        }
-
-        if (settings().isMarketingTelemetryEnabled) {
-            components.analytics.metrics.start(MetricServiceType.Marketing)
+    @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
+    private fun queueCollectProcessExitInfo() {
+        if (SDK_INT >= Build.VERSION_CODES.R && settings().isTelemetryEnabled) {
+            GlobalScope.launch(IO) {
+                ApplicationExitInfoMetrics.recordProcessExits(applicationContext)
+            }
         }
     }
 
@@ -497,7 +513,20 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         components
             .analytics
             .crashReporter
-            .install(this)
+            .install(this, ::handleCaughtException)
+    }
+
+    private fun handleCaughtException() {
+        if (
+            isMainProcess() &&
+            Config.channel.isNightlyOrDebug &&
+            !components.performance.visualCompletenessQueue.isReady()
+        ) {
+            val intent = Intent(applicationContext, StartupCrashActivity::class.java)
+
+            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
+            applicationContext.startActivity(intent)
+        }
     }
 
     protected open fun initializeNimbus() {
@@ -516,33 +545,27 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
      * megazord - it contains everything that fenix needs, and (currently) nothing more.
      *
      * Documentation on what megazords are, and why they're needed:
-     * - https://github.com/mozilla/application-services/blob/master/docs/design/megazords.md
-     * - https://mozilla.github.io/application-services/docs/applications/consuming-megazord-libraries.html
+     * - https://github.com/mozilla/application-services/blob/main/docs/design/megazords.md
+     * - https://mozilla.github.io/application-services/book/design/megazords.html
      *
      * This is the initialization of the megazord without setting up networking, i.e. needing the
      * engine for networking. This should do the minimum work necessary as it is done on the main
      * thread, early in the app startup sequence.
      */
     private fun beginSetupMegazord() {
-        // Note: Megazord.init() must be called as soon as possible ...
-        Megazord.init()
-
-        initializeRustErrors(components.analytics.crashReporter)
-        // ... but RustHttpConfig.setClient() and RustLog.enable() can be called later.
-
-        RustLog.enable()
+        // Rust components must be initialized at the very beginning, before any other Rust call, ...
+        AppServicesInitializer.init(
+            AppServicesConfig(components.analytics.crashReporter),
+        )
     }
 
     @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
     private fun finishSetupMegazord(): Deferred<Unit> {
-        return GlobalScope.async(Dispatchers.IO) {
+        return GlobalScope.async(IO) {
             if (Config.channel.isDebug) {
                 RustHttpConfig.allowEmulatorLoopback()
             }
             RustHttpConfig.setClient(lazy { components.core.client })
-
-            // Now viaduct (the RustHttp client) is initialized we can ask Nimbus to fetch
-            // experiments recipes from the server.
         }
     }
 
@@ -553,6 +576,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
     }
 
+    @SuppressLint("NewApi")
     override fun onTrimMemory(level: Int) {
         super.onTrimMemory(level)
 
@@ -561,17 +585,21 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
         logger.info("onTrimMemory(), level=$level, main=${isMainProcess()}")
 
-        components.analytics.crashReporter.recordCrashBreadcrumb(
-            Breadcrumb(
-                category = "Memory",
-                message = "onTrimMemory()",
-                data = mapOf(
-                    "level" to level.toString(),
-                    "main" to isMainProcess().toString(),
+        // See Bug 1969818: Crash reporting requires updates to be compatible with
+        // isolated content process.
+        if (!android.os.Process.isIsolated()) {
+            components.analytics.crashReporter.recordCrashBreadcrumb(
+                Breadcrumb(
+                    category = "Memory",
+                    message = "onTrimMemory()",
+                    data = mapOf(
+                        "level" to level.toString(),
+                        "main" to isMainProcess().toString(),
+                    ),
+                    level = Breadcrumb.Level.INFO,
                 ),
-                level = Breadcrumb.Level.INFO,
-            ),
-        )
+            )
+        }
 
         runOnlyInMainProcess {
             components.core.icons.onTrimMemory(level)
@@ -579,8 +607,6 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
     }
 
-    @SuppressLint("WrongConstant")
-    // Suppressing erroneous lint warning about using MODE_NIGHT_AUTO_BATTERY, a likely library bug
     private fun setDayNightTheme() {
         val settings = this.settings()
         when {
@@ -621,21 +647,6 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
     }
 
-    /**
-     * Migrate the topic specific engine to the first general or custom search engine available.
-     */
-    private fun migrateTopicSpecificSearchEngines() {
-        components.core.store.state.search.selectedOrDefaultSearchEngine.let { currentSearchEngine ->
-            if (currentSearchEngine?.isGeneral == false) {
-                components.core.store.state.search.searchEngines.firstOrNull { nextSearchEngine ->
-                    nextSearchEngine.isGeneral
-                }?.let {
-                    components.useCases.searchUseCases.selectSearchEngine(it)
-                }
-            }
-        }
-    }
-
     @OptIn(DelicateCoroutinesApi::class) // GlobalScope usage
     private fun warmBrowsersCache() {
         // We avoid blocking the main thread for BrowsersCache on startup by loading it on
@@ -645,6 +656,12 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         }
     }
 
+    private fun initializeRemoteSettingsSupport() {
+        GlobalRemoteSettingsDependencyProvider.initialize(components.remoteSettingsService.value)
+        components.remoteSettingsSyncScheduler.registerForSync()
+    }
+
+    @Suppress("ForbiddenComment")
     private fun initializeWebExtensionSupport() {
         try {
             GlobalAddonDependencyProvider.initialize(
@@ -678,6 +695,15 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                 onExtensionsLoaded = { extensions ->
                     components.addonUpdater.registerForFutureUpdates(extensions)
                     subscribeForNewAddonsIfNeeded(components.supportedAddonsChecker, extensions)
+
+                    // Bug 1948634 - Make sure the webcompat-reporter extension is fully uninstalled.
+                    // This is added here because we need gecko to load the extension first.
+                    //
+                    // TODO: Bug 1953359 - remove the code below in the next release.
+                    if (Config.channel.isNightlyOrDebug || Config.channel.isBeta) {
+                        logger.debug("Attempting to uninstall the WebCompat Reporter extension")
+                        WebCompatReporterFeature.uninstall(components.core.engine)
+                    }
                 },
                 onUpdatePermissionRequest = components.addonUpdater::onUpdatePermissionRequest,
             )
@@ -714,30 +740,27 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
     internal fun setStartupMetrics(
         browserStore: BrowserStore,
         settings: Settings,
+        dohSettingsProvider: DohSettingsProvider = DefaultDohSettingsProvider(
+            components.core.engine,
+            settings,
+        ),
         browsersCache: BrowsersCache = BrowsersCache,
         mozillaProductDetector: MozillaProductDetector = MozillaProductDetector,
     ) {
-        setPreferenceMetrics(settings)
+        setPreferenceMetrics(settings, dohSettingsProvider)
         with(Metrics) {
             // Set this early to guarantee it's in every ping from here on.
-            distributionId.set(
-                when (Config.channel.isMozillaOnline) {
-                    true -> "MozillaOnline"
-                    false -> "Mozilla"
-                },
-            )
+            distributionId.set(components.distributionIdManager.getDistributionId())
+
+            if (settings.hasAcceptedTermsOfService) {
+                setTermsOfUseStartUpMetrics(settings)
+            }
 
             defaultBrowser.set(browsersCache.all(applicationContext).isDefaultBrowser)
             mozillaProductDetector.getMozillaBrowserDefault(applicationContext)?.also {
                 defaultMozBrowser.set(it)
             }
 
-            if (settings.contileContextId.isEmpty()) {
-                settings.contileContextId = TopSites.contextId.generateAndSet().toString()
-            } else {
-                TopSites.contextId.set(UUID.fromString(settings.contileContextId))
-            }
-
             mozillaProducts.set(
                 mozillaProductDetector.getInstalledMozillaProducts(
                     applicationContext,
@@ -796,14 +819,6 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             tabViewSetting.set(settings.getTabViewPingString())
             closeTabSetting.set(settings.getTabTimeoutPingString())
 
-            val installSourcePackage = if (SDK_INT >= Build.VERSION_CODES.R) {
-                packageManager.getInstallSourceInfo(packageName).installingPackageName
-            } else {
-                @Suppress("DEPRECATION")
-                packageManager.getInstallerPackageName(packageName)
-            }
-            installSource.set(installSourcePackage.orEmpty())
-
             val isDefaultTheCurrentWallpaper =
                 Wallpaper.nameIsDefault(settings.currentWallpaperName)
 
@@ -817,6 +832,8 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
             ramMoreThanThreshold.set(isDeviceRamAboveThreshold)
             deviceTotalRam.set(getDeviceTotalRAM())
+
+            isLargeDevice.set(isLargeScreenSize())
         }
 
         with(AndroidAutofill) {
@@ -831,7 +848,13 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                     !searchEngine.isCustomEngine() || searchEngine.isKnownSearchDomain()
                 if (sendSearchUrl) {
                     SearchDefaultEngine.apply {
-                        code.set(searchEngine.id)
+                        code.set(
+                            if (searchEngine.telemetrySuffix.isNullOrEmpty()) {
+                                searchEngine.id
+                            } else {
+                                "${searchEngine.id}-${searchEngine.telemetrySuffix}"
+                            },
+                        )
                         name.set(searchEngine.name)
                         searchUrl.set(searchEngine.buildSearchUrl(""))
                     }
@@ -841,37 +864,15 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                         name.set("custom")
                     }
                 }
-
-                migrateTopicSpecificSearchEngines()
             }
         }
 
-        @OptIn(DelicateCoroutinesApi::class)
-        GlobalScope.launch(IO) {
-            try {
-                val autoFillStorage = applicationContext.components.core.autofillStorage
-                Addresses.savedAll.set(autoFillStorage.getAllAddresses().size.toLong())
-                CreditCards.savedAll.set(autoFillStorage.getAllCreditCards().size.toLong())
-            } catch (e: AutofillApiException) {
-                logger.error("Failed to fetch autofill data", e)
-            }
+        setAutofillMetrics()
+    }
 
-            try {
-                val passwordsStorage = applicationContext.components.core.passwordsStorage
-                Logins.savedAll.set(passwordsStorage.list().size.toLong())
-            } catch (e: LoginsApiException) {
-                logger.error("Failed to fetch list of logins", e)
-            }
-        }
-
-        with(ShoppingSettings) {
-            componentOptedOut.set(!settings.isReviewQualityCheckEnabled)
-            nimbusDisabledShopping.set(!FxNimbus.features.shoppingExperience.value().enabled)
-            userHasOnboarded.set(settings.reviewQualityCheckOptInTimeInMillis != 0L)
-            disabledAds.set(!settings.isReviewQualityCheckProductRecommendationsEnabled)
-        }
-
-        TabStrip.enabled.set(settings.isTabStripEnabled)
+    private fun setTermsOfUseStartUpMetrics(settings: Settings) {
+        TermsOfUse.version.set(settings.termsOfUseAcceptedVersion.toLong())
+        TermsOfUse.date.set(Date(settings.termsOfUseAcceptedTimeInMillis))
     }
 
     @VisibleForTesting
@@ -905,9 +906,12 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
     @Suppress("ComplexMethod")
     private fun setPreferenceMetrics(
         settings: Settings,
+        dohSettingsProvider: DohSettingsProvider,
     ) {
         with(Preferences) {
             searchSuggestionsEnabled.set(settings.shouldShowSearchSuggestions)
+            showSponsorSuggestionsEnabled.set(settings.showSponsoredSuggestions)
+            showNonSponsorSuggestionsEnabled.set(settings.showNonSponsoredSuggestions)
             remoteDebuggingEnabled.set(settings.isRemoteDebuggingEnabled)
             studiesEnabled.set(settings.isExperimentationEnabled)
             telemetryEnabled.set(settings.isTelemetryEnabled)
@@ -918,6 +922,8 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             voiceSearchEnabled.set(settings.shouldShowVoiceSearch)
             openLinksInAppEnabled.set(settings.openLinksInExternalApp)
             signedInSync.set(settings.signedInFxaAccount)
+            isolatedContentProcessesEnabled.set(settings.isIsolatedProcessEnabled)
+            appZygoteIsolatedContentProcessesEnabled.set(settings.isAppZygoteEnabled)
 
             val syncedItems = SyncEnginesStorage(applicationContext).getStatus().entries.filter {
                 it.value
@@ -932,6 +938,13 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
                 },
             )
 
+            toolbarModeSetting.set(
+                when {
+                    settings.shouldUseExpandedToolbar -> "expanded"
+                    else -> "simple"
+                },
+            )
+
             enhancedTrackingProtection.set(
                 when {
                     !settings.shouldUseTrackingProtection -> ""
@@ -966,10 +979,35 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             )
 
             inactiveTabsEnabled.set(settings.inactiveTabsAreEnabled)
+            dohProtectionLevel.set(dohSettingsProvider.getSelectedProtectionLevel().toString())
+            httpsOnlyMode.set(settings.getHttpsOnlyMode().toString())
+            globalPrivacyControlEnabled.set(settings.shouldEnableGlobalPrivacyControl)
         }
         reportHomeScreenMetrics(settings)
     }
 
+    @VisibleForTesting
+    @OpenForTesting
+    internal open fun setAutofillMetrics() {
+        @OptIn(DelicateCoroutinesApi::class)
+        GlobalScope.launch(IO) {
+            try {
+                val autoFillStorage = applicationContext.components.core.autofillStorage
+                Addresses.savedAll.set(autoFillStorage.getAllAddresses().size.toLong())
+                CreditCards.savedAll.set(autoFillStorage.getAllCreditCards().size.toLong())
+            } catch (e: AutofillApiException) {
+                logger.error("Failed to fetch autofill data", e)
+            }
+
+            try {
+                val passwordsStorage = applicationContext.components.core.passwordsStorage
+                Logins.savedAll.set(passwordsStorage.list().size.toLong())
+            } catch (e: LoginsApiException) {
+                logger.error("Failed to fetch list of logins", e)
+            }
+        }
+    }
+
     @VisibleForTesting
     internal fun reportHomeScreenMetrics(settings: Settings) {
         reportOpeningScreenMetrics(settings)
@@ -1000,7 +1038,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
         CustomizeHome.contile.set(settings.showContileFeature)
     }
 
-    protected fun recordOnInit() {
+    private fun recordOnInit() {
         // This gets called by more than one process. Ideally we'd only run this in the main process
         // but the code to check which process we're in crashes because the Context isn't valid yet.
         //
@@ -1021,7 +1059,7 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
 
             // There's a strict mode violation in A-Cs LocaleAwareApplication which
             // reads from shared prefs: https://github.com/mozilla-mobile/android-components/issues/8816
-            components.strictMode.resetAfter(StrictMode.allowThreadDiskReads()) {
+            components.strictMode.allowViolation(StrictMode::allowThreadDiskReads) {
                 super.onConfigurationChanged(config)
             }
         } else {
@@ -1037,14 +1075,4 @@ open class FenixApplication : LocaleAwareApplication(), Provider {
             components.useCases.wallpaperUseCases.initialize()
         }
     }
-
-    /**
-     * Checks whether or not a privacy notice needs to be displayed before
-     * the application can continue to initialize.
-     */
-    internal fun shouldShowPrivacyNotice(): Boolean {
-        return Config.channel.isMozillaOnline &&
-            settings().shouldShowPrivacyPopWindow &&
-            !components.fenixOnboarding.userHasBeenOnboarded()
-    }
 }
