@@ -4,8 +4,8 @@
 
 package org.mozilla.fenix.browser.tabstrip
 
+import android.annotation.SuppressLint
 import android.graphics.Bitmap
-import androidx.compose.foundation.ExperimentalFoundationApi
 import androidx.compose.foundation.Image
 import androidx.compose.foundation.background
 import androidx.compose.foundation.clickable
@@ -22,16 +22,15 @@ import androidx.compose.foundation.layout.height
 import androidx.compose.foundation.layout.padding
 import androidx.compose.foundation.layout.size
 import androidx.compose.foundation.layout.width
-import androidx.compose.foundation.lazy.LazyListItemInfo
-import androidx.compose.foundation.lazy.LazyListState
 import androidx.compose.foundation.lazy.LazyRow
 import androidx.compose.foundation.lazy.itemsIndexed
 import androidx.compose.foundation.lazy.rememberLazyListState
+import androidx.compose.foundation.selection.selectableGroup
 import androidx.compose.foundation.shape.CircleShape
 import androidx.compose.foundation.systemGestureExclusion
-import androidx.compose.material.Icon
-import androidx.compose.material.IconButton
-import androidx.compose.material.Text
+import androidx.compose.material3.Icon
+import androidx.compose.material3.IconButton
+import androidx.compose.material3.Text
 import androidx.compose.runtime.Composable
 import androidx.compose.runtime.LaunchedEffect
 import androidx.compose.runtime.getValue
@@ -43,6 +42,14 @@ import androidx.compose.ui.graphics.asImageBitmap
 import androidx.compose.ui.res.dimensionResource
 import androidx.compose.ui.res.painterResource
 import androidx.compose.ui.res.stringResource
+import androidx.compose.ui.semantics.CustomAccessibilityAction
+import androidx.compose.ui.semantics.Role
+import androidx.compose.ui.semantics.clearAndSetSemantics
+import androidx.compose.ui.semantics.customActions
+import androidx.compose.ui.semantics.role
+import androidx.compose.ui.semantics.selected
+import androidx.compose.ui.semantics.semantics
+import androidx.compose.ui.semantics.traversalIndex
 import androidx.compose.ui.tooling.preview.Devices
 import androidx.compose.ui.tooling.preview.Preview
 import androidx.compose.ui.tooling.preview.PreviewParameter
@@ -53,6 +60,7 @@ import androidx.core.text.BidiFormatter
 import mozilla.components.browser.state.action.TabListAction
 import mozilla.components.browser.state.state.createTab
 import mozilla.components.browser.state.store.BrowserStore
+import mozilla.components.compose.base.modifier.thenConditional
 import mozilla.components.feature.tabs.TabsUseCases
 import mozilla.components.lib.state.ext.observeAsState
 import org.mozilla.fenix.R
@@ -60,24 +68,29 @@ import org.mozilla.fenix.components.AppStore
 import org.mozilla.fenix.components.components
 import org.mozilla.fenix.compose.Favicon
 import org.mozilla.fenix.compose.HorizontalFadingEdgeBox
+import org.mozilla.fenix.compose.ext.isItemPartiallyVisible
 import org.mozilla.fenix.tabstray.browser.compose.DragItemContainer
 import org.mozilla.fenix.tabstray.browser.compose.createListReorderState
 import org.mozilla.fenix.tabstray.browser.compose.detectListPressAndDrag
 import org.mozilla.fenix.theme.FirefoxTheme
 import org.mozilla.fenix.theme.Theme
+import mozilla.components.ui.icons.R as iconsR
+import org.mozilla.fenix.GleanMetrics.TabStrip as TabStripMetrics
 
-private val minTabStripItemWidth = 160.dp
+private val minTabStripItemWidth = 130.dp
 private val maxTabStripItemWidth = 280.dp
+private val tabItemHeight = 40.dp
 private val tabStripIconSize = 24.dp
 private val spaceBetweenTabs = 4.dp
-private val tabStripStartPadding = 8.dp
-private val addTabIconSize = 20.dp
+private val tabStripListContentStartPadding = 8.dp
 private val titleFadeWidth = 16.dp
+private val tabStripHorizontalPadding = 16.dp
 
 /**
  * Top level composable for the tabs strip.
  *
- * @param onHome Whether or not the tabs strip is in the home screen.
+ * @param isSelectDisabled Whether or not the tabs can be shown as selected.
+ * @param showActionButtons Show the action buttons in the tabs strip when true.
  * @param browserStore The [BrowserStore] instance used to observe tabs state.
  * @param appStore The [AppStore] instance used to observe browsing mode.
  * @param tabsUseCases The [TabsUseCases] instance to perform tab actions.
@@ -85,93 +98,144 @@ private val titleFadeWidth = 16.dp
  * @param onCloseTabClick Invoked when a tab is closed.
  * @param onLastTabClose Invoked when the last remaining open tab is closed.
  * @param onSelectedTabClick Invoked when a tab is selected.
+ * @param onTabCounterClick Invoked when tab counter is clicked.
  */
 @Composable
 fun TabStrip(
-    onHome: Boolean = false,
+    isSelectDisabled: Boolean = false,
+    showActionButtons: Boolean = true,
     browserStore: BrowserStore = components.core.store,
     appStore: AppStore = components.appStore,
     tabsUseCases: TabsUseCases = components.useCases.tabsUseCases,
     onAddTabClick: () -> Unit,
     onCloseTabClick: (isPrivate: Boolean) -> Unit,
     onLastTabClose: (isPrivate: Boolean) -> Unit,
-    onSelectedTabClick: () -> Unit,
+    onSelectedTabClick: (url: String) -> Unit,
+    onTabCounterClick: () -> Unit,
 ) {
-    val isPrivateMode by appStore.observeAsState(false) { it.mode.isPrivate }
+    val isPossiblyPrivateMode by appStore.observeAsState(false) { it.mode.isPrivate }
     val state by browserStore.observeAsState(TabStripState.initial) {
-        it.toTabStripState(isSelectDisabled = onHome, isPrivateMode = isPrivateMode)
+        it.toTabStripState(
+            isSelectDisabled = isSelectDisabled,
+            isPossiblyPrivateMode = isPossiblyPrivateMode,
+            addTab = onAddTabClick,
+            closeTab = { isPrivate, numberOfTabs ->
+                it.selectedTabId?.let { selectedTabId ->
+                    closeTab(
+                        numberOfTabs = numberOfTabs,
+                        isPrivate = isPrivate,
+                        tabsUseCases = tabsUseCases,
+                        tabId = selectedTabId,
+                        onLastTabClose = onLastTabClose,
+                        onCloseTabClick = onCloseTabClick,
+                    )
+                }
+            },
+        )
     }
 
     TabStripContent(
         state = state,
-        onAddTabClick = onAddTabClick,
-        onCloseTabClick = { id, isPrivate ->
-            if (state.tabs.size == 1) {
-                onLastTabClose(isPrivate)
-            }
-            tabsUseCases.removeTab(id)
-            onCloseTabClick(isPrivate)
+        showActionButtons = showActionButtons,
+        onAddTabClick = {
+            onAddTabClick()
+            TabStripMetrics.newTabTapped.record()
         },
-        onSelectedTabClick = {
-            tabsUseCases.selectTab(it)
-            onSelectedTabClick()
+        onCloseTabClick = { tabId, isPrivate ->
+            closeTab(
+                numberOfTabs = state.tabs.size,
+                isPrivate = isPrivate,
+                tabsUseCases = tabsUseCases,
+                tabId = tabId,
+                onLastTabClose = onLastTabClose,
+                onCloseTabClick = onCloseTabClick,
+            )
+        },
+        onSelectedTabClick = { tabId, url ->
+            tabsUseCases.selectTab(tabId)
+            onSelectedTabClick(url)
+            TabStripMetrics.selectTab.record()
         },
         onMove = { tabId, targetId, placeAfter ->
             if (tabId != targetId) {
                 tabsUseCases.moveTabs(listOf(tabId), targetId, placeAfter)
             }
         },
+        onTabCounterClick = onTabCounterClick,
     )
 }
 
 @Composable
 private fun TabStripContent(
     state: TabStripState,
+    showActionButtons: Boolean = true,
     onAddTabClick: () -> Unit,
     onCloseTabClick: (id: String, isPrivate: Boolean) -> Unit,
-    onSelectedTabClick: (id: String) -> Unit,
+    onSelectedTabClick: (tabId: String, url: String) -> Unit,
     onMove: (tabId: String, targetId: String, placeAfter: Boolean) -> Unit,
+    onTabCounterClick: () -> Unit,
 ) {
     Row(
         modifier = Modifier
-            .fillMaxSize()
-            .background(FirefoxTheme.colors.layer1)
-            .systemGestureExclusion(),
+            .fillMaxWidth()
+            .height(dimensionResource(R.dimen.tab_strip_height))
+            .background(FirefoxTheme.colors.layer3)
+            .systemGestureExclusion()
+            .padding(horizontal = tabStripHorizontalPadding),
         verticalAlignment = Alignment.CenterVertically,
+        horizontalArrangement = Arrangement.SpaceBetween,
     ) {
-        TabsList(
-            state = state,
+        Row(
             modifier = Modifier.weight(1f, fill = false),
-            onCloseTabClick = onCloseTabClick,
-            onSelectedTabClick = onSelectedTabClick,
-            onMove = onMove,
-        )
+            verticalAlignment = Alignment.CenterVertically,
+        ) {
+            TabsList(
+                state = state,
+                modifier = Modifier.weight(1f, fill = false),
+                onCloseTabClick = onCloseTabClick,
+                onSelectedTabClick = onSelectedTabClick,
+                onMove = onMove,
+            )
 
-        IconButton(onClick = onAddTabClick) {
-            Icon(
-                painter = painterResource(R.drawable.mozac_ic_plus_24),
-                modifier = Modifier.size(addTabIconSize),
-                tint = FirefoxTheme.colors.iconPrimary,
-                contentDescription = stringResource(R.string.add_tab),
+            if (showActionButtons) {
+                IconButton(onClick = onAddTabClick) {
+                    Icon(
+                        painter = painterResource(iconsR.drawable.mozac_ic_plus_24),
+                        tint = FirefoxTheme.colors.iconPrimary,
+                        contentDescription = stringResource(R.string.add_tab),
+                    )
+                }
+            }
+        }
+
+        if (showActionButtons) {
+            TabStripTabCounterButton(
+                tabCount = state.tabs.size,
+                size = dimensionResource(R.dimen.tab_strip_height),
+                menuItems = state.menuItems,
+                privacyBadgeVisible = state.isPrivateMode,
+                onClick = onTabCounterClick,
             )
         }
     }
 }
 
+// There is a bug with `BoxWithConstraints` where it flags the `BoxWithConstraintsScope` being unused
+// even though it's being used implicitly below via the `maxWidth` property of `BoxWithConstraintsScope`.
+@SuppressLint("UnusedBoxWithConstraintsScope")
 @Composable
-@OptIn(ExperimentalFoundationApi::class)
 private fun TabsList(
     state: TabStripState,
     modifier: Modifier = Modifier,
     onCloseTabClick: (id: String, isPrivate: Boolean) -> Unit,
-    onSelectedTabClick: (id: String) -> Unit,
+    onSelectedTabClick: (tabId: String, url: String) -> Unit,
     onMove: (tabId: String, targetId: String, placeAfter: Boolean) -> Unit,
 ) {
     BoxWithConstraints(modifier = modifier) {
         val listState = rememberLazyListState()
         // Calculate the width of each tab item based on available width and the number of tabs and
         // taking into account the space between tabs.
-        val availableWidth = maxWidth - tabStripStartPadding
+        val availableWidth = maxWidth - tabStripListContentStartPadding
         val tabWidth = (availableWidth / state.tabs.size) - spaceBetweenTabs
 
         val reorderState = createListReorderState(
@@ -187,13 +251,15 @@ private fun TabsList(
         )
 
         LazyRow(
-            modifier = Modifier.detectListPressAndDrag(
-                reorderState = reorderState,
-                listState = listState,
-                shouldLongPressToDrag = true,
-            ),
+            modifier = Modifier
+                .detectListPressAndDrag(
+                    reorderState = reorderState,
+                    listState = listState,
+                    shouldLongPressToDrag = true,
+                )
+                .selectableGroup(),
             state = listState,
-            contentPadding = PaddingValues(start = tabStripStartPadding),
+            contentPadding = PaddingValues(start = tabStripListContentStartPadding),
         ) {
             itemsIndexed(
                 items = state.tabs,
@@ -210,12 +276,16 @@ private fun TabsList(
                         onSelectedTabClick = onSelectedTabClick,
                         modifier = Modifier
                             .padding(end = spaceBetweenTabs)
-                            .animateItemPlacement()
+                            .animateItem()
                             .width(
                                 tabWidth.coerceIn(
                                     minimumValue = minTabStripItemWidth,
                                     maximumValue = maxTabStripItemWidth,
                                 ),
+                            )
+                            .thenConditional(
+                                modifier = Modifier.semantics { traversalIndex = -1f },
+                                predicate = { itemState.isSelected },
                             ),
                     )
                 }
@@ -239,7 +309,7 @@ private fun TabsList(
                     val selectedItemInfo =
                         listState.layoutInfo.visibleItemsInfo.firstOrNull { it.key == selectedTab.id }
 
-                    if (listState.isItemPartiallyVisible(selectedItemInfo) || selectedItemInfo == null) {
+                    if (selectedItemInfo == null || listState.isItemPartiallyVisible(selectedItemInfo)) {
                         listState.animateScrollToItem(state.tabs.indexOf(selectedTab))
                     }
                 }
@@ -248,33 +318,23 @@ private fun TabsList(
     }
 }
 
-private fun LazyListState.isItemPartiallyVisible(itemInfo: LazyListItemInfo?) =
-    itemInfo != null &&
-        (itemInfo.offset + itemInfo.size > layoutInfo.viewportEndOffset || itemInfo.offset < 0)
-
 @Composable
+@Suppress("LongMethod")
 private fun TabItem(
     state: TabStripItem,
     modifier: Modifier = Modifier,
     onCloseTabClick: (id: String, isPrivate: Boolean) -> Unit,
-    onSelectedTabClick: (id: String) -> Unit,
+    onSelectedTabClick: (id: String, url: String) -> Unit,
 ) {
-    val backgroundColor = if (state.isPrivate) {
-        if (state.isSelected) {
-            FirefoxTheme.colors.layer3
-        } else {
-            FirefoxTheme.colors.layer2
-        }
+    val backgroundColor = if (state.isSelected) {
+        FirefoxTheme.colors.tabActive
     } else {
-        if (state.isSelected) {
-            FirefoxTheme.colors.layer2
-        } else {
-            FirefoxTheme.colors.layer3
-        }
+        FirefoxTheme.colors.tabInactive
     }
+    val closeTabLabel = stringResource(R.string.close_tab)
 
     TabStripCard(
-        modifier = modifier.fillMaxSize(),
+        modifier = modifier.height(tabItemHeight),
         backgroundColor = backgroundColor,
         elevation = if (state.isSelected) {
             selectedTabStripCardElevation
@@ -285,7 +345,19 @@ private fun TabItem(
         Row(
             modifier = Modifier
                 .fillMaxSize()
-                .clickable { onSelectedTabClick(state.id) },
+                .clickable { onSelectedTabClick(state.id, state.url) }
+                .semantics {
+                    role = Role.Tab
+                    selected = state.isSelected
+                    customActions = listOf(
+                        CustomAccessibilityAction(
+                            label = closeTabLabel,
+                        ) {
+                            onCloseTabClick(state.id, state.isPrivate)
+                            true
+                        },
+                    )
+                },
             verticalAlignment = Alignment.CenterVertically,
             horizontalArrangement = Arrangement.SpaceBetween,
         ) {
@@ -326,12 +398,30 @@ private fun TabItem(
                 }
             }
 
-            IconButton(onClick = { onCloseTabClick(state.id, state.isPrivate) }) {
-                Icon(
-                    painter = painterResource(R.drawable.mozac_ic_cross_20),
-                    tint = FirefoxTheme.colors.iconPrimary,
-                    contentDescription = stringResource(R.string.close_tab),
-                )
+            if (state.isCloseButtonVisible) {
+                IconButton(
+                    onClick = { onCloseTabClick(state.id, state.isPrivate) },
+                    modifier = if (state.isSelected) {
+                        Modifier.semantics {}
+                    } else {
+                        Modifier.clearAndSetSemantics {}
+                    },
+                ) {
+                    Icon(
+                        painter = painterResource(iconsR.drawable.mozac_ic_cross_20),
+                        tint = if (state.isSelected) {
+                            FirefoxTheme.colors.iconPrimary
+                        } else {
+                            FirefoxTheme.colors.iconSecondary
+                        },
+                        contentDescription = stringResource(
+                            id = R.string.close_tab_title,
+                            state.title,
+                        ),
+                    )
+                }
+            } else {
+                Spacer(modifier = Modifier.size(8.dp))
             }
         }
     }
@@ -365,6 +455,22 @@ private fun TabStripIcon(
     }
 }
 
+private fun closeTab(
+    numberOfTabs: Int,
+    isPrivate: Boolean,
+    tabsUseCases: TabsUseCases,
+    tabId: String,
+    onLastTabClose: (isPrivate: Boolean) -> Unit,
+    onCloseTabClick: (isPrivate: Boolean) -> Unit,
+) {
+    if (numberOfTabs == 1) {
+        onLastTabClose(isPrivate)
+    }
+    tabsUseCases.removeTab(tabId)
+    onCloseTabClick(isPrivate)
+    TabStripMetrics.closeTab.record()
+}
+
 private class TabUIStateParameterProvider : PreviewParameterProvider<TabStripState> {
     override val values: Sequence<TabStripState>
         get() = sequenceOf(
@@ -406,11 +512,13 @@ private class TabUIStateParameterProvider : PreviewParameterProvider<TabStripSta
                         isSelected = true,
                     ),
                 ),
+                isPrivateMode = false,
+                tabCounterMenuItems = emptyList(),
             ),
         )
 }
 
-@Preview(device = Devices.TABLET)
+@Preview(device = Devices.PIXEL_TABLET)
 @Composable
 private fun TabStripPreview(
     @PreviewParameter(TabUIStateParameterProvider::class) tabStripState: TabStripState,
@@ -420,7 +528,7 @@ private fun TabStripPreview(
     }
 }
 
-@Preview(device = Devices.TABLET)
+@Preview(device = Devices.PIXEL_TABLET)
 @Composable
 private fun TabStripPreviewDarkMode(
     @PreviewParameter(TabUIStateParameterProvider::class) tabStripState: TabStripState,
@@ -430,7 +538,7 @@ private fun TabStripPreviewDarkMode(
     }
 }
 
-@Preview(device = Devices.TABLET)
+@Preview(device = Devices.PIXEL_TABLET)
 @Composable
 private fun TabStripPreviewPrivateMode(
     @PreviewParameter(TabUIStateParameterProvider::class) tabStripState: TabStripState,
@@ -451,16 +559,19 @@ private fun TabStripContentPreview(tabs: List<TabStripItem>) {
         TabStripContent(
             state = TabStripState(
                 tabs = tabs,
+                isPrivateMode = false,
+                tabCounterMenuItems = emptyList(),
             ),
             onAddTabClick = {},
             onCloseTabClick = { _, _ -> },
-            onSelectedTabClick = {},
+            onSelectedTabClick = { _, _ -> },
             onMove = { _, _, _ -> },
+            onTabCounterClick = {},
         )
     }
 }
 
-@Preview(device = Devices.TABLET)
+@Preview(device = Devices.PIXEL_TABLET)
 @Composable
 private fun TabStripPreview() {
     val browserStore = BrowserStore()
@@ -485,6 +596,7 @@ private fun TabStripPreview() {
                 onLastTabClose = {},
                 onCloseTabClick = {},
                 onSelectedTabClick = {},
+                onTabCounterClick = {},
             )
         }
     }
