@@ -32,17 +32,20 @@ import mozilla.components.browser.state.state.TabSessionState
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.storage.BookmarksStorage
 import mozilla.components.feature.top.sites.PinnedSiteStorage
-import mozilla.components.feature.webcompat.reporter.WebCompatReporterFeature
 import mozilla.components.lib.state.ext.flowScoped
 import mozilla.components.support.ktx.android.content.getColorFromAttr
+import mozilla.components.support.ktx.kotlin.isAboutUrl
+import mozilla.components.support.ktx.kotlin.isContentUrl
 import mozilla.components.support.ktx.kotlinx.coroutines.flow.ifAnyChanged
 import org.mozilla.fenix.Config
 import org.mozilla.fenix.R
+import org.mozilla.fenix.automotive.isAndroidAutomotiveAvailable
 import org.mozilla.fenix.components.accounts.FenixAccountManager
 import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.settings
 import org.mozilla.fenix.nimbus.FxNimbus
 import org.mozilla.fenix.theme.ThemeManager
+import mozilla.components.ui.icons.R as iconsR
 
 /**
  * Builds the toolbar object used with the 3-dot menu in the browser fragment.
@@ -73,7 +76,6 @@ open class DefaultToolbarMenu(
 
     private val shouldDeleteDataOnQuit = context.settings().shouldDeleteBrowsingDataOnQuit
     private val shouldUseBottomToolbar = context.settings().shouldUseBottomToolbar
-    private val shouldShowMenuToolbar = !IncompleteRedesignToolbarFeature(context.settings()).isEnabled
     private val shouldShowTopSites = context.settings().showTopSitesFeature
     private val accountManager = FenixAccountManager(context)
 
@@ -99,7 +101,7 @@ open class DefaultToolbarMenu(
 
     override val menuToolbar by lazy {
         val back = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = mozilla.components.ui.icons.R.drawable.mozac_ic_back_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_back_24,
             primaryContentDescription = context.getString(R.string.browser_menu_back),
             primaryImageTintResource = primaryTextColor(),
             isInPrimaryState = {
@@ -113,7 +115,7 @@ open class DefaultToolbarMenu(
         }
 
         val forward = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = mozilla.components.ui.icons.R.drawable.mozac_ic_forward_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_forward_24,
             primaryContentDescription = context.getString(R.string.browser_menu_forward),
             primaryImageTintResource = primaryTextColor(),
             isInPrimaryState = {
@@ -127,13 +129,13 @@ open class DefaultToolbarMenu(
         }
 
         val refresh = BrowserMenuItemToolbar.TwoStateButton(
-            primaryImageResource = mozilla.components.ui.icons.R.drawable.mozac_ic_arrow_clockwise_24,
+            primaryImageResource = iconsR.drawable.mozac_ic_arrow_clockwise_24,
             primaryContentDescription = context.getString(R.string.browser_menu_refresh),
             primaryImageTintResource = primaryTextColor(),
             isInPrimaryState = {
                 selectedSession?.content?.loading == false
             },
-            secondaryImageResource = mozilla.components.ui.icons.R.drawable.mozac_ic_stop,
+            secondaryImageResource = iconsR.drawable.mozac_ic_stop,
             secondaryContentDescription = context.getString(R.string.browser_menu_stop),
             secondaryImageTintResource = primaryTextColor(),
             disableInSecondaryState = false,
@@ -155,17 +157,22 @@ open class DefaultToolbarMenu(
             },
         )
 
-        registerForIsBookmarkedUpdates()
-
-        registerForScreenReaderUpdates()
-
         BrowserMenuItemToolbar(listOf(back, forward, share, refresh), isSticky = true)
     }
 
     // Predicates that need to be repeatedly called as the session changes
     @VisibleForTesting(otherwise = PRIVATE)
     fun canAddToHomescreen(): Boolean =
-        selectedSession != null && isPinningSupported
+        selectedSession != null && isPinningSupported &&
+            !context.components.useCases.webAppUseCases.isInstallable()
+
+    /**
+     * Should the menu item to install as PWA be visible?
+     */
+    @VisibleForTesting(otherwise = PRIVATE)
+    fun canAddAppToHomescreen(): Boolean =
+        selectedSession != null && isPinningSupported &&
+            context.components.useCases.webAppUseCases.isInstallable()
 
     /**
      * Should the "Open in regular tab" menu item be visible?
@@ -197,20 +204,27 @@ open class DefaultToolbarMenu(
     @VisibleForTesting(otherwise = PRIVATE)
     fun shouldShowTranslations(): Boolean {
         val isEngineSupported = store.state.translationEngine.isEngineSupported
-        if (isEngineSupported == true) {
-            FxNimbus.features.translations.recordExposure()
-        }
         return selectedSession?.let {
             isEngineSupported == true &&
                 FxNimbus.features.translations.value().mainFlowBrowserMenuEnabled
         } ?: false
     }
+
+    /**
+     * Return whether Report Broken Site menu item is visible
+     */
+    private fun shouldShowWebCompatReporter(): Boolean {
+        val url = store.state.selectedTab?.content?.url
+        val isAboutUrl = url?.isAboutUrl() ?: false
+        val isContentUrl = url?.isContentUrl() ?: false
+        return !isAboutUrl && !isContentUrl
+    }
     // End of predicates //
 
     @VisibleForTesting
     internal val newTabItem = BrowserMenuImageText(
         context.getString(R.string.library_new_tab),
-        R.drawable.ic_new,
+        iconsR.drawable.mozac_ic_plus_24,
         primaryTextColor(),
     ) {
         onItemTapped.invoke(ToolbarMenu.Item.NewTab)
@@ -234,7 +248,7 @@ open class DefaultToolbarMenu(
 
     private val passwordsItem = BrowserMenuImageText(
         context.getString(R.string.preferences_sync_logins_2),
-        R.drawable.mozac_ic_login_24,
+        iconsR.drawable.mozac_ic_login_24,
         primaryTextColor(),
     ) {
         onItemTapped.invoke(ToolbarMenu.Item.Passwords)
@@ -246,7 +260,7 @@ open class DefaultToolbarMenu(
 
     private val findInPageItem = BrowserMenuImageText(
         label = context.getString(R.string.browser_menu_find_in_page),
-        imageResource = R.drawable.mozac_ic_search_24,
+        imageResource = iconsR.drawable.mozac_ic_search_24,
         iconTintColorResource = primaryTextColor(),
     ) {
         onItemTapped.invoke(ToolbarMenu.Item.FindInPage)
@@ -254,7 +268,7 @@ open class DefaultToolbarMenu(
 
     private val translationsItem = BrowserMenuImageText(
         label = context.getString(R.string.browser_menu_translations),
-        imageResource = R.drawable.mozac_ic_translate_24,
+        imageResource = iconsR.drawable.mozac_ic_translate_24,
         iconTintColorResource = primaryTextColor(),
     ) {
         onItemTapped.invoke(ToolbarMenu.Item.Translate)
@@ -298,22 +312,22 @@ open class DefaultToolbarMenu(
         onItemTapped.invoke(ToolbarMenu.Item.OpenInApp)
     }
 
-    private val reportSiteIssuePlaceholder = WebExtensionPlaceholderMenuItem(
-        id = WebCompatReporterFeature.WEBCOMPAT_REPORTER_EXTENSION_ID,
-        iconTintColorResource = primaryTextColor(),
-    )
-
     private val addToHomeScreenItem = BrowserMenuImageText(
         label = context.getString(R.string.browser_menu_add_to_homescreen),
-        imageResource = R.drawable.mozac_ic_add_to_homescreen_24,
+        imageResource = iconsR.drawable.mozac_ic_add_to_homescreen_24,
         iconTintColorResource = primaryTextColor(),
         isCollapsingMenuLimit = true,
     ) {
-        if (context.components.useCases.webAppUseCases.isInstallable()) {
-            onItemTapped.invoke(ToolbarMenu.Item.InstallPwaToHomeScreen)
-        } else {
-            onItemTapped.invoke(ToolbarMenu.Item.AddToHomeScreen)
-        }
+        onItemTapped.invoke(ToolbarMenu.Item.AddToHomeScreen)
+    }
+
+    private val addAppToHomeScreenItem = BrowserMenuImageText(
+        label = context.getString(R.string.browser_menu_add_app_to_homescreen),
+        imageResource = iconsR.drawable.mozac_ic_add_to_homescreen_24,
+        iconTintColorResource = primaryTextColor(),
+        isCollapsingMenuLimit = true,
+    ) {
+        onItemTapped.invoke(ToolbarMenu.Item.InstallPwaToHomeScreen)
     }
 
     private val addRemoveTopSitesItem = TwoStateBrowserMenuImageText(
@@ -353,7 +367,7 @@ open class DefaultToolbarMenu(
     @VisibleForTesting
     internal val settingsItem = BrowserMenuHighlightableItem(
         label = context.getString(R.string.browser_menu_settings),
-        startImageResource = R.drawable.mozac_ic_settings_24,
+        startImageResource = iconsR.drawable.mozac_ic_settings_24,
         iconTintColorResource = if (hasAccountProblem) {
             ThemeManager.resolveAttribute(R.attr.syncDisconnected, context)
         } else {
@@ -393,7 +407,7 @@ open class DefaultToolbarMenu(
 
     private val deleteDataOnQuit = BrowserMenuImageText(
         label = context.getString(R.string.delete_browsing_data_on_quit_action),
-        imageResource = R.drawable.mozac_ic_cross_circle_24,
+        imageResource = iconsR.drawable.mozac_ic_cross_circle_24,
         iconTintColorResource = primaryTextColor(),
     ) {
         onItemTapped.invoke(ToolbarMenu.Item.Quit)
@@ -407,11 +421,19 @@ open class DefaultToolbarMenu(
         }
     }
 
+    private val reportBrokenSite = BrowserMenuImageText(
+        label = context.getString(R.string.browser_menu_webcompat_reporter),
+        imageResource = iconsR.drawable.mozac_ic_lightbulb_24,
+        iconTintColorResource = primaryTextColor(),
+    ) {
+        onItemTapped.invoke(ToolbarMenu.Item.ReportBrokenSite)
+    }
+
     @VisibleForTesting(otherwise = PRIVATE)
     val coreMenuItems by lazy {
         val menuItems =
             listOfNotNull(
-                if (shouldUseBottomToolbar || !shouldShowMenuToolbar) null else menuToolbar,
+                if (shouldUseBottomToolbar) null else menuToolbar,
                 newTabItem,
                 BrowserMenuDivider(),
                 bookmarksItem,
@@ -423,23 +445,33 @@ open class DefaultToolbarMenu(
                 BrowserMenuDivider(),
                 findInPageItem,
                 translationsItem.apply { visible = ::shouldShowTranslations },
-                desktopSiteItem,
+                desktopSiteItem.apply { visible = { store.state.selectedTab?.content?.isPdf == false } },
                 openInRegularTabItem.apply { visible = ::shouldShowOpenInRegularTab },
                 customizeReaderView.apply { visible = ::shouldShowReaderViewCustomization },
                 openInApp.apply { visible = ::shouldShowOpenInApp },
-                reportSiteIssuePlaceholder,
+                reportBrokenSite.apply { visible = ::shouldShowWebCompatReporter },
                 BrowserMenuDivider(),
                 addToHomeScreenItem.apply { visible = ::canAddToHomescreen },
+                addAppToHomeScreenItem.apply { visible = ::canAddAppToHomescreen },
                 if (shouldShowTopSites) addRemoveTopSitesItem else null,
                 saveToCollectionItem,
-                if (FxNimbus.features.print.value().browserPrintEnabled) printPageItem else null,
+                if (FxNimbus.features.print.value().browserPrintEnabled &&
+                    !context.isAndroidAutomotiveAvailable()
+                ) {
+                    printPageItem
+                } else {
+                    null
+                },
                 BrowserMenuDivider(),
                 settingsItem,
                 if (shouldDeleteDataOnQuit) deleteDataOnQuit else null,
                 if (shouldUseBottomToolbar) BrowserMenuDivider() else null,
-                if (shouldUseBottomToolbar && shouldShowMenuToolbar) menuToolbar else null,
+                if (shouldUseBottomToolbar) menuToolbar else null,
             )
 
+        registerForIsBookmarkedUpdates()
+        registerForScreenReaderUpdates()
+
         menuItems
     }
 
@@ -491,6 +523,7 @@ open class DefaultToolbarMenu(
         isBookmarkedJob = lifecycleOwner.lifecycleScope.launch {
             isCurrentUrlBookmarked = bookmarksStorage
                 .getBookmarksWithUrl(newUrl)
+                .getOrDefault(listOf())
                 .any { it.url == newUrl }
         }
     }
