@@ -5,25 +5,32 @@
 package org.mozilla.fenix.customtabs
 
 import android.content.Context
+import androidx.annotation.ColorInt
 import mozilla.components.browser.toolbar.BrowserToolbar
 import mozilla.components.concept.toolbar.ScrollableToolbar
 import mozilla.components.feature.toolbar.ToolbarFeature
 import org.mozilla.fenix.components.toolbar.ToolbarIntegration
 import org.mozilla.fenix.components.toolbar.ToolbarMenu
+import org.mozilla.fenix.components.toolbar.interactor.BrowserToolbarInteractor
 
+@Suppress("LongParameterList")
 class CustomTabToolbarIntegration(
     context: Context,
     toolbar: BrowserToolbar,
     scrollableToolbar: ScrollableToolbar,
     toolbarMenu: ToolbarMenu,
-    sessionId: String,
+    interactor: BrowserToolbarInteractor,
+    customTabId: String,
     isPrivate: Boolean,
+    @ColorInt backgroundColor: Int? = null,
 ) : ToolbarIntegration(
     context = context,
     toolbar = toolbar,
     scrollableToolbar = scrollableToolbar,
     toolbarMenu = toolbarMenu,
-    sessionId = sessionId,
+    interactor = interactor,
+    customTabId = customTabId,
     isPrivate = isPrivate,
-    renderStyle = ToolbarFeature.RenderStyle.RegistrableDomain,
+    renderStyle = ToolbarFeature.RenderStyle.ColoredDomain,
+    backgroundColor = backgroundColor,
 )
