@@ -6,9 +6,15 @@ package org.mozilla.fenix.extension
 
 import android.content.Context
 import android.view.Gravity
+import android.view.LayoutInflater
+import android.view.View
+import android.widget.TextView
+import androidx.annotation.UiContext
 import androidx.annotation.VisibleForTesting
 import androidx.appcompat.app.AlertDialog
 import androidx.fragment.app.FragmentManager
+import androidx.navigation.NavController
+import com.google.android.material.dialog.MaterialAlertDialogBuilder
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.cancel
 import kotlinx.coroutines.flow.distinctUntilChanged
@@ -16,6 +22,7 @@ import kotlinx.coroutines.flow.mapNotNull
 import mozilla.components.browser.state.action.WebExtensionAction
 import mozilla.components.browser.state.state.extension.WebExtensionPromptRequest
 import mozilla.components.browser.state.store.BrowserStore
+import mozilla.components.concept.engine.webextension.PermissionPromptResponse
 import mozilla.components.concept.engine.webextension.WebExtensionInstallException
 import mozilla.components.feature.addons.Addon
 import mozilla.components.feature.addons.AddonManager
@@ -26,28 +33,27 @@ import mozilla.components.lib.state.ext.flowScoped
 import mozilla.components.support.base.feature.LifecycleAwareFeature
 import mozilla.components.support.ktx.android.content.appVersionName
 import mozilla.components.ui.widgets.withCenterAlignedButtons
+import org.mozilla.fenix.BuildConfig
 import org.mozilla.fenix.R
+import org.mozilla.fenix.addons.AddonsManagementFragmentDirections
 import org.mozilla.fenix.ext.components
+import org.mozilla.fenix.ext.pixelSizeFor
+import org.mozilla.fenix.settings.SupportUtils
 import org.mozilla.fenix.theme.ThemeManager
-import java.lang.ref.WeakReference
+import mozilla.components.feature.addons.R as addonsR
 
 /**
  * Feature implementation for handling [WebExtensionPromptRequest] and showing the respective UI.
  */
 class WebExtensionPromptFeature(
     private val store: BrowserStore,
-    private val context: Context,
-    private val addonManager: AddonManager = context.components.addonManager,
+    @param:UiContext private val context: Context,
     private val fragmentManager: FragmentManager,
+    private val onLinkClicked: (String, Boolean) -> Unit,
+    private val navController: NavController,
+    private val addonManager: AddonManager = context.components.addonManager,
 ) : LifecycleAwareFeature {
 
-    /**
-     * (optional) callback invoked when an add-on was updated due to an interaction with a
-     * [WebExtensionPromptRequest].
-     * Won't be needed after https://bugzilla.mozilla.org/show_bug.cgi?id=1858484.
-     */
-    var onAddonChanged: (Addon) -> Unit = {}
-
     /**
      * Whether or not an add-on installation is in progress.
      */
@@ -121,7 +127,13 @@ class WebExtensionPromptFeature(
         addon: Addon,
         promptRequest: WebExtensionPromptRequest.AfterInstallation.Permissions.Required,
     ) {
-        showPermissionDialog(addon = addon, promptRequest = promptRequest)
+        showPermissionDialog(
+            addon = addon,
+            promptRequest = promptRequest,
+            permissions = promptRequest.permissions,
+            origins = promptRequest.origins,
+            dataCollectionPermissions = promptRequest.dataCollectionPermissions,
+        )
     }
 
     @VisibleForTesting
@@ -129,11 +141,19 @@ class WebExtensionPromptFeature(
         addon: Addon,
         promptRequest: WebExtensionPromptRequest.AfterInstallation.Permissions.Optional,
     ) {
-        val shouldGrantWithoutPrompt = Addon.localizePermissions(promptRequest.permissions, context).isEmpty()
+        val shouldGrantWithoutPrompt = Addon.localizePermissions(
+            promptRequest.permissions,
+            context,
+        ).isEmpty() && promptRequest.origins.isEmpty() && promptRequest.dataCollectionPermissions.isEmpty()
 
         // If we don't have any promptable permissions, just proceed.
         if (shouldGrantWithoutPrompt) {
-            handlePermissions(promptRequest, granted = true)
+            handlePermissions(
+                promptRequest,
+                granted = true,
+                privateBrowsingAllowed = false,
+                technicalAndInteractionDataGranted = false,
+            )
             return
         }
 
@@ -141,59 +161,73 @@ class WebExtensionPromptFeature(
             addon = addon,
             promptRequest = promptRequest,
             forOptionalPermissions = true,
-            optionalPermissions = promptRequest.permissions,
+            permissions = promptRequest.permissions,
+            origins = promptRequest.origins,
+            dataCollectionPermissions = promptRequest.dataCollectionPermissions,
         )
     }
 
     @VisibleForTesting
     internal fun handleInstallationFailedRequest(
         exception: WebExtensionInstallException,
-    ) {
+    ): AlertDialog? {
         val addonName = exception.extensionName ?: ""
-        var title = context.getString(R.string.mozac_feature_addons_failed_to_install, "")
+        val appName = context.getString(R.string.app_name)
+
+        var title = context.getString(addonsR.string.mozac_feature_addons_cant_install_extension)
+        var url: String? = null
         val message = when (exception) {
             is WebExtensionInstallException.Blocklisted -> {
-                context.getString(R.string.mozac_feature_addons_blocklisted_1, addonName)
+                url = formatBlocklistURL(exception)
+                context.getString(addonsR.string.mozac_feature_addons_blocklisted_2, addonName, appName)
+            }
+
+            is WebExtensionInstallException.SoftBlocked -> {
+                url = formatBlocklistURL(exception)
+                context.getString(addonsR.string.mozac_feature_addons_soft_blocked_2, addonName, appName)
             }
 
             is WebExtensionInstallException.UserCancelled -> {
                 // We don't want to show an error message when users cancel installation.
-                return
+                return null
             }
 
             is WebExtensionInstallException.UnsupportedAddonType,
             is WebExtensionInstallException.Unknown,
             -> {
                 // Making sure we don't have a
-                // Title = Failed to install
+                // Title = Can't install extension
                 // Message = Failed to install $addonName
                 title = ""
                 if (addonName.isNotEmpty()) {
-                    context.getString(R.string.mozac_feature_addons_failed_to_install, addonName)
+                    context.getString(addonsR.string.mozac_feature_addons_failed_to_install, addonName)
                 } else {
-                    context.getString(R.string.mozac_feature_addons_extension_failed_to_install)
+                    context.getString(addonsR.string.mozac_feature_addons_extension_failed_to_install)
                 }
             }
 
+            is WebExtensionInstallException.AdminInstallOnly -> {
+                context.getString(addonsR.string.mozac_feature_addons_admin_install_only, addonName)
+            }
+
             is WebExtensionInstallException.NetworkFailure -> {
-                context.getString(R.string.mozac_feature_addons_extension_failed_to_install_network_error)
+                context.getString(addonsR.string.mozac_feature_addons_extension_failed_to_install_network_error)
             }
 
             is WebExtensionInstallException.CorruptFile -> {
-                context.getString(R.string.mozac_feature_addons_extension_failed_to_install_corrupt_error)
+                context.getString(addonsR.string.mozac_feature_addons_extension_failed_to_install_corrupt_error)
             }
 
             is WebExtensionInstallException.NotSigned -> {
                 context.getString(
-                    R.string.mozac_feature_addons_extension_failed_to_install_not_signed_error,
+                    addonsR.string.mozac_feature_addons_extension_failed_to_install_not_signed_error,
                 )
             }
 
             is WebExtensionInstallException.Incompatible -> {
-                val appName = context.getString(R.string.app_name)
                 val version = context.appVersionName
                 context.getString(
-                    R.string.mozac_feature_addons_failed_to_install_incompatible_error,
+                    addonsR.string.mozac_feature_addons_failed_to_install_incompatible_error,
                     addonName,
                     appName,
                     version,
@@ -201,12 +235,24 @@ class WebExtensionPromptFeature(
             }
         }
 
-        showDialog(
+        return showDialog(
             title = title,
             message = message,
+            url = url,
         )
     }
 
+    private fun formatBlocklistURL(exception: WebExtensionInstallException): String? {
+        var url: String? = exception.extensionId?.let { AMO_BLOCKED_PAGE_URL.format(it) }
+        // Only append the version if the URL is valid and we have a version. The AMO "blocked" page
+        // can be loaded without a version, but it's always better to specify a version if we have one.
+        if (url != null && exception.extensionVersion != null) {
+            url += "${exception.extensionVersion}/"
+        }
+
+        return url
+    }
+
     /**
      * Stops observing the selected session for incoming window requests.
      */
@@ -219,73 +265,120 @@ class WebExtensionPromptFeature(
         addon: Addon,
         promptRequest: WebExtensionPromptRequest.AfterInstallation.Permissions,
         forOptionalPermissions: Boolean = false,
-        optionalPermissions: List<String> = emptyList(),
-    ) {
+        permissions: List<String> = emptyList(),
+        origins: List<String> = emptyList(),
+        dataCollectionPermissions: List<String> = emptyList(),
+    ): PermissionsDialogFragment? {
         if (isInstallationInProgress || hasExistingPermissionDialogFragment()) {
-            return
+            return null
         }
 
         val dialog = PermissionsDialogFragment.newInstance(
             addon = addon,
             forOptionalPermissions = forOptionalPermissions,
-            optionalPermissions = optionalPermissions,
+            permissions = permissions,
+            origins = origins,
+            dataCollectionPermissions = dataCollectionPermissions,
             promptsStyling = AddonDialogFragment.PromptsStyling(
                 gravity = Gravity.BOTTOM,
                 shouldWidthMatchParent = true,
                 confirmButtonBackgroundColor = ThemeManager.resolveAttribute(
-                    R.attr.accent,
+                    R.attr.actionPrimary,
                     context,
                 ),
                 confirmButtonTextColor = ThemeManager.resolveAttribute(
-                    R.attr.textOnColorPrimary,
+                    R.attr.textActionPrimary,
+                    context,
+                ),
+                confirmButtonDisabledBackgroundColor = ThemeManager.resolveAttribute(
+                    R.attr.actionPrimaryDisabled,
+                    context,
+                ),
+                confirmButtonDisabledTextColor = ThemeManager.resolveAttribute(
+                    R.attr.textActionPrimaryDisabled,
+                    context,
+                ),
+                confirmButtonRadius = context.pixelSizeFor(R.dimen.tab_corner_radius).toFloat(),
+                learnMoreLinkTextColor = ThemeManager.resolveAttribute(
+                    R.attr.textAccent,
                     context,
                 ),
-                confirmButtonRadius =
-                (context.resources.getDimensionPixelSize(R.dimen.tab_corner_radius)).toFloat(),
             ),
-            onPositiveButtonClicked = { handlePermissions(promptRequest, granted = true) },
-            onNegativeButtonClicked = { handlePermissions(promptRequest, granted = false) },
-        )
-        dialog.show(
-            fragmentManager,
-            PERMISSIONS_DIALOG_FRAGMENT_TAG,
+            onPositiveButtonClicked = { _, privateBrowsingAllowed, technicalAndInteractionDataAllowed ->
+                handlePermissions(
+                    promptRequest,
+                    granted = true,
+                    privateBrowsingAllowed,
+                    technicalAndInteractionDataAllowed,
+                )
+            },
+            onNegativeButtonClicked = {
+                handlePermissions(
+                    promptRequest,
+                    granted = false,
+                    privateBrowsingAllowed = false,
+                    technicalAndInteractionDataGranted = false,
+                )
+            },
+            onLearnMoreClicked = {
+                onLinkClicked.invoke(
+                    SupportUtils.getSumoURLForTopic(
+                        context,
+                        SupportUtils.SumoTopic.EXTENSION_PERMISSIONS,
+                    ),
+                    false,
+                )
+            },
         )
+        dialog.show(fragmentManager, PERMISSIONS_DIALOG_FRAGMENT_TAG)
+
+        return dialog
     }
 
     private fun tryToReAttachButtonHandlersToPreviousDialog() {
         findPreviousPermissionDialogFragment()?.let { dialog ->
-            dialog.onPositiveButtonClicked = { addon ->
+            dialog.onPositiveButtonClicked = { addon, privateBrowsingAllowed, technicalAndInteractionDataGranted ->
                 store.state.webExtensionPromptRequest?.let { promptRequest ->
                     if (promptRequest is WebExtensionPromptRequest.AfterInstallation.Permissions &&
                         addon.id == promptRequest.extension.id
                     ) {
-                        handlePermissions(promptRequest, granted = true)
+                        handlePermissions(
+                            promptRequest,
+                            granted = true,
+                            privateBrowsingAllowed,
+                            technicalAndInteractionDataGranted,
+                        )
                     }
                 }
             }
             dialog.onNegativeButtonClicked = {
                 store.state.webExtensionPromptRequest?.let { promptRequest ->
                     if (promptRequest is WebExtensionPromptRequest.AfterInstallation.Permissions) {
-                        handlePermissions(promptRequest, granted = false)
+                        handlePermissions(
+                            promptRequest,
+                            granted = false,
+                            privateBrowsingAllowed = false,
+                            technicalAndInteractionDataGranted = false,
+                        )
+                    }
+                }
+            }
+            dialog.onLearnMoreClicked = {
+                store.state.webExtensionPromptRequest?.let { promptRequest ->
+                    if (promptRequest is WebExtensionPromptRequest.AfterInstallation.Permissions) {
+                        onLinkClicked.invoke(
+                            SupportUtils.getSumoURLForTopic(
+                                context,
+                                SupportUtils.SumoTopic.EXTENSION_PERMISSIONS,
+                            ),
+                            false,
+                        )
                     }
                 }
             }
         }
 
         findPreviousPostInstallationDialogFragment()?.let { dialog ->
-            dialog.onConfirmButtonClicked = { addon, allowInPrivateBrowsing ->
-                store.state.webExtensionPromptRequest?.let { promptRequest ->
-                    if (promptRequest is WebExtensionPromptRequest.AfterInstallation.PostInstallation &&
-                        addon.id == promptRequest.extension.id
-                    ) {
-                        handlePostInstallationButtonClicked(
-                            allowInPrivateBrowsing = allowInPrivateBrowsing,
-                            context = WeakReference(context),
-                            addon = addon,
-                        )
-                    }
-                }
-            }
             dialog.onDismissed = {
                 store.state.webExtensionPromptRequest?.let { _ ->
                     consumePromptRequest()
@@ -297,8 +390,23 @@ class WebExtensionPromptFeature(
     private fun handlePermissions(
         promptRequest: WebExtensionPromptRequest.AfterInstallation.Permissions,
         granted: Boolean,
+        privateBrowsingAllowed: Boolean,
+        technicalAndInteractionDataGranted: Boolean,
     ) {
-        promptRequest.onConfirm(granted)
+        when (promptRequest) {
+            is WebExtensionPromptRequest.AfterInstallation.Permissions.Optional -> {
+                promptRequest.onConfirm(granted)
+            }
+
+            is WebExtensionPromptRequest.AfterInstallation.Permissions.Required -> {
+                val response = PermissionPromptResponse(
+                    isPermissionsGranted = granted,
+                    isPrivateModeGranted = privateBrowsingAllowed,
+                    isTechnicalAndInteractionDataGranted = technicalAndInteractionDataGranted,
+                )
+                promptRequest.onConfirm(response)
+            }
+        }
         consumePromptRequest()
     }
 
@@ -326,78 +434,77 @@ class WebExtensionPromptFeature(
         ) as? AddonInstallationDialogFragment
     }
 
-    private fun showPostInstallationDialog(addon: Addon) {
+    @VisibleForTesting
+    internal fun showPostInstallationDialog(addon: Addon): AddonInstallationDialogFragment? {
         if (!isInstallationInProgress && !hasExistingAddonPostInstallationDialogFragment()) {
-            // Fragment may not be attached to the context anymore during onConfirmButtonClicked handling,
-            // but we still want to be able to process user selection of the 'allowInPrivateBrowsing' pref.
-            // This is a best-effort attempt to do so - retain a weak reference to the application context
-            // (to avoid a leak), which we attempt to use to access addonManager.
-            // See https://github.com/mozilla-mobile/fenix/issues/15816
-            val weakApplicationContext: WeakReference<Context> = WeakReference(context)
-
             val dialog = AddonInstallationDialogFragment.newInstance(
                 addon = addon,
                 promptsStyling = AddonDialogFragment.PromptsStyling(
                     gravity = Gravity.BOTTOM,
                     shouldWidthMatchParent = true,
                     confirmButtonBackgroundColor = ThemeManager.resolveAttribute(
-                        R.attr.accent,
+                        R.attr.actionPrimary,
                         context,
                     ),
                     confirmButtonTextColor = ThemeManager.resolveAttribute(
-                        R.attr.textOnColorPrimary,
+                        R.attr.textActionPrimary,
                         context,
                     ),
-                    confirmButtonRadius =
-                    (context.resources.getDimensionPixelSize(R.dimen.tab_corner_radius)).toFloat(),
+                    confirmButtonRadius = context.pixelSizeFor(R.dimen.tab_corner_radius).toFloat(),
                 ),
                 onDismissed = {
                     consumePromptRequest()
                 },
-                onConfirmButtonClicked = { _, allowInPrivateBrowsing ->
-                    handlePostInstallationButtonClicked(
-                        addon = addon,
-                        context = weakApplicationContext,
-                        allowInPrivateBrowsing = allowInPrivateBrowsing,
+                onConfirmButtonClicked = { _ ->
+                    consumePromptRequest()
+                },
+                onExtensionSettingsLinkClicked = {
+                    navController.navigate(
+                        AddonsManagementFragmentDirections.actionGlobalToInstalledAddonDetailsFragment(it),
                     )
+                    consumePromptRequest()
                 },
             )
             dialog.show(fragmentManager, POST_INSTALLATION_DIALOG_FRAGMENT_TAG)
-        }
-    }
 
-    private fun handlePostInstallationButtonClicked(
-        context: WeakReference<Context>,
-        allowInPrivateBrowsing: Boolean,
-        addon: Addon,
-    ) {
-        if (allowInPrivateBrowsing) {
-            context.get()?.components?.addonManager?.setAddonAllowedInPrivateBrowsing(
-                addon = addon,
-                allowed = true,
-                onSuccess = { updatedAddon ->
-                    onAddonChanged(updatedAddon)
-                },
-            )
+            return dialog
         }
-        consumePromptRequest()
+
+        return null
     }
 
     @VisibleForTesting
     internal fun showDialog(
         title: String,
         message: String,
-    ) {
+        url: String? = null,
+    ): AlertDialog {
         context.let {
-            AlertDialog.Builder(it)
+            var dialog: AlertDialog? = null
+            val inflater = LayoutInflater.from(it)
+            val view = inflater.inflate(R.layout.addon_installation_failed_dialog, null)
+            val messageView = view.findViewById<TextView>(R.id.message)
+            messageView.text = message
+
+            if (url != null) {
+                val linkView = view.findViewById<TextView>(R.id.link)
+                linkView.visibility = View.VISIBLE
+                linkView.setOnClickListener {
+                    onLinkClicked(url, true) // shouldOpenInBrowser
+                    dialog?.dismiss()
+                }
+            }
+
+            dialog = MaterialAlertDialogBuilder(it)
                 .setTitle(title)
                 .setPositiveButton(android.R.string.ok) { _, _ -> }
                 .setCancelable(false)
-                .setMessage(
-                    message,
-                )
-                .show()
+                .setView(view)
+                .create()
                 .withCenterAlignedButtons()
+            dialog.show()
+
+            return dialog
         }
     }
 
@@ -405,5 +512,6 @@ class WebExtensionPromptFeature(
         private const val PERMISSIONS_DIALOG_FRAGMENT_TAG = "ADDONS_PERMISSIONS_DIALOG_FRAGMENT"
         private const val POST_INSTALLATION_DIALOG_FRAGMENT_TAG =
             "ADDONS_INSTALLATION_DIALOG_FRAGMENT"
+        private const val AMO_BLOCKED_PAGE_URL = "${BuildConfig.AMO_BASE_URL}/android/blocked-addon/%s/"
     }
 }
