@@ -4,20 +4,12 @@
 
 package org.mozilla.fenix.home.sessioncontrol
 
-import android.annotation.SuppressLint
-import android.view.LayoutInflater
-import android.widget.EditText
 import androidx.annotation.VisibleForTesting
-import androidx.appcompat.app.AlertDialog
 import androidx.navigation.NavController
+import androidx.navigation.NavDirections
 import kotlinx.coroutines.CoroutineScope
-import kotlinx.coroutines.Dispatchers
 import kotlinx.coroutines.launch
-import mozilla.components.browser.state.search.SearchEngine
 import mozilla.components.browser.state.selector.getNormalOrPrivateTabs
-import mozilla.components.browser.state.state.availableSearchEngines
-import mozilla.components.browser.state.state.searchEngines
-import mozilla.components.browser.state.state.selectedOrDefaultSearchEngine
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.Engine
 import mozilla.components.concept.engine.prompt.ShareData
@@ -25,38 +17,30 @@ import mozilla.components.feature.session.SessionUseCases
 import mozilla.components.feature.tab.collections.TabCollection
 import mozilla.components.feature.tab.collections.ext.invoke
 import mozilla.components.feature.tabs.TabsUseCases
-import mozilla.components.feature.top.sites.TopSite
 import mozilla.components.service.nimbus.messaging.Message
-import mozilla.components.support.ktx.android.view.showKeyboard
-import mozilla.components.ui.widgets.withCenterAlignedButtons
 import mozilla.telemetry.glean.private.NoExtras
-import org.mozilla.fenix.BrowserDirection
 import org.mozilla.fenix.GleanMetrics.Collections
 import org.mozilla.fenix.GleanMetrics.HomeBookmarks
-import org.mozilla.fenix.GleanMetrics.HomeScreen
-import org.mozilla.fenix.GleanMetrics.Pings
-import org.mozilla.fenix.GleanMetrics.Pocket
 import org.mozilla.fenix.GleanMetrics.RecentTabs
-import org.mozilla.fenix.GleanMetrics.TopSites
 import org.mozilla.fenix.HomeActivity
 import org.mozilla.fenix.R
-import org.mozilla.fenix.browser.browsingmode.BrowsingMode
 import org.mozilla.fenix.collections.SaveCollectionStep
 import org.mozilla.fenix.components.AppStore
 import org.mozilla.fenix.components.TabCollectionStorage
+import org.mozilla.fenix.components.accounts.FenixFxAEntryPoint
 import org.mozilla.fenix.components.appstate.AppAction
 import org.mozilla.fenix.components.appstate.AppState
-import org.mozilla.fenix.components.metrics.MetricsUtils
-import org.mozilla.fenix.ext.components
+import org.mozilla.fenix.components.appstate.setup.checklist.ChecklistItem
+import org.mozilla.fenix.components.usecases.FenixBrowserUseCases
 import org.mozilla.fenix.ext.nav
 import org.mozilla.fenix.home.HomeFragment
 import org.mozilla.fenix.home.HomeFragmentDirections
 import org.mozilla.fenix.messaging.MessageController
 import org.mozilla.fenix.onboarding.WallpaperOnboardingDialogFragment.Companion.THUMBNAILS_SELECTION_COUNT
-import org.mozilla.fenix.settings.SupportUtils
 import org.mozilla.fenix.utils.Settings
 import org.mozilla.fenix.wallpapers.Wallpaper
 import org.mozilla.fenix.wallpapers.WallpaperState
+import java.lang.ref.WeakReference
 import mozilla.components.feature.tab.collections.Tab as ComponentTab
 
 /**
@@ -95,46 +79,11 @@ interface SessionControlController {
      */
     fun handleDeleteCollectionTapped(collection: TabCollection)
 
-    /**
-     * @see [TopSiteInteractor.onOpenInPrivateTabClicked]
-     */
-    fun handleOpenInPrivateTabClicked(topSite: TopSite)
-
-    /**
-     * @see [TopSiteInteractor.onRenameTopSiteClicked]
-     */
-    fun handleRenameTopSiteClicked(topSite: TopSite)
-
-    /**
-     * @see [TopSiteInteractor.onRemoveTopSiteClicked]
-     */
-    fun handleRemoveTopSiteClicked(topSite: TopSite)
-
     /**
      * @see [CollectionInteractor.onRenameCollectionTapped]
      */
     fun handleRenameCollectionTapped(collection: TabCollection)
 
-    /**
-     * @see [TopSiteInteractor.onSelectTopSite]
-     */
-    fun handleSelectTopSite(topSite: TopSite, position: Int)
-
-    /**
-     * @see [TopSiteInteractor.onSettingsClicked]
-     */
-    fun handleTopSiteSettingsClicked()
-
-    /**
-     * @see [TopSiteInteractor.onSponsorPrivacyClicked]
-     */
-    fun handleSponsorPrivacyClicked()
-
-    /**
-     * @see [TopSiteInteractor.onTopSiteLongClicked]
-     */
-    fun handleTopSiteLongClicked(topSite: TopSite)
-
     /**
      * @see [CollectionInteractor.onToggleCollectionExpanded]
      */
@@ -160,11 +109,6 @@ interface SessionControlController {
      */
     fun handleMessageClosed(message: Message)
 
-    /**
-     * @see [CustomizeHomeIteractor.openCustomizeHomePage]
-     */
-    fun handleCustomizeHomeTapped()
-
     /**
      * @see [WallpaperInteractor.showWallpapersOnboardingDialog]
      */
@@ -174,11 +118,52 @@ interface SessionControlController {
      * @see [SessionControlInteractor.reportSessionMetrics]
      */
     fun handleReportSessionMetrics(state: AppState)
+
+    /**
+     * @see [SetupChecklistInteractor.onChecklistItemClicked]
+     */
+    fun onChecklistItemClicked(item: ChecklistItem)
+
+    /**
+     * @see [SetupChecklistInteractor.onRemoveChecklistButtonClicked]
+     */
+    fun onRemoveChecklistButtonClicked()
+
+    /**
+     * Registers a [SessionControlControllerCallback] to handle callbacks that are implemented in the UI layer.
+     */
+    fun registerCallback(callback: SessionControlControllerCallback)
+
+    /**
+     * Unregisters the callback is typically called as part of cleaning up.
+     */
+    fun unregisterCallback()
+}
+
+/**
+ * Interface for [SessionControlController] callbacks that are implemented in the UI.
+ */
+interface SessionControlControllerCallback {
+
+    /**
+     * Callback to register the [TabCollectionStorage.Observer].
+     */
+    fun registerCollectionStorageObserver()
+
+    /**
+     * Callback to remove collection.
+     */
+    fun removeCollection(tabCollection: TabCollection)
+
+    /**
+     * Callback to show tab tray.
+     */
+    fun showTabTray()
 }
 
 @Suppress("TooManyFunctions", "LargeClass", "LongParameterList")
 class DefaultSessionControlController(
-    private val activity: HomeActivity,
+    private val activityRef: WeakReference<HomeActivity>,
     private val settings: Settings,
     private val engine: Engine,
     private val messageController: MessageController,
@@ -186,17 +171,30 @@ class DefaultSessionControlController(
     private val tabCollectionStorage: TabCollectionStorage,
     private val addTabUseCase: TabsUseCases.AddNewTabUseCase,
     private val restoreUseCase: TabsUseCases.RestoreUseCase,
-    private val reloadUrlUseCase: SessionUseCases.ReloadUrlUseCase,
     private val selectTabUseCase: TabsUseCases.SelectTabUseCase,
+    private val reloadUrlUseCase: SessionUseCases.ReloadUrlUseCase,
+    private val fenixBrowserUseCases: FenixBrowserUseCases,
     private val appStore: AppStore,
-    private val navController: NavController,
+    private val navControllerRef: WeakReference<NavController>,
     private val viewLifecycleScope: CoroutineScope,
-    private val registerCollectionStorageObserver: () -> Unit,
-    private val removeCollectionWithUndo: (tabCollection: TabCollection) -> Unit,
-    private val showUndoSnackbarForTopSite: (topSite: TopSite) -> Unit,
-    private val showTabTray: () -> Unit,
+    private val showAddSearchWidgetPrompt: () -> Unit,
 ) : SessionControlController {
 
+    private var callback: SessionControlControllerCallback? = null
+    private val activity: HomeActivity
+        get() = requireNotNull(activityRef.get())
+
+    private val navController: NavController
+        get() = requireNotNull(navControllerRef.get())
+
+    override fun registerCallback(callback: SessionControlControllerCallback) {
+        this.callback = callback
+    }
+
+    override fun unregisterCallback() {
+        this.callback = null
+    }
+
     override fun handleCollectionAddTabTapped(collection: TabCollection) {
         Collections.addTabButton.record(NoExtras())
         showCollectionCreationFragment(
@@ -211,15 +209,16 @@ class DefaultSessionControlController(
             engine,
             tab,
             onTabRestored = {
-                activity.openToBrowser(BrowserDirection.FromHome)
+                navController.navigate(R.id.browserFragment)
                 selectTabUseCase.invoke(it)
                 reloadUrlUseCase.invoke(it)
             },
             onFailure = {
-                activity.openToBrowserAndLoad(
+                navController.navigate(R.id.browserFragment)
+                fenixBrowserUseCases.loadUrlOrSearch(
                     searchTermOrURL = tab.url,
-                    newTab = true,
-                    from = BrowserDirection.FromHome,
+                    newTab = !settings.enableHomepageAsNewTab,
+                    private = appStore.state.mode.isPrivate,
                 )
             },
         )
@@ -233,11 +232,11 @@ class DefaultSessionControlController(
             engine,
             collection,
             onFailure = { url ->
-                addTabUseCase.invoke(url)
+                addTabUseCase(url)
             },
         )
 
-        showTabTray()
+        callback?.showTabTray()
         Collections.allTabsRestored.record(NoExtras())
     }
 
@@ -247,8 +246,14 @@ class DefaultSessionControlController(
     ) {
         Collections.tabRemoved.record(NoExtras())
 
-        if (collection.tabs.size == 1) {
-            removeCollectionWithUndo(collection)
+        // collection tabs hold a reference to the initial collection that could have changed since
+        val updatedCollection =
+            tabCollectionStorage.cachedTabCollections.firstOrNull {
+                it.id == collection.id
+            }
+
+        if (updatedCollection?.tabs?.size == 1) {
+            callback?.removeCollection(collection)
         } else {
             viewLifecycleScope.launch {
                 tabCollectionStorage.removeTabFromCollection(collection, tab)
@@ -265,77 +270,10 @@ class DefaultSessionControlController(
     }
 
     override fun handleDeleteCollectionTapped(collection: TabCollection) {
-        removeCollectionWithUndo(collection)
+        callback?.removeCollection(collection)
         Collections.removed.record(NoExtras())
     }
 
-    override fun handleOpenInPrivateTabClicked(topSite: TopSite) {
-        if (topSite is TopSite.Provided) {
-            TopSites.openContileInPrivateTab.record(NoExtras())
-        } else {
-            TopSites.openInPrivateTab.record(NoExtras())
-        }
-        with(activity) {
-            browsingModeManager.mode = BrowsingMode.Private
-            openToBrowserAndLoad(
-                searchTermOrURL = topSite.url,
-                newTab = true,
-                from = BrowserDirection.FromHome,
-            )
-        }
-    }
-
-    @SuppressLint("InflateParams")
-    override fun handleRenameTopSiteClicked(topSite: TopSite) {
-        activity.let {
-            val customLayout =
-                LayoutInflater.from(it).inflate(R.layout.top_sites_rename_dialog, null)
-            val topSiteLabelEditText: EditText =
-                customLayout.findViewById(R.id.top_site_title)
-            topSiteLabelEditText.setText(topSite.title)
-
-            AlertDialog.Builder(it).apply {
-                setTitle(R.string.rename_top_site)
-                setView(customLayout)
-                setPositiveButton(R.string.top_sites_rename_dialog_ok) { dialog, _ ->
-                    viewLifecycleScope.launch(Dispatchers.IO) {
-                        with(activity.components.useCases.topSitesUseCase) {
-                            updateTopSites(
-                                topSite,
-                                topSiteLabelEditText.text.toString(),
-                                topSite.url,
-                            )
-                        }
-                    }
-                    dialog.dismiss()
-                }
-                setNegativeButton(R.string.top_sites_rename_dialog_cancel) { dialog, _ ->
-                    dialog.cancel()
-                }
-            }.show().withCenterAlignedButtons().also {
-                topSiteLabelEditText.setSelection(0, topSiteLabelEditText.text.length)
-                topSiteLabelEditText.showKeyboard()
-            }
-        }
-    }
-
-    override fun handleRemoveTopSiteClicked(topSite: TopSite) {
-        TopSites.remove.record(NoExtras())
-        when (topSite.url) {
-            SupportUtils.POCKET_TRENDING_URL -> Pocket.pocketTopSiteRemoved.record(NoExtras())
-            SupportUtils.GOOGLE_URL -> TopSites.googleTopSiteRemoved.record(NoExtras())
-            SupportUtils.BAIDU_URL -> TopSites.baiduTopSiteRemoved.record(NoExtras())
-        }
-
-        viewLifecycleScope.launch(Dispatchers.IO) {
-            with(activity.components.useCases.topSitesUseCase) {
-                removeTopSites(topSite)
-            }
-        }
-
-        showUndoSnackbarForTopSite(topSite)
-    }
-
     override fun handleRenameCollectionTapped(collection: TabCollection) {
         showCollectionCreationFragment(
             step = SaveCollectionStep.RenameCollection,
@@ -344,127 +282,8 @@ class DefaultSessionControlController(
         Collections.renameButton.record(NoExtras())
     }
 
-    override fun handleSelectTopSite(topSite: TopSite, position: Int) {
-        when (topSite) {
-            is TopSite.Default -> TopSites.openDefault.record(NoExtras())
-            is TopSite.Frecent -> TopSites.openFrecency.record(NoExtras())
-            is TopSite.Pinned -> TopSites.openPinned.record(NoExtras())
-            is TopSite.Provided -> TopSites.openContileTopSite.record(NoExtras()).also {
-                submitTopSitesImpressionPing(topSite, position)
-            }
-        }
-
-        when (topSite.url) {
-            SupportUtils.GOOGLE_URL -> TopSites.openGoogleSearchAttribution.record(NoExtras())
-            SupportUtils.BAIDU_URL -> TopSites.openBaiduSearchAttribution.record(NoExtras())
-            SupportUtils.POCKET_TRENDING_URL -> Pocket.pocketTopSiteClicked.record(NoExtras())
-        }
-
-        val availableEngines: List<SearchEngine> = getAvailableSearchEngines()
-        val searchAccessPoint = MetricsUtils.Source.TOPSITE
-
-        availableEngines.firstOrNull { engine ->
-            engine.resultUrls.firstOrNull { it.contains(topSite.url) } != null
-        }?.let { searchEngine ->
-            MetricsUtils.recordSearchMetrics(
-                searchEngine,
-                searchEngine == store.state.search.selectedOrDefaultSearchEngine,
-                searchAccessPoint,
-            )
-        }
-
-        val existingTabForUrl = when (topSite) {
-            is TopSite.Frecent, is TopSite.Pinned -> {
-                store.state.tabs.firstOrNull { topSite.url == it.content.url }
-            }
-
-            else -> null
-        }
-
-        if (existingTabForUrl == null) {
-            TopSites.openInNewTab.record(NoExtras())
-
-            val tabId = addTabUseCase.invoke(
-                url = appendSearchAttributionToUrlIfNeeded(topSite.url),
-                selectTab = true,
-                startLoading = true,
-            )
-
-            if (settings.openNextTabInDesktopMode) {
-                activity.handleRequestDesktopMode(tabId)
-            }
-        } else {
-            selectTabUseCase.invoke(existingTabForUrl.id)
-        }
-        navController.navigate(R.id.browserFragment)
-    }
-
-    @VisibleForTesting
-    internal fun submitTopSitesImpressionPing(topSite: TopSite.Provided, position: Int) {
-        TopSites.contileClick.record(
-            TopSites.ContileClickExtra(
-                position = position + 1,
-                source = "newtab",
-            ),
-        )
-
-        topSite.id?.let { TopSites.contileTileId.set(it) }
-        topSite.title?.let { TopSites.contileAdvertiser.set(it.lowercase()) }
-        TopSites.contileReportingUrl.set(topSite.clickUrl)
-        Pings.topsitesImpression.submit()
-    }
-
-    override fun handleTopSiteSettingsClicked() {
-        TopSites.contileSettings.record(NoExtras())
-        navController.nav(
-            R.id.homeFragment,
-            HomeFragmentDirections.actionGlobalHomeSettingsFragment(),
-        )
-    }
-
-    override fun handleSponsorPrivacyClicked() {
-        TopSites.contileSponsorsAndPrivacy.record(NoExtras())
-        activity.openToBrowserAndLoad(
-            searchTermOrURL = SupportUtils.getGenericSumoURLForTopic(SupportUtils.SumoTopic.SPONSOR_PRIVACY),
-            newTab = true,
-            from = BrowserDirection.FromHome,
-        )
-    }
-
-    override fun handleTopSiteLongClicked(topSite: TopSite) {
-        TopSites.longPress.record(TopSites.LongPressExtra(topSite.type))
-    }
-
-    @VisibleForTesting
-    internal fun getAvailableSearchEngines() =
-        activity.components.core.store.state.search.searchEngines +
-            activity.components.core.store.state.search.availableSearchEngines
-
-    /**
-     * Append a search attribution query to any provided search engine URL based on the
-     * user's current region.
-     */
-    private fun appendSearchAttributionToUrlIfNeeded(url: String): String {
-        if (url == SupportUtils.GOOGLE_URL) {
-            store.state.search.region?.let { region ->
-                return when (region.current) {
-                    "US" -> SupportUtils.GOOGLE_US_URL
-                    else -> SupportUtils.GOOGLE_XX_URL
-                }
-            }
-        }
-
-        return url
-    }
-
-    override fun handleCustomizeHomeTapped() {
-        val directions = HomeFragmentDirections.actionGlobalHomeSettingsFragment()
-        navController.nav(navController.currentDestination?.id, directions)
-        HomeScreen.customizeHomeClicked.record(NoExtras())
-    }
-
     override fun handleShowWallpapersOnboardingDialog(state: WallpaperState): Boolean {
-        return if (activity.browsingModeManager.mode.isPrivate) {
+        return if (appStore.state.mode.isPrivate) {
             false
         } else {
             state.availableWallpapers.filter { wallpaper ->
@@ -487,9 +306,15 @@ class DefaultSessionControlController(
     }
 
     private fun showTabTrayCollectionCreation() {
-        val directions = HomeFragmentDirections.actionGlobalTabsTrayFragment(
-            enterMultiselect = true,
-        )
+        val directions = if (settings.tabManagerEnhancementsEnabled) {
+            HomeFragmentDirections.actionGlobalTabManagementFragment(
+                enterMultiselect = true,
+            )
+        } else {
+            HomeFragmentDirections.actionGlobalTabsTrayFragment(
+                enterMultiselect = true,
+            )
+        }
         navController.nav(R.id.homeFragment, directions)
     }
 
@@ -501,7 +326,7 @@ class DefaultSessionControlController(
         if (navController.currentDestination?.id == R.id.collectionCreationFragment) return
 
         // Only register the observer right before moving to collection creation
-        registerCollectionStorageObserver()
+        callback?.registerCollectionStorageObserver()
 
         val tabIds = store.state
             .getNormalOrPrivateTabs(private = activity.browsingModeManager.mode.isPrivate)
@@ -553,4 +378,39 @@ class DefaultSessionControlController(
 
         HomeBookmarks.bookmarksCount.set(state.bookmarks.size.toLong())
     }
+
+    override fun onChecklistItemClicked(item: ChecklistItem) {
+        if (item is ChecklistItem.Task) {
+            // The navigation actions are required to be called on the main thread.
+            navigationActionFor(item)
+        }
+
+        appStore.dispatch(AppAction.SetupChecklistAction.ChecklistItemClicked(item))
+    }
+
+    @VisibleForTesting
+    internal fun navigationActionFor(item: ChecklistItem.Task) = when (item.type) {
+        ChecklistItem.Task.Type.SET_AS_DEFAULT -> activity.showSetDefaultBrowserPrompt()
+
+        ChecklistItem.Task.Type.SIGN_IN ->
+            navigateTo(HomeFragmentDirections.actionGlobalTurnOnSync(FenixFxAEntryPoint.NewUserOnboarding))
+
+        ChecklistItem.Task.Type.SELECT_THEME ->
+            navigateTo(HomeFragmentDirections.actionGlobalCustomizationFragment())
+
+        ChecklistItem.Task.Type.CHANGE_TOOLBAR_PLACEMENT ->
+            navigateTo(HomeFragmentDirections.actionGlobalCustomizationFragment())
+
+        ChecklistItem.Task.Type.INSTALL_SEARCH_WIDGET -> showAddSearchWidgetPrompt()
+
+        ChecklistItem.Task.Type.EXPLORE_EXTENSION ->
+            navigateTo(HomeFragmentDirections.actionGlobalAddonsManagementFragment())
+    }
+
+    private fun navigateTo(directions: NavDirections) =
+        navController.nav(R.id.homeFragment, directions)
+
+    override fun onRemoveChecklistButtonClicked() {
+        appStore.dispatch(AppAction.SetupChecklistAction.Closed)
+    }
 }
