@@ -4,72 +4,113 @@
 
 package org.mozilla.fenix.home.topsites
 
-import androidx.compose.foundation.ExperimentalFoundationApi
 import androidx.compose.foundation.Image
 import androidx.compose.foundation.background
 import androidx.compose.foundation.combinedClickable
 import androidx.compose.foundation.interaction.MutableInteractionSource
-import androidx.compose.foundation.isSystemInDarkTheme
 import androidx.compose.foundation.layout.Arrangement
 import androidx.compose.foundation.layout.Box
 import androidx.compose.foundation.layout.Column
 import androidx.compose.foundation.layout.Row
 import androidx.compose.foundation.layout.Spacer
 import androidx.compose.foundation.layout.defaultMinSize
+import androidx.compose.foundation.layout.fillMaxSize
 import androidx.compose.foundation.layout.fillMaxWidth
 import androidx.compose.foundation.layout.height
 import androidx.compose.foundation.layout.padding
 import androidx.compose.foundation.layout.size
 import androidx.compose.foundation.layout.width
-import androidx.compose.foundation.pager.HorizontalPager
-import androidx.compose.foundation.pager.rememberPagerState
+import androidx.compose.foundation.layout.wrapContentSize
+import androidx.compose.foundation.shape.CircleShape
 import androidx.compose.foundation.shape.RoundedCornerShape
-import androidx.compose.material.Card
-import androidx.compose.material.Surface
-import androidx.compose.material.Text
+import androidx.compose.material3.Card
+import androidx.compose.material3.CardDefaults
+import androidx.compose.material3.Surface
+import androidx.compose.material3.Text
 import androidx.compose.runtime.Composable
 import androidx.compose.runtime.LaunchedEffect
 import androidx.compose.runtime.getValue
 import androidx.compose.runtime.mutableStateOf
 import androidx.compose.runtime.remember
 import androidx.compose.runtime.setValue
+import androidx.compose.ui.AbsoluteAlignment
 import androidx.compose.ui.Alignment
-import androidx.compose.ui.ExperimentalComposeUiApi
 import androidx.compose.ui.Modifier
-import androidx.compose.ui.draw.alpha
 import androidx.compose.ui.graphics.Color
-import androidx.compose.ui.graphics.painter.Painter
-import androidx.compose.ui.layout.ContentScale
+import androidx.compose.ui.graphics.ColorFilter
 import androidx.compose.ui.platform.testTag
 import androidx.compose.ui.res.painterResource
 import androidx.compose.ui.res.stringResource
 import androidx.compose.ui.semantics.semantics
+import androidx.compose.ui.semantics.testTag
 import androidx.compose.ui.semantics.testTagsAsResourceId
 import androidx.compose.ui.text.style.TextAlign
 import androidx.compose.ui.text.style.TextOverflow
+import androidx.compose.ui.tooling.preview.PreviewLightDark
 import androidx.compose.ui.unit.dp
 import androidx.compose.ui.unit.sp
+import mozilla.components.compose.base.modifier.rightClickable
 import mozilla.components.feature.top.sites.TopSite
-import org.mozilla.fenix.GleanMetrics.Pings
 import org.mozilla.fenix.R
 import org.mozilla.fenix.compose.ContextualMenu
 import org.mozilla.fenix.compose.Favicon
 import org.mozilla.fenix.compose.MenuItem
-import org.mozilla.fenix.compose.PagerIndicator
-import org.mozilla.fenix.compose.annotation.LightDarkPreview
-import org.mozilla.fenix.settings.SupportUtils
+import org.mozilla.fenix.home.fake.FakeHomepagePreview
+import org.mozilla.fenix.home.topsites.TopSitesTestTag.TOP_SITE_CARD_FAVICON
+import org.mozilla.fenix.home.topsites.interactor.TopSiteInteractor
 import org.mozilla.fenix.theme.FirefoxTheme
 import org.mozilla.fenix.wallpapers.WallpaperState
-import kotlin.math.ceil
-import org.mozilla.fenix.GleanMetrics.TopSites as TopSitesMetrics
 
-private const val TOP_SITES_PER_PAGE = 8
+/**
+ * The size of a top site item.
+ */
+const val TOP_SITES_ITEM_SIZE = 84
+
+private const val TOP_SITES_TO_SHOW = 8
 private const val TOP_SITES_PER_ROW = 4
-private const val TOP_SITES_ITEM_SIZE = 84
 private const val TOP_SITES_ROW_WIDTH = TOP_SITES_PER_ROW * TOP_SITES_ITEM_SIZE
 private const val TOP_SITES_FAVICON_CARD_SIZE = 60
 private const val TOP_SITES_FAVICON_SIZE = 36
 
+// These intentionally do not reference the design system because they should always be the same colors
+private const val PIN_BACKGROUND_COLOR = 0xFFF0F0F4
+private const val PIN_COLOR = 0xFF15141A
+
+/**
+ * A list of top sites.
+ *
+ * @param topSites List of [TopSite] to display.
+ * @param topSiteColors The color set defined by [TopSiteColors] used to style a top site.
+ * @param interactor The interactor which handles user actions with the widget.
+ * @param onTopSitesItemBound Invoked during the composition of a top site item.
+ */
+@Composable
+fun TopSites(
+    topSites: List<TopSite>,
+    topSiteColors: TopSiteColors = TopSiteColors.colors(),
+    interactor: TopSiteInteractor,
+    onTopSitesItemBound: () -> Unit,
+) {
+    TopSites(
+        topSites = topSites,
+        topSiteColors = topSiteColors,
+        onTopSiteClick = { topSite ->
+            interactor.onSelectTopSite(
+                topSite = topSite,
+                position = topSites.indexOf(topSite),
+            )
+        },
+        onTopSiteLongClick = interactor::onTopSiteLongClicked,
+        onTopSiteImpression = interactor::onTopSiteImpression,
+        onOpenInPrivateTabClicked = interactor::onOpenInPrivateTabClicked,
+        onEditTopSiteClicked = interactor::onEditTopSiteClicked,
+        onRemoveTopSiteClicked = interactor::onRemoveTopSiteClicked,
+        onSettingsClicked = interactor::onSettingsClicked,
+        onSponsorPrivacyClicked = interactor::onSponsorPrivacyClicked,
+        onTopSitesItemBound = onTopSitesItemBound,
+    )
+}
+
 /**
  * A list of top sites.
  *
@@ -77,31 +118,33 @@ private const val TOP_SITES_FAVICON_SIZE = 36
  * @param topSiteColors The color set defined by [TopSiteColors] used to style a top site.
  * @param onTopSiteClick Invoked when the user clicks on a top site.
  * @param onTopSiteLongClick Invoked when the user long clicks on a top site.
+ * @param onTopSiteImpression Invoked when the user sees a provided top site.
  * @param onOpenInPrivateTabClicked Invoked when the user clicks on the "Open in private tab"
  * menu item.
- * @param onRenameTopSiteClicked Invoked when the user clicks on the "Rename" menu item.
+ * @param onEditTopSiteClicked Invoked when the user clicks on the "Edit" menu item.
  * @param onRemoveTopSiteClicked Invoked when the user clicks on the "Remove" menu item.
  * @param onSettingsClicked Invoked when the user clicks on the "Settings" menu item.
  * @param onSponsorPrivacyClicked Invoked when the user clicks on the "Our sponsors & your privacy"
  * menu item.
  * @param onTopSitesItemBound Invoked during the composition of a top site item.
  */
-@OptIn(ExperimentalFoundationApi::class, ExperimentalComposeUiApi::class)
 @Composable
-@Suppress("LongParameterList", "LongMethod")
+@Suppress("LongParameterList")
 fun TopSites(
     topSites: List<TopSite>,
     topSiteColors: TopSiteColors = TopSiteColors.colors(),
     onTopSiteClick: (TopSite) -> Unit,
     onTopSiteLongClick: (TopSite) -> Unit,
+    onTopSiteImpression: (TopSite.Provided, Int) -> Unit,
     onOpenInPrivateTabClicked: (topSite: TopSite) -> Unit,
-    onRenameTopSiteClicked: (topSite: TopSite) -> Unit,
+    onEditTopSiteClicked: (topSite: TopSite) -> Unit,
     onRemoveTopSiteClicked: (topSite: TopSite) -> Unit,
     onSettingsClicked: () -> Unit,
     onSponsorPrivacyClicked: () -> Unit,
     onTopSitesItemBound: () -> Unit,
 ) {
-    val pageCount = ceil((topSites.size.toDouble() / TOP_SITES_PER_PAGE)).toInt()
+    val topSitesToShow = topSites.take(TOP_SITES_TO_SHOW).chunked(TOP_SITES_PER_ROW)
+    val needsInvisibleRow = topSites.size <= (TOP_SITES_TO_SHOW - TOP_SITES_PER_ROW)
 
     Column(
         modifier = Modifier
@@ -109,70 +152,74 @@ fun TopSites(
             .semantics {
                 testTagsAsResourceId = true
             }
-            .testTag(TopSitesTestTag.topSites),
+            .testTag(TopSitesTestTag.TOP_SITES),
         horizontalAlignment = Alignment.CenterHorizontally,
     ) {
-        val pagerState = rememberPagerState(
-            pageCount = { pageCount },
-        )
-
         Box(
             modifier = Modifier.fillMaxWidth(),
             contentAlignment = Alignment.Center,
         ) {
-            HorizontalPager(
-                state = pagerState,
-            ) { page ->
-                Column(
-                    modifier = Modifier.fillMaxWidth(),
-                    horizontalAlignment = Alignment.CenterHorizontally,
-                ) {
-                    val topSitesWindows = topSites.windowed(
-                        size = TOP_SITES_PER_PAGE,
-                        step = TOP_SITES_PER_PAGE,
-                        partialWindows = true,
-                    )[page].chunked(TOP_SITES_PER_ROW)
-
-                    for (items in topSitesWindows) {
-                        Row(modifier = Modifier.defaultMinSize(minWidth = TOP_SITES_ROW_WIDTH.dp)) {
-                            items.forEachIndexed { position, topSite ->
-                                TopSiteItem(
+            Column(
+                modifier = Modifier.fillMaxWidth(),
+                horizontalAlignment = Alignment.CenterHorizontally,
+            ) {
+                for (items in topSitesToShow) {
+                    Row(modifier = Modifier.defaultMinSize(minWidth = TOP_SITES_ROW_WIDTH.dp)) {
+                        items.forEachIndexed { position, topSite ->
+                            TopSiteItem(
+                                topSite = topSite,
+                                menuItems = getMenuItems(
                                     topSite = topSite,
-                                    menuItems = getMenuItems(
-                                        topSite = topSite,
-                                        onOpenInPrivateTabClicked = onOpenInPrivateTabClicked,
-                                        onRenameTopSiteClicked = onRenameTopSiteClicked,
-                                        onRemoveTopSiteClicked = onRemoveTopSiteClicked,
-                                        onSettingsClicked = onSettingsClicked,
-                                        onSponsorPrivacyClicked = onSponsorPrivacyClicked,
-                                    ),
-                                    position = position,
-                                    topSiteColors = topSiteColors,
-                                    onTopSiteClick = { item -> onTopSiteClick(item) },
-                                    onTopSiteLongClick = onTopSiteLongClick,
-                                    onTopSitesItemBound = onTopSitesItemBound,
-                                )
-                            }
+                                    onOpenInPrivateTabClicked = onOpenInPrivateTabClicked,
+                                    onEditTopSiteClicked = onEditTopSiteClicked,
+                                    onRemoveTopSiteClicked = onRemoveTopSiteClicked,
+                                    onSettingsClicked = onSettingsClicked,
+                                    onSponsorPrivacyClicked = onSponsorPrivacyClicked,
+                                ),
+                                position = position,
+                                topSiteColors = topSiteColors,
+                                onTopSiteClick = { item -> onTopSiteClick(item) },
+                                onTopSiteLongClick = onTopSiteLongClick,
+                                onTopSiteImpression = onTopSiteImpression,
+                                onTopSitesItemBound = onTopSitesItemBound,
+                            )
                         }
+                    }
 
+                    if (items != topSitesToShow.last()) {
                         Spacer(modifier = Modifier.height(12.dp))
                     }
                 }
+
+                if (needsInvisibleRow) {
+                    InvisibleRow()
+                }
             }
         }
-
-        if (pagerState.pageCount > 1) {
-            Spacer(modifier = Modifier.height(8.dp))
-
-            PagerIndicator(
-                pagerState = pagerState,
-                modifier = Modifier.padding(horizontal = 16.dp),
-                spacing = 4.dp,
-            )
-        }
     }
 }
 
+/**
+ * Workaround for when the second pager page only has one row, and the pager shrinks to fit. This
+ * invisible row mimics top sites items to match the correct height.
+ */
+@Composable
+private fun InvisibleRow() {
+    Spacer(modifier = Modifier.height(4.dp + TOP_SITES_FAVICON_CARD_SIZE.dp + 6.dp))
+
+    Text(
+        text = "",
+        style = FirefoxTheme.typography.caption,
+    )
+
+    Text(
+        text = "",
+        fontSize = 10.sp,
+    )
+
+    Spacer(modifier = Modifier.height(12.dp))
+}
+
 /**
  * Represents the colors used by top sites.
  */
@@ -209,20 +256,10 @@ data class TopSiteColors(
                 Color(textColor) to Color(textColor)
             }
 
-            var faviconCardBackgroundColor = FirefoxTheme.colors.layer2
-
-            wallpaperState.composeRunIfWallpaperCardColorsAreAvailable { cardColorLight, cardColorDark ->
-                faviconCardBackgroundColor = if (isSystemInDarkTheme()) {
-                    cardColorDark
-                } else {
-                    cardColorLight
-                }
-            }
-
             return TopSiteColors(
                 titleTextColor = titleTextColor,
                 sponsoredTextColor = sponsoredTextColor,
-                faviconCardBackgroundColor = faviconCardBackgroundColor,
+                faviconCardBackgroundColor = FirefoxTheme.colors.layer2,
             )
         }
     }
@@ -237,28 +274,37 @@ data class TopSiteColors(
  * @param topSiteColors The color set defined by [TopSiteColors] used to style a top site.
  * @param onTopSiteClick Invoked when the user clicks on a top site.
  * @param onTopSiteLongClick Invoked when the user long clicks on a top site.
+ * @param onTopSiteImpression Invoked when the user sees a provided top site.
  * @param onTopSitesItemBound Invoked during the composition of a top site item.
  */
-@Suppress("LongMethod")
-@OptIn(ExperimentalFoundationApi::class, ExperimentalComposeUiApi::class)
+@Suppress(
+    "LongMethod",
+    "LongParameterList",
+    "Deprecation",
+) // https://bugzilla.mozilla.org/show_bug.cgi?id=1927713
 @Composable
-private fun TopSiteItem(
+fun TopSiteItem(
     topSite: TopSite,
     menuItems: List<MenuItem>,
     position: Int,
     topSiteColors: TopSiteColors,
     onTopSiteClick: (TopSite) -> Unit,
     onTopSiteLongClick: (TopSite) -> Unit,
+    onTopSiteImpression: (TopSite.Provided, Int) -> Unit,
     onTopSitesItemBound: () -> Unit,
 ) {
     var menuExpanded by remember { mutableStateOf(false) }
+    val onLongClick = {
+        onTopSiteLongClick(topSite)
+        menuExpanded = true
+    }
 
     Box(
         modifier = Modifier
             .semantics {
                 testTagsAsResourceId = true
             }
-            .testTag(TopSitesTestTag.topSiteItemRoot),
+            .testTag(TopSitesTestTag.TOP_SITE_ITEM_ROOT),
     ) {
         Column(
             modifier = Modifier
@@ -266,43 +312,57 @@ private fun TopSiteItem(
                     interactionSource = remember { MutableInteractionSource() },
                     indication = null,
                     onClick = { onTopSiteClick(topSite) },
-                    onLongClick = {
-                        onTopSiteLongClick(topSite)
-                        menuExpanded = true
-                    },
+                    onLongClick = onLongClick,
+                )
+                .rightClickable(
+                    interactionSource = remember { MutableInteractionSource() },
+                    indication = null,
+                    onRightClick = onLongClick,
                 )
                 .width(TOP_SITES_ITEM_SIZE.dp),
             horizontalAlignment = Alignment.CenterHorizontally,
         ) {
             Spacer(modifier = Modifier.height(4.dp))
 
-            TopSiteFaviconCard(
-                topSite = topSite,
-                backgroundColor = topSiteColors.faviconCardBackgroundColor,
-            )
+            Box(
+                modifier = Modifier.wrapContentSize(),
+                contentAlignment = AbsoluteAlignment.TopLeft,
+            ) {
+                TopSiteFaviconCard(
+                    topSite = topSite,
+                    backgroundColor = topSiteColors.faviconCardBackgroundColor,
+                )
+
+                if (topSite is TopSite.Pinned || topSite is TopSite.Default) {
+                    Box(
+                        modifier = Modifier
+                            .size(16.dp)
+                            .background(color = Color(PIN_BACKGROUND_COLOR), shape = CircleShape),
+                        contentAlignment = Alignment.Center,
+                    ) {
+                        Image(
+                            modifier = Modifier.size(10.dp),
+                            painter = painterResource(id = R.drawable.ic_new_pin),
+                            colorFilter = ColorFilter.tint(Color(PIN_COLOR)),
+                            contentDescription = null,
+                        )
+                    }
+                }
+            }
 
             Spacer(modifier = Modifier.height(6.dp))
 
             Row(
                 modifier = Modifier.width(TOP_SITES_ITEM_SIZE.dp),
-                horizontalArrangement = Arrangement.Center,
+                horizontalArrangement = Arrangement.Absolute.Center,
                 verticalAlignment = Alignment.CenterVertically,
             ) {
-                if (topSite is TopSite.Pinned || topSite is TopSite.Default) {
-                    Image(
-                        painter = painterResource(id = R.drawable.ic_new_pin),
-                        contentDescription = null,
-                    )
-
-                    Spacer(modifier = Modifier.width(2.dp))
-                }
-
                 Text(
                     modifier = Modifier
                         .semantics {
                             testTagsAsResourceId = true
                         }
-                        .testTag(TopSitesTestTag.topSiteTitle),
+                        .testTag(TopSitesTestTag.TOP_SITE_TITLE),
                     text = topSite.title ?: topSite.url,
                     color = topSiteColors.titleTextColor,
                     overflow = TextOverflow.Ellipsis,
@@ -312,10 +372,9 @@ private fun TopSiteItem(
             }
 
             Text(
-                text = stringResource(id = R.string.top_sites_sponsored_label),
+                text = if (topSite is TopSite.Provided) stringResource(id = R.string.top_sites_sponsored_label) else "",
                 modifier = Modifier
-                    .width(TOP_SITES_ITEM_SIZE.dp)
-                    .alpha(alpha = if (topSite is TopSite.Provided) 1f else 0f),
+                    .width(TOP_SITES_ITEM_SIZE.dp),
                 color = topSiteColors.sponsoredTextColor,
                 fontSize = 10.sp,
                 textAlign = TextAlign.Center,
@@ -326,7 +385,7 @@ private fun TopSiteItem(
 
         ContextualMenu(
             modifier = Modifier
-                .testTag(TopSitesTestTag.topSiteContextualMenu),
+                .testTag(TopSitesTestTag.TOP_SITE_CONTEXTUAL_MENU),
             menuItems = menuItems,
             showMenu = menuExpanded,
             onDismissRequest = { menuExpanded = false },
@@ -334,7 +393,7 @@ private fun TopSiteItem(
 
         if (topSite is TopSite.Provided) {
             LaunchedEffect(topSite) {
-                submitTopSitesImpressionPing(topSite = topSite, position = position)
+                onTopSiteImpression(topSite, position)
             }
         }
     }
@@ -356,77 +415,75 @@ private fun TopSiteFaviconCard(
     backgroundColor: Color,
 ) {
     Card(
-        modifier = Modifier.size(TOP_SITES_FAVICON_CARD_SIZE.dp),
-        shape = RoundedCornerShape(8.dp),
-        backgroundColor = backgroundColor,
-        elevation = 6.dp,
+        modifier = Modifier
+            .semantics {
+                testTagsAsResourceId = true
+                testTag = TOP_SITE_CARD_FAVICON
+            }
+            .size(TOP_SITES_FAVICON_CARD_SIZE.dp),
+        shape = CircleShape,
+        colors = CardDefaults.cardColors(containerColor = backgroundColor),
+        elevation = CardDefaults.cardElevation(defaultElevation = 6.dp),
     ) {
-        Box(contentAlignment = Alignment.Center) {
+        Box(
+            modifier = Modifier.fillMaxSize(),
+            contentAlignment = Alignment.Center,
+        ) {
             Surface(
                 modifier = Modifier.size(TOP_SITES_FAVICON_SIZE.dp),
                 color = backgroundColor,
                 shape = RoundedCornerShape(4.dp),
             ) {
-                if (topSite is TopSite.Provided) {
-                    TopSiteFavicon(topSite.url, topSite.imageUrl)
-                } else {
-                    TopSiteFavicon(topSite.url)
-                }
+                TopSiteFavicon(topSite = topSite)
             }
         }
     }
 }
 
 @Composable
-private fun FaviconImage(painter: Painter) {
-    Image(
-        painter = painter,
-        contentDescription = null,
-        modifier = Modifier.size(TOP_SITES_FAVICON_SIZE.dp),
-        contentScale = ContentScale.Crop,
-    )
-}
+private fun TopSiteFavicon(topSite: TopSite) {
+    when (val favicon = getTopSitesFavicon(topSite)) {
+        is TopSitesFavicon.ImageUrl -> Favicon(
+            url = topSite.url,
+            size = TOP_SITES_FAVICON_SIZE.dp,
+            imageUrl = favicon.imageUrl,
+        )
 
-@Composable
-private fun TopSiteFavicon(url: String, imageUrl: String? = null) {
-    when (url) {
-        SupportUtils.POCKET_TRENDING_URL -> FaviconImage(painterResource(R.drawable.ic_pocket))
-        SupportUtils.BAIDU_URL -> FaviconImage(painterResource(R.drawable.ic_baidu))
-        SupportUtils.JD_URL -> FaviconImage(painterResource(R.drawable.ic_jd))
-        SupportUtils.PDD_URL -> FaviconImage(painterResource(R.drawable.ic_pdd))
-        SupportUtils.TC_URL -> FaviconImage(painterResource(R.drawable.ic_tc))
-        SupportUtils.MEITUAN_URL -> FaviconImage(painterResource(R.drawable.ic_meituan))
-        else -> Favicon(url = url, size = TOP_SITES_FAVICON_SIZE.dp, imageUrl = imageUrl)
+        is TopSitesFavicon.Drawable -> Favicon(
+            size = TOP_SITES_FAVICON_SIZE.dp,
+            imageResource = favicon.drawableResId,
+        )
     }
 }
 
 @Composable
-private fun getMenuItems(
+internal fun getMenuItems(
     topSite: TopSite,
     onOpenInPrivateTabClicked: (topSite: TopSite) -> Unit,
-    onRenameTopSiteClicked: (topSite: TopSite) -> Unit,
+    onEditTopSiteClicked: (topSite: TopSite) -> Unit,
     onRemoveTopSiteClicked: (topSite: TopSite) -> Unit,
     onSettingsClicked: () -> Unit,
     onSponsorPrivacyClicked: () -> Unit,
 ): List<MenuItem> {
     val isPinnedSite = topSite is TopSite.Pinned || topSite is TopSite.Default
     val isProvidedSite = topSite is TopSite.Provided
+    val isFrecentSite = topSite is TopSite.Frecent
     val result = mutableListOf<MenuItem>()
 
     result.add(
         MenuItem(
             title = stringResource(id = R.string.bookmark_menu_open_in_private_tab_button),
-            testTag = TopSitesTestTag.openInPrivateTab,
+            testTag = TopSitesTestTag.OPEN_IN_PRIVATE_TAB,
             onClick = { onOpenInPrivateTabClicked(topSite) },
         ),
     )
 
-    if (isPinnedSite) {
+    if (isPinnedSite || isFrecentSite) {
         result.add(
             MenuItem(
-                title = stringResource(id = R.string.rename_top_site),
-                testTag = TopSitesTestTag.rename,
-                onClick = { onRenameTopSiteClicked(topSite) },
+                title = stringResource(id = R.string.top_sites_edit_top_site),
+                testTag = TopSitesTestTag.EDIT,
+                onClick = { onEditTopSiteClicked(topSite) },
             ),
         )
     }
@@ -441,36 +498,28 @@ private fun getMenuItems(
                         R.string.delete_from_history
                     },
                 ),
-                testTag = TopSitesTestTag.remove,
+                testTag = TopSitesTestTag.REMOVE,
                 onClick = { onRemoveTopSiteClicked(topSite) },
             ),
         )
     }
 
     if (isProvidedSite) {
-        result.add(
-            MenuItem(
-                title = stringResource(id = R.string.delete_from_history),
-                testTag = TopSitesTestTag.remove,
-                onClick = { onRemoveTopSiteClicked(topSite) },
-            ),
-        )
-    }
-
-    if (isProvidedSite) {
-        result.add(
-            MenuItem(
-                title = stringResource(id = R.string.top_sites_menu_settings),
-                onClick = onSettingsClicked,
-            ),
-        )
-    }
-
-    if (isProvidedSite) {
-        result.add(
-            MenuItem(
-                title = stringResource(id = R.string.top_sites_menu_sponsor_privacy),
-                onClick = onSponsorPrivacyClicked,
+        result.addAll(
+            listOf(
+                MenuItem(
+                    title = stringResource(id = R.string.delete_from_history),
+                    testTag = TopSitesTestTag.REMOVE,
+                    onClick = { onRemoveTopSiteClicked(topSite) },
+                ),
+                MenuItem(
+                    title = stringResource(id = R.string.top_sites_menu_settings),
+                    onClick = onSettingsClicked,
+                ),
+                MenuItem(
+                    title = stringResource(id = R.string.top_sites_menu_sponsor_privacy),
+                    onClick = onSponsorPrivacyClicked,
+                ),
             ),
         )
     }
@@ -478,76 +527,22 @@ private fun getMenuItems(
     return result
 }
 
-private fun submitTopSitesImpressionPing(topSite: TopSite.Provided, position: Int) {
-    TopSitesMetrics.contileImpression.record(
-        TopSitesMetrics.ContileImpressionExtra(
-            position = position + 1,
-            source = "newtab",
-        ),
-    )
-
-    topSite.id?.let { TopSitesMetrics.contileTileId.set(it) }
-    topSite.title?.let { TopSitesMetrics.contileAdvertiser.set(it.lowercase()) }
-    TopSitesMetrics.contileReportingUrl.set(topSite.impressionUrl)
-    Pings.topsitesImpression.submit()
-}
-
 @Composable
-@LightDarkPreview
+@PreviewLightDark
 private fun TopSitesPreview() {
     FirefoxTheme {
-        Box(modifier = Modifier.background(color = FirefoxTheme.colors.layer1)) {
+        Box(
+            modifier = Modifier
+                .background(color = FirefoxTheme.colors.layer1)
+                .padding(16.dp),
+        ) {
             TopSites(
-                topSites = mutableListOf<TopSite>().apply {
-                    for (index in 0 until 2) {
-                        add(
-                            TopSite.Pinned(
-                                id = index.toLong(),
-                                title = "Mozilla$index",
-                                url = "mozilla.com",
-                                createdAt = 0L,
-                            ),
-                        )
-                    }
-
-                    for (index in 0 until 2) {
-                        add(
-                            TopSite.Provided(
-                                id = index.toLong(),
-                                title = "Mozilla$index",
-                                url = "mozilla.com",
-                                clickUrl = "https://mozilla.com/click",
-                                imageUrl = "https://test.com/image2.jpg",
-                                impressionUrl = "https://example.com",
-                                createdAt = 0L,
-                            ),
-                        )
-                    }
-
-                    for (index in 0 until 2) {
-                        add(
-                            TopSite.Default(
-                                id = index.toLong(),
-                                title = "Mozilla$index",
-                                url = "mozilla.com",
-                                createdAt = 0L,
-                            ),
-                        )
-                    }
-
-                    add(
-                        TopSite.Default(
-                            id = null,
-                            title = "Top Articles",
-                            url = "https://getpocket.com/fenix-top-articles",
-                            createdAt = 0L,
-                        ),
-                    )
-                },
+                topSites = FakeHomepagePreview.topSites(),
                 onTopSiteClick = {},
                 onTopSiteLongClick = {},
+                onTopSiteImpression = { _, _ -> },
                 onOpenInPrivateTabClicked = {},
-                onRenameTopSiteClicked = {},
+                onEditTopSiteClicked = {},
                 onRemoveTopSiteClicked = {},
                 onSettingsClicked = {},
                 onSponsorPrivacyClicked = {},
