@@ -13,7 +13,6 @@ import android.content.Intent
 import android.content.pm.PackageManager
 import android.graphics.Color
 import android.graphics.Typeface
-import android.graphics.drawable.ColorDrawable
 import android.os.Build
 import android.os.Bundle
 import android.speech.RecognizerIntent
@@ -23,14 +22,17 @@ import android.view.MotionEvent
 import android.view.View
 import android.view.ViewGroup
 import android.view.ViewStub
+import android.view.Window
 import android.view.WindowManager
-import android.view.accessibility.AccessibilityEvent
 import android.view.inputmethod.InputMethodManager
+import android.window.OnBackInvokedDispatcher
+import androidx.activity.ComponentDialog
+import androidx.activity.OnBackPressedCallback
 import androidx.activity.result.ActivityResultLauncher
 import androidx.annotation.VisibleForTesting
-import androidx.appcompat.app.AlertDialog
 import androidx.appcompat.app.AppCompatDialogFragment
 import androidx.appcompat.content.res.AppCompatResources
+import androidx.compose.ui.platform.ViewCompositionStrategy
 import androidx.constraintlayout.widget.ConstraintProperties.BOTTOM
 import androidx.constraintlayout.widget.ConstraintProperties.PARENT_ID
 import androidx.constraintlayout.widget.ConstraintProperties.TOP
@@ -43,7 +45,9 @@ import androidx.navigation.NavBackStackEntry
 import androidx.navigation.NavGraph
 import androidx.navigation.fragment.findNavController
 import androidx.navigation.fragment.navArgs
+import com.google.android.material.dialog.MaterialAlertDialogBuilder
 import kotlinx.coroutines.flow.distinctUntilChanged
+import kotlinx.coroutines.flow.distinctUntilChangedBy
 import kotlinx.coroutines.flow.map
 import kotlinx.coroutines.launch
 import mozilla.components.browser.state.action.AwesomeBarAction
@@ -58,7 +62,6 @@ import mozilla.components.concept.toolbar.Toolbar
 import mozilla.components.feature.qr.QrFeature
 import mozilla.components.lib.state.ext.consumeFlow
 import mozilla.components.lib.state.ext.consumeFrom
-import mozilla.components.service.glean.private.NoExtras
 import mozilla.components.support.base.coroutines.Dispatchers
 import mozilla.components.support.base.feature.UserInteractionHandler
 import mozilla.components.support.base.feature.ViewBoundFeatureWrapper
@@ -67,33 +70,37 @@ import mozilla.components.support.ktx.android.content.hasCamera
 import mozilla.components.support.ktx.android.content.isPermissionGranted
 import mozilla.components.support.ktx.android.content.res.getSpanned
 import mozilla.components.support.ktx.android.net.isHttpOrHttps
-import mozilla.components.support.ktx.android.view.findViewInHierarchy
+import mozilla.components.support.ktx.android.view.ImeInsetsSynchronizer
 import mozilla.components.support.ktx.android.view.hideKeyboard
+import mozilla.components.support.ktx.android.view.setupPersistentInsets
 import mozilla.components.support.ktx.android.view.showKeyboard
 import mozilla.components.support.ktx.kotlin.toNormalizedUrl
 import mozilla.components.ui.autocomplete.InlineAutocompleteEditText
 import mozilla.components.ui.widgets.withCenterAlignedButtons
+import mozilla.telemetry.glean.private.NoExtras
 import org.mozilla.fenix.BrowserDirection
 import org.mozilla.fenix.GleanMetrics.Awesomebar
 import org.mozilla.fenix.GleanMetrics.Events
 import org.mozilla.fenix.GleanMetrics.VoiceSearch
 import org.mozilla.fenix.HomeActivity
 import org.mozilla.fenix.R
-import org.mozilla.fenix.components.Core.Companion.BOOKMARKS_SEARCH_ENGINE_ID
-import org.mozilla.fenix.components.Core.Companion.HISTORY_SEARCH_ENGINE_ID
-import org.mozilla.fenix.components.Core.Companion.TABS_SEARCH_ENGINE_ID
 import org.mozilla.fenix.components.appstate.AppAction
-import org.mozilla.fenix.components.toolbar.IncompleteRedesignToolbarFeature
+import org.mozilla.fenix.components.search.BOOKMARKS_SEARCH_ENGINE_ID
+import org.mozilla.fenix.components.search.HISTORY_SEARCH_ENGINE_ID
+import org.mozilla.fenix.components.search.TABS_SEARCH_ENGINE_ID
 import org.mozilla.fenix.components.toolbar.ToolbarPosition
 import org.mozilla.fenix.databinding.FragmentSearchDialogBinding
 import org.mozilla.fenix.databinding.SearchSuggestionsHintBinding
 import org.mozilla.fenix.ext.components
-import org.mozilla.fenix.ext.getRectWithScreenLocation
 import org.mozilla.fenix.ext.increaseTapArea
 import org.mozilla.fenix.ext.registerForActivityResult
 import org.mozilla.fenix.ext.requireComponents
+import org.mozilla.fenix.ext.runIfFragmentIsAttached
 import org.mozilla.fenix.ext.secure
 import org.mozilla.fenix.ext.settings
+import org.mozilla.fenix.navigation.DefaultNavControllerProvider
+import org.mozilla.fenix.navigation.NavControllerProvider
+import org.mozilla.fenix.nimbus.FxNimbus
 import org.mozilla.fenix.search.awesomebar.AwesomeBarView
 import org.mozilla.fenix.search.awesomebar.toSearchProviderState
 import org.mozilla.fenix.search.ext.searchEngineShortcuts
@@ -102,28 +109,34 @@ import org.mozilla.fenix.search.toolbar.SearchSelectorMenu
 import org.mozilla.fenix.search.toolbar.SearchSelectorToolbarAction
 import org.mozilla.fenix.search.toolbar.ToolbarView
 import org.mozilla.fenix.settings.SupportUtils
+import org.mozilla.fenix.telemetry.ACTION_QR_CLICKED
+import org.mozilla.fenix.telemetry.SOURCE_ADDRESS_BAR
+import mozilla.components.browser.toolbar.R as toolbarR
+import org.mozilla.fenix.GleanMetrics.Toolbar as GleanMetricsToolbar
 
 typealias SearchDialogFragmentStore = SearchFragmentStore
 
-/**
- * Possible extra actions when opening search dialog fragment.
- */
-enum class ExtraAction {
-    QR_READER, VOICE_SEARCH, NONE,
-}
-
 @SuppressWarnings("LargeClass", "TooManyFunctions")
 class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     private var _binding: FragmentSearchDialogBinding? = null
     private val binding get() = _binding!!
 
-    @VisibleForTesting internal lateinit var interactor: SearchDialogInteractor
+    private var controller: SearchDialogController? = null
+
+    @VisibleForTesting
+    internal var nullableInteractor: SearchDialogInteractor? = null
+
+    @VisibleForTesting internal val interactor: SearchDialogInteractor get() = nullableInteractor!!
+
     private lateinit var store: SearchDialogFragmentStore
 
-    @VisibleForTesting internal lateinit var toolbarView: ToolbarView
+    private var _toolbarView: ToolbarView? = null
+
+    @VisibleForTesting internal val toolbarView: ToolbarView get() = _toolbarView!!
 
     @VisibleForTesting internal lateinit var inlineAutocompleteEditText: InlineAutocompleteEditText
-    private lateinit var awesomeBarView: AwesomeBarView
+    private var _awesomeBarView: AwesomeBarView? = null
+    private val awesomeBarView: AwesomeBarView get() = _awesomeBarView!!
     private lateinit var startForResult: ActivityResultLauncher<Intent>
 
     private val searchSelectorMenu by lazy {
@@ -136,7 +149,6 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     private val qrFeature = ViewBoundFeatureWrapper<QrFeature>()
     private val speechIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
 
-    private var isPrivateButtonClicked = false
     private var dialogHandledAction = false
     private var searchSelectorAlreadyAdded = false
     private var qrButtonAction: Toolbar.Action? = null
@@ -169,10 +181,17 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
 
     override fun onCreate(savedInstanceState: Bundle?) {
         super.onCreate(savedInstanceState)
-        setStyle(STYLE_NO_TITLE, R.style.SearchDialogStyle)
+
+        val args by navArgs<SearchDialogFragmentArgs>()
+
+        if (context?.settings()?.isTabStripEnabled == true) {
+            setStyle(STYLE_NORMAL, R.style.SearchDialogStyleTabStrip)
+        } else {
+            setStyle(STYLE_NORMAL, R.style.SearchDialogStyle)
+        }
 
         startForResult = registerForActivityResult { result ->
-            result.data?.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)?.first()?.also {
+            result.data?.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)?.firstOrNull()?.also {
                 val updatedUrl = toolbarView.view.edit.updateUrl(url = it, shouldHighlight = false, shouldAppend = true)
                 interactor.onTextChanged(updatedUrl)
                 toolbarView.view.edit.focus()
@@ -182,20 +201,39 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
         }
 
         requireComponents.appStore.dispatch(
-            AppAction.UpdateSearchDialogVisibility(isVisible = true),
+            AppAction.SearchAction.SearchStarted(
+                tabId = args.sessionId,
+                source = args.searchAccessPoint,
+            ),
         )
     }
 
     override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
-        return object : Dialog(requireContext(), this.theme) {
-            @Deprecated("Deprecated in Java")
-            override fun onBackPressed() {
-                this@SearchDialogFragment.onBackPressed()
-            }
-        }.apply {
+        return ComponentDialog(requireContext(), this.theme).apply {
             if ((requireActivity() as HomeActivity).browsingModeManager.mode.isPrivate) {
                 this.secure(requireActivity())
             }
+
+            onBackPressedDispatcher.addCallback(
+                owner = this,
+                onBackPressedCallback = object : OnBackPressedCallback(true) {
+                    override fun handleOnBackPressed() {
+                        this@SearchDialogFragment.onBackPressed()
+                    }
+                },
+            )
+
+            // This makes sure that we don't miss any onBackPressed calls because
+            // of the introduction of predictive back gesture to Android OS.
+            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
+                onBackInvokedDispatcher.registerOnBackInvokedCallback(
+                    OnBackInvokedDispatcher.PRIORITY_OVERLAY,
+                ) {
+                    this@SearchDialogFragment.onBackPressed()
+                }
+            }
+            requestWindowFeature(Window.FEATURE_NO_TITLE)
+            window?.setupPersistentInsets()
         }
     }
 
@@ -224,35 +262,37 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
             ),
         )
 
-        interactor = SearchDialogInteractor(
-            SearchDialogController(
-                activity = activity,
-                store = requireComponents.core.store,
-                tabsUseCases = requireComponents.useCases.tabsUseCases,
-                fragmentStore = store,
-                navController = findNavController(),
-                settings = requireContext().settings(),
-                dismissDialog = {
-                    dialogHandledAction = true
-                    dismissAllowingStateLoss()
-                },
-                clearToolbarFocus = {
-                    dialogHandledAction = true
-                    toolbarView.view.hideKeyboard()
-                    toolbarView.view.clearFocus()
-                },
-                focusToolbar = { toolbarView.view.edit.focus() },
-                clearToolbar = {
-                    inlineAutocompleteEditText.setText("")
-                },
-                dismissDialogAndGoBack = ::dismissDialogAndGoBack,
-            ),
+        controller = SearchDialogController(
+            activity = activity,
+            store = requireComponents.core.store,
+            tabsUseCases = requireComponents.useCases.tabsUseCases,
+            fenixBrowserUseCases = requireComponents.useCases.fenixBrowserUseCases,
+            fragmentStore = store,
+            navController = findNavController(),
+            settings = requireContext().settings(),
+            dismissDialog = {
+                dialogHandledAction = true
+                dismissAllowingStateLoss()
+            },
+            clearToolbarFocus = {
+                dialogHandledAction = true
+                toolbarView.view.hideKeyboard()
+                toolbarView.view.clearFocus()
+            },
+            focusToolbar = { toolbarView.view.edit.focus() },
+            clearToolbar = {
+                inlineAutocompleteEditText.setText("")
+            },
+            dismissDialogAndGoBack = {
+                dismissDialogAndGoBack()
+            },
         )
+        nullableInteractor = SearchDialogInteractor(searchController = requireNotNull(controller))
 
         val fromHomeFragment =
             getPreviousDestination()?.destination?.id == R.id.homeFragment
 
-        toolbarView = ToolbarView(
+        _toolbarView = ToolbarView(
             requireContext().settings(),
             requireComponents,
             interactor,
@@ -260,23 +300,18 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
             binding.toolbar,
             fromHomeFragment,
         ).also {
-            if (!IncompleteRedesignToolbarFeature(requireContext().settings()).isEnabled) {
-                it.view.hidePageActionSeparator()
-            } else {
-                it.view.showPageActionSeparator()
-                it.view.isNavBarEnabled = true
-            }
-            inlineAutocompleteEditText = it.view.findViewById(R.id.mozac_browser_toolbar_edit_url_view)
+            inlineAutocompleteEditText = it.view.findViewById(toolbarR.id.mozac_browser_toolbar_edit_url_view)
             inlineAutocompleteEditText.increaseTapArea(TAP_INCREASE_DPS_4)
         }
 
         val awesomeBar = binding.awesomeBar
 
-        awesomeBarView = AwesomeBarView(
+        _awesomeBarView = AwesomeBarView(
             activity,
             interactor,
             awesomeBar,
             fromHomeFragment,
+            browsingModeManager = activity.browsingModeManager,
         )
 
         binding.awesomeBar.setOnTouchListener { _, _ ->
@@ -296,31 +331,22 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
         when (getPreviousDestination()?.destination?.id) {
             R.id.homeFragment -> {
                 // When displayed above home, dispatches the touch events to scrim area to the HomeFragment
-                binding.searchWrapper.background = ColorDrawable(Color.TRANSPARENT)
+                binding.searchWrapper.background = Color.TRANSPARENT.toDrawable()
                 dialog?.window?.decorView?.setOnTouchListener { _, event ->
                     when (event?.action) {
                         MotionEvent.ACTION_DOWN, MotionEvent.ACTION_MOVE -> {
-                            isPrivateButtonClicked = isTouchingPrivateButton(event.x, event.y)
-                            // Immediately drop Search Bar focus when the touch is not on the private button.
-                            if (!isPrivateButtonClicked) {
-                                toolbarView.view.clearFocus()
-                            }
+                            // Immediately drop Search Bar focus on touch.
+                            toolbarView.view.clearFocus()
                         }
                         MotionEvent.ACTION_UP -> {
-                            if (!isTouchingPrivateButton(
-                                    event.x,
-                                    event.y,
-                                ) && !isPrivateButtonClicked
-                            ) {
-                                findNavController().popBackStack()
-                                isPrivateButtonClicked = false
-                            }
+                            findNavController().popBackStack()
                         }
-                        else -> isPrivateButtonClicked = false
                     }
+
                     if (binding.awesomeBar.visibility != View.VISIBLE) {
                         requireActivity().dispatchTouchEvent(event)
                     }
+
                     false
                 }
             }
@@ -348,6 +374,8 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
         super.onViewCreated(view, savedInstanceState)
 
+        binding.awesomeBar.setViewCompositionStrategy(ViewCompositionStrategy.DisposeOnViewTreeLifecycleDestroyed)
+
         val showUnifiedSearchFeature = requireContext().settings().showUnifiedSearchFeature
 
         consumeFlow(requireComponents.core.store) { flow ->
@@ -465,10 +493,21 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
             updateAccessibilityTraversalOrder()
         }
 
+        ImeInsetsSynchronizer.setup(view)
         observeClipboardState()
-        observeAwesomeBarState()
         observeSuggestionProvidersState()
 
+        val shouldShowSuggestions = store.state.run {
+            (showTrendingSearches || showRecentSearches) &&
+                (query.isNotEmpty() || FxNimbus.features.searchSuggestionsOnHomepage.value().enabled)
+        }
+
+        if (shouldShowSuggestions) {
+            binding.awesomeBar.isVisible = true
+        } else {
+            observeAwesomeBarState()
+        }
+
         consumeFrom(store) {
             updateSearchSuggestionsHintVisibility(it)
             updateToolbarContentDescription(
@@ -479,16 +518,11 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
             awesomeBarView.update(it)
 
             addSearchSelector()
-            updateQrButton(it)
+            if (it.showQrButton) {
+                updateQrButton(it)
+            }
             updateVoiceSearchButton()
         }
-
-        val args by navArgs<SearchDialogFragmentArgs>()
-        when (args.extraAction) {
-            ExtraAction.QR_READER -> launchQr()
-            ExtraAction.VOICE_SEARCH -> launchVoiceSearch()
-            ExtraAction.NONE -> {}
-        }
     }
 
     /**
@@ -511,13 +545,6 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
         }
     }
 
-    private fun isTouchingPrivateButton(x: Float, y: Float): Boolean {
-        val view = parentFragmentManager.primaryNavigationFragment?.view?.findViewInHierarchy {
-            it.id == R.id.privateBrowsingButton
-        } ?: return false
-        return view.getRectWithScreenLocation().contains(x.toInt(), y.toInt())
-    }
-
     private fun hideClipboardSection() {
         binding.fillLinkFromClipboard.isVisible = false
         binding.fillLinkDivider.isVisible = false
@@ -528,8 +555,8 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     }
 
     private fun observeSuggestionProvidersState() = consumeFlow(store) { flow ->
-        flow.map { state -> state.toSearchProviderState() }
-            .distinctUntilChanged()
+        flow
+            .distinctUntilChangedBy { it.toSearchProviderState() }
             .collect { state -> awesomeBarView.updateSuggestionProvidersVisibility(state) }
     }
 
@@ -565,13 +592,7 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     }
 
     private fun updateAccessibilityTraversalOrder() {
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
-            binding.fillLinkFromClipboard.accessibilityTraversalAfter = binding.searchWrapper.id
-        } else {
-            viewLifecycleOwner.lifecycleScope.launch {
-                binding.searchWrapper.sendAccessibilityEvent(AccessibilityEvent.TYPE_VIEW_FOCUSED)
-            }
-        }
+        binding.fillLinkFromClipboard.accessibilityTraversalAfter = binding.searchWrapper.id
     }
 
     override fun onResume() {
@@ -600,7 +621,18 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
 
     override fun onDestroyView() {
         super.onDestroyView()
-
+        _awesomeBarView?.dispose()
+        _awesomeBarView = null
+        nullableInteractor = null
+        controller?.apply {
+            dismissDialog = null
+            clearToolbarFocus = null
+            focusToolbar = null
+            clearToolbar = null
+            dismissDialogAndGoBack = null
+        }
+        controller = null
+        _toolbarView = null
         _binding = null
     }
 
@@ -625,7 +657,7 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
         }
 
         requireComponents.appStore.dispatch(
-            AppAction.UpdateSearchDialogVisibility(isVisible = false),
+            AppAction.SearchAction.SearchEnded,
         )
     }
 
@@ -643,20 +675,22 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
     }
 
     private fun dismissDialogAndGoBack() {
-        // In case we're displaying search results, we wouldn't have navigated to home, and
-        // so we don't need to navigate "back to" browser fragment.
-        // See mirror of this logic in BrowserToolbarController#handleToolbarClick.
-        if (store.state.searchTerms.isBlank()) {
-            val args by navArgs<SearchDialogFragmentArgs>()
-            args.sessionId?.let {
-                findNavController().navigate(
-                    SearchDialogFragmentDirections.actionGlobalBrowser(null),
-                )
+        runIfFragmentIsAttached {
+            // In case we're displaying search results, we wouldn't have navigated to home, and
+            // so we don't need to navigate "back to" browser fragment.
+            // See mirror of this logic in BrowserToolbarController#handleToolbarClick.
+            if (store.state.searchTerms.isBlank()) {
+                val args by navArgs<SearchDialogFragmentArgs>()
+                args.sessionId?.let {
+                    findNavController().navigate(
+                        SearchDialogFragmentDirections.actionGlobalBrowser(null),
+                    )
+                }
             }
-        }
 
-        view?.hideKeyboard()
-        dismissAllowingStateLoss()
+            view?.hideKeyboard()
+            dismissAllowingStateLoss()
+        }
     }
 
     @Suppress("DEPRECATION")
@@ -672,7 +706,7 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
                 val normalizedUrl = result.toNormalizedUrl()
                 if (!normalizedUrl.toUri().isHttpOrHttps) {
                     activity?.let {
-                        AlertDialog.Builder(it).apply {
+                        MaterialAlertDialogBuilder(it).apply {
                             setMessage(R.string.qr_scanner_dialog_invalid)
                             setPositiveButton(R.string.qr_scanner_dialog_invalid_ok) { dialog: DialogInterface, _ ->
                                 dialog.dismiss()
@@ -682,7 +716,7 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
                     }
                 } else {
                     activity?.let {
-                        AlertDialog.Builder(it).apply {
+                        MaterialAlertDialogBuilder(it).apply {
                             val spannable = resources.getSpanned(
                                 R.string.qr_scanner_confirmation_dialog_message,
                                 getString(R.string.app_name) to StyleSpan(Typeface.BOLD),
@@ -710,8 +744,8 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
         )
     }
 
-    @Suppress("DEPRECATION")
-    // https://github.com/mozilla-mobile/fenix/issues/19920
+    @Suppress("DEPRECATION", "OVERRIDE_DEPRECATION")
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=1813657
     override fun onRequestPermissionsResult(
         requestCode: Int,
         permissions: Array<String>,
@@ -872,7 +906,7 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
                     qrButtonAction = IncreasedTapAreaActionDecorator(
                         BrowserToolbar.Button(
                             AppCompatResources.getDrawable(requireContext(), R.drawable.ic_qr)!!,
-                            requireContext().getString(R.string.search_scan_button),
+                            requireContext().getString(R.string.search_scan_button_2),
                             autoHide = { true },
                             listener = ::launchQr,
                         ),
@@ -898,7 +932,9 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
             return
         }
 
-        Events.browserToolbarQrScanTapped.record(NoExtras())
+        GleanMetricsToolbar.buttonTapped.record(
+            GleanMetricsToolbar.ButtonTappedExtra(source = SOURCE_ADDRESS_BAR, item = ACTION_QR_CLICKED),
+        )
 
         view?.hideKeyboard()
         toolbarView.view.clearFocus()
@@ -989,10 +1025,13 @@ class SearchDialogFragment : AppCompatDialogFragment(), UserInteractionHandler {
      */
     @VisibleForTesting
     @SuppressLint("RestrictedApi")
-    internal fun getPreviousDestination(): NavBackStackEntry? {
+    internal fun getPreviousDestination(
+        navControllerProvider: NavControllerProvider = DefaultNavControllerProvider(),
+    ): NavBackStackEntry? {
         // This duplicates the platform functionality for "previousBackStackEntry" but additionally skips this entry.
 
-        val descendingEntries = findNavController().currentBackStack.value.reversed().iterator()
+        val descendingEntries =
+            navControllerProvider.getNavController(this).currentBackStack.value.reversed().iterator()
         // Throw the topmost destination away.
         if (descendingEntries.hasNext()) {
             descendingEntries.next()
