@@ -2,49 +2,37 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-@file:OptIn(ExperimentalFoundationApi::class)
-
 package org.mozilla.fenix.tabstray
 
-import androidx.compose.foundation.ExperimentalFoundationApi
 import androidx.compose.foundation.background
 import androidx.compose.foundation.layout.Box
 import androidx.compose.foundation.layout.Column
 import androidx.compose.foundation.layout.fillMaxSize
-import androidx.compose.foundation.layout.padding
 import androidx.compose.foundation.pager.HorizontalPager
 import androidx.compose.foundation.pager.rememberPagerState
 import androidx.compose.foundation.shape.RoundedCornerShape
-import androidx.compose.material.Text
+import androidx.compose.material3.HorizontalDivider
 import androidx.compose.runtime.Composable
 import androidx.compose.runtime.LaunchedEffect
+import androidx.compose.runtime.derivedStateOf
 import androidx.compose.runtime.getValue
 import androidx.compose.runtime.mutableStateOf
 import androidx.compose.runtime.remember
 import androidx.compose.runtime.setValue
-import androidx.compose.runtime.toMutableStateList
 import androidx.compose.ui.Alignment
 import androidx.compose.ui.Modifier
 import androidx.compose.ui.draw.clip
 import androidx.compose.ui.input.nestedscroll.nestedScroll
 import androidx.compose.ui.platform.rememberNestedScrollInteropConnection
 import androidx.compose.ui.platform.testTag
-import androidx.compose.ui.res.stringResource
+import androidx.compose.ui.tooling.preview.PreviewLightDark
 import androidx.compose.ui.unit.dp
-import mozilla.components.browser.state.state.BrowserState
 import mozilla.components.browser.state.state.ContentState
 import mozilla.components.browser.state.state.TabSessionState
-import mozilla.components.browser.state.store.BrowserStore
+import mozilla.components.browser.state.state.createTab
 import mozilla.components.browser.storage.sync.TabEntry
-import mozilla.components.lib.state.ext.observeAsComposableState
-import org.mozilla.fenix.R
-import org.mozilla.fenix.components.AppStore
-import org.mozilla.fenix.components.appstate.AppState
-import org.mozilla.fenix.compose.Divider
-import org.mozilla.fenix.compose.annotation.LightDarkPreview
+import mozilla.components.lib.state.ext.observeAsState
 import org.mozilla.fenix.tabstray.ext.isNormalTab
-import org.mozilla.fenix.tabstray.inactivetabs.InactiveTabsList
-import org.mozilla.fenix.tabstray.syncedtabs.SyncedTabsList
 import org.mozilla.fenix.tabstray.syncedtabs.SyncedTabsListItem
 import org.mozilla.fenix.theme.FirefoxTheme
 import mozilla.components.browser.storage.sync.Tab as SyncTab
@@ -54,13 +42,13 @@ import org.mozilla.fenix.tabstray.syncedtabs.OnTabCloseClick as OnSyncedTabClose
 /**
  * Top-level UI for displaying the Tabs Tray feature.
  *
- * @param appStore [AppStore] used to listen for changes to [AppState].
- * @param browserStore [BrowserStore] used to listen for changes to [BrowserState].
  * @param tabsTrayStore [TabsTrayStore] used to listen for changes to [TabsTrayState].
  * @param displayTabsInGrid Whether the normal and private tabs should be displayed in a grid.
  * @param isInDebugMode True for debug variant or if secret menu is enabled for this session.
  * @param shouldShowTabAutoCloseBanner Whether the tab auto closer banner should be displayed.
+ * @param shouldShowLockPbmBanner Whether the lock private browsing banner should be displayed.
  * @param shouldShowInactiveTabsAutoCloseDialog Whether the inactive tabs auto close dialog should be displayed.
+ * @param isPbmLocked Whether the private browsing mode is currently locked.
  * @param onTabPageClick Invoked when the user clicks on the Normal, Private, or Synced tabs page button.
  * @param onTabClose Invoked when the user clicks to close a tab.
  * @param onTabMediaClick Invoked when the user interacts with a tab's media controls.
@@ -92,6 +80,9 @@ import org.mozilla.fenix.tabstray.syncedtabs.OnTabCloseClick as OnSyncedTabClose
  * @param onForceSelectedTabsAsInactiveClick Invoked when the user clicks on the make inactive banner menu item.
  * @param onTabsTrayDismiss Invoked when accessibility services or UI automation requests dismissal.
  * @param onTabAutoCloseBannerViewOptionsClick Invoked when the user clicks to view the auto close options.
+ * @param onTabsTrayPbmLockedClick Invoked when the user interacts with the lock private browsing mode banner.
+ * @param onTabsTrayPbmLockedDismiss Invoked when the user clicks either button on the
+ * lock private browsing mode banner.
  * @param onTabAutoCloseBannerDismiss Invoked when the user clicks to dismiss the auto close banner.
  * @param onTabAutoCloseBannerShown Invoked when the auto close banner has been shown to the user.
  * @param onMove Invoked after the drag and drop gesture completed. Swaps positions of two tabs.
@@ -99,18 +90,18 @@ import org.mozilla.fenix.tabstray.syncedtabs.OnTabCloseClick as OnSyncedTabClose
  * @param onInactiveTabsCFRShown Invoked when the inactive tabs CFR is displayed.
  * @param onInactiveTabsCFRClick Invoked when the inactive tabs CFR is clicked.
  * @param onInactiveTabsCFRDismiss Invoked when the inactive tabs CFR is dismissed.
+ * @param onUnlockPbmClick Invoked when user clicks on the Unlock button.
  */
-@OptIn(ExperimentalFoundationApi::class)
-@Suppress("LongMethod", "LongParameterList", "ComplexMethod")
+@Suppress("LongMethod", "LongParameterList")
 @Composable
 fun TabsTray(
-    appStore: AppStore,
-    browserStore: BrowserStore,
     tabsTrayStore: TabsTrayStore,
     displayTabsInGrid: Boolean,
     isInDebugMode: Boolean,
     shouldShowTabAutoCloseBanner: Boolean,
+    shouldShowLockPbmBanner: Boolean,
     shouldShowInactiveTabsAutoCloseDialog: (Int) -> Boolean,
+    isPbmLocked: Boolean = false,
     onTabPageClick: (Page) -> Unit,
     onTabClose: (TabSessionState) -> Unit,
     onTabMediaClick: (TabSessionState) -> Unit,
@@ -137,6 +128,8 @@ fun TabsTray(
     onForceSelectedTabsAsInactiveClick: () -> Unit,
     onTabsTrayDismiss: () -> Unit,
     onTabAutoCloseBannerViewOptionsClick: () -> Unit,
+    onTabsTrayPbmLockedClick: () -> Unit,
+    onTabsTrayPbmLockedDismiss: () -> Unit,
     onTabAutoCloseBannerDismiss: () -> Unit,
     onTabAutoCloseBannerShown: () -> Unit,
     onMove: (String, String?, Boolean) -> Unit,
@@ -144,23 +137,34 @@ fun TabsTray(
     onInactiveTabsCFRShown: () -> Unit,
     onInactiveTabsCFRClick: () -> Unit,
     onInactiveTabsCFRDismiss: () -> Unit,
+    onUnlockPbmClick: () -> Unit,
 ) {
-    val multiselectMode = tabsTrayStore
-        .observeAsComposableState { state -> state.mode }.value ?: TabsTrayState.Mode.Normal
-    val selectedPage = tabsTrayStore
-        .observeAsComposableState { state -> state.selectedPage }.value ?: Page.NormalTabs
-    val pagerState =
-        rememberPagerState(initialPage = selectedPage.ordinal, pageCount = { Page.values().size })
-    val isInMultiSelectMode = multiselectMode is TabsTrayState.Mode.Select
-
+    val tabsTrayState by tabsTrayStore.observeAsState(initialValue = tabsTrayStore.state) { it }
+    val pagerState = rememberPagerState(
+        initialPage = Page.pageToPosition(page = tabsTrayState.selectedPage, enhancementsEnabled = false),
+        pageCount = { Page.entries.size },
+    )
+    val isInMultiSelectMode by remember(tabsTrayState.mode) {
+        derivedStateOf {
+            tabsTrayState.mode is TabsTrayState.Mode.Select
+        }
+    }
     val shapeModifier = if (isInMultiSelectMode) {
         Modifier
     } else {
         Modifier.clip(RoundedCornerShape(topStart = 16.dp, topEnd = 16.dp))
     }
 
-    LaunchedEffect(selectedPage) {
-        pagerState.animateScrollToPage(selectedPage.ordinal)
+    val syncedTabCount = remember(tabsTrayState.syncedTabs) {
+        tabsTrayState.syncedTabs
+            .filterIsInstance<SyncedTabsListItem.DeviceSection>()
+            .sumOf { deviceSection: SyncedTabsListItem.DeviceSection -> deviceSection.tabs.size }
+    }
+
+    LaunchedEffect(tabsTrayState.selectedPage) {
+        pagerState.animateScrollToPage(
+            page = Page.pageToPosition(page = tabsTrayState.selectedPage, enhancementsEnabled = false),
+        )
     }
 
     Column(
@@ -168,13 +172,18 @@ fun TabsTray(
             .fillMaxSize()
             .then(shapeModifier)
             .background(FirefoxTheme.colors.layer1)
-            .testTag(TabsTrayTestTag.tabsTray),
+            .testTag(TabsTrayTestTag.TABS_TRAY),
     ) {
         Box(modifier = Modifier.nestedScroll(rememberNestedScrollInteropConnection())) {
             TabsTrayBanner(
-                tabsTrayStore = tabsTrayStore,
+                selectedPage = tabsTrayState.selectedPage,
+                normalTabCount = tabsTrayState.normalTabs.size + tabsTrayState.inactiveTabs.size,
+                privateTabCount = tabsTrayState.privateTabs.size,
+                syncedTabCount = syncedTabCount,
+                selectionMode = tabsTrayState.mode,
                 isInDebugMode = isInDebugMode,
                 shouldShowTabAutoCloseBanner = shouldShowTabAutoCloseBanner,
+                shouldShowLockPbmBanner = shouldShowLockPbmBanner,
                 onTabPageIndicatorClicked = onTabPageClick,
                 onSaveToCollectionClick = onSaveToCollectionClick,
                 onShareSelectedTabsClick = onShareSelectedTabsClick,
@@ -188,26 +197,39 @@ fun TabsTray(
                 onForceSelectedTabsAsInactiveClick = onForceSelectedTabsAsInactiveClick,
                 onDismissClick = onTabsTrayDismiss,
                 onTabAutoCloseBannerViewOptionsClick = onTabAutoCloseBannerViewOptionsClick,
+                onTabsTrayPbmLockedClick = onTabsTrayPbmLockedClick,
+                onTabsTrayPbmLockedDismiss = onTabsTrayPbmLockedDismiss,
                 onTabAutoCloseBannerDismiss = onTabAutoCloseBannerDismiss,
                 onTabAutoCloseBannerShown = onTabAutoCloseBannerShown,
+                onEnterMultiselectModeClick = {
+                    tabsTrayStore.dispatch(TabsTrayAction.EnterSelectMode)
+                },
+                onExitSelectModeClick = {
+                    tabsTrayStore.dispatch(TabsTrayAction.ExitSelectMode)
+                },
+                onThreeDotMenuShown = {
+                    tabsTrayStore.dispatch(TabsTrayAction.ThreeDotMenuShown)
+                },
             )
         }
 
-        Divider()
+        HorizontalDivider()
 
         Box(modifier = Modifier.fillMaxSize()) {
             HorizontalPager(
                 modifier = Modifier.fillMaxSize(),
                 state = pagerState,
-                beyondBoundsPageCount = 2,
+                beyondViewportPageCount = 2,
                 userScrollEnabled = false,
             ) { position ->
-                when (Page.positionToPage(position)) {
+                when (Page.positionToPage(position = position, enhancementsEnabled = false)) {
                     Page.NormalTabs -> {
                         NormalTabsPage(
-                            appStore = appStore,
-                            browserStore = browserStore,
-                            tabsTrayStore = tabsTrayStore,
+                            normalTabs = tabsTrayState.normalTabs,
+                            inactiveTabs = tabsTrayState.inactiveTabs,
+                            selectedTabId = tabsTrayState.selectedTabId,
+                            selectionMode = tabsTrayState.mode,
+                            inactiveTabsExpanded = tabsTrayState.inactiveTabsExpanded,
                             displayTabsInGrid = displayTabsInGrid,
                             onTabClose = onTabClose,
                             onTabMediaClick = onTabMediaClick,
@@ -226,25 +248,31 @@ fun TabsTray(
                             onInactiveTabsCFRShown = onInactiveTabsCFRShown,
                             onInactiveTabsCFRClick = onInactiveTabsCFRClick,
                             onInactiveTabsCFRDismiss = onInactiveTabsCFRDismiss,
+                            onTabDragStart = {
+                                tabsTrayStore.dispatch(TabsTrayAction.ExitSelectMode)
+                            },
                         )
                     }
 
                     Page.PrivateTabs -> {
                         PrivateTabsPage(
-                            browserStore = browserStore,
-                            tabsTrayStore = tabsTrayStore,
+                            privateTabs = tabsTrayState.privateTabs,
+                            selectedTabId = tabsTrayState.selectedTabId,
+                            selectionMode = tabsTrayState.mode,
                             displayTabsInGrid = displayTabsInGrid,
+                            isPbmLocked = isPbmLocked,
                             onTabClose = onTabClose,
                             onTabMediaClick = onTabMediaClick,
                             onTabClick = onTabClick,
                             onTabLongClick = onTabLongClick,
                             onMove = onMove,
+                            onUnlockPbmClick = onUnlockPbmClick,
                         )
                     }
 
                     Page.SyncedTabs -> {
                         SyncedTabsPage(
-                            tabsTrayStore = tabsTrayStore,
+                            syncedTabs = tabsTrayState.syncedTabs,
                             onTabClick = onSyncedTabClick,
                             onTabClose = onSyncedTabClose,
                         )
@@ -255,184 +283,7 @@ fun TabsTray(
     }
 }
 
-@Composable
-@Suppress("LongParameterList")
-private fun NormalTabsPage(
-    appStore: AppStore,
-    browserStore: BrowserStore,
-    tabsTrayStore: TabsTrayStore,
-    displayTabsInGrid: Boolean,
-    onTabClose: (TabSessionState) -> Unit,
-    onTabMediaClick: (TabSessionState) -> Unit,
-    onTabClick: (TabSessionState) -> Unit,
-    onTabLongClick: (TabSessionState) -> Unit,
-    shouldShowInactiveTabsAutoCloseDialog: (Int) -> Boolean,
-    onInactiveTabsHeaderClick: (Boolean) -> Unit,
-    onDeleteAllInactiveTabsClick: () -> Unit,
-    onInactiveTabsAutoCloseDialogShown: () -> Unit,
-    onInactiveTabAutoCloseDialogCloseButtonClick: () -> Unit,
-    onEnableInactiveTabAutoCloseClick: () -> Unit,
-    onInactiveTabClick: (TabSessionState) -> Unit,
-    onInactiveTabClose: (TabSessionState) -> Unit,
-    onMove: (String, String?, Boolean) -> Unit,
-    shouldShowInactiveTabsCFR: () -> Boolean,
-    onInactiveTabsCFRShown: () -> Unit,
-    onInactiveTabsCFRClick: () -> Unit,
-    onInactiveTabsCFRDismiss: () -> Unit,
-) {
-    val inactiveTabsExpanded = appStore
-        .observeAsComposableState { state -> state.inactiveTabsExpanded }.value ?: false
-    val selectedTabId = browserStore
-        .observeAsComposableState { state -> state.selectedTabId }.value
-    val normalTabs = tabsTrayStore
-        .observeAsComposableState { state -> state.normalTabs }.value ?: emptyList()
-    val inactiveTabs = tabsTrayStore
-        .observeAsComposableState { state -> state.inactiveTabs }.value ?: emptyList()
-    val selectionMode = tabsTrayStore
-        .observeAsComposableState { state -> state.mode }.value ?: TabsTrayState.Mode.Normal
-
-    if (normalTabs.isNotEmpty() || inactiveTabs.isNotEmpty()) {
-        val showInactiveTabsAutoCloseDialog =
-            shouldShowInactiveTabsAutoCloseDialog(inactiveTabs.size)
-        var showAutoCloseDialog by remember { mutableStateOf(showInactiveTabsAutoCloseDialog) }
-
-        val optionalInactiveTabsHeader: (@Composable () -> Unit)? = if (inactiveTabs.isEmpty()) {
-            null
-        } else {
-            {
-                InactiveTabsList(
-                    inactiveTabs = inactiveTabs,
-                    expanded = inactiveTabsExpanded,
-                    showAutoCloseDialog = showAutoCloseDialog,
-                    showCFR = shouldShowInactiveTabsCFR(),
-                    onHeaderClick = onInactiveTabsHeaderClick,
-                    onDeleteAllButtonClick = onDeleteAllInactiveTabsClick,
-                    onAutoCloseDismissClick = {
-                        onInactiveTabAutoCloseDialogCloseButtonClick()
-                        showAutoCloseDialog = !showAutoCloseDialog
-                    },
-                    onEnableAutoCloseClick = {
-                        onEnableInactiveTabAutoCloseClick()
-                        showAutoCloseDialog = !showAutoCloseDialog
-                    },
-                    onTabClick = onInactiveTabClick,
-                    onTabCloseClick = onInactiveTabClose,
-                    onCFRShown = onInactiveTabsCFRShown,
-                    onCFRClick = onInactiveTabsCFRClick,
-                    onCFRDismiss = onInactiveTabsCFRDismiss,
-                )
-            }
-        }
-
-        if (showInactiveTabsAutoCloseDialog) {
-            onInactiveTabsAutoCloseDialogShown()
-        }
-
-        TabLayout(
-            tabs = normalTabs,
-            displayTabsInGrid = displayTabsInGrid,
-            selectedTabId = selectedTabId,
-            selectionMode = selectionMode,
-            modifier = Modifier.testTag(TabsTrayTestTag.normalTabsList),
-            onTabClose = onTabClose,
-            onTabMediaClick = onTabMediaClick,
-            onTabClick = onTabClick,
-            onTabLongClick = onTabLongClick,
-            header = optionalInactiveTabsHeader,
-            onTabDragStart = { tabsTrayStore.dispatch(TabsTrayAction.ExitSelectMode) },
-            onMove = onMove,
-        )
-    } else {
-        EmptyTabPage(isPrivate = false)
-    }
-}
-
-@Composable
-@Suppress("LongParameterList")
-private fun PrivateTabsPage(
-    browserStore: BrowserStore,
-    tabsTrayStore: TabsTrayStore,
-    displayTabsInGrid: Boolean,
-    onTabClose: (TabSessionState) -> Unit,
-    onTabMediaClick: (TabSessionState) -> Unit,
-    onTabClick: (TabSessionState) -> Unit,
-    onTabLongClick: (TabSessionState) -> Unit,
-    onMove: (String, String?, Boolean) -> Unit,
-) {
-    val selectedTabId = browserStore
-        .observeAsComposableState { state -> state.selectedTabId }.value
-    val privateTabs = tabsTrayStore
-        .observeAsComposableState { state -> state.privateTabs }.value ?: emptyList()
-    val selectionMode = tabsTrayStore
-        .observeAsComposableState { state -> state.mode }.value ?: TabsTrayState.Mode.Normal
-
-    if (privateTabs.isNotEmpty()) {
-        TabLayout(
-            tabs = privateTabs,
-            displayTabsInGrid = displayTabsInGrid,
-            selectedTabId = selectedTabId,
-            selectionMode = selectionMode,
-            modifier = Modifier.testTag(TabsTrayTestTag.privateTabsList),
-            onTabClose = onTabClose,
-            onTabMediaClick = onTabMediaClick,
-            onTabClick = onTabClick,
-            onTabLongClick = onTabLongClick,
-            onTabDragStart = {
-                // Because we don't currently support selection mode for private tabs,
-                // there's no need to exit selection mode when dragging tabs.
-            },
-            onMove = onMove,
-        )
-    } else {
-        EmptyTabPage(isPrivate = true)
-    }
-}
-
-@Composable
-private fun SyncedTabsPage(
-    tabsTrayStore: TabsTrayStore,
-    onTabClick: OnSyncedTabClick,
-    onTabClose: OnSyncedTabClose,
-) {
-    val syncedTabs = tabsTrayStore
-        .observeAsComposableState { state -> state.syncedTabs }.value ?: emptyList()
-
-    SyncedTabsList(
-        syncedTabs = syncedTabs,
-        onTabClick = onTabClick,
-        onTabCloseClick = onTabClose,
-    )
-}
-
-@Composable
-private fun EmptyTabPage(isPrivate: Boolean) {
-    val testTag: String
-    val emptyTextId: Int
-    if (isPrivate) {
-        testTag = TabsTrayTestTag.emptyPrivateTabsList
-        emptyTextId = R.string.no_private_tabs_description
-    } else {
-        testTag = TabsTrayTestTag.emptyNormalTabsList
-        emptyTextId = R.string.no_open_tabs_description
-    }
-
-    Box(
-        modifier = Modifier
-            .fillMaxSize()
-            .testTag(testTag),
-    ) {
-        Text(
-            text = stringResource(id = emptyTextId),
-            modifier = Modifier
-                .align(Alignment.TopCenter)
-                .padding(top = 80.dp),
-            color = FirefoxTheme.colors.textSecondary,
-            style = FirefoxTheme.typography.body1,
-        )
-    }
-}
-
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTrayPreview() {
     val tabs = generateFakeTabsList()
@@ -449,7 +300,7 @@ private fun TabsTrayPreview() {
 }
 
 @Suppress("MagicNumber")
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTrayMultiSelectPreview() {
     val tabs = generateFakeTabsList()
@@ -460,7 +311,7 @@ private fun TabsTrayMultiSelectPreview() {
     )
 }
 
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTrayInactiveTabsPreview() {
     TabsTrayPreviewRoot(
@@ -471,7 +322,7 @@ private fun TabsTrayInactiveTabsPreview() {
     )
 }
 
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTrayPrivateTabsPreview() {
     TabsTrayPreviewRoot(
@@ -480,7 +331,7 @@ private fun TabsTrayPrivateTabsPreview() {
     )
 }
 
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTraySyncedTabsPreview() {
     TabsTrayPreviewRoot(
@@ -489,7 +340,7 @@ private fun TabsTraySyncedTabsPreview() {
     )
 }
 
-@LightDarkPreview
+@PreviewLightDark
 @Composable
 private fun TabsTrayAutoCloseBannerPreview() {
     TabsTrayPreviewRoot(
@@ -498,6 +349,16 @@ private fun TabsTrayAutoCloseBannerPreview() {
     )
 }
 
+@PreviewLightDark
+@Composable
+private fun TabsTrayLockedPreview() {
+    TabsTrayPreviewRoot(
+        privateTabs = generateFakeTabsList(isPrivate = true),
+        selectedPage = Page.PrivateTabs,
+        isPbmLocked = true,
+    )
+}
+
 @Suppress("LongMethod")
 @Composable
 private fun TabsTrayPreviewRoot(
@@ -512,107 +373,139 @@ private fun TabsTrayPreviewRoot(
     inactiveTabsExpanded: Boolean = false,
     showInactiveTabsAutoCloseDialog: Boolean = false,
     showTabAutoCloseBanner: Boolean = false,
+    isPbmLocked: Boolean = false,
+    isSignedIn: Boolean = true,
 ) {
-    var selectedPageState by remember { mutableStateOf(selectedPage) }
-    val normalTabsState = remember { normalTabs.toMutableStateList() }
-    val inactiveTabsState = remember { inactiveTabs.toMutableStateList() }
-    val privateTabsState = remember { privateTabs.toMutableStateList() }
-    val syncedTabsState = remember { syncedTabs.toMutableStateList() }
-    var inactiveTabsExpandedState by remember { mutableStateOf(inactiveTabsExpanded) }
     var showInactiveTabsAutoCloseDialogState by remember { mutableStateOf(showInactiveTabsAutoCloseDialog) }
 
-    val appStore = AppStore(
-        initialState = AppState(
-            inactiveTabsExpanded = inactiveTabsExpandedState,
-        ),
-    )
-    val browserStore = BrowserStore(
-        initialState = BrowserState(
-            tabs = normalTabs + privateTabs,
-            selectedTabId = selectedTabId,
-        ),
-    )
-    val tabsTrayStore = TabsTrayStore(
-        initialState = TabsTrayState(
-            selectedPage = selectedPageState,
-            mode = mode,
-            inactiveTabs = inactiveTabsState,
-            normalTabs = normalTabsState,
-            privateTabs = privateTabsState,
-            syncedTabs = syncedTabsState,
-        ),
-    )
+    val tabsTrayStore = remember {
+        TabsTrayStore(
+            initialState = TabsTrayState(
+                selectedPage = selectedPage,
+                mode = mode,
+                inactiveTabs = inactiveTabs,
+                inactiveTabsExpanded = inactiveTabsExpanded,
+                normalTabs = normalTabs,
+                privateTabs = privateTabs,
+                syncedTabs = syncedTabs,
+                selectedTabId = selectedTabId,
+            ),
+        )
+    }
 
     FirefoxTheme {
-        TabsTray(
-            appStore = appStore,
-            browserStore = browserStore,
-            tabsTrayStore = tabsTrayStore,
-            displayTabsInGrid = displayTabsInGrid,
-            isInDebugMode = false,
-            shouldShowInactiveTabsAutoCloseDialog = { true },
-            shouldShowTabAutoCloseBanner = showTabAutoCloseBanner,
-            onTabPageClick = { page ->
-                selectedPageState = page
-            },
-            onTabClose = { tab ->
-                if (tab.isNormalTab()) {
-                    normalTabsState.remove(tab)
-                } else {
-                    privateTabsState.remove(tab)
-                }
-            },
-            onTabMediaClick = {},
-            onTabClick = { tab ->
-                when (tabsTrayStore.state.mode) {
-                    TabsTrayState.Mode.Normal -> {}
-                    is TabsTrayState.Mode.Select -> {
-                        if (tabsTrayStore.state.mode.selectedTabs.contains(tab)) {
-                            tabsTrayStore.dispatch(TabsTrayAction.RemoveSelectTab(tab))
-                        } else {
-                            tabsTrayStore.dispatch(TabsTrayAction.AddSelectTab(tab))
+        Box {
+            TabsTray(
+                tabsTrayStore = tabsTrayStore,
+                displayTabsInGrid = displayTabsInGrid,
+                isInDebugMode = false,
+                shouldShowInactiveTabsAutoCloseDialog = { true },
+                shouldShowTabAutoCloseBanner = showTabAutoCloseBanner,
+                isPbmLocked = isPbmLocked,
+                onTabPageClick = { page ->
+                    tabsTrayStore.dispatch(TabsTrayAction.PageSelected(page))
+                },
+                onTabClose = { tab ->
+                    if (tab.isNormalTab()) {
+                        val newTabs = tabsTrayStore.state.normalTabs - tab
+                        tabsTrayStore.dispatch(TabsTrayAction.UpdateNormalTabs(newTabs))
+                    } else {
+                        val newTabs = tabsTrayStore.state.privateTabs - tab
+                        tabsTrayStore.dispatch(TabsTrayAction.UpdatePrivateTabs(newTabs))
+                    }
+                },
+                onTabMediaClick = {},
+                onTabClick = { tab ->
+                    when (tabsTrayStore.state.mode) {
+                        TabsTrayState.Mode.Normal -> {
+                            tabsTrayStore.dispatch(TabsTrayAction.UpdateSelectedTabId(tabId = tab.id))
+                        }
+
+                        is TabsTrayState.Mode.Select -> {
+                            if (tabsTrayStore.state.mode.selectedTabs.contains(tab)) {
+                                tabsTrayStore.dispatch(TabsTrayAction.RemoveSelectTab(tab))
+                            } else {
+                                tabsTrayStore.dispatch(TabsTrayAction.AddSelectTab(tab))
+                            }
                         }
                     }
-                }
-            },
-            onTabLongClick = { tab ->
-                tabsTrayStore.dispatch(TabsTrayAction.AddSelectTab(tab))
-            },
-            onInactiveTabsHeaderClick = {
-                inactiveTabsExpandedState = !inactiveTabsExpandedState
-            },
-            onDeleteAllInactiveTabsClick = inactiveTabsState::clear,
-            onInactiveTabsAutoCloseDialogShown = {},
-            onInactiveTabAutoCloseDialogCloseButtonClick = {
-                showInactiveTabsAutoCloseDialogState = !showInactiveTabsAutoCloseDialogState
-            },
-            onEnableInactiveTabAutoCloseClick = {
-                showInactiveTabsAutoCloseDialogState = !showInactiveTabsAutoCloseDialogState
-            },
-            onInactiveTabClick = {},
-            onInactiveTabClose = inactiveTabsState::remove,
-            onSyncedTabClick = {},
-            onSyncedTabClose = { _, _ -> },
-            onSaveToCollectionClick = {},
-            onShareSelectedTabsClick = {},
-            onShareAllTabsClick = {},
-            onTabSettingsClick = {},
-            onRecentlyClosedClick = {},
-            onAccountSettingsClick = {},
-            onDeleteAllTabsClick = {},
-            onDeleteSelectedTabsClick = {},
-            onBookmarkSelectedTabsClick = {},
-            onForceSelectedTabsAsInactiveClick = {},
-            onTabsTrayDismiss = {},
-            onTabAutoCloseBannerViewOptionsClick = {},
-            onTabAutoCloseBannerDismiss = {},
-            onTabAutoCloseBannerShown = {},
-            onMove = { _, _, _ -> },
-            shouldShowInactiveTabsCFR = { false },
-            onInactiveTabsCFRShown = {},
-            onInactiveTabsCFRClick = {},
-            onInactiveTabsCFRDismiss = {},
-        )
+                },
+                onTabLongClick = { tab ->
+                    tabsTrayStore.dispatch(TabsTrayAction.AddSelectTab(tab))
+                },
+                onInactiveTabsHeaderClick = { expanded ->
+                    tabsTrayStore.dispatch(TabsTrayAction.UpdateInactiveExpanded(expanded))
+                },
+                onDeleteAllInactiveTabsClick = {
+                    tabsTrayStore.dispatch(TabsTrayAction.UpdateInactiveTabs(emptyList()))
+                },
+                onInactiveTabsAutoCloseDialogShown = {},
+                onInactiveTabAutoCloseDialogCloseButtonClick = {
+                    showInactiveTabsAutoCloseDialogState = !showInactiveTabsAutoCloseDialogState
+                },
+                onEnableInactiveTabAutoCloseClick = {
+                    showInactiveTabsAutoCloseDialogState = !showInactiveTabsAutoCloseDialogState
+                },
+                onInactiveTabClick = {},
+                onInactiveTabClose = { tab ->
+                    val newTabs = tabsTrayStore.state.inactiveTabs - tab
+                    tabsTrayStore.dispatch(TabsTrayAction.UpdateInactiveTabs(newTabs))
+                },
+                onSyncedTabClick = {},
+                onSyncedTabClose = { _, _ -> },
+                onSaveToCollectionClick = {},
+                onShareSelectedTabsClick = {},
+                onShareAllTabsClick = {},
+                onTabSettingsClick = {},
+                onRecentlyClosedClick = {},
+                onAccountSettingsClick = {},
+                onDeleteAllTabsClick = {},
+                onDeleteSelectedTabsClick = {},
+                onBookmarkSelectedTabsClick = {},
+                onForceSelectedTabsAsInactiveClick = {},
+                onTabsTrayDismiss = {},
+                onTabAutoCloseBannerViewOptionsClick = {},
+                onTabsTrayPbmLockedClick = {},
+                onTabsTrayPbmLockedDismiss = {},
+                onTabAutoCloseBannerDismiss = {},
+                onTabAutoCloseBannerShown = {},
+                onMove = { _, _, _ -> },
+                shouldShowInactiveTabsCFR = { false },
+                onInactiveTabsCFRShown = {},
+                onInactiveTabsCFRClick = {},
+                onInactiveTabsCFRDismiss = {},
+                shouldShowLockPbmBanner = false,
+                onUnlockPbmClick = {},
+            )
+
+            Box(modifier = Modifier.align(alignment = Alignment.BottomEnd)) {
+                TabsTrayFab(
+                    tabsTrayStore = tabsTrayStore,
+                    isSignedIn = isSignedIn,
+                    isPbmLocked = isPbmLocked,
+                    onNormalTabsFabClicked = {
+                        val newTab = createTab(
+                            url = "www.mozilla.com",
+                            private = false,
+                        )
+                        val allTabs = tabsTrayStore.state.normalTabs + newTab
+                        tabsTrayStore.dispatch(TabsTrayAction.UpdateNormalTabs(allTabs))
+                    },
+                    onPrivateTabsFabClicked = {
+                        val newTab = createTab(
+                            url = "www.mozilla.com",
+                            private = true,
+                        )
+                        val allTabs = tabsTrayStore.state.privateTabs + newTab
+                        tabsTrayStore.dispatch(TabsTrayAction.UpdatePrivateTabs(allTabs))
+                    },
+                    onSyncedTabsFabClicked = {
+                        val newSyncedTabList = tabsTrayStore.state.syncedTabs + generateFakeSyncedTabsList()
+                        tabsTrayStore.dispatch(TabsTrayAction.UpdateSyncedTabs(newSyncedTabList))
+                    },
+                )
+            }
+        }
     }
 }
 
