@@ -5,7 +5,7 @@
 package org.mozilla.fenix.translations
 
 import android.content.res.Configuration
-import androidx.compose.foundation.clickable
+import androidx.compose.foundation.background
 import androidx.compose.foundation.layout.Arrangement
 import androidx.compose.foundation.layout.Box
 import androidx.compose.foundation.layout.Column
@@ -14,50 +14,42 @@ import androidx.compose.foundation.layout.Spacer
 import androidx.compose.foundation.layout.fillMaxWidth
 import androidx.compose.foundation.layout.height
 import androidx.compose.foundation.layout.padding
-import androidx.compose.foundation.layout.requiredSizeIn
 import androidx.compose.foundation.layout.size
 import androidx.compose.foundation.layout.width
 import androidx.compose.foundation.layout.wrapContentSize
-import androidx.compose.material.Divider
-import androidx.compose.material.Icon
-import androidx.compose.material.IconButton
-import androidx.compose.material.Text
+import androidx.compose.material3.Icon
+import androidx.compose.material3.IconButton
+import androidx.compose.material3.Text
 import androidx.compose.runtime.Composable
-import androidx.compose.runtime.getValue
-import androidx.compose.runtime.mutableStateOf
-import androidx.compose.runtime.remember
-import androidx.compose.runtime.setValue
 import androidx.compose.ui.Alignment
 import androidx.compose.ui.Modifier
-import androidx.compose.ui.layout.onGloballyPositioned
 import androidx.compose.ui.platform.LocalConfiguration
 import androidx.compose.ui.platform.LocalDensity
 import androidx.compose.ui.res.painterResource
 import androidx.compose.ui.res.stringResource
-import androidx.compose.ui.semantics.Role
 import androidx.compose.ui.semantics.heading
-import androidx.compose.ui.semantics.role
 import androidx.compose.ui.semantics.semantics
+import androidx.compose.ui.text.TextStyle
+import androidx.compose.ui.text.rememberTextMeasurer
 import androidx.compose.ui.text.style.TextDecoration
-import androidx.compose.ui.unit.DpOffset
+import androidx.compose.ui.tooling.preview.PreviewLightDark
+import androidx.compose.ui.unit.Dp
 import androidx.compose.ui.unit.dp
+import mozilla.components.compose.base.Dropdown
+import mozilla.components.compose.base.button.FilledButton
+import mozilla.components.compose.base.button.TextButton
+import mozilla.components.compose.base.menu.MenuItem.CheckableItem
+import mozilla.components.compose.base.text.Text
 import mozilla.components.concept.engine.translate.Language
 import mozilla.components.concept.engine.translate.TranslationError
 import org.mozilla.fenix.R
-import org.mozilla.fenix.compose.BetaLabel
-import org.mozilla.fenix.compose.ContextualMenu
+import org.mozilla.fenix.compose.InfoCard
+import org.mozilla.fenix.compose.InfoType
 import org.mozilla.fenix.compose.LinkText
 import org.mozilla.fenix.compose.LinkTextState
-import org.mozilla.fenix.compose.MenuItem
-import org.mozilla.fenix.compose.annotation.LightDarkPreview
-import org.mozilla.fenix.compose.button.PrimaryButton
-import org.mozilla.fenix.compose.button.TextButton
-import org.mozilla.fenix.shopping.ui.ReviewQualityCheckInfoCard
-import org.mozilla.fenix.shopping.ui.ReviewQualityCheckInfoType
 import org.mozilla.fenix.theme.FirefoxTheme
 import java.util.Locale
-
-private val ICON_SIZE = 24.dp
+import mozilla.components.ui.icons.R as iconsR
 
 /**
  * Firefox Translations bottom sheet dialog.
@@ -87,16 +79,7 @@ fun TranslationsDialogBottomSheet(
     onFromDropdownSelected: (Language) -> Unit,
     onToDropdownSelected: (Language) -> Unit,
 ) {
-    Column(
-        modifier = Modifier.padding(16.dp),
-    ) {
-        if (LocalConfiguration.current.orientation == Configuration.ORIENTATION_PORTRAIT) {
-            BetaLabel(
-                modifier = Modifier
-                    .padding(bottom = 8.dp),
-            )
-        }
-
+    Column(modifier = Modifier.padding(16.dp)) {
         TranslationsDialogHeader(
             title = if (translationsDialogState.isTranslated && translationsDialogState.translatedPageTitle != null) {
                 translationsDialogState.translatedPageTitle
@@ -109,9 +92,8 @@ fun TranslationsDialogBottomSheet(
             onSettingClicked = onSettingClicked,
         )
 
-        Spacer(modifier = Modifier.height(8.dp))
-
         if (showFirstTimeFlow) {
+            Spacer(modifier = Modifier.height(8.dp))
             TranslationsDialogInfoMessage(
                 learnMoreUrl = learnMoreUrl,
                 onLearnMoreClicked = onLearnMoreClicked,
@@ -165,7 +147,7 @@ private fun DialogContentBaseOnTranslationState(
             initialTo = translationsDialogState.initialTo,
         )
     } else {
-        Spacer(modifier = Modifier.height(14.dp))
+        Spacer(modifier = Modifier.height(16.dp))
 
         TranslationsDialogContent(
             translateFromLanguages = translationsDialogState.fromLanguages,
@@ -209,7 +191,7 @@ private fun DialogContentTranslated(
     positiveButtonType: PositiveButtonType? = null,
     initialTo: Language? = null,
 ) {
-    Spacer(modifier = Modifier.height(14.dp))
+    Spacer(modifier = Modifier.height(16.dp))
 
     TranslationsDialogContent(
         translateToLanguages = translateToLanguages,
@@ -253,7 +235,7 @@ private fun DialogContentAnErrorOccurred(
             documentLangDisplayName = translationsDialogState.documentLangDisplayName,
         )
 
-        Spacer(modifier = Modifier.height(14.dp))
+        Spacer(modifier = Modifier.height(16.dp))
 
         if (translationError !is TranslationError.CouldNotLoadLanguagesError) {
             TranslationsDialogContent(
@@ -288,6 +270,8 @@ private fun DialogContentAnErrorOccurred(
                 translationsDialogState.positiveButtonType
             }
 
+        Spacer(modifier = Modifier.height(16.dp))
+
         TranslationsDialogActionButtons(
             positiveButtonText = positiveButtonTitle,
             negativeButtonText = negativeButtonTitle,
@@ -308,9 +292,19 @@ private fun TranslationsDialogContent(
     onFromDropdownSelected: (Language) -> Unit,
     onToDropdownSelected: (Language) -> Unit,
 ) {
+    val allLanguagesList = mutableListOf<Language>()
+
+    with(allLanguagesList) {
+        translateToLanguages?.let { addAll(it) }
+        translateFromLanguages?.let { addAll(it) }
+    }
+
+    val longestLanguageSize = getLongestLanguageWidth(allLanguagesList, FirefoxTheme.typography.subtitle1)
+
     when (LocalConfiguration.current.orientation) {
         Configuration.ORIENTATION_LANDSCAPE -> {
             TranslationsDialogContentInLandscapeMode(
+                longestLanguageSize = longestLanguageSize,
                 translateFromLanguages = translateFromLanguages,
                 translateToLanguages = translateToLanguages,
                 initialFrom = initialFrom,
@@ -323,6 +317,7 @@ private fun TranslationsDialogContent(
 
         else -> {
             TranslationsDialogContentInPortraitMode(
+                longestLanguageSize = longestLanguageSize,
                 translateFromLanguages = translateFromLanguages,
                 translateToLanguages = translateToLanguages,
                 initialFrom = initialFrom,
@@ -333,12 +328,11 @@ private fun TranslationsDialogContent(
             )
         }
     }
-
-    Spacer(modifier = Modifier.height(16.dp))
 }
 
 @Composable
 private fun TranslationsDialogContentInPortraitMode(
+    longestLanguageSize: Dp,
     translateFromLanguages: List<Language>? = null,
     translateToLanguages: List<Language>? = null,
     initialFrom: Language? = null,
@@ -357,13 +351,19 @@ private fun TranslationsDialogContentInPortraitMode(
             }
 
         translateFromLanguages?.let {
-            TranslationsDropdown(
-                header = header,
+            Dropdown(
+                label = header,
+                placeholder = stringResource(R.string.translations_bottom_sheet_default_dropdown_selection),
+                dropdownItems = getContextMenuItems(
+                    translateLanguages = it,
+                    selectedLanguage = initialFrom,
+                    onClickItem = { language ->
+                        onFromDropdownSelected(language)
+                    },
+                ),
                 modifier = Modifier.fillMaxWidth(),
+                dropdownMenuTextWidth = longestLanguageSize,
                 isInLandscapeMode = false,
-                translateLanguages = translateFromLanguages,
-                initiallySelected = initialFrom,
-                onLanguageSelection = onFromDropdownSelected,
             )
 
             Spacer(modifier = Modifier.height(16.dp))
@@ -371,13 +371,19 @@ private fun TranslationsDialogContentInPortraitMode(
 
         if (translationError !is TranslationError.LanguageNotSupportedError) {
             translateToLanguages?.let {
-                TranslationsDropdown(
-                    header = stringResource(id = R.string.translations_bottom_sheet_translate_to),
+                Dropdown(
+                    label = stringResource(id = R.string.translations_bottom_sheet_translate_to),
+                    placeholder = stringResource(R.string.translations_bottom_sheet_default_dropdown_selection),
+                    dropdownItems = getContextMenuItems(
+                        translateLanguages = it,
+                        selectedLanguage = initialTo,
+                        onClickItem = { language ->
+                            onToDropdownSelected(language)
+                        },
+                    ),
                     modifier = Modifier.fillMaxWidth(),
+                    dropdownMenuTextWidth = longestLanguageSize,
                     isInLandscapeMode = false,
-                    translateLanguages = it,
-                    initiallySelected = initialTo,
-                    onLanguageSelection = onToDropdownSelected,
                 )
             }
         }
@@ -386,6 +392,7 @@ private fun TranslationsDialogContentInPortraitMode(
 
 @Composable
 private fun TranslationsDialogContentInLandscapeMode(
+    longestLanguageSize: Dp,
     translateFromLanguages: List<Language>? = null,
     translateToLanguages: List<Language>? = null,
     initialFrom: Language? = null,
@@ -405,13 +412,19 @@ private fun TranslationsDialogContentInLandscapeMode(
                 }
 
             translateFromLanguages?.let {
-                TranslationsDropdown(
-                    header = header,
+                Dropdown(
+                    label = header,
+                    placeholder = stringResource(R.string.translations_bottom_sheet_default_dropdown_selection),
+                    dropdownItems = getContextMenuItems(
+                        translateLanguages = it,
+                        selectedLanguage = initialFrom,
+                        onClickItem = { language ->
+                            onFromDropdownSelected(language)
+                        },
+                    ),
                     modifier = Modifier.weight(1f),
+                    dropdownMenuTextWidth = longestLanguageSize,
                     isInLandscapeMode = true,
-                    translateLanguages = translateFromLanguages,
-                    initiallySelected = initialFrom,
-                    onLanguageSelection = onFromDropdownSelected,
                 )
 
                 Spacer(modifier = Modifier.width(16.dp))
@@ -419,13 +432,19 @@ private fun TranslationsDialogContentInLandscapeMode(
 
             if (translationError !is TranslationError.LanguageNotSupportedError) {
                 translateToLanguages?.let {
-                    TranslationsDropdown(
-                        header = stringResource(id = R.string.translations_bottom_sheet_translate_to),
+                    Dropdown(
+                        label = stringResource(id = R.string.translations_bottom_sheet_translate_to),
+                        placeholder = stringResource(R.string.translations_bottom_sheet_default_dropdown_selection),
+                        dropdownItems = getContextMenuItems(
+                            translateLanguages = it,
+                            selectedLanguage = initialTo,
+                            onClickItem = { language ->
+                                onToDropdownSelected(language)
+                            },
+                        ),
                         modifier = Modifier.weight(1f),
+                        dropdownMenuTextWidth = longestLanguageSize,
                         isInLandscapeMode = true,
-                        translateLanguages = it,
-                        initiallySelected = initialTo,
-                        onLanguageSelection = onToDropdownSelected,
                     )
                 }
             }
@@ -439,8 +458,29 @@ private fun TranslationsDialogHeader(
     showPageSettings: Boolean,
     onSettingClicked: () -> Unit,
 ) {
+    if (showPageSettings) {
+        Row(
+            modifier = Modifier.fillMaxWidth(),
+            horizontalArrangement = Arrangement.End,
+        ) {
+            IconButton(
+                onClick = { onSettingClicked() },
+                modifier = Modifier.size(24.dp),
+            ) {
+                Icon(
+                    painter = painterResource(id = iconsR.drawable.mozac_ic_settings_24),
+                    contentDescription = stringResource(
+                        id = R.string.translation_option_bottom_sheet_title_heading,
+                    ),
+                    tint = FirefoxTheme.colors.iconPrimary,
+                )
+            }
+        }
+    }
+
     Row(
         verticalAlignment = Alignment.CenterVertically,
+        modifier = Modifier.padding(top = if (showPageSettings) { 12.dp } else { 0.dp }),
     ) {
         Text(
             text = title,
@@ -450,25 +490,6 @@ private fun TranslationsDialogHeader(
             color = FirefoxTheme.colors.textPrimary,
             style = FirefoxTheme.typography.headline7,
         )
-
-        if (LocalConfiguration.current.orientation == Configuration.ORIENTATION_LANDSCAPE) {
-            BetaLabel()
-        }
-
-        Spacer(modifier = Modifier.width(8.dp))
-
-        if (showPageSettings) {
-            IconButton(
-                onClick = { onSettingClicked() },
-                modifier = Modifier.size(24.dp),
-            ) {
-                Icon(
-                    painter = painterResource(id = R.drawable.mozac_ic_settings_24),
-                    contentDescription = stringResource(id = R.string.translation_option_bottom_sheet_title_heading),
-                    tint = FirefoxTheme.colors.iconPrimary,
-                )
-            }
-        }
     }
 }
 
@@ -478,23 +499,23 @@ private fun TranslationErrorWarning(
     documentLangDisplayName: String? = null,
 ) {
     val modifier = Modifier
-        .padding(top = 8.dp)
+        .padding(top = 12.dp)
         .fillMaxWidth()
 
     when (translationError) {
         is TranslationError.CouldNotTranslateError -> {
-            ReviewQualityCheckInfoCard(
+            InfoCard(
                 description = stringResource(id = R.string.translation_error_could_not_translate_warning_text),
-                type = ReviewQualityCheckInfoType.Error,
+                type = InfoType.Error,
                 verticalRowAlignment = Alignment.CenterVertically,
                 modifier = modifier,
             )
         }
 
         is TranslationError.CouldNotLoadLanguagesError -> {
-            ReviewQualityCheckInfoCard(
+            InfoCard(
                 description = stringResource(id = R.string.translation_error_could_not_load_languages_warning_text),
-                type = ReviewQualityCheckInfoType.Error,
+                type = InfoType.Error,
                 verticalRowAlignment = Alignment.CenterVertically,
                 modifier = modifier,
             )
@@ -502,12 +523,12 @@ private fun TranslationErrorWarning(
 
         is TranslationError.LanguageNotSupportedError -> {
             documentLangDisplayName?.let {
-                ReviewQualityCheckInfoCard(
+                InfoCard(
                     description = stringResource(
                         id = R.string.translation_error_language_not_supported_warning_text,
                         it,
                     ),
-                    type = ReviewQualityCheckInfoType.Info,
+                    type = InfoType.Info,
                     verticalRowAlignment = Alignment.CenterVertically,
                     modifier = modifier,
                 )
@@ -539,7 +560,7 @@ private fun TranslationsDialogInfoMessage(
                 learnMoreText,
             ),
             linkTextStates = listOf(learnMoreState),
-            style = FirefoxTheme.typography.subtitle1.copy(
+            style = FirefoxTheme.typography.body2.copy(
                 color = FirefoxTheme.colors.textPrimary,
             ),
             linkTextDecoration = TextDecoration.Underline,
@@ -547,104 +568,6 @@ private fun TranslationsDialogInfoMessage(
     }
 }
 
-@Suppress("LongMethod")
-@Composable
-private fun TranslationsDropdown(
-    header: String,
-    translateLanguages: List<Language>,
-    modifier: Modifier = Modifier,
-    isInLandscapeMode: Boolean,
-    initiallySelected: Language? = null,
-    onLanguageSelection: (Language) -> Unit,
-) {
-    val density = LocalDensity.current
-
-    var expanded by remember { mutableStateOf(false) }
-
-    var contextMenuWidthDp by remember {
-        mutableStateOf(0.dp)
-    }
-
-    Column(
-        modifier = modifier
-            .clickable {
-                expanded = true
-            }
-            .semantics { role = Role.DropdownList },
-    ) {
-        Text(
-            text = header,
-            modifier = Modifier.wrapContentSize(),
-            color = FirefoxTheme.colors.textPrimary,
-            style = FirefoxTheme.typography.caption,
-        )
-
-        Spacer(modifier = Modifier.height(4.dp))
-
-        var initialValue =
-            stringResource(R.string.translations_bottom_sheet_default_dropdown_selection)
-        initiallySelected?.localizedDisplayName?.let {
-            initialValue = it
-        }
-
-        Row {
-            Text(
-                text = initialValue,
-                modifier = Modifier.weight(1f),
-                color = FirefoxTheme.colors.textPrimary,
-                style = FirefoxTheme.typography.subtitle1,
-            )
-
-            Spacer(modifier = Modifier.width(10.dp))
-
-            Box {
-                Icon(
-                    painter = painterResource(id = R.drawable.mozac_ic_dropdown_arrow),
-                    contentDescription = null,
-                    tint = FirefoxTheme.colors.iconPrimary,
-                )
-
-                if (expanded) {
-                    ContextualMenu(
-                        showMenu = expanded,
-                        onDismissRequest = {
-                            expanded = false
-                        },
-                        menuItems = getContextMenuItems(
-                            translateLanguages = translateLanguages,
-                            selectedLanguage = initiallySelected,
-                            onClickItem = {
-                                onLanguageSelection(it)
-                            },
-                        ),
-                        modifier = Modifier
-                            .onGloballyPositioned { coordinates ->
-                                contextMenuWidthDp = with(density) {
-                                    coordinates.size.width.toDp()
-                                }
-                            }
-                            .requiredSizeIn(maxHeight = 200.dp)
-                            .padding(horizontal = if (initiallySelected == null) 36.dp else 4.dp),
-                        offset = if (isInLandscapeMode) {
-                            DpOffset(
-                                -contextMenuWidthDp + ICON_SIZE,
-                                ICON_SIZE,
-                            )
-                        } else {
-                            DpOffset(
-                                0.dp,
-                                ICON_SIZE,
-                            )
-                        },
-                    )
-                }
-            }
-        }
-
-        Divider(color = FirefoxTheme.colors.formDefault)
-    }
-}
-
 @Composable
 private fun getTranslationsDialogTitle(
     showFirstTime: Boolean = false,
@@ -661,23 +584,18 @@ private fun getContextMenuItems(
     translateLanguages: List<Language>,
     selectedLanguage: Language? = null,
     onClickItem: (Language) -> Unit,
-): List<MenuItem> {
-    val menuItems = mutableListOf<MenuItem>()
-    translateLanguages.map { item ->
+): List<CheckableItem> =
+    translateLanguages.mapNotNull { item ->
         item.localizedDisplayName?.let {
-            menuItems.add(
-                MenuItem(
-                    title = it,
-                    isChecked = item == selectedLanguage,
-                    onClick = {
-                        onClickItem(item)
-                    },
-                ),
+            CheckableItem(
+                text = Text.String(it),
+                isChecked = item == selectedLanguage,
+                onClick = {
+                    onClickItem(item)
+                },
             )
         }
     }
-    return menuItems
-}
 
 @Composable
 private fun TranslationsDialogActionButtons(
@@ -696,7 +614,6 @@ private fun TranslationsDialogActionButtons(
             text = negativeButtonText,
             modifier = Modifier,
             onClick = onNegativeButtonClicked,
-            upperCaseText = false,
         )
 
         Spacer(modifier = Modifier.width(10.dp))
@@ -708,12 +625,12 @@ private fun TranslationsDialogActionButtons(
                     contentDescription = stringResource(
                         id = R.string.translations_bottom_sheet_translating_in_progress_content_description,
                     ),
-                    icon = painterResource(id = R.drawable.mozac_ic_sync_24),
+                    icon = painterResource(id = iconsR.drawable.mozac_ic_sync_24),
                 )
             }
 
             PositiveButtonType.Enabled -> {
-                PrimaryButton(
+                FilledButton(
                     text = positiveButtonText,
                     modifier = Modifier.wrapContentSize(),
                 ) {
@@ -722,7 +639,7 @@ private fun TranslationsDialogActionButtons(
             }
 
             else -> {
-                PrimaryButton(
+                FilledButton(
                     text = positiveButtonText,
                     enabled = false,
                     modifier = Modifier.wrapContentSize(),
@@ -735,25 +652,39 @@ private fun TranslationsDialogActionButtons(
 }
 
 @Composable
-@LightDarkPreview
+private fun getLongestLanguageWidth(languages: List<Language>, style: TextStyle): Dp {
+    val textMeasurer = rememberTextMeasurer(cacheSize = languages.size)
+
+    val maxWidth = languages.mapNotNull { it.localizedDisplayName }
+        .maxOfOrNull { text ->
+            textMeasurer.measure(text = text, style = style).size.width
+        }
+
+    return with(LocalDensity.current) { maxWidth?.toDp() ?: 0.dp }
+}
+
+@Composable
+@PreviewLightDark
 private fun TranslationsDialogBottomSheetPreview() {
     FirefoxTheme {
-        TranslationsDialogBottomSheet(
-            translationsDialogState = TranslationsDialogState(
-                positiveButtonType = PositiveButtonType.Enabled,
-                toLanguages = getTranslateToLanguageList(),
-                fromLanguages = getTranslateFromLanguageList(),
-            ),
-            learnMoreUrl = "",
-            showPageSettings = true,
-            showFirstTimeFlow = true,
-            onSettingClicked = {},
-            onLearnMoreClicked = {},
-            onPositiveButtonClicked = {},
-            onNegativeButtonClicked = {},
-            onFromDropdownSelected = {},
-            onToDropdownSelected = {},
-        )
+        Column(modifier = Modifier.background(color = FirefoxTheme.colors.layer1)) {
+            TranslationsDialogBottomSheet(
+                translationsDialogState = TranslationsDialogState(
+                    positiveButtonType = PositiveButtonType.Enabled,
+                    toLanguages = getTranslateToLanguageList(),
+                    fromLanguages = getTranslateFromLanguageList(),
+                ),
+                learnMoreUrl = "",
+                showPageSettings = true,
+                showFirstTimeFlow = true,
+                onSettingClicked = {},
+                onLearnMoreClicked = {},
+                onPositiveButtonClicked = {},
+                onNegativeButtonClicked = {},
+                onFromDropdownSelected = {},
+                onToDropdownSelected = {},
+            )
+        }
     }
 }
 
