@@ -23,8 +23,6 @@ import androidx.compose.ui.platform.LocalDensity
 import androidx.compose.ui.platform.LocalView
 import androidx.compose.ui.platform.ViewCompositionStrategy
 import androidx.compose.ui.unit.dp
-import androidx.core.os.bundleOf
-import androidx.fragment.app.setFragmentResult
 import androidx.navigation.fragment.findNavController
 import androidx.navigation.fragment.navArgs
 import com.google.android.material.bottomsheet.BottomSheetBehavior
@@ -33,14 +31,18 @@ import mozilla.components.browser.state.selector.selectedTab
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.translate.Language
 import mozilla.components.concept.engine.translate.TranslationError
+import mozilla.components.feature.downloads.FileSizeFormatter
 import mozilla.components.lib.state.ext.observeAsComposableState
 import mozilla.components.support.base.feature.ViewBoundFeatureWrapper
 import org.mozilla.fenix.BrowserDirection
 import org.mozilla.fenix.GleanMetrics.Translations
 import org.mozilla.fenix.HomeActivity
 import org.mozilla.fenix.R
+import org.mozilla.fenix.components.AppStore
+import org.mozilla.fenix.components.appstate.AppAction
 import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.requireComponents
+import org.mozilla.fenix.ext.runIfFragmentIsAttached
 import org.mozilla.fenix.ext.settings
 import org.mozilla.fenix.nimbus.FxNimbus
 import org.mozilla.fenix.settings.SupportUtils
@@ -48,6 +50,7 @@ import org.mozilla.fenix.theme.FirefoxTheme
 import org.mozilla.fenix.translations.preferences.downloadlanguages.DownloadLanguageFileDialog
 import org.mozilla.fenix.translations.preferences.downloadlanguages.DownloadLanguageFileDialogType
 import org.mozilla.fenix.translations.preferences.downloadlanguages.DownloadLanguagesFeature
+import com.google.android.material.R as materialR
 
 // Friction should be increased, since peek height on this dialog is to fill the screen.
 private const val DIALOG_FRICTION = .65f
@@ -64,25 +67,34 @@ enum class TranslationsDialogAccessPoint {
  */
 class TranslationsDialogFragment : BottomSheetDialogFragment() {
 
-    private var behavior: BottomSheetBehavior<View>? = null
     private val args by navArgs<TranslationsDialogFragmentArgs>()
+
+    private val appStore: AppStore by lazy { requireComponents.appStore }
     private val browserStore: BrowserStore by lazy { requireComponents.core.store }
+
     private val translationDialogBinding = ViewBoundFeatureWrapper<TranslationsDialogBinding>()
     private val downloadLanguagesFeature =
         ViewBoundFeatureWrapper<DownloadLanguagesFeature>()
+
     private lateinit var translationsDialogStore: TranslationsDialogStore
+
+    private var behavior: BottomSheetBehavior<View>? = null
     private var isTranslationInProgress: Boolean? = null
     private var isDataSaverEnabledAndWifiDisabled = false
 
     override fun onCreateDialog(savedInstanceState: Bundle?): Dialog =
         super.onCreateDialog(savedInstanceState).apply {
             setOnShowListener {
-                val bottomSheet = findViewById<View?>(R.id.design_bottom_sheet)
-                bottomSheet?.setBackgroundResource(android.R.color.transparent)
-                behavior = BottomSheetBehavior.from(bottomSheet)
-                behavior?.peekHeight = resources.displayMetrics.heightPixels
-                behavior?.state = BottomSheetBehavior.STATE_EXPANDED
-                behavior?.hideFriction = DIALOG_FRICTION
+                runIfFragmentIsAttached {
+                    val bottomSheet = findViewById<View?>(materialR.id.design_bottom_sheet)
+                    bottomSheet?.let {
+                        it.setBackgroundResource(android.R.color.transparent)
+                        behavior = BottomSheetBehavior.from(it)
+                        behavior?.peekHeight = resources.displayMetrics.heightPixels
+                        behavior?.state = BottomSheetBehavior.STATE_EXPANDED
+                        behavior?.hideFriction = DIALOG_FRICTION
+                    }
+                }
             }
         }
 
@@ -132,9 +144,9 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                     SupportUtils.SumoTopic.TRANSLATIONS,
                 )
 
-                isTranslationInProgress = translationsDialogState?.isTranslationInProgress
+                isTranslationInProgress = translationsDialogState.isTranslationInProgress
 
-                if (translationsDialogState?.dismissDialogState is DismissDialogState.Dismiss) {
+                if (translationsDialogState.dismissDialogState is DismissDialogState.Dismiss) {
                     dismissDialog()
                 }
 
@@ -145,16 +157,13 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                 var revertAlwaysTranslateLanguageCheckBox by remember {
                     mutableStateOf(false)
                 }
-
                 TranslationDialogBottomSheet(
                     onRequestDismiss = { behavior?.state = BottomSheetBehavior.STATE_HIDDEN },
                 ) {
                     TranslationsAnimation(
-                        translationsVisibility = translationsVisibility,
-                        density = density,
-                        translationsOptionsHeightDp = translationsOptionsHeightDp,
-                    ) {
-                        if (translationsVisibility) {
+                        showMainSheet = translationsVisibility,
+                    ) { showMainPage ->
+                        if (showMainPage) {
                             Column(
                                 modifier = Modifier.onGloballyPositioned { coordinates ->
                                     translationsHeightDp = with(density) {
@@ -165,13 +174,17 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                                     }
                                 },
                             ) {
-                                translationsDialogState?.let {
+                                translationsDialogState.let {
                                     TranslationsDialogContent(
                                         learnMoreUrl = learnMoreUrl,
                                         showPageSettings = FxNimbus.features.translations.value().pageSettingsEnabled,
                                         translationsDialogState = it,
                                         onSettingClicked = {
-                                            Translations.action.record(Translations.ActionExtra("page_settings"))
+                                            Translations.action.record(
+                                                Translations.ActionExtra(
+                                                    "page_settings",
+                                                ),
+                                            )
                                             translationsVisibility = false
                                         },
                                         onShowDownloadLanguageFileDialog = {
@@ -180,16 +193,7 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                                     )
                                 }
                             }
-                        }
-                    }
-
-                    TranslationsOptionsAnimation(
-                        translationsVisibility = !translationsVisibility,
-                        density = density,
-                        translationsHeightDp = translationsHeightDp,
-                        translationsWidthDp = translationsWidthDp,
-                    ) {
-                        if (!translationsVisibility) {
+                        } else {
                             Column(
                                 modifier = Modifier.onGloballyPositioned { coordinates ->
                                     translationsOptionsHeightDp = with(density) {
@@ -200,8 +204,8 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                                 TranslationsOptionsDialogContent(
                                     learnMoreUrl = learnMoreUrl,
                                     showGlobalSettings = FxNimbus.features.translations.value().globalSettingsEnabled,
-                                    isTranslated = translationsDialogState?.isTranslated == true,
-                                    initialFrom = translationsDialogState?.initialFrom,
+                                    isTranslated = translationsDialogState.isTranslated == true,
+                                    initialFrom = translationsDialogState.initialFrom,
                                     onBackClicked = { translationsVisibility = true },
                                     onTranslate = {
                                         translate(
@@ -218,9 +222,10 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                     }
 
                     if (showDownloadLanguageFileDialog) {
-                        translationsDialogState?.translationDownloadSize?.size?.let { fileSize ->
+                        translationsDialogState.translationDownloadSize?.size?.let { fileSize ->
                             DownloadLanguageFileDialog(
                                 fileSize = fileSize,
+                                fileSizeFormatter = requireComponents.core.fileSizeFormatter,
                                 onConfirmDownload = {
                                     showDownloadLanguageFileDialog = false
                                 },
@@ -299,12 +304,14 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                 )
             },
             onNegativeButtonClicked = {
-                if (translationsDialogState.isTranslated) {
+                if (translationsDialogState.isTranslated || translationsDialogState.isTranslationInProgress) {
+                    @Suppress("DEPRECATION")
                     localView.announceForAccessibility(
                         requireContext().getString(
                             R.string.translations_bottom_sheet_restore_accessibility_announcement,
                         ),
                     )
+                    isTranslationInProgress = false
                     translationsDialogStore.dispatch(TranslationsDialogAction.RestoreTranslation)
                 }
                 dismiss()
@@ -365,12 +372,14 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
     @Composable
     private fun DownloadLanguageFileDialog(
         fileSize: Long,
+        fileSizeFormatter: FileSizeFormatter,
         onConfirmDownload: () -> Unit,
         onCancel: () -> Unit,
     ) {
         var checkBoxEnabled by remember { mutableStateOf(false) }
         DownloadLanguageFileDialog(
             downloadLanguageDialogType = DownloadLanguageFileDialogType.TranslationRequest,
+            fileSizeFormatter = fileSizeFormatter,
             fileSize = fileSize,
             isCheckBoxEnabled = checkBoxEnabled,
             onSavingModeStateChange = { checkBoxEnabled = it },
@@ -401,11 +410,16 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
             state.translationEngine.offerTranslation
         }.value
 
+        val pageSettingsError = browserStore.observeAsComposableState { state ->
+            state.selectedTab?.translationsState?.settingsError
+        }.value
+
         val localView = LocalView.current
 
         TranslationsOptionsDialog(
             context = requireContext(),
             translationPageSettings = pageSettingsState,
+            translationPageSettingsError = pageSettingsError,
             offerTranslation = offerTranslation,
             showGlobalSettings = showGlobalSettings,
             initialFrom = initialFrom,
@@ -421,6 +435,7 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
                 )
 
                 if (checked) {
+                    @Suppress("DEPRECATION")
                     localView.announceForAccessibility(type.descriptionId?.let { getString(it) })
                 }
             },
@@ -440,12 +455,10 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
 
     override fun onDismiss(dialog: DialogInterface) {
         super.onDismiss(dialog)
+
         if (isTranslationInProgress == true) {
-            setFragmentResult(
-                TRANSLATION_IN_PROGRESS,
-                bundleOf(
-                    SESSION_ID to browserStore.state.selectedTab?.id,
-                ),
+            appStore.dispatch(
+                AppAction.TranslationsAction.TranslationStarted(sessionId = browserStore.state.selectedTab?.id),
             )
         }
     }
@@ -464,9 +477,4 @@ class TranslationsDialogFragment : BottomSheetDialogFragment() {
         }
         dismiss()
     }
-
-    companion object {
-        const val TRANSLATION_IN_PROGRESS = "translationInProgress"
-        const val SESSION_ID = "sessionId"
-    }
 }
