@@ -19,6 +19,7 @@ import mozilla.components.feature.awesomebar.provider.SearchSuggestionProvider
 import mozilla.components.feature.awesomebar.provider.SessionSuggestionProvider
 import mozilla.components.feature.contextmenu.facts.ContextMenuFacts
 import mozilla.components.feature.media.facts.MediaFacts
+import mozilla.components.feature.prompts.dialog.GeneratedPasswordFacts
 import mozilla.components.feature.prompts.dialog.LoginDialogFacts
 import mozilla.components.feature.prompts.facts.CreditCardAutofillDialogFacts
 import mozilla.components.feature.pwa.ProgressiveWebAppFacts
@@ -33,7 +34,6 @@ import mozilla.components.support.base.facts.Fact
 import mozilla.components.support.base.log.logger.Logger
 import mozilla.components.support.test.robolectric.testContext
 import mozilla.components.support.webextensions.facts.WebExtensionFacts
-import mozilla.telemetry.glean.testing.GleanTestRule
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertNotNull
 import org.junit.Assert.assertNull
@@ -46,6 +46,7 @@ import org.mozilla.fenix.GleanMetrics.Awesomebar
 import org.mozilla.fenix.GleanMetrics.BrowserSearch
 import org.mozilla.fenix.GleanMetrics.ContextualMenu
 import org.mozilla.fenix.GleanMetrics.CreditCards
+import org.mozilla.fenix.GleanMetrics.GeneratedPasswordDialog
 import org.mozilla.fenix.GleanMetrics.LoginDialog
 import org.mozilla.fenix.GleanMetrics.MediaNotification
 import org.mozilla.fenix.GleanMetrics.PerfAwesomebar
@@ -53,16 +54,17 @@ import org.mozilla.fenix.GleanMetrics.ProgressiveWebApp
 import org.mozilla.fenix.GleanMetrics.SitePermissions
 import org.mozilla.fenix.GleanMetrics.SyncedTabs
 import org.mozilla.fenix.components.metrics.ReleaseMetricController.Companion
-import org.mozilla.fenix.helpers.FenixRobolectricTestRunner
+import org.mozilla.fenix.helpers.FenixGleanTestRule
 import org.mozilla.fenix.search.awesomebar.ShortcutsSuggestionProvider
 import org.mozilla.fenix.utils.Settings
+import org.robolectric.RobolectricTestRunner
 import mozilla.components.compose.browser.awesomebar.AwesomeBarFacts as ComposeAwesomeBarFacts
 
-@RunWith(FenixRobolectricTestRunner::class)
+@RunWith(RobolectricTestRunner::class)
 class MetricControllerTest {
 
     @get:Rule
-    val gleanTestRule = GleanTestRule(testContext)
+    val gleanTestRule = FenixGleanTestRule(testContext)
 
     @MockK(relaxUnitFun = true)
     private lateinit var dataService1: MetricsService
@@ -76,6 +78,9 @@ class MetricControllerTest {
     @MockK(relaxUnitFun = true)
     private lateinit var marketingService2: MetricsService
 
+    @MockK(relaxUnitFun = true)
+    private lateinit var usageReportingService: MetricsService
+
     @Before
     fun setup() {
         MockKAnnotations.init(this)
@@ -84,6 +89,7 @@ class MetricControllerTest {
         every { dataService2.type } returns MetricServiceType.Data
         every { marketingService1.type } returns MetricServiceType.Marketing
         every { marketingService2.type } returns MetricServiceType.Marketing
+        every { usageReportingService.type } returns MetricServiceType.UsageReporting
     }
 
     @Test
@@ -101,11 +107,11 @@ class MetricControllerTest {
     @Test
     fun `release metric controller starts and stops all data services`() {
         var enabled = true
-        val controller = ReleaseMetricController(
-            services = listOf(dataService1, marketingService1, dataService2, marketingService2),
+        val controller = createReleaseMetricController(
+            services = listOf(dataService1, marketingService1, dataService2, marketingService2, usageReportingService),
             isDataTelemetryEnabled = { enabled },
             isMarketingDataTelemetryEnabled = { enabled },
-            mockk(),
+            isUsageTelemetryEnabled = { enabled },
         )
 
         controller.start(MetricServiceType.Data)
@@ -123,16 +129,16 @@ class MetricControllerTest {
             marketingService1.stop()
             marketingService2.start()
             marketingService2.stop()
+            usageReportingService.start()
+            usageReportingService.stop()
         }
     }
 
     @Test
     fun `release metric controller starts data service only if enabled`() {
-        val controller = ReleaseMetricController(
+        val controller = createReleaseMetricController(
             services = listOf(dataService1),
             isDataTelemetryEnabled = { false },
-            isMarketingDataTelemetryEnabled = { true },
-            mockk(),
         )
 
         controller.start(MetricServiceType.Data)
@@ -142,14 +148,64 @@ class MetricControllerTest {
         verify(inverse = true) { dataService1.stop() }
     }
 
+    @Test
+    fun `release metric controller doesn't start usage reporting service if disabled`() {
+        val controller = createReleaseMetricController(
+            services = listOf(usageReportingService),
+            isUsageTelemetryEnabled = { false },
+        )
+        controller.start(MetricServiceType.UsageReporting)
+        verify(inverse = true) { usageReportingService.start() }
+    }
+
+    @Test
+    fun `release metric controller starts usage reporting service if enabled`() {
+        val controller = createReleaseMetricController(
+            services = listOf(usageReportingService),
+        )
+        controller.start(MetricServiceType.UsageReporting)
+        verify { usageReportingService.start() }
+    }
+
+    @Test
+    fun `release metric controller starts usage reporting service if data service disabled`() {
+        val controller = createReleaseMetricController(
+            services = listOf(dataService1, usageReportingService),
+            isDataTelemetryEnabled = { false },
+        )
+        controller.start(MetricServiceType.UsageReporting)
+        verify { usageReportingService.start() }
+    }
+
+    @Test
+    fun `release metric controller doesn't stop usage reporting service if enabled`() {
+        val controller = createReleaseMetricController(
+            services = listOf(dataService1, usageReportingService),
+        )
+        controller.start(MetricServiceType.UsageReporting)
+        controller.stop(MetricServiceType.UsageReporting)
+        verify(inverse = true) { usageReportingService.stop() }
+    }
+
+    @Test
+    fun `release metric controller stops usage reporting service if disabled`() {
+        var enabled = true
+        val controller = createReleaseMetricController(
+            services = listOf(dataService1, usageReportingService),
+            isUsageTelemetryEnabled = { enabled },
+        )
+        controller.start(MetricServiceType.UsageReporting)
+        enabled = false
+        controller.stop(MetricServiceType.UsageReporting)
+        verify { usageReportingService.stop() }
+    }
+
     @Test
     fun `release metric controller starts service only once`() {
         var enabled = true
-        val controller = ReleaseMetricController(
+        val controller = createReleaseMetricController(
             services = listOf(dataService1),
             isDataTelemetryEnabled = { enabled },
-            isMarketingDataTelemetryEnabled = { true },
-            mockk(),
         )
 
         controller.start(MetricServiceType.Data)
@@ -165,7 +221,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN AwesomeBar duration fact is processed THEN the correct metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val action = mockk<Action>()
         val duration = 1000L
         var metadata = mapOf<String, Pair<*, Long>>(
@@ -273,11 +329,11 @@ class MetricControllerTest {
     @Test
     fun `release metric controller starts and stops all marketing services`() {
         var enabled = true
-        val controller = ReleaseMetricController(
+        val controller = createReleaseMetricController(
             services = listOf(dataService1, marketingService1, dataService2, marketingService2),
             isDataTelemetryEnabled = { enabled },
             isMarketingDataTelemetryEnabled = { enabled },
-            mockk(),
+            isUsageTelemetryEnabled = { enabled },
         )
 
         controller.start(MetricServiceType.Marketing)
@@ -302,11 +358,12 @@ class MetricControllerTest {
     fun `topsites fact should set value in SharedPreference`() {
         val enabled = true
         val settings: Settings = mockk(relaxed = true)
-        val controller = ReleaseMetricController(
+        val controller = createReleaseMetricController(
             services = listOf(dataService1),
             isDataTelemetryEnabled = { enabled },
             isMarketingDataTelemetryEnabled = { enabled },
-            settings,
+            isUsageTelemetryEnabled = { enabled },
+            settings = settings,
         )
 
         val fact = Fact(
@@ -327,11 +384,12 @@ class MetricControllerTest {
     fun `web extension fact should set value in SharedPreference`() {
         val enabled = true
         val settings = Settings(testContext)
-        val controller = ReleaseMetricController(
+        val controller = createReleaseMetricController(
             services = listOf(dataService1),
             isDataTelemetryEnabled = { enabled },
             isMarketingDataTelemetryEnabled = { enabled },
-            settings,
+            isUsageTelemetryEnabled = { enabled },
+            settings = settings,
         )
         val fact = Fact(
             Component.SUPPORT_WEBEXTENSIONS,
@@ -358,7 +416,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN processing a fact with FEATURE_PROMPTS component THEN the right metric is recorded with no extras`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val action = mockk<Action>()
 
         // Verify display interaction
@@ -408,11 +466,35 @@ class MetricControllerTest {
         assertNotNull(LoginDialog.saved.testGetValue())
         assertEquals(1, LoginDialog.saved.testGetValue()!!.size)
         assertNull(LoginDialog.saved.testGetValue()!!.single().extra)
+
+        // Verify generated password shown interaction
+        assertNull(GeneratedPasswordDialog.shown.testGetValue())
+        fact = Fact(Component.FEATURE_PROMPTS, action, GeneratedPasswordFacts.Items.SHOWN)
+
+        controller.run {
+            fact.process()
+        }
+
+        assertNotNull(GeneratedPasswordDialog.shown.testGetValue())
+        assertEquals(1, GeneratedPasswordDialog.shown.testGetValue()!!.size)
+        assertNull(GeneratedPasswordDialog.shown.testGetValue()!!.single().extra)
+
+        // Verify generated password filled interaction
+        assertNull(GeneratedPasswordDialog.filled.testGetValue())
+        fact = Fact(Component.FEATURE_PROMPTS, action, GeneratedPasswordFacts.Items.FILLED)
+
+        controller.run {
+            fact.process()
+        }
+
+        assertNotNull(GeneratedPasswordDialog.filled.testGetValue())
+        assertEquals(1, GeneratedPasswordDialog.filled.testGetValue()!!.size)
+        assertNull(GeneratedPasswordDialog.filled.testGetValue()!!.single().extra)
     }
 
     @Test
     fun `WHEN processing a FEATURE_MEDIA NOTIFICATION fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         // Verify the play action
         var fact = Fact(Component.FEATURE_MEDIA, Action.PLAY, MediaFacts.Items.NOTIFICATION)
         assertNull(MediaNotification.play.testGetValue())
@@ -440,7 +522,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN processing a FEATURE_AUTOFILL fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         var fact = Fact(
             Component.FEATURE_AUTOFILL,
             mockk(relaxed = true),
@@ -508,7 +590,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN processing a ContextualMenu fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val action = mockk<Action>()
         // Verify copy button interaction
         var fact = Fact(
@@ -581,7 +663,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN processing a CreditCardAutofillDialog fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val action = mockk<Action>(relaxed = true)
         val itemsToEvents = listOf(
             CreditCardAutofillDialogFacts.Items.AUTOFILL_CREDIT_CARD_FORM_DETECTED to CreditCards.formDetected,
@@ -607,7 +689,7 @@ class MetricControllerTest {
 
     @Test
     fun `GIVEN pwa facts WHEN they are processed THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val action = mockk<Action>(relaxed = true)
 
         // a PWA shortcut from homescreen was opened
@@ -641,7 +723,7 @@ class MetricControllerTest {
 
     @Test
     fun `WHEN processing a suggestion fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
 
         // Verify synced tabs suggestion clicked
         assertNull(SyncedTabs.syncedTabsSuggestionClicked.testGetValue())
@@ -693,6 +775,52 @@ class MetricControllerTest {
 
         assertNotNull(Awesomebar.searchActionClicked.testGetValue())
 
+        // Verify search suggestion action clicked
+        assertNull(Awesomebar.searchSuggestionClicked.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.SEARCH_SUGGESTION_CLICKED)
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.searchSuggestionClicked.testGetValue())
+
+        // Verify trending search suggestion action clicked
+        assertNull(Awesomebar.trendingSearchSuggestionClicked.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.TRENDING_SEARCH_SUGGESTION_CLICKED, "3")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.trendingSearchSuggestionClicked.testGetValue())
+        assertNotNull(Awesomebar.trendingSearchSuggestionClicked.testGetValue()!![0].extra)
+        assertEquals("3", Awesomebar.trendingSearchSuggestionClicked.testGetValue()!![0].extra!!["position"])
+
+        // Verify top site suggestion action clicked
+        assertNull(Awesomebar.topSiteSuggestionClicked.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.TOP_SITE_SUGGESTION_CLICKED, "2")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.topSiteSuggestionClicked.testGetValue())
+        assertNotNull(Awesomebar.topSiteSuggestionClicked.testGetValue()!![0].extra)
+        assertEquals("2", Awesomebar.topSiteSuggestionClicked.testGetValue()!![0].extra!!["position"])
+
+        // Verify recent search suggestion action clicked
+        assertNull(Awesomebar.recentSearchSuggestionClicked.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.RECENT_SEARCH_SUGGESTION_CLICKED, "1")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.recentSearchSuggestionClicked.testGetValue())
+        assertNotNull(Awesomebar.recentSearchSuggestionClicked.testGetValue()!![0].extra)
+        assertEquals("1", Awesomebar.recentSearchSuggestionClicked.testGetValue()!![0].extra!!["position"])
+
         // Verify bookmark opened tab suggestion clicked
         assertNull(Awesomebar.openedTabSuggestionClicked.testGetValue())
         fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.OPENED_TAB_SUGGESTION_CLICKED)
@@ -712,11 +840,49 @@ class MetricControllerTest {
         }
 
         assertNotNull(Awesomebar.searchTermSuggestionClicked.testGetValue())
+
+        // Verify trending search suggestions displayed
+        assertNull(Awesomebar.trendingSearchSuggestionsDisplayed.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.TRENDING_SEARCH_SUGGESTIONS_DISPLAYED, "6")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.trendingSearchSuggestionsDisplayed.testGetValue())
+        assertNotNull(Awesomebar.trendingSearchSuggestionsDisplayed.testGetValue()!![0].extra)
+        assertEquals("6", Awesomebar.trendingSearchSuggestionsDisplayed.testGetValue()!![0].extra!!["count"])
+
+        // Verify top site suggestions displayed
+        assertNull(Awesomebar.topSiteSuggestionsDisplayed.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.TOP_SITE_SUGGESTIONS_DISPLAYED, "5")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.topSiteSuggestionsDisplayed.testGetValue())
+        assertNotNull(Awesomebar.topSiteSuggestionsDisplayed.testGetValue()!![0].extra)
+        assertEquals("5", Awesomebar.topSiteSuggestionsDisplayed.testGetValue()!![0].extra!!["count"])
+
+        // Verify recent site suggestions displayed
+        assertNull(Awesomebar.recentSearchSuggestionsDisplayed.testGetValue())
+        fact = Fact(Component.FEATURE_AWESOMEBAR, Action.CANCEL, AwesomeBarFacts.Items.RECENT_SEARCH_SUGGESTIONS_DISPLAYED, "4")
+
+        with(controller) {
+            fact.process()
+        }
+
+        assertNotNull(Awesomebar.recentSearchSuggestionsDisplayed.testGetValue())
+        assertNotNull(Awesomebar.recentSearchSuggestionsDisplayed.testGetValue()!![0].extra)
+        assertEquals("4", Awesomebar.recentSearchSuggestionsDisplayed.testGetValue()!![0].extra!!["count"])
     }
 
     @Test
     fun `GIVEN advertising search facts WHEN the list is processed THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { false }, mockk())
+        val controller = createReleaseMetricController(
+            isMarketingDataTelemetryEnabled = { false },
+        )
         val action = mockk<Action>()
 
         // an ad was clicked in a Search Engine Result Page
@@ -778,7 +944,7 @@ class MetricControllerTest {
 
     @Test
     fun `GIVEN a site permissions prompt is shown WHEN processing the fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val fact = Fact(
             component = Component.FEATURE_SITEPERMISSIONS,
             action = Action.DISPLAY,
@@ -792,12 +958,15 @@ class MetricControllerTest {
         }
 
         assertEquals(1, SitePermissions.promptShown.testGetValue()!!.size)
-        assertEquals("test", SitePermissions.promptShown.testGetValue()!!.single().extra!!["permissions"])
+        assertEquals(
+            "test",
+            SitePermissions.promptShown.testGetValue()!!.single().extra!!["permissions"],
+        )
     }
 
     @Test
     fun `GIVEN site permissions are allowed WHEN processing the fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val fact = Fact(
             component = Component.FEATURE_SITEPERMISSIONS,
             action = Action.CONFIRM,
@@ -811,12 +980,15 @@ class MetricControllerTest {
         }
 
         assertEquals(1, SitePermissions.permissionsAllowed.testGetValue()!!.size)
-        assertEquals("allow", SitePermissions.permissionsAllowed.testGetValue()!!.single().extra!!["permissions"])
+        assertEquals(
+            "allow",
+            SitePermissions.permissionsAllowed.testGetValue()!!.single().extra!!["permissions"],
+        )
     }
 
     @Test
     fun `GIVEN site permissions are denied WHEN processing the fact THEN the right metric is recorded`() {
-        val controller = ReleaseMetricController(emptyList(), { true }, { true }, mockk())
+        val controller = createReleaseMetricController()
         val fact = Fact(
             component = Component.FEATURE_SITEPERMISSIONS,
             action = Action.CANCEL,
@@ -832,4 +1004,19 @@ class MetricControllerTest {
         assertEquals(1, SitePermissions.permissionsDenied.testGetValue()!!.size)
         assertEquals("deny", SitePermissions.permissionsDenied.testGetValue()!!.single().extra!!["permissions"])
     }
+
+    private fun createReleaseMetricController(
+        services: List<MetricsService> = emptyList(),
+        isDataTelemetryEnabled: () -> Boolean = { true },
+        isMarketingDataTelemetryEnabled: () -> Boolean = { true },
+        isUsageTelemetryEnabled: () -> Boolean = { true },
+        settings: Settings = mockk(),
+    ) =
+        ReleaseMetricController(
+            services = services,
+            isDataTelemetryEnabled = isDataTelemetryEnabled,
+            isMarketingDataTelemetryEnabled = isMarketingDataTelemetryEnabled,
+            isUsageTelemetryEnabled = isUsageTelemetryEnabled,
+            settings = settings,
+        )
 }
