@@ -13,14 +13,18 @@ import io.mockk.impl.annotations.MockK
 import io.mockk.impl.annotations.RelaxedMockK
 import io.mockk.just
 import io.mockk.mockk
+import io.mockk.spyk
 import io.mockk.verify
 import mozilla.components.browser.state.action.BrowserAction
 import mozilla.components.browser.state.action.ContentAction
+import mozilla.components.browser.state.action.ShareResourceAction
 import mozilla.components.browser.state.action.TabListAction
 import mozilla.components.browser.state.state.BrowserState
+import mozilla.components.browser.state.state.content.ShareResourceState
 import mozilla.components.browser.state.state.createTab
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.EngineView
+import mozilla.components.concept.engine.prompt.ShareData
 import mozilla.components.feature.search.SearchUseCases
 import mozilla.components.feature.session.SessionUseCases
 import mozilla.components.feature.tabs.TabsUseCases
@@ -43,22 +47,32 @@ import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.fenix.GleanMetrics.Events
 import org.mozilla.fenix.GleanMetrics.ReaderMode
+import org.mozilla.fenix.GleanMetrics.Toolbar
 import org.mozilla.fenix.GleanMetrics.Translations
 import org.mozilla.fenix.HomeActivity
+import org.mozilla.fenix.NavGraphDirections
 import org.mozilla.fenix.R
 import org.mozilla.fenix.browser.BrowserAnimator
 import org.mozilla.fenix.browser.BrowserFragmentDirections
 import org.mozilla.fenix.browser.browsingmode.BrowsingMode
 import org.mozilla.fenix.browser.browsingmode.SimpleBrowsingModeManager
 import org.mozilla.fenix.browser.readermode.ReaderModeController
+import org.mozilla.fenix.components.AppStore
+import org.mozilla.fenix.components.appstate.AppAction.SnackbarAction
+import org.mozilla.fenix.components.menu.MenuAccessPoint
+import org.mozilla.fenix.components.usecases.FenixBrowserUseCases
 import org.mozilla.fenix.ext.components
+import org.mozilla.fenix.ext.directionsEq
 import org.mozilla.fenix.ext.settings
-import org.mozilla.fenix.helpers.FenixRobolectricTestRunner
-import org.mozilla.fenix.home.HomeFragment
 import org.mozilla.fenix.home.HomeScreenViewModel
+import org.mozilla.fenix.telemetry.ACTION_ADD_NEW_TAB
+import org.mozilla.fenix.telemetry.ACTION_ADD_NEW_TAB_LONG_CLICKED
+import org.mozilla.fenix.telemetry.ACTION_HOME_CLICKED
+import org.mozilla.fenix.telemetry.SOURCE_ADDRESS_BAR
 import org.mozilla.fenix.utils.Settings
+import org.robolectric.RobolectricTestRunner
 
-@RunWith(FenixRobolectricTestRunner::class)
+@RunWith(RobolectricTestRunner::class)
 class DefaultBrowserToolbarControllerTest {
 
     @RelaxedMockK
@@ -81,6 +95,9 @@ class DefaultBrowserToolbarControllerTest {
     @RelaxedMockK
     private lateinit var tabsUseCases: TabsUseCases
 
+    @RelaxedMockK
+    private lateinit var fenixBrowserUseCases: FenixBrowserUseCases
+
     @RelaxedMockK
     private lateinit var browserAnimator: BrowserAnimator
 
@@ -93,7 +110,11 @@ class DefaultBrowserToolbarControllerTest {
     @RelaxedMockK
     private lateinit var homeViewModel: HomeScreenViewModel
 
+    @RelaxedMockK
+    private lateinit var settings: Settings
+
     private lateinit var store: BrowserStore
+    private lateinit var appStore: AppStore
     private val captureMiddleware = CaptureActionsMiddleware<BrowserState, BrowserAction>()
 
     @get:Rule
@@ -127,6 +148,7 @@ class DefaultBrowserToolbarControllerTest {
             ),
             middleware = listOf(captureMiddleware),
         )
+        appStore = AppStore()
     }
 
     @After
@@ -360,91 +382,26 @@ class DefaultBrowserToolbarControllerTest {
 
     @Test
     fun handleHomeButtonClick() {
-        assertNull(Events.browserToolbarHomeTapped.testGetValue())
+        assertNull(Toolbar.buttonTapped.testGetValue())
 
         val controller = createController()
         controller.handleHomeButtonClick()
 
         verify { navController.navigate(BrowserFragmentDirections.actionGlobalHome()) }
-        assertNotNull(Events.browserToolbarHomeTapped.testGetValue())
+        assertToolbarTapped(ACTION_HOME_CLICKED)
     }
 
     @Test
-    fun handleEraseButtonClicked() {
-        assertNull(Events.browserToolbarEraseTapped.testGetValue())
+    fun `GIVEN homepage as a new tab is enabled WHEN home button is clicked THEN navigate to ABOUT_HOME`() {
+        every { settings.enableHomepageAsNewTab } returns true
+
+        assertNull(Toolbar.buttonTapped.testGetValue())
+
         val controller = createController()
-        controller.handleEraseButtonClick()
+        controller.handleHomeButtonClick()
 
-        verify {
-            homeViewModel.sessionToDelete = HomeFragment.ALL_PRIVATE_TABS
-            navController.navigate(BrowserFragmentDirections.actionGlobalHome())
-        }
-        assertNotNull(Events.browserToolbarEraseTapped.testGetValue())
-    }
-
-    @Test
-    fun handleShoppingCfrActionClick() {
-        val controller = createController()
-
-        controller.handleShoppingCfrActionClick()
-
-        verify {
-            navController.navigate(BrowserFragmentDirections.actionBrowserFragmentToReviewQualityCheckDialogFragment())
-        }
-    }
-
-    @Test
-    fun handleShoppingCfrDisplayedOnce() {
-        val controller = createController()
-        val mockSettings = mockk<Settings> {
-            every { reviewQualityCheckCfrDisplayTimeInMillis } returns System.currentTimeMillis()
-            every { reviewQualityCheckCfrDisplayTimeInMillis = any() } just Runs
-            every { reviewQualityCheckCFRClosedCounter } returns 1
-            every { reviewQualityCheckCFRClosedCounter = 2 } just Runs
-            every { shouldShowReviewQualityCheckCFR } returns true
-        }
-        every { activity.settings() } returns mockSettings
-
-        controller.handleShoppingCfrDisplayed()
-
-        verify(exactly = 0) { mockSettings.shouldShowReviewQualityCheckCFR = false }
-        verify { mockSettings.reviewQualityCheckCfrDisplayTimeInMillis = any() }
-    }
-
-    @Test
-    fun handleShoppingCfrDisplayedTwice() {
-        val controller = createController()
-        val mockSettings = mockk<Settings> {
-            every { reviewQualityCheckCfrDisplayTimeInMillis } returns System.currentTimeMillis()
-            every { reviewQualityCheckCfrDisplayTimeInMillis = any() } just Runs
-            every { reviewQualityCheckCFRClosedCounter } returns 2
-            every { reviewQualityCheckCFRClosedCounter = 3 } just Runs
-            every { shouldShowReviewQualityCheckCFR } returns true
-        }
-        every { activity.settings() } returns mockSettings
-
-        controller.handleShoppingCfrDisplayed()
-
-        verify(exactly = 0) { mockSettings.shouldShowReviewQualityCheckCFR = false }
-        verify { mockSettings.reviewQualityCheckCfrDisplayTimeInMillis = any() }
-    }
-
-    @Test
-    fun handleShoppingCfrDisplayedThreeTimes() {
-        val controller = createController()
-        val mockSettings = mockk<Settings> {
-            every { reviewQualityCheckCfrDisplayTimeInMillis } returns System.currentTimeMillis()
-            every { reviewQualityCheckCFRClosedCounter } returns 3
-            every { reviewQualityCheckCFRClosedCounter = 4 } just Runs
-            every { shouldShowReviewQualityCheckCFR } returns true
-            every { shouldShowReviewQualityCheckCFR = any() } just Runs
-        }
-        every { activity.settings() } returns mockSettings
-
-        controller.handleShoppingCfrDisplayed()
-
-        verify { mockSettings.shouldShowReviewQualityCheckCFR = false }
-        verify(exactly = 0) { mockSettings.reviewQualityCheckCfrDisplayTimeInMillis = any() }
+        verify { fenixBrowserUseCases.navigateToHomepage() }
+        assertToolbarTapped(ACTION_HOME_CLICKED)
     }
 
     @Test
@@ -453,6 +410,8 @@ class DefaultBrowserToolbarControllerTest {
         controller.handleTranslationsButtonClick()
 
         verify {
+            appStore.dispatch(SnackbarAction.SnackbarDismissed)
+
             navController.navigate(
                 BrowserFragmentDirections.actionBrowserFragmentToTranslationsDialogFragment(),
             )
@@ -462,13 +421,148 @@ class DefaultBrowserToolbarControllerTest {
         assertEquals("main_flow_toolbar", telemetry?.extra?.get("item"))
     }
 
+    @Test
+    fun `WHEN new tab button is clicked THEN navigate to homepage`() {
+        val controller = createController()
+        controller.handleNewTabButtonClick()
+
+        verify {
+            navController.navigate(
+                BrowserFragmentDirections.actionGlobalHome(focusOnAddressBar = true),
+            )
+        }
+
+        assertToolbarTapped(ACTION_ADD_NEW_TAB)
+    }
+
+    @Test
+    fun `GIVEN homepage as a new tab is enabled WHEN new tab button is clicked THEN a new homepage tab is displayed`() {
+        every { settings.enableHomepageAsNewTab } returns true
+
+        val controller = createController()
+        controller.handleNewTabButtonClick()
+
+        verify {
+            fenixBrowserUseCases.addNewHomepageTab(
+                private = false,
+            )
+
+            navController.navigate(
+                BrowserFragmentDirections.actionGlobalHome(focusOnAddressBar = true),
+            )
+        }
+    }
+
+    @Test
+    fun `WHEN new tab button is long clicked THEN record the navigation bar telemetry event`() {
+        val controller = createController()
+        assertNull(Toolbar.buttonTapped.testGetValue())
+
+        controller.handleNewTabButtonLongClick()
+
+        assertToolbarTapped(ACTION_ADD_NEW_TAB_LONG_CLICKED)
+    }
+
+    @Test
+    fun `GIVEN that the menu access point is not a custom tab WHEN menu button is clicked THEN handle menu navigation`() {
+        val controller = createController()
+        val accessPoint = MenuAccessPoint.Browser
+
+        controller.handleMenuButtonClicked(accessPoint)
+
+        verify {
+            navController.navigate(
+                BrowserFragmentDirections.actionGlobalMenuDialogFragment(
+                    accesspoint = accessPoint,
+                ),
+            )
+        }
+    }
+
+    @Test
+    fun `GIVEN that the menu access point is a custom tab WHEN menu button is clicked THEN handle menu navigation`() {
+        val controller = createController()
+        val accessPoint = MenuAccessPoint.External
+        val customTabSessionId = "1"
+
+        controller.handleMenuButtonClicked(accessPoint, customTabSessionId)
+
+        verify {
+            navController.navigate(
+                BrowserFragmentDirections.actionGlobalMenuDialogFragment(
+                    accesspoint = accessPoint,
+                    customTabSessionId = customTabSessionId,
+                ),
+            )
+        }
+    }
+
+    @Test
+    fun `GIVEN that the tab is a local PDF WHEN share button is clicked THEN start the shareResource process`() {
+        val store = spyk(
+            BrowserStore(
+                initialState = BrowserState(
+                    tabs = listOf(
+                        createTab("content://pdf.pdf", id = "1"),
+                    ),
+                    selectedTabId = "1",
+                ),
+                middleware = listOf(captureMiddleware),
+            ),
+        )
+
+        val controller = createController(store = store)
+        controller.onShareActionClicked()
+
+        verify {
+            store.dispatch(
+                ShareResourceAction.AddShareAction(
+                    tabId = "1",
+                    ShareResourceState.LocalResource("content://pdf.pdf"),
+                ),
+            )
+        }
+    }
+
+    @Test
+    fun `GIVEN that the tab is an internet resource WHEN share button is clicked THEN navigate to ShareFragment`() {
+        val store = BrowserStore(
+            initialState = BrowserState(
+                tabs = listOf(
+                    createTab("https://mozilla.com", id = "1"),
+                ),
+                selectedTabId = "1",
+            ),
+            middleware = listOf(captureMiddleware),
+        )
+
+        val controller = createController(store = store)
+        controller.onShareActionClicked()
+
+        verify {
+            navController.navigate(
+                directionsEq(
+                    NavGraphDirections.actionGlobalShareFragment(
+                        sessionId = "1",
+                        data = arrayOf(ShareData(url = "https://mozilla.com", title = "")),
+                        showPage = true,
+                    ),
+                ),
+            )
+        }
+    }
+
     private fun createController(
         activity: HomeActivity = this.activity,
         customTabSessionId: String? = null,
+        store: BrowserStore = this.store,
     ) = DefaultBrowserToolbarController(
         store = store,
+        appStore = appStore,
         tabsUseCases = tabsUseCases,
+        fenixBrowserUseCases = fenixBrowserUseCases,
         activity = activity,
+        settings = settings,
         navController = navController,
         engineView = engineView,
         homeViewModel = homeViewModel,
@@ -480,4 +574,12 @@ class DefaultBrowserToolbarControllerTest {
         },
         onCloseTab = {},
     )
+
+    private fun assertToolbarTapped(expectedItem: String, expectedSource: String = SOURCE_ADDRESS_BAR) {
+        val values = Toolbar.buttonTapped.testGetValue()
+        assertNotNull(values)
+        val last = values!!.last()
+        assertEquals(expectedItem, last.extra?.get("item"))
+        assertEquals(expectedSource, last.extra?.get("source"))
+    }
 }
