@@ -10,25 +10,24 @@ import androidx.fragment.app.Fragment
 import androidx.navigation.NavController
 import io.mockk.MockKAnnotations
 import io.mockk.Runs
-import io.mockk.coEvery
 import io.mockk.every
 import io.mockk.impl.annotations.MockK
 import io.mockk.impl.annotations.RelaxedMockK
 import io.mockk.just
 import io.mockk.mockk
-import io.mockk.mockkObject
-import io.mockk.mockkStatic
 import io.mockk.slot
-import io.mockk.unmockkObject
-import io.mockk.unmockkStatic
+import io.mockk.spyk
 import io.mockk.verify
+import io.mockk.verifyOrder
 import kotlinx.coroutines.CoroutineScope
 import kotlinx.coroutines.test.runTest
 import mozilla.appservices.places.BookmarkRoot
 import mozilla.components.browser.state.action.CustomTabListAction
+import mozilla.components.browser.state.action.ShareResourceAction
 import mozilla.components.browser.state.state.BrowserState
 import mozilla.components.browser.state.state.ReaderState
 import mozilla.components.browser.state.state.TabSessionState
+import mozilla.components.browser.state.state.content.ShareResourceState
 import mozilla.components.browser.state.state.createCustomTab
 import mozilla.components.browser.state.state.createTab
 import mozilla.components.browser.state.store.BrowserStore
@@ -39,17 +38,15 @@ import mozilla.components.feature.session.SessionFeature
 import mozilla.components.feature.session.SessionUseCases
 import mozilla.components.feature.tab.collections.TabCollection
 import mozilla.components.feature.tabs.CustomTabsUseCases
+import mozilla.components.feature.tabs.TabsUseCases
 import mozilla.components.feature.top.sites.DefaultTopSitesStorage
 import mozilla.components.feature.top.sites.PinnedSiteStorage
-import mozilla.components.feature.top.sites.TopSite
 import mozilla.components.feature.top.sites.TopSitesUseCases
-import mozilla.components.service.glean.testing.GleanTestRule
 import mozilla.components.support.base.feature.ViewBoundFeatureWrapper
 import mozilla.components.support.test.ext.joinBlocking
 import mozilla.components.support.test.robolectric.testContext
 import mozilla.components.support.test.rule.MainCoroutineRule
 import mozilla.components.support.test.rule.runTestOnMain
-import org.junit.After
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertNotNull
 import org.junit.Assert.assertNull
@@ -58,6 +55,7 @@ import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
 import org.junit.runner.RunWith
+import org.mozilla.fenix.BrowserDirection
 import org.mozilla.fenix.GleanMetrics.Collections
 import org.mozilla.fenix.GleanMetrics.Events
 import org.mozilla.fenix.GleanMetrics.ReaderMode
@@ -69,24 +67,28 @@ import org.mozilla.fenix.browser.BrowserAnimator
 import org.mozilla.fenix.browser.BrowserFragmentDirections
 import org.mozilla.fenix.browser.readermode.ReaderModeController
 import org.mozilla.fenix.collections.SaveCollectionStep
-import org.mozilla.fenix.components.FenixSnackbar
+import org.mozilla.fenix.components.AppStore
 import org.mozilla.fenix.components.TabCollectionStorage
 import org.mozilla.fenix.components.accounts.AccountState
 import org.mozilla.fenix.components.accounts.FenixFxAEntryPoint
+import org.mozilla.fenix.components.appstate.AppAction.ShortcutAction
+import org.mozilla.fenix.components.usecases.FenixBrowserUseCases
+import org.mozilla.fenix.compose.snackbar.Snackbar
 import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.directionsEq
-import org.mozilla.fenix.helpers.FenixRobolectricTestRunner
-import org.mozilla.fenix.settings.deletebrowsingdata.deleteAndQuit
+import org.mozilla.fenix.helpers.FenixGleanTestRule
 import org.mozilla.fenix.utils.Settings
+import org.mozilla.fenix.webcompat.WEB_COMPAT_REPORTER_URL
+import org.robolectric.RobolectricTestRunner
 
-@RunWith(FenixRobolectricTestRunner::class)
+@RunWith(RobolectricTestRunner::class)
 class DefaultBrowserToolbarMenuControllerTest {
 
     @get:Rule
     val coroutinesTestRule = MainCoroutineRule()
 
     @get:Rule
-    val gleanTestRule = GleanTestRule(testContext)
+    val gleanTestRule = FenixGleanTestRule(testContext)
 
     @MockK private lateinit var snackbarParent: ViewGroup
 
@@ -108,12 +110,16 @@ class DefaultBrowserToolbarMenuControllerTest {
 
     @RelaxedMockK private lateinit var browserAnimator: BrowserAnimator
 
-    @RelaxedMockK private lateinit var snackbar: FenixSnackbar
+    @RelaxedMockK private lateinit var snackbar: Snackbar
 
     @RelaxedMockK private lateinit var tabCollectionStorage: TabCollectionStorage
 
     @RelaxedMockK private lateinit var topSitesUseCase: TopSitesUseCases
 
+    @RelaxedMockK private lateinit var tabsUseCases: TabsUseCases
+
+    @RelaxedMockK private lateinit var fenixBrowserUseCases: FenixBrowserUseCases
+
     @RelaxedMockK private lateinit var readerModeController: ReaderModeController
 
     @MockK private lateinit var sessionFeatureWrapper: ViewBoundFeatureWrapper<SessionFeature>
@@ -124,25 +130,22 @@ class DefaultBrowserToolbarMenuControllerTest {
 
     @RelaxedMockK private lateinit var pinnedSiteStorage: PinnedSiteStorage
 
+    @RelaxedMockK private lateinit var appStore: AppStore
+
     private lateinit var browserStore: BrowserStore
     private lateinit var selectedTab: TabSessionState
+    private var deleteAndQuitCalled = false
 
     @Before
     fun setUp() {
         MockKAnnotations.init(this)
 
-        mockkStatic(
-            "org.mozilla.fenix.settings.deletebrowsingdata.DeleteAndQuitKt",
-        )
-        every { deleteAndQuit(any(), any(), any()) } just Runs
-
-        mockkObject(FenixSnackbar.Companion)
-        every { FenixSnackbar.make(any(), any(), any(), any()) } returns snackbar
-
         every { activity.components.useCases.sessionUseCases } returns sessionUseCases
         every { activity.components.useCases.customTabsUseCases } returns customTabUseCases
         every { activity.components.useCases.searchUseCases } returns searchUseCases
         every { activity.components.useCases.topSitesUseCase } returns topSitesUseCase
+        every { activity.components.useCases.tabsUseCases } returns tabsUseCases
+        every { activity.components.useCases.fenixBrowserUseCases } returns fenixBrowserUseCases
         every { sessionFeatureWrapper.get() } returns sessionFeature
         every { navController.currentDestination } returns mockk {
             every { id } returns R.id.browserFragment
@@ -161,12 +164,6 @@ class DefaultBrowserToolbarMenuControllerTest {
         )
     }
 
-    @After
-    fun tearDown() {
-        unmockkStatic("org.mozilla.fenix.settings.deletebrowsingdata.DeleteAndQuitKt")
-        unmockkObject(FenixSnackbar.Companion)
-    }
-
     @Test
     fun handleToolbarBookmarkPressWithReaderModeInactive() = runTest {
         val item = ToolbarMenu.Item.Bookmark
@@ -248,7 +245,7 @@ class DefaultBrowserToolbarMenuControllerTest {
             customTabSessionId = customTab.id,
         )
 
-        val item = ToolbarMenu.Item.OpenInFenix
+        val item = ToolbarMenu.Item.OpenInFenix()
 
         every { activity.startActivity(any()) } just Runs
         controller.handleToolbarItemInteraction(item)
@@ -276,13 +273,12 @@ class DefaultBrowserToolbarMenuControllerTest {
     @Test
     fun `WHEN quit menu item is pressed THEN menu item is handled correctly`() = runTest {
         val item = ToolbarMenu.Item.Quit
-        val testScope = this
 
-        val controller = createController(scope = this, store = browserStore)
+        val controller = createController(scope = mockk(), store = browserStore)
 
         controller.handleToolbarItemInteraction(item)
 
-        verify { deleteAndQuit(activity, testScope, null) }
+        assertTrue(deleteAndQuitCalled)
     }
 
     @Test
@@ -538,36 +534,7 @@ class DefaultBrowserToolbarMenuControllerTest {
         assertEquals(1, snapshot.size)
         assertEquals("add_to_top_sites", snapshot.single().extra?.getValue("item"))
 
-        verify { addPinnedSiteUseCase.invoke(selectedTab.content.title, selectedTab.content.url) }
-        verify { snackbar.setText("Added to shortcuts!") }
-    }
-
-    @Test
-    fun `GIVEN a shortcut page is open WHEN remove from shortcuts is pressed THEN show snackbar`() = runTestOnMain {
-        val snackbarMessage = "Site removed"
-        val item = ToolbarMenu.Item.RemoveFromTopSites
-        val removePinnedSiteUseCase: TopSitesUseCases.RemoveTopSiteUseCase =
-            mockk(relaxed = true)
-        val topSite: TopSite = mockk()
-        every { topSite.url } returns selectedTab.content.url
-        coEvery { pinnedSiteStorage.getPinnedSites() } returns listOf(topSite)
-        every { topSitesUseCase.removeTopSites } returns removePinnedSiteUseCase
-        every {
-            snackbarParent.context.getString(R.string.snackbar_top_site_removed)
-        } returns snackbarMessage
-
-        val controller = createController(scope = this, store = browserStore)
-        assertNull(Events.browserMenuAction.testGetValue())
-
-        controller.handleToolbarItemInteraction(item)
-
-        assertNotNull(Events.browserMenuAction.testGetValue())
-        val snapshot = Events.browserMenuAction.testGetValue()!!
-        assertEquals(1, snapshot.size)
-        assertEquals("remove_from_top_sites", snapshot.single().extra?.getValue("item"))
-
-        verify { snackbar.setText(snackbarMessage) }
-        verify { removePinnedSiteUseCase.invoke(topSite) }
+        verify { appStore.dispatch(ShortcutAction.ShortcutAdded) }
     }
 
     @Test
@@ -668,6 +635,71 @@ class DefaultBrowserToolbarMenuControllerTest {
         }
     }
 
+    @Test
+    fun `IF in customtab WHEN share menu item is pressed THEN navigate to share screen`() = runTest {
+        val item = ToolbarMenu.Item.Share
+        val title = "Mozilla"
+        val url = "https://mozilla.org"
+        val customTab = createCustomTab(
+            url = url,
+            title = title,
+        )
+        browserStore = BrowserStore(BrowserState(customTabs = listOf(customTab)))
+        val controller = createController(
+            scope = this,
+            store = browserStore,
+            customTabSessionId = customTab.id,
+        )
+        assertNull(Events.browserMenuAction.testGetValue())
+
+        controller.handleToolbarItemInteraction(item)
+
+        assertNotNull(Events.browserMenuAction.testGetValue())
+        val snapshot = Events.browserMenuAction.testGetValue()!!
+        assertEquals(1, snapshot.size)
+        assertEquals("share", snapshot.single().extra?.getValue("item"))
+
+        verify {
+            navController.navigate(
+                directionsEq(
+                    NavGraphDirections.actionGlobalShareFragment(
+                        sessionId = customTab.id,
+                        data = arrayOf(ShareData(url = "https://mozilla.org", title = "Mozilla")),
+                        showPage = true,
+                    ),
+                ),
+            )
+        }
+    }
+
+    @Test
+    fun `GIVEN tab is a local PDF WHEN share menu item is pressed THEN trigger ShareResourceAtion`() = runTest {
+        val item = ToolbarMenu.Item.Share
+        val url = "content://pdf.pdf"
+        val tab = createTab(
+            url = url,
+            id = "1",
+        )
+        browserStore = spyk(BrowserStore(BrowserState(tabs = listOf(tab), selectedTabId = "1")))
+        val controller = createController(scope = this, store = browserStore)
+        assertNull(Events.browserMenuAction.testGetValue())
+
+        controller.handleToolbarItemInteraction(item)
+
+        val snapshot = requireNotNull(Events.browserMenuAction.testGetValue()) { "Snapshot should'nt be null" }
+        assertEquals(1, snapshot.size)
+        assertEquals("share", snapshot.single().extra?.getValue("item"))
+
+        verify {
+            browserStore.dispatch(
+                ShareResourceAction.AddShareAction(
+                    tabId = "1",
+                    ShareResourceState.LocalResource("content://pdf.pdf"),
+                ),
+            )
+        }
+    }
+
     @Test
     fun `WHEN Find In Page menu item is pressed THEN launch finder`() = runTest {
         val item = ToolbarMenu.Item.FindInPage
@@ -688,7 +720,7 @@ class DefaultBrowserToolbarMenuControllerTest {
     @Test
     fun `IF one or more collection exists WHEN Save To Collection menu item is pressed THEN navigate to save collection page`() = runTest {
         val item = ToolbarMenu.Item.SaveToCollection
-        val cachedTabCollections: List<TabCollection> = mockk(relaxed = true)
+        val cachedTabCollections: List<TabCollection> = listOf(mockk(relaxed = true))
         every { tabCollectionStorage.cachedTabCollections } returns cachedTabCollections
 
         val controller = createController(scope = this, store = browserStore)
@@ -788,6 +820,28 @@ class DefaultBrowserToolbarMenuControllerTest {
         }
     }
 
+    @Test
+    fun `GIVEN homepage as a new tab is enabled WHEN New Tab menu item is pressed THEN navigate to a new homepage tab`() = runTest {
+        every { settings.enableHomepageAsNewTab } returns true
+
+        val controller = createController(scope = this, store = browserStore)
+        val item = ToolbarMenu.Item.NewTab
+
+        controller.handleToolbarItemInteraction(item)
+
+        verifyOrder {
+            fenixBrowserUseCases.addNewHomepageTab(private = false)
+
+            navController.navigate(
+                directionsEq(
+                    NavGraphDirections.actionGlobalHome(
+                        focusOnAddressBar = true,
+                    ),
+                ),
+            )
+        }
+    }
+
     @Test
     fun `GIVEN account exists and the user is signed in WHEN sign in to sync menu item is pressed THEN navigate to account settings`() = runTest {
         val item = ToolbarMenu.Item.SyncAccount(AccountState.AUTHENTICATED)
@@ -846,6 +900,45 @@ class DefaultBrowserToolbarMenuControllerTest {
             assertEquals("main_flow_browser", telemetry?.extra?.get("item"))
         }
 
+    @Test
+    fun `GIVEN telemetry is enabled WHEN the Report broken site menu item is pressed THEN navigate to the web compat reporter AND post telemetry`() =
+        runTest {
+            val item = ToolbarMenu.Item.ReportBrokenSite
+            every { settings.isTelemetryEnabled } returns true
+
+            createController(scope = this, store = browserStore).handleToolbarItemInteraction(item)
+
+            verify {
+                navController.navigate(
+                    directions =
+                    BrowserFragmentDirections.actionBrowserFragmentToWebCompatReporterFragment(
+                        tabUrl = selectedTab.content.url,
+                    ),
+                )
+            }
+
+            val telemetry = Events.browserMenuAction.testGetValue()?.firstOrNull()
+            assertEquals("report_broken_site", telemetry?.extra?.get("item"))
+        }
+
+    @Test
+    fun `GIVEN telemetry is disabled WHEN the Report broken site menu item is pressed THEN navigate to the web compat reporter web page`() =
+        runTest {
+            val item = ToolbarMenu.Item.ReportBrokenSite
+            every { settings.isTelemetryEnabled } returns false
+            val controller = createController(scope = this, store = browserStore)
+
+            controller.handleToolbarItemInteraction(item)
+
+            verify {
+                activity.openToBrowserAndLoad(
+                    searchTermOrURL = "$WEB_COMPAT_REPORTER_URL${selectedTab.content.url}",
+                    newTab = true,
+                    from = BrowserDirection.FromGlobal,
+                )
+            }
+        }
+
     private fun createController(
         scope: CoroutineScope,
         store: BrowserStore,
@@ -856,6 +949,7 @@ class DefaultBrowserToolbarMenuControllerTest {
     ) = DefaultBrowserToolbarMenuController(
         fragment = fragment,
         store = store,
+        appStore = appStore,
         activity = activity,
         navController = navController,
         settings = settings,
@@ -864,15 +958,13 @@ class DefaultBrowserToolbarMenuControllerTest {
         customTabSessionId = customTabSessionId,
         openInFenixIntent = openInFenixIntent,
         scope = scope,
-        snackbarParent = snackbarParent,
         tabCollectionStorage = tabCollectionStorage,
         bookmarkTapped = bookmarkTapped,
         readerModeController = readerModeController,
         sessionFeature = sessionFeatureWrapper,
         topSitesStorage = topSitesStorage,
         pinnedSiteStorage = pinnedSiteStorage,
-        onShowPinVerification = {},
-        onBiometricAuthenticationSuccessful = {},
+        deleteAndQuit = { _ -> deleteAndQuitCalled = true },
     ).apply {
         ioScope = scope
     }
