@@ -8,6 +8,8 @@ import io.mockk.coEvery
 import io.mockk.coVerify
 import io.mockk.every
 import io.mockk.mockk
+import io.mockk.verify
+import kotlinx.coroutines.ExperimentalCoroutinesApi
 import kotlinx.coroutines.test.advanceUntilIdle
 import mozilla.components.service.nimbus.messaging.Message
 import mozilla.components.service.nimbus.messaging.MessageData
@@ -24,25 +26,32 @@ import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
 import org.mozilla.fenix.components.AppStore
+import org.mozilla.fenix.components.appstate.AppAction
 import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.Evaluate
 import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.MessageClicked
 import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.MessageDismissed
+import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.MicrosurveyAction
+import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.MicrosurveyAction.Dismissed
 import org.mozilla.fenix.components.appstate.AppAction.MessagingAction.Restore
 import org.mozilla.fenix.components.appstate.AppState
 import org.mozilla.fenix.messaging.FenixMessageSurfaceId
 import org.mozilla.fenix.messaging.MessagingState
+import org.mozilla.fenix.utils.Settings
 
 class MessagingMiddlewareTest {
     @get:Rule
     val coroutinesTestRule = MainCoroutineRule()
     private val coroutineScope = coroutinesTestRule.scope
     private lateinit var controller: NimbusMessagingController
+    private lateinit var settings: Settings
 
     @Before
     fun setUp() {
         controller = mockk(relaxed = true)
+        settings = mockk(relaxed = true)
     }
 
+    @OptIn(ExperimentalCoroutinesApi::class) // advanceUntilIdle
     @Test
     fun `WHEN restored THEN get messages from the storage`() = runTestOnMain {
         val store = AppStore(
@@ -52,7 +61,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
         val message = createMessage()
@@ -78,7 +87,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -108,7 +117,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -121,6 +130,31 @@ class MessagingMiddlewareTest {
         coVerify { controller.onMessageClicked(message = message) }
     }
 
+    @Test
+    fun `GVIEN a microsurvey WHEN Started THEN only notify the controller`() =
+        runTestOnMain {
+            val message = createMessage(data = mockk<MessageData>(relaxed = true))
+            val store = AppStore(
+                AppState(
+                    messaging = MessagingState(
+                        messages = listOf(message),
+                        messageToShow = mapOf(message.id to message),
+                    ),
+                ),
+                listOf(
+                    MessagingMiddleware(controller, settings, coroutineScope),
+                ),
+            )
+
+            assertEquals(message, store.state.messaging.messages.first())
+
+            store.dispatch(MicrosurveyAction.Started(message.id)).joinBlocking()
+            store.waitUntilIdle()
+
+            assertFalse(store.state.messaging.messages.isEmpty())
+            coVerify { controller.onMicrosurveyStarted(id = message.id) }
+        }
+
     @Test
     fun `WHEN MessageDismissed THEN update storage`() = runTestOnMain {
         val metadata = createMetadata(displayCount = 1, "same-id")
@@ -132,7 +166,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
         store.dispatch(MessageDismissed(message)).joinBlocking()
@@ -142,6 +176,105 @@ class MessagingMiddlewareTest {
         coVerify { controller.onMessageDismissed(message = message) }
     }
 
+    @Test
+    fun `WHEN onMicrosurveyDismissed THEN update storage`() = runTestOnMain {
+        val metadata = createMetadata(displayCount = 1, "same-id")
+        val message = createMessage(metadata)
+        val store = AppStore(
+            AppState(
+                messaging = MessagingState(
+                    messages = listOf(message),
+                ),
+            ),
+            listOf(
+                MessagingMiddleware(controller, settings, coroutineScope),
+            ),
+        )
+        store.dispatch(Dismissed(message.id)).joinBlocking()
+        store.waitUntilIdle()
+
+        assertTrue(store.state.messaging.messages.isEmpty())
+        coVerify { controller.onMicrosurveyDismissed(message = message) }
+    }
+
+    @Test
+    fun `GIVEN a microsurvey WHEN onMicrosurveyShown THEN only notify the controller`() =
+        runTestOnMain {
+            val message = createMessage()
+            val store = AppStore(
+                AppState(
+                    messaging = MessagingState(
+                        messages = listOf(message),
+                        messageToShow = mapOf(message.id to message),
+                    ),
+                ),
+                listOf(
+                    MessagingMiddleware(controller, settings, coroutineScope),
+                ),
+            )
+
+            assertEquals(message, store.state.messaging.messages.first())
+
+            store.dispatch(AppAction.MessagingAction.MicrosurveyAction.Shown(message.id))
+                .joinBlocking()
+            store.waitUntilIdle()
+
+            assertFalse(store.state.messaging.messages.isEmpty())
+            coVerify { controller.onMicrosurveyShown(id = message.id) }
+        }
+
+    @Test
+    fun `GVIEN a microsurvey WHEN onMicrosurveyConfirmationShown THEN only notify the controller`() =
+        runTestOnMain {
+            val message = createMessage()
+            val store = AppStore(
+                AppState(
+                    messaging = MessagingState(
+                        messages = listOf(message),
+                        messageToShow = mapOf(message.id to message),
+                    ),
+                ),
+                listOf(
+                    MessagingMiddleware(controller, settings, coroutineScope),
+                ),
+            )
+
+            assertEquals(message, store.state.messaging.messages.first())
+
+            store.dispatch(AppAction.MessagingAction.MicrosurveyAction.SentConfirmationShown(message.id))
+                .joinBlocking()
+            store.waitUntilIdle()
+
+            assertFalse(store.state.messaging.messages.isEmpty())
+            coVerify { controller.onMicrosurveySentConfirmationShown(id = message.id) }
+        }
+
+    @Test
+    fun `GVIEN a microsurvey WHEN onPrivacyNoticeTapped THEN only notify the controller`() =
+        runTestOnMain {
+            val message = createMessage()
+            val store = AppStore(
+                AppState(
+                    messaging = MessagingState(
+                        messages = listOf(message),
+                        messageToShow = mapOf(message.id to message),
+                    ),
+                ),
+                listOf(
+                    MessagingMiddleware(controller, settings, coroutineScope),
+                ),
+            )
+
+            assertEquals(message, store.state.messaging.messages.first())
+
+            store.dispatch(AppAction.MessagingAction.MicrosurveyAction.OnPrivacyNoticeTapped(message.id))
+                .joinBlocking()
+            store.waitUntilIdle()
+
+            assertFalse(store.state.messaging.messages.isEmpty())
+            coVerify { controller.onMicrosurveyPrivacyNoticeTapped(id = message.id) }
+        }
+
     @Test
     fun `WHEN onMessageDismissed THEN remove the message from storage and state`() = runTestOnMain {
         val message = createMessage(createMetadata(displayCount = 1))
@@ -155,7 +288,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -168,6 +301,32 @@ class MessagingMiddlewareTest {
         assertEquals(0, store.state.messaging.messageToShow.size)
     }
 
+    @Test
+    fun `WHEN onMicrosurveyDismissed THEN remove the message from storage and state`() = runTestOnMain {
+        val message = createMessage(createMetadata(displayCount = 1))
+        val store = AppStore(
+            AppState(
+                messaging = MessagingState(
+                    messages = listOf(
+                        message,
+                    ),
+                    messageToShow = mapOf(message.surface to message),
+                ),
+            ),
+            listOf(
+                MessagingMiddleware(controller, settings, coroutineScope),
+            ),
+        )
+
+        store.dispatch(Dismissed(message.id)).joinBlocking()
+        store.waitUntilIdle()
+
+        // removeMessages causes messages size to be 0
+        assertEquals(0, store.state.messaging.messages.size)
+        // consumeMessageToShowIfNeeded causes messageToShow map size to be 0
+        assertEquals(0, store.state.messaging.messageToShow.size)
+    }
+
     @Test
     fun `WHEN consumeMessageToShowIfNeeded THEN consume the message`() = runTestOnMain {
         val message = createMessage()
@@ -181,7 +340,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -205,7 +364,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -243,7 +402,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -282,7 +441,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -304,10 +463,12 @@ class MessagingMiddlewareTest {
         store.waitUntilIdle()
 
         assertEquals(messageDisplayed.displayCount, store.state.messaging.messages[0].displayCount)
+        assertEquals(1, store.state.messaging.messages.size)
+        assertEquals(1, store.state.messaging.messageToShow.size)
     }
 
     @Test
-    fun `GIVEN a message with that surpassed the maxDisplayCount WHEN onMessagedDisplayed THEN remove the message and consume it`() = runTestOnMain {
+    fun `GIVEN a message has reached the maxDisplayCount WHEN onMessagedDisplayed THEN remove the message`() = runTestOnMain {
         val message = createMessage(createMetadata(displayCount = 4))
         val messageDisplayed = createMessage(createMetadata(displayCount = 5))
         assertFalse(message.isExpired)
@@ -323,7 +484,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
@@ -345,7 +506,41 @@ class MessagingMiddlewareTest {
         store.waitUntilIdle()
 
         assertEquals(0, store.state.messaging.messages.size)
-        assertEquals(0, store.state.messaging.messageToShow.size)
+        assertEquals(1, store.state.messaging.messageToShow.size)
+    }
+
+    @Test
+    fun `GIVEN a microsurvey message that surpassed the maxDisplayCount WHEN onMessagedDisplayed THEN remove the message, consume it & reset the pref`() = runTestOnMain {
+        val message = createMessage(metadata = createMetadata(displayCount = 4), data = MessageData(surface = "microsurvey"))
+        val messageDisplayed = createMessage(createMetadata(displayCount = 5), data = MessageData(surface = "microsurvey"))
+
+        assertFalse(message.isExpired)
+        assertTrue(messageDisplayed.isExpired)
+        settings.shouldShowMicrosurveyPrompt = true
+        verify { settings.shouldShowMicrosurveyPrompt = true }
+
+        val store = AppStore(
+            AppState(
+                messaging = MessagingState(
+                    messages = listOf(message),
+                    messageToShow = mapOf(message.surface to message),
+                ),
+            ),
+            listOf(MessagingMiddleware(controller, settings, coroutineScope)),
+        )
+
+        every { controller.getNextMessage(FenixMessageSurfaceId.MICROSURVEY, any()) } returns message
+        coEvery { controller.onMessageDisplayed(message, any()) } returns incrementDisplayCount(
+            message,
+        )
+        coEvery { controller.onMessageDisplayed(eq(message), any()) } returns messageDisplayed
+
+        store.dispatch(Evaluate(FenixMessageSurfaceId.MICROSURVEY)).joinBlocking()
+        store.waitUntilIdle()
+
+        verify { settings.shouldShowMicrosurveyPrompt = false }
+        assertEquals(0, store.state.messaging.messages.size)
+        assertEquals(1, store.state.messaging.messageToShow.size)
     }
 
     @Test
@@ -361,7 +556,7 @@ class MessagingMiddlewareTest {
                 ),
             ),
             listOf(
-                MessagingMiddleware(controller, coroutineScope),
+                MessagingMiddleware(controller, settings, coroutineScope),
             ),
         )
 
