@@ -4,19 +4,16 @@
 
 package org.mozilla.fenix.search
 
-import androidx.appcompat.app.AlertDialog
 import androidx.navigation.NavController
 import androidx.navigation.NavDirections
+import com.google.android.material.dialog.MaterialAlertDialogBuilder
 import io.mockk.MockKAnnotations
-import io.mockk.Runs
 import io.mockk.every
 import io.mockk.impl.annotations.MockK
-import io.mockk.just
 import io.mockk.mockk
-import io.mockk.mockkObject
 import io.mockk.spyk
-import io.mockk.unmockkObject
 import io.mockk.verify
+import io.mockk.verifyOrder
 import kotlinx.coroutines.test.runTest
 import mozilla.components.browser.state.action.AwesomeBarAction
 import mozilla.components.browser.state.action.BrowserAction
@@ -26,11 +23,9 @@ import mozilla.components.browser.state.state.BrowserState
 import mozilla.components.browser.state.store.BrowserStore
 import mozilla.components.concept.engine.EngineSession
 import mozilla.components.feature.tabs.TabsUseCases
-import mozilla.components.service.glean.testing.GleanTestRule
 import mozilla.components.support.test.libstate.ext.waitUntilIdle
 import mozilla.components.support.test.middleware.CaptureActionsMiddleware
 import mozilla.components.support.test.robolectric.testContext
-import org.junit.After
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertFalse
 import org.junit.Assert.assertNotNull
@@ -42,20 +37,29 @@ import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.fenix.BrowserDirection
 import org.mozilla.fenix.GleanMetrics.Events
-import org.mozilla.fenix.GleanMetrics.UnifiedSearch
+import org.mozilla.fenix.GleanMetrics.Toolbar
 import org.mozilla.fenix.HomeActivity
 import org.mozilla.fenix.R
 import org.mozilla.fenix.browser.browsingmode.BrowsingMode
-import org.mozilla.fenix.components.Core
-import org.mozilla.fenix.components.metrics.MetricsUtils
-import org.mozilla.fenix.helpers.FenixRobolectricTestRunner
+import org.mozilla.fenix.components.search.BOOKMARKS_SEARCH_ENGINE_ID
+import org.mozilla.fenix.components.search.HISTORY_SEARCH_ENGINE_ID
+import org.mozilla.fenix.components.search.TABS_SEARCH_ENGINE_ID
+import org.mozilla.fenix.components.usecases.FenixBrowserUseCases
+import org.mozilla.fenix.ext.components
+import org.mozilla.fenix.helpers.FenixGleanTestRule
+import org.mozilla.fenix.search.SearchDialogFragmentDirections.Companion.actionCrashListFragment
+import org.mozilla.fenix.search.SearchDialogFragmentDirections.Companion.actionGleanDebugToolsFragment
 import org.mozilla.fenix.search.SearchDialogFragmentDirections.Companion.actionGlobalAddonsManagementFragment
+import org.mozilla.fenix.search.SearchDialogFragmentDirections.Companion.actionGlobalBrowser
 import org.mozilla.fenix.search.SearchDialogFragmentDirections.Companion.actionGlobalSearchEngineFragment
 import org.mozilla.fenix.search.toolbar.SearchSelectorMenu
 import org.mozilla.fenix.settings.SupportUtils
+import org.mozilla.fenix.telemetry.ACTION_SEARCH_ENGINE_SELECTED
+import org.mozilla.fenix.telemetry.SOURCE_ADDRESS_BAR
 import org.mozilla.fenix.utils.Settings
+import org.robolectric.RobolectricTestRunner
 
-@RunWith(FenixRobolectricTestRunner::class) // for gleanTestRule
+@RunWith(RobolectricTestRunner::class) // for gleanTestRule
 class SearchDialogControllerTest {
 
     @MockK(relaxed = true)
@@ -72,16 +76,18 @@ class SearchDialogControllerTest {
     @MockK(relaxed = true)
     private lateinit var settings: Settings
 
+    @MockK(relaxed = true)
+    private lateinit var fenixBrowserUseCases: FenixBrowserUseCases
+
     private lateinit var middleware: CaptureActionsMiddleware<BrowserState, BrowserAction>
     private lateinit var browserStore: BrowserStore
 
     @get:Rule
-    val gleanTestRule = GleanTestRule(testContext)
+    val gleanTestRule = FenixGleanTestRule(testContext)
 
     @Before
     fun setUp() {
         MockKAnnotations.init(this)
-        mockkObject(MetricsUtils)
         middleware = CaptureActionsMiddleware()
         browserStore = BrowserStore(
             middleware = listOf(middleware),
@@ -89,15 +95,13 @@ class SearchDialogControllerTest {
         every { store.state.tabId } returns "test-tab-id"
         every { store.state.searchEngineSource.searchEngine } returns searchEngine
         every { searchEngine.type } returns SearchEngine.Type.BUNDLED
+        every { searchEngine.id } returns "test-search-engine-id"
+        every { searchEngine.telemetrySuffix } returns "test-telemetry-suffix"
+
         every { navController.currentDestination } returns mockk {
             every { id } returns R.id.searchDialogFragment
         }
-        every { MetricsUtils.recordSearchMetrics(searchEngine, any(), any()) } just Runs
-    }
-
-    @After
-    fun teardown() {
-        unmockkObject(MetricsUtils)
+        every { activity.components.nimbus.events } returns mockk(relaxUnitFun = true)
     }
 
     @Test
@@ -111,13 +115,50 @@ class SearchDialogControllerTest {
 
         browserStore.waitUntilIdle()
 
-        verify {
-            activity.openToBrowserAndLoad(
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
                 searchTermOrURL = url,
                 newTab = false,
-                from = BrowserDirection.FromSearchDialog,
-                engine = searchEngine,
                 forceSearch = false,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
+            )
+        }
+
+        middleware.assertLastAction(AwesomeBarAction.EngagementFinished::class) { action ->
+            assertFalse(action.abandoned)
+        }
+
+        assertNotNull(Events.enteredUrl.testGetValue())
+        val snapshot = Events.enteredUrl.testGetValue()!!
+        assertEquals(1, snapshot.size)
+        assertEquals("false", snapshot.single().extra?.getValue("autocomplete"))
+    }
+
+    @Test
+    fun `GIVEN default search engine is selected and homepage as a new tab is enabled WHEN url is committed THEN load the url`() {
+        val url = "https://www.google.com/"
+        assertNull(Events.enteredUrl.testGetValue())
+
+        every { store.state.defaultEngine } returns searchEngine
+        every { settings.enableHomepageAsNewTab } returns true
+        every { store.state.tabId } returns null
+
+        createController().handleUrlCommitted(url)
+
+        browserStore.waitUntilIdle()
+
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
+                searchTermOrURL = url,
+                newTab = false,
+                forceSearch = false,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
             )
         }
 
@@ -142,13 +183,15 @@ class SearchDialogControllerTest {
 
         browserStore.waitUntilIdle()
 
-        verify {
-            activity.openToBrowserAndLoad(
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
                 searchTermOrURL = url,
                 newTab = false,
-                from = BrowserDirection.FromSearchDialog,
-                engine = searchEngine,
                 forceSearch = true,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
             )
         }
 
@@ -190,13 +233,43 @@ class SearchDialogControllerTest {
 
         browserStore.waitUntilIdle()
 
-        verify {
-            activity.openToBrowserAndLoad(
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
                 searchTermOrURL = searchTerm,
                 newTab = false,
-                from = BrowserDirection.FromSearchDialog,
-                engine = searchEngine,
                 forceSearch = true,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
+            )
+        }
+
+        middleware.assertLastAction(AwesomeBarAction.EngagementFinished::class) { action ->
+            assertFalse(action.abandoned)
+        }
+    }
+
+    @Test
+    fun `GIVEN homepage as a new tab is enabled WHEN search term is committed THEN perform search in the existing tab`() {
+        val searchTerm = "Firefox"
+
+        every { settings.enableHomepageAsNewTab } returns true
+        every { store.state.tabId } returns null
+
+        createController().handleUrlCommitted(searchTerm)
+
+        browserStore.waitUntilIdle()
+
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
+                searchTermOrURL = searchTerm,
+                newTab = false,
+                forceSearch = true,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
             )
         }
 
@@ -237,19 +310,13 @@ class SearchDialogControllerTest {
     @Test
     fun handleCrashesUrlCommitted() {
         val url = "about:crashes"
-        every { activity.packageName } returns "org.mozilla.fenix"
+        val directions = actionCrashListFragment()
 
         createController().handleUrlCommitted(url)
 
         browserStore.waitUntilIdle()
 
-        verify {
-            activity.startActivity(any())
-        }
-
-        middleware.assertLastAction(AwesomeBarAction.EngagementFinished::class) { action ->
-            assertFalse(action.abandoned)
-        }
+        verify { navController.navigate(directions) }
     }
 
     @Test
@@ -268,6 +335,18 @@ class SearchDialogControllerTest {
         }
     }
 
+    @Test
+    fun handleGleanUrlCommitted() {
+        val url = "about:glean"
+        val directions = actionGleanDebugToolsFragment()
+
+        createController().handleUrlCommitted(url)
+
+        browserStore.waitUntilIdle()
+
+        verify { navController.navigate(directions) }
+    }
+
     @Test
     fun handleMozillaUrlCommitted() {
         val url = "moz://a"
@@ -279,12 +358,15 @@ class SearchDialogControllerTest {
 
         browserStore.waitUntilIdle()
 
-        verify {
-            activity.openToBrowserAndLoad(
+        verifyOrder {
+            navController.navigate(actionGlobalBrowser())
+
+            fenixBrowserUseCases.loadUrlOrSearch(
                 searchTermOrURL = SupportUtils.getMozillaPageUrl(SupportUtils.MozillaPage.MANIFESTO),
                 newTab = false,
-                from = BrowserDirection.FromSearchDialog,
-                engine = searchEngine,
+                forceSearch = false,
+                private = activity.browsingModeManager.mode.isPrivate,
+                searchEngine = searchEngine,
             )
         }
 
@@ -329,6 +411,10 @@ class SearchDialogControllerTest {
 
         verify { store.dispatch(SearchFragmentAction.UpdateQuery(text)) }
 
+        val actionSlot = mutableListOf<SearchFragmentAction>()
+        verify { store.dispatch(capture(actionSlot)) }
+        assertTrue(actionSlot.any { it is SearchFragmentAction.AllowSearchSuggestionsInPrivateModePrompt })
+
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
     }
 
@@ -345,40 +431,6 @@ class SearchDialogControllerTest {
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
     }
 
-    @Test
-    fun `WHEN felt privacy is enabled THEN do not dispatch AllowSearchSuggestionsInPrivateModePrompt`() {
-        every { settings.feltPrivateBrowsingEnabled } returns true
-
-        val text = "mozilla"
-
-        createController().handleTextChanged(text)
-
-        browserStore.waitUntilIdle()
-
-        val actionSlot = mutableListOf<SearchFragmentAction>()
-        verify { store.dispatch(capture(actionSlot)) }
-        assertFalse(actionSlot.any { it is SearchFragmentAction.AllowSearchSuggestionsInPrivateModePrompt })
-
-        middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
-    }
-
-    @Test
-    fun `WHEN felt privacy is disabled THEN dispatch AllowSearchSuggestionsInPrivateModePrompt`() {
-        every { settings.feltPrivateBrowsingEnabled } returns false
-
-        val text = "mozilla"
-
-        createController().handleTextChanged(text)
-
-        browserStore.waitUntilIdle()
-
-        val actionSlot = mutableListOf<SearchFragmentAction>()
-        verify { store.dispatch(capture(actionSlot)) }
-        assertTrue(actionSlot.any { it is SearchFragmentAction.AllowSearchSuggestionsInPrivateModePrompt })
-
-        middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
-    }
-
     @Test
     fun handleUrlTapped() {
         val url = "https://www.google.com/"
@@ -410,6 +462,67 @@ class SearchDialogControllerTest {
         }
     }
 
+    @Test
+    fun `GIVEN homepage as a new tab is enabled WHEN an url suggestion is tapped THEN load url in the existing tab`() {
+        val url = "https://www.google.com/"
+        val flags = EngineSession.LoadUrlFlags.all()
+
+        assertNull(Events.enteredUrl.testGetValue())
+
+        every { settings.enableHomepageAsNewTab } returns true
+        every { store.state.tabId } returns null
+
+        createController().handleUrlTapped(url, flags)
+        createController().handleUrlTapped(url)
+
+        browserStore.waitUntilIdle()
+
+        verify {
+            activity.openToBrowserAndLoad(
+                searchTermOrURL = url,
+                newTab = false,
+                from = BrowserDirection.FromSearchDialog,
+                flags = flags,
+            )
+        }
+
+        assertNotNull(Events.enteredUrl.testGetValue())
+        val snapshot = Events.enteredUrl.testGetValue()!!
+        assertEquals(2, snapshot.size)
+        assertEquals("false", snapshot.first().extra?.getValue("autocomplete"))
+        assertEquals("false", snapshot[1].extra?.getValue("autocomplete"))
+
+        middleware.assertLastAction(AwesomeBarAction.EngagementFinished::class) { action ->
+            assertFalse(action.abandoned)
+        }
+    }
+
+    @Test
+    fun `GIVEN homepage as a new tab is enabled WHEN a search suggestion is tapped THEN perform search in the existing tab`() {
+        val searchTerms = "fenix"
+
+        every { settings.enableHomepageAsNewTab } returns true
+        every { store.state.tabId } returns null
+
+        createController().handleSearchTermsTapped(searchTerms)
+
+        browserStore.waitUntilIdle()
+
+        verify {
+            activity.openToBrowserAndLoad(
+                searchTermOrURL = searchTerms,
+                newTab = false,
+                from = BrowserDirection.FromSearchDialog,
+                engine = searchEngine,
+                forceSearch = true,
+            )
+        }
+
+        middleware.assertLastAction(AwesomeBarAction.EngagementFinished::class) { action ->
+            assertFalse(action.abandoned)
+        }
+    }
+
     @Test
     fun handleSearchTermsTapped() {
         val searchTerms = "fenix"
@@ -452,23 +565,16 @@ class SearchDialogControllerTest {
         verify { store.dispatch(SearchFragmentAction.SearchShortcutEngineSelected(searchEngine, browsingMode, settings)) }
 
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
-
-        assertNotNull(UnifiedSearch.engineSelected.testGetValue())
-        val recordedEvents = UnifiedSearch.engineSelected.testGetValue()!!
-        assertEquals(1, recordedEvents.size)
-        val eventExtra = recordedEvents.single().extra
-        assertNotNull(eventExtra)
-        assertTrue(eventExtra!!.containsKey("engine"))
-        assertEquals(searchEngine.name, eventExtra["engine"])
+        assertSearchEngineSelectedTelemetryRecorded(searchEngine.name)
     }
 
     @Test
     fun `WHEN history search engine is selected THEN dispatch correct action`() {
         val searchEngine: SearchEngine = mockk(relaxed = true)
         every { searchEngine.type } returns SearchEngine.Type.APPLICATION
-        every { searchEngine.id } returns Core.HISTORY_SEARCH_ENGINE_ID
+        every { searchEngine.id } returns HISTORY_SEARCH_ENGINE_ID
 
-        assertNull(UnifiedSearch.engineSelected.testGetValue())
+        assertNull(Toolbar.buttonTapped.testGetValue())
 
         var focusToolbarInvoked = false
         createController(
@@ -484,22 +590,16 @@ class SearchDialogControllerTest {
 
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
 
-        assertNotNull(UnifiedSearch.engineSelected.testGetValue())
-        val recordedEvents = UnifiedSearch.engineSelected.testGetValue()!!
-        assertEquals(1, recordedEvents.size)
-        val eventExtra = recordedEvents.single().extra
-        assertNotNull(eventExtra)
-        assertTrue(eventExtra!!.containsKey("engine"))
-        assertEquals("history", eventExtra["engine"])
+        assertSearchEngineSelectedTelemetryRecorded("history")
     }
 
     @Test
     fun `WHEN bookmarks search engine is selected THEN dispatch correct action`() {
         val searchEngine: SearchEngine = mockk(relaxed = true)
         every { searchEngine.type } returns SearchEngine.Type.APPLICATION
-        every { searchEngine.id } returns Core.BOOKMARKS_SEARCH_ENGINE_ID
+        every { searchEngine.id } returns BOOKMARKS_SEARCH_ENGINE_ID
 
-        assertNull(UnifiedSearch.engineSelected.testGetValue())
+        assertNull(Toolbar.buttonTapped.testGetValue())
 
         var focusToolbarInvoked = false
         createController(
@@ -515,22 +615,16 @@ class SearchDialogControllerTest {
 
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
 
-        assertNotNull(UnifiedSearch.engineSelected.testGetValue())
-        val recordedEvents = UnifiedSearch.engineSelected.testGetValue()!!
-        assertEquals(1, recordedEvents.size)
-        val eventExtra = recordedEvents.single().extra
-        assertNotNull(eventExtra)
-        assertTrue(eventExtra!!.containsKey("engine"))
-        assertEquals("bookmarks", eventExtra["engine"])
+        assertSearchEngineSelectedTelemetryRecorded("bookmarks")
     }
 
     @Test
     fun `WHEN tabs search engine is selected THEN dispatch correct action`() {
         val searchEngine: SearchEngine = mockk(relaxed = true)
         every { searchEngine.type } returns SearchEngine.Type.APPLICATION
-        every { searchEngine.id } returns Core.TABS_SEARCH_ENGINE_ID
+        every { searchEngine.id } returns TABS_SEARCH_ENGINE_ID
 
-        assertNull(UnifiedSearch.engineSelected.testGetValue())
+        assertNull(Toolbar.buttonTapped.testGetValue())
 
         var focusToolbarInvoked = false
         createController(
@@ -546,13 +640,7 @@ class SearchDialogControllerTest {
 
         middleware.assertNotDispatched(AwesomeBarAction.EngagementFinished::class)
 
-        assertNotNull(UnifiedSearch.engineSelected.testGetValue())
-        val recordedEvents = UnifiedSearch.engineSelected.testGetValue()!!
-        assertEquals(1, recordedEvents.size)
-        val eventExtra = recordedEvents.single().extra
-        assertNotNull(eventExtra)
-        assertTrue(eventExtra!!.containsKey("engine"))
-        assertEquals("tabs", eventExtra["engine"])
+        assertSearchEngineSelectedTelemetryRecorded("tabs")
     }
 
     @Test
@@ -605,7 +693,7 @@ class SearchDialogControllerTest {
 
     @Test
     fun `show camera permissions needed dialog`() {
-        val dialogBuilder: AlertDialog.Builder = mockk(relaxed = true)
+        val dialogBuilder: MaterialAlertDialogBuilder = mockk(relaxed = true)
 
         val spyController = spyk(createController())
         every { spyController.buildDialog() } returns dialogBuilder
@@ -635,6 +723,7 @@ class SearchDialogControllerTest {
             activity = activity,
             store = browserStore,
             tabsUseCases = TabsUseCases(browserStore),
+            fenixBrowserUseCases = fenixBrowserUseCases,
             fragmentStore = store,
             navController = navController,
             settings = settings,
@@ -645,4 +734,15 @@ class SearchDialogControllerTest {
             dismissDialogAndGoBack = dismissDialogAndGoBack,
         )
     }
+
+    private fun assertSearchEngineSelectedTelemetryRecorded(
+        extra: String,
+    ) {
+        val values = Toolbar.buttonTapped.testGetValue()
+        assertNotNull(values)
+        val last = values!!.last()
+        assertEquals(ACTION_SEARCH_ENGINE_SELECTED, last.extra?.get("item"))
+        assertEquals(SOURCE_ADDRESS_BAR, last.extra?.get("source"))
+        assertEquals(extra, last.extra?.get("extra"))
+    }
 }
