@@ -4,26 +4,18 @@
 
 package org.mozilla.fenix.settings
 
-import android.os.Build
-import androidx.core.app.NotificationManagerCompat
 import androidx.fragment.app.FragmentActivity
 import androidx.preference.Preference
 import io.mockk.every
 import io.mockk.mockk
-import io.mockk.mockkObject
-import io.mockk.mockkStatic
-import io.mockk.unmockkObject
-import io.mockk.unmockkStatic
 import io.mockk.verify
+import kotlinx.coroutines.ExperimentalCoroutinesApi
 import kotlinx.coroutines.test.advanceUntilIdle
-import mozilla.components.browser.state.state.SearchState
-import mozilla.components.browser.state.state.selectedOrDefaultSearchEngine
 import mozilla.components.concept.fetch.Client
 import mozilla.components.service.fxa.manager.FxaAccountManager
 import mozilla.components.support.test.robolectric.testContext
 import mozilla.components.support.test.rule.MainCoroutineRule
 import mozilla.components.support.test.rule.runTestOnMain
-import org.junit.After
 import org.junit.Assert.assertEquals
 import org.junit.Assert.assertFalse
 import org.junit.Assert.assertNotNull
@@ -32,18 +24,16 @@ import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
 import org.junit.runner.RunWith
-import org.mozilla.fenix.Config
-import org.mozilla.fenix.FeatureFlags
 import org.mozilla.fenix.R
 import org.mozilla.fenix.ext.components
 import org.mozilla.fenix.ext.getPreferenceKey
 import org.mozilla.fenix.ext.settings
-import org.mozilla.fenix.helpers.FenixRobolectricTestRunner
 import org.mozilla.fenix.utils.Settings
 import org.robolectric.Robolectric
+import org.robolectric.RobolectricTestRunner
 import java.io.IOException
 
-@RunWith(FenixRobolectricTestRunner::class)
+@RunWith(RobolectricTestRunner::class)
 class SettingsFragmentTest {
 
     @get:Rule
@@ -59,18 +49,18 @@ class SettingsFragmentTest {
         every { testContext.components.core.engine.profiler } returns mockk(relaxed = true)
         every { testContext.components.core.client } returns client
         every { testContext.components.settings } returns mockk(relaxed = true)
+        every { testContext.components.core.store.state.translationEngine } returns mockk(relaxed = true)
         every { testContext.components.addonManager } returns mockk(relaxed = true)
         every { testContext.components.analytics } returns mockk(relaxed = true)
         every { testContext.components.backgroundServices } returns mockk(relaxed = true)
 
-        mockkObject(FeatureFlags)
-
         val activity = Robolectric.buildActivity(FragmentActivity::class.java).create().get()
         activity.supportFragmentManager.beginTransaction()
             .add(settingsFragment, "test")
             .commitNow()
     }
 
+    @OptIn(ExperimentalCoroutinesApi::class) // advanceUntilIdle
     @Test
     fun `Add-on collection override pref is visible if debug menu active and feature is enabled`() = runTestOnMain {
         val settingsFragment = SettingsFragment()
@@ -82,24 +72,28 @@ class SettingsFragmentTest {
 
         advanceUntilIdle()
 
-        every { FeatureFlags.customExtensionCollectionFeature } returns true
-
         val preferenceAmoCollectionOverride = settingsFragment.findPreference<Preference>(
             settingsFragment.getPreferenceKey(R.string.pref_key_override_amo_collection),
         )
 
-        settingsFragment.setupAmoCollectionOverridePreference(mockk(relaxed = true))
+        settingsFragment.setupAmoCollectionOverridePreference(
+            mockk(relaxed = true),
+            true,
+        )
         assertNotNull(preferenceAmoCollectionOverride)
         assertFalse(preferenceAmoCollectionOverride!!.isVisible)
 
         val settings: Settings = mockk(relaxed = true)
         every { settings.showSecretDebugMenuThisSession } returns true
-        settingsFragment.setupAmoCollectionOverridePreference(settings)
+        settingsFragment.setupAmoCollectionOverridePreference(
+            settings,
+            true,
+        )
         assertTrue(preferenceAmoCollectionOverride.isVisible)
     }
 
+    @OptIn(ExperimentalCoroutinesApi::class) // advanceUntilIdle
     @Test
-    @org.robolectric.annotation.Config(sdk = [Build.VERSION_CODES.Q])
     fun `Install add-on from file pref is visible if debug menu active and feature is enabled`() = runTestOnMain {
         val settingsFragment = SettingsFragment()
         val activity = Robolectric.buildActivity(FragmentActivity::class.java).create().get()
@@ -123,32 +117,9 @@ class SettingsFragmentTest {
         every { settings.showSecretDebugMenuThisSession } returns true
         settingsFragment.setupInstallAddonFromFilePreference(settings)
         assertTrue(preference.isVisible)
-        unmockkObject(Config)
-    }
-
-    @Test
-    @org.robolectric.annotation.Config(sdk = [Build.VERSION_CODES.P])
-    fun `Install add-on from file pref is invisible below Android 10`() = runTestOnMain {
-        val settingsFragment = SettingsFragment()
-        val activity = Robolectric.buildActivity(FragmentActivity::class.java).create().get()
-
-        activity.supportFragmentManager.beginTransaction()
-            .add(settingsFragment, "test")
-            .commitNow()
-
-        advanceUntilIdle()
-
-        val preference = settingsFragment.findPreference<Preference>(
-            settingsFragment.getPreferenceKey(R.string.pref_key_install_local_addon),
-        )
-
-        val settings: Settings = mockk(relaxed = true)
-
-        every { settings.showSecretDebugMenuThisSession } returns true
-        settingsFragment.setupInstallAddonFromFilePreference(settings)
-        assertFalse(preference!!.isVisible)
     }
 
+    @OptIn(ExperimentalCoroutinesApi::class) // advanceUntilIdle
     @Test
     fun `Add-on collection override pref is visible if already configured and feature is enabled`() = runTestOnMain {
         val settingsFragment = SettingsFragment()
@@ -160,13 +131,14 @@ class SettingsFragmentTest {
 
         advanceUntilIdle()
 
-        every { FeatureFlags.customExtensionCollectionFeature } returns true
-
         val preferenceAmoCollectionOverride = settingsFragment.findPreference<Preference>(
             settingsFragment.getPreferenceKey(R.string.pref_key_override_amo_collection),
         )
 
-        settingsFragment.setupAmoCollectionOverridePreference(mockk(relaxed = true))
+        settingsFragment.setupAmoCollectionOverridePreference(
+            mockk(relaxed = true),
+            true,
+        )
         assertNotNull(preferenceAmoCollectionOverride)
         assertFalse(preferenceAmoCollectionOverride!!.isVisible)
 
@@ -174,14 +146,21 @@ class SettingsFragmentTest {
         every { settings.showSecretDebugMenuThisSession } returns false
 
         every { settings.amoCollectionOverrideConfigured() } returns false
-        settingsFragment.setupAmoCollectionOverridePreference(settings)
+        settingsFragment.setupAmoCollectionOverridePreference(
+            settings,
+            true,
+        )
         assertFalse(preferenceAmoCollectionOverride.isVisible)
 
         every { settings.amoCollectionOverrideConfigured() } returns true
-        settingsFragment.setupAmoCollectionOverridePreference(settings)
+        settingsFragment.setupAmoCollectionOverridePreference(
+            settings,
+            true,
+        )
         assertTrue(preferenceAmoCollectionOverride.isVisible)
     }
 
+    @OptIn(ExperimentalCoroutinesApi::class) // advanceUntilIdle
     @Test
     fun `Add-on collection override pref is not visible if feature is disabled`() = runTestOnMain {
         val settingsFragment = SettingsFragment()
@@ -193,20 +172,24 @@ class SettingsFragmentTest {
 
         advanceUntilIdle()
 
-        every { FeatureFlags.customExtensionCollectionFeature } returns false
-
         val preferenceAmoCollectionOverride = settingsFragment.findPreference<Preference>(
             settingsFragment.getPreferenceKey(R.string.pref_key_override_amo_collection),
         )
 
         val settings: Settings = mockk(relaxed = true)
-        settingsFragment.setupAmoCollectionOverridePreference(settings)
+        settingsFragment.setupAmoCollectionOverridePreference(
+            settings,
+            false,
+        )
         assertNotNull(preferenceAmoCollectionOverride)
         assertFalse(preferenceAmoCollectionOverride!!.isVisible)
 
         every { settings.showSecretDebugMenuThisSession } returns true
         every { settings.amoCollectionOverrideConfigured() } returns true
-        settingsFragment.setupAmoCollectionOverridePreference(settings)
+        settingsFragment.setupAmoCollectionOverridePreference(
+            settings,
+            false,
+        )
         assertFalse(preferenceAmoCollectionOverride.isVisible)
     }
 
@@ -216,14 +199,12 @@ class SettingsFragmentTest {
             R.string.pref_key_notifications,
         )
         val summary = testContext.getString(R.string.notifications_not_allowed_summary)
-        mockkStatic(NotificationManagerCompat::class)
-        every { NotificationManagerCompat.from(any()).areNotificationsEnabled() } returns false
+
         assertTrue(notificationPreference.summary.isNullOrEmpty())
 
-        settingsFragment.setupNotificationPreference()
+        settingsFragment.setupNotificationPreference(false)
 
         assertEquals(summary, notificationPreference.summary)
-        unmockkStatic(NotificationManagerCompat::class)
     }
 
     @Test
@@ -232,14 +213,12 @@ class SettingsFragmentTest {
             R.string.pref_key_notifications,
         )
         val summary = testContext.getString(R.string.notifications_allowed_summary)
-        mockkStatic(NotificationManagerCompat::class)
-        every { NotificationManagerCompat.from(any()).areNotificationsEnabled() } returns true
+
         assertTrue(notificationPreference.summary.isNullOrEmpty())
 
-        settingsFragment.setupNotificationPreference()
+        settingsFragment.setupNotificationPreference(true)
 
         assertEquals(summary, notificationPreference.summary)
-        unmockkStatic(NotificationManagerCompat::class)
     }
 
     @Test
@@ -254,7 +233,7 @@ class SettingsFragmentTest {
         val summary =
             testContext.getString(R.string.opening_screen_after_four_hours_of_inactivity_summary)
 
-        settingsFragment.setupHomepagePreference()
+        settingsFragment.setupHomepagePreference(testContext.settings())
 
         assertEquals(summary, homepagePreference.summary)
     }
@@ -270,7 +249,7 @@ class SettingsFragmentTest {
         assertTrue(homepagePreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.opening_screen_last_tab_summary)
 
-        settingsFragment.setupHomepagePreference()
+        settingsFragment.setupHomepagePreference(testContext.settings())
 
         assertEquals(summary, homepagePreference.summary)
     }
@@ -286,7 +265,7 @@ class SettingsFragmentTest {
         assertTrue(homepagePreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.opening_screen_homepage_summary)
 
-        settingsFragment.setupHomepagePreference()
+        settingsFragment.setupHomepagePreference(testContext.settings())
 
         assertEquals(summary, homepagePreference.summary)
     }
@@ -297,14 +276,10 @@ class SettingsFragmentTest {
         val searchPreference = settingsFragment.requirePreference<Preference>(
             R.string.pref_key_search_settings,
         )
-        mockkStatic("mozilla.components.browser.state.state.SearchStateKt")
-        every { testContext.components.core.store.state.search } returns mockk(relaxed = true)
-        every { any<SearchState>().selectedOrDefaultSearchEngine } returns mockk {
-            every { name } returns searchEngineName
-        }
+
         assertTrue(searchPreference.summary.isNullOrEmpty())
 
-        settingsFragment.setupSearchPreference()
+        settingsFragment.setupSearchPreference(searchEngineName)
 
         assertEquals(searchEngineName, searchPreference.summary)
     }
@@ -321,7 +296,7 @@ class SettingsFragmentTest {
         assertTrue(trackingProtectionPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.tracking_protection_custom)
 
-        settingsFragment.setupTrackingProtectionPreference()
+        settingsFragment.setupTrackingProtectionPreference(testContext.settings())
 
         assertEquals(summary, trackingProtectionPreference.summary)
     }
@@ -338,7 +313,7 @@ class SettingsFragmentTest {
         assertTrue(trackingProtectionPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.tracking_protection_strict)
 
-        settingsFragment.setupTrackingProtectionPreference()
+        settingsFragment.setupTrackingProtectionPreference(testContext.settings())
 
         assertEquals(summary, trackingProtectionPreference.summary)
     }
@@ -355,7 +330,7 @@ class SettingsFragmentTest {
         assertTrue(trackingProtectionPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.tracking_protection_standard)
 
-        settingsFragment.setupTrackingProtectionPreference()
+        settingsFragment.setupTrackingProtectionPreference(testContext.settings())
 
         assertEquals(summary, trackingProtectionPreference.summary)
     }
@@ -369,7 +344,7 @@ class SettingsFragmentTest {
         assertTrue(trackingProtectionPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.tracking_protection_off)
 
-        settingsFragment.setupTrackingProtectionPreference()
+        settingsFragment.setupTrackingProtectionPreference(testContext.settings())
 
         assertEquals(summary, trackingProtectionPreference.summary)
     }
@@ -385,7 +360,7 @@ class SettingsFragmentTest {
         assertTrue(httpsOnlyPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.preferences_https_only_on_private)
 
-        settingsFragment.setupHttpsOnlyPreferences()
+        settingsFragment.setupHttpsOnlyPreferences(testContext.settings())
 
         assertEquals(summary, httpsOnlyPreference.summary)
     }
@@ -401,7 +376,7 @@ class SettingsFragmentTest {
         assertTrue(httpsOnlyPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.preferences_https_only_on_all)
 
-        settingsFragment.setupHttpsOnlyPreferences()
+        settingsFragment.setupHttpsOnlyPreferences(testContext.settings())
 
         assertEquals(summary, httpsOnlyPreference.summary)
     }
@@ -415,7 +390,7 @@ class SettingsFragmentTest {
         assertTrue(httpsOnlyPreference.summary.isNullOrEmpty())
         val summary = testContext.getString(R.string.preferences_https_only_off)
 
-        settingsFragment.setupHttpsOnlyPreferences()
+        settingsFragment.setupHttpsOnlyPreferences(testContext.settings())
 
         assertEquals(summary, httpsOnlyPreference.summary)
     }
@@ -439,9 +414,4 @@ class SettingsFragmentTest {
 
         verify { accountManager.unregister(settingsFragment.accountObserver) }
     }
-
-    @After
-    fun tearDown() {
-        unmockkObject(FeatureFlags)
-    }
 }
