@@ -121,6 +121,7 @@ class GleanMetricsService(context: Context) : MetricsService {
                 httpClient = ConceptFetchHttpUploader(
                     client = lazy(LazyThreadSafetyMode.NONE) { components.client },
                     usePrivateRequest = true,
+                    supportsOhttp = true,
                 ),
             ),
             buildInfo = GleanBuildInfo.buildInfo,
@@ -196,10 +197,10 @@ class GleanMetricsService(context: Context) : MetricsService {
         MozillaProducts.isFenixDefaultBrowser.set(isFenixDefaultBrowser)
 
         // tracking protection metrics
-        TrackingProtection.hasAdvertisingBlocked.set(settings.hasAdvertisingBlocked())
-        TrackingProtection.hasAnalyticsBlocked.set(settings.hasAnalyticsBlocked())
+        TrackingProtection.hasAdvertisingBlocked.set(settings.shouldBlockAdTrackers())
+        TrackingProtection.hasAnalyticsBlocked.set(settings.shouldBlockAnalyticTrackers())
         TrackingProtection.hasContentBlocked.set(settings.shouldBlockOtherTrackers())
-        TrackingProtection.hasSocialBlocked.set(settings.hasSocialBlocked())
+        TrackingProtection.hasSocialBlocked.set(settings.shouldBlockSocialTrackers())
 
         // theme telemetry
         val currentTheme =
