@@ -9,9 +9,14 @@ import android.os.Looper
 import android.view.KeyEvent
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import androidx.test.filters.MediumTest
-import org.hamcrest.Matchers.* // ktlint-disable no-wildcard-imports
+import junit.framework.TestCase.assertTrue
+import org.hamcrest.Matchers.equalTo
+import org.hamcrest.Matchers.isEmptyOrNullString
+import org.hamcrest.Matchers.not
+import org.hamcrest.Matchers.notNullValue
 import org.junit.Test
 import org.junit.runner.RunWith
+import org.mozilla.geckoview.Autocomplete
 import org.mozilla.geckoview.Autocomplete.Address
 import org.mozilla.geckoview.Autocomplete.AddressSelectOption
 import org.mozilla.geckoview.Autocomplete.CreditCard
@@ -27,6 +32,7 @@ import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoSession
 import org.mozilla.geckoview.GeckoSession.PromptDelegate
 import org.mozilla.geckoview.GeckoSession.PromptDelegate.AutocompleteRequest
+import org.mozilla.geckoview.TranslationsController
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
 
@@ -104,6 +110,7 @@ class AutocompleteTest : BaseSessionTest() {
                 // Enable login management since it's disabled in automation.
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -503,7 +510,7 @@ class AutocompleteTest : BaseSessionTest() {
                     equalTo(savedAddresses.size),
                 )
 
-                val addressOption = prompt.options.find { it.value.familyName == selectedAddress.familyName }
+                val addressOption = prompt.options.find { it.value.guid == selectedAddress.guid }
                 val address = addressOption?.value
 
                 assertThat("Address should not be null", address, notNullValue())
@@ -654,6 +661,38 @@ class AutocompleteTest : BaseSessionTest() {
         checkAddressesForCorrectness(savedAddresses.toTypedArray(), savedAddress)
     }
 
+    @Test
+    fun addressSelectAndFillWithoutGivenName() {
+        val name = "Peter Parker"
+        val streetAddress = "20 Ingram Street, Forest Hills Gardens, Queens"
+        val postalCode = "11375"
+        val country = "US"
+        val email = "spiderman@newyork.com"
+        val tel = "+1 180090021"
+        val organization = ""
+        val guid = "test-guid"
+        val builder = Address.Builder()
+            .guid(guid)
+            .name(name)
+            .streetAddress(streetAddress)
+            .postalCode(postalCode)
+            .country(country)
+            .email(email)
+            .tel(tel)
+            .organization(organization)
+
+        val savedAddress = builder.build()
+
+        val expectedAddress = builder
+            .givenName("Peter")
+            .familyName("Parker")
+            .build()
+
+        val savedAddresses = arrayOf(savedAddress)
+
+        checkAddressesForCorrectness(savedAddresses, expectedAddress)
+    }
+
     @Test
     fun addressSelectAndFillMultipleAddresses() {
         val names = arrayOf("Peter Parker", "Wade Wilson")
@@ -772,6 +811,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -835,6 +875,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -909,6 +950,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -991,6 +1033,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -1551,6 +1594,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -1661,6 +1705,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -1747,6 +1792,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.autofillForms.http" to true,
                 "dom.disable_open_during_load" to false,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -2025,6 +2071,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.autofillForms.http" to true,
                 "dom.disable_open_during_load" to false,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -2290,6 +2337,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.generation.available" to true,
                 "dom.disable_open_during_load" to false,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -2306,9 +2354,12 @@ class AutocompleteTest : BaseSessionTest() {
         var genPass = ""
 
         val saveHandled1 = GeckoResult<Void>()
+        val saveHandled2 = GeckoResult<Void>()
         val selectHandled = GeckoResult<Void>()
         var numSelects = 0
 
+        var onLoginSaveCount = 0
+
         sessionRule.delegateUntilTestEnd(object : StorageDelegate {
             @AssertCalled
             override fun onLoginFetch(domain: String): GeckoResult<Array<LoginEntry>>? {
@@ -2317,13 +2368,9 @@ class AutocompleteTest : BaseSessionTest() {
                 return GeckoResult.fromValue(null)
             }
 
-            @AssertCalled(count = 1)
+            @AssertCalled(count = 2)
             override fun onLoginSave(login: LoginEntry) {
-                assertThat(
-                    "Username should match",
-                    login.username,
-                    equalTo(user1),
-                )
+                onLoginSaveCount++
 
                 assertThat(
                     "Password should match",
@@ -2331,7 +2378,21 @@ class AutocompleteTest : BaseSessionTest() {
                     equalTo(genPass),
                 )
 
-                saveHandled1.complete(null)
+                if (onLoginSaveCount == 1) {
+                    assertThat(
+                        "Username should be empty",
+                        login.username,
+                        equalTo(""),
+                    )
+                    saveHandled1.complete(null)
+                } else if (onLoginSaveCount == 2) {
+                    assertThat(
+                        "Username should match",
+                        login.username,
+                        equalTo(user1),
+                    )
+                    saveHandled2.complete(null)
+                }
             }
 
             @AssertCalled(false)
@@ -2452,9 +2513,23 @@ class AutocompleteTest : BaseSessionTest() {
             equalTo(genPass),
         )
 
+        val filledConfirmPass = mainSession.evaluateJS(
+            "document.querySelector('#passConfirm').value",
+        ) as String
+
+        assertThat(
+            "Confirm password should match filled password",
+            filledConfirmPass,
+            equalTo(genPass),
+        )
+
+        sessionRule.waitForResult(saveHandled1)
+
         // Submit the selection.
         mainSession.evaluateJS("document.querySelector('#form1').submit()")
         mainSession.waitForPageStop()
+
+        sessionRule.waitForResult(saveHandled2)
     }
 
     @Test
@@ -2465,6 +2540,7 @@ class AutocompleteTest : BaseSessionTest() {
                 "signon.rememberSignons" to true,
                 "signon.autofillForms.http" to true,
                 "signon.userInputRequiredToCapture.enabled" to false,
+                "signon.testOnlyNotWaitForPaint" to true,
             ),
         )
 
@@ -2529,4 +2605,69 @@ class AutocompleteTest : BaseSessionTest() {
         mainSession.evaluateJS("document.querySelector('#user1').blur()")
         sessionRule.waitForResult(result)
     }
+
+    @Test
+    fun testAddressStructureGetFieldsForUS() {
+        val structureResult = Autocomplete.AddressStructure.getAddressStructure("US")
+
+        try {
+            sessionRule.waitForResult(structureResult)
+            assertTrue("Should not be able to retreive a structure.", true)
+        } catch (e: Exception) {
+            assertTrue("Should not have an exception.", false)
+        }
+
+        sessionRule.waitForResult(structureResult).let { fields ->
+            val expectedResult = listOf(
+                Pair("name", "autofill-address-name"),
+                Pair("organization", "autofill-address-organization"),
+                Pair("street-address", "autofill-address-street"),
+                Pair("address-level2", "autofill-address-city"),
+                Pair("address-level1", "autofill-address-state"),
+                Pair("postal-code", "autofill-address-zip"),
+                Pair("country", "autofill-address-country"),
+                Pair("tel", "autofill-address-tel"),
+                Pair("email", "autofill-address-email"),
+            )
+
+            expectedResult.forEachIndexed { index, pair ->
+                assertTrue(
+                    "Result should have id: ${pair.first}, localizationKey: ${pair.second}",
+                    fields[index].id == pair.first && fields[index].localizationKey == pair.second,
+                )
+            }
+        }
+    }
+
+    @Test
+    fun testAddressStructureGetFieldsForJP() {
+        val structureResult = Autocomplete.AddressStructure.getAddressStructure("JP")
+
+        try {
+            sessionRule.waitForResult(structureResult)
+            assertTrue("Should not be able to retreive a structure.", true)
+        } catch (e: Exception) {
+            assertTrue("Should not have an exception.", false)
+        }
+
+        sessionRule.waitForResult(structureResult).let { fields ->
+            val expectedResult = listOf(
+                Pair("postal-code", "autofill-address-postal-code"),
+                Pair("address-level1", "autofill-address-prefecture"),
+                Pair("street-address", "autofill-address-street"),
+                Pair("organization", "autofill-address-organization"),
+                Pair("name", "autofill-address-name"),
+                Pair("country", "autofill-address-country"),
+                Pair("tel", "autofill-address-tel"),
+                Pair("email", "autofill-address-email"),
+            )
+
+            expectedResult.forEachIndexed { index, pair ->
+                assertTrue(
+                    "Result should have id: ${pair.first}, localizationKey: ${pair.second}",
+                    fields[index].id == pair.first && fields[index].localizationKey == pair.second,
+                )
+            }
+        }
+    }
 }
