@@ -2,13 +2,13 @@ package org.mozilla.geckoview.test
 
 import android.graphics.Bitmap
 import android.graphics.BitmapFactory
-import androidx.test.platform.app.InstrumentationRegistry
+import androidx.core.graphics.get
 import androidx.test.filters.MediumTest
+import androidx.test.platform.app.InstrumentationRegistry
 import org.hamcrest.Matchers.equalTo
 import org.json.JSONObject
 import org.junit.After
 import org.junit.Assert.assertEquals
-import org.junit.Assert.assertTrue
 import org.junit.Assume.assumeThat
 import org.junit.Before
 import org.junit.Ignore
@@ -17,8 +17,8 @@ import org.junit.runner.RunWith
 import org.junit.runners.Parameterized
 import org.mozilla.geckoview.GeckoResult
 import org.mozilla.geckoview.GeckoSession
-import org.mozilla.geckoview.WebExtension
 import org.mozilla.geckoview.Image.ImageProcessingException
+import org.mozilla.geckoview.WebExtension
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule
 import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
 
@@ -26,6 +26,7 @@ import org.mozilla.geckoview.test.rule.GeckoSessionTestRule.AssertCalled
 @RunWith(Parameterized::class)
 class ExtensionActionTest : BaseSessionTest() {
     private var extension: WebExtension? = null
+    private var otherExtension: WebExtension? = null
     private var default: WebExtension.Action? = null
     private var backgroundPort: WebExtension.Port? = null
     private var windowPort: WebExtension.Port? = null
@@ -34,11 +35,14 @@ class ExtensionActionTest : BaseSessionTest() {
         @get:Parameterized.Parameters(name = "{0}")
         @JvmStatic
         val parameters = listOf(
-                arrayOf("#pageAction"),
-                arrayOf("#browserAction"))
+            arrayOf("#pageAction"),
+            arrayOf("#browserAction"),
+        )
     }
 
-    @field:Parameterized.Parameter(0) @JvmField var id: String = ""
+    @field:Parameterized.Parameter(0)
+    @JvmField
+    var id: String = ""
 
     private val controller
         get() = sessionRule.runtime.webExtensionController
@@ -56,35 +60,47 @@ class ExtensionActionTest : BaseSessionTest() {
         val backgroundPortResult = GeckoResult<WebExtension.Port>()
 
         extension = sessionRule.waitForResult(
-                controller.installBuiltIn("resource://android/assets/web_extensions/actions/"));
+            controller.installBuiltIn("resource://android/assets/web_extensions/actions/"),
+        )
+        // Another dummy extension, only used to check restrictions related to setting
+        // another extension url as a popup url, and so there is no delegate needed for it.
+        otherExtension = sessionRule.waitForResult(
+            controller.installBuiltIn("resource://android/assets/web_extensions/dummy/"),
+        )
 
         mainSession.webExtensionController.setMessageDelegate(
-                extension!!,
-                object : WebExtension.MessageDelegate {
-                    override fun onConnect(port: WebExtension.Port) {
-                        windowPortResult.complete(port)
-                    }
-                }, "browser")
-        extension!!.setMessageDelegate(object : WebExtension.MessageDelegate {
-            override fun onConnect(port: WebExtension.Port) {
-                backgroundPortResult.complete(port)
-            }
-        }, "browser")
+            extension!!,
+            object : WebExtension.MessageDelegate {
+                override fun onConnect(port: WebExtension.Port) {
+                    windowPortResult.complete(port)
+                }
+            },
+            "browser",
+        )
+        extension!!.setMessageDelegate(
+            object : WebExtension.MessageDelegate {
+                override fun onConnect(port: WebExtension.Port) {
+                    backgroundPortResult.complete(port)
+                }
+            },
+            "browser",
+        )
 
         sessionRule.addExternalDelegateDuringNextWait(
-                WebExtension.ActionDelegate::class,
-                extension!!::setActionDelegate,
-                { extension!!.setActionDelegate(null) },
-        object : WebExtension.ActionDelegate {
-            override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                assertEquals(action.title, "Test action default")
-                browserActionDefaultResult.complete(action)
-            }
-            override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                assertEquals(action.title, "Test action default")
-                pageActionDefaultResult.complete(action)
-            }
-        })
+            WebExtension.ActionDelegate::class,
+            extension!!::setActionDelegate,
+            { extension!!.setActionDelegate(null) },
+            object : WebExtension.ActionDelegate {
+                override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    assertEquals(action.title, "Test action default")
+                    browserActionDefaultResult.complete(action)
+                }
+                override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    assertEquals(action.title, "Test action default")
+                    pageActionDefaultResult.complete(action)
+                }
+            },
+        )
 
         mainSession.loadUri("http://example.com")
         sessionRule.waitForPageStop()
@@ -110,7 +126,7 @@ class ExtensionActionTest : BaseSessionTest() {
     }
 
     private val type: String
-        get() = when(id) {
+        get() = when (id) {
             "#pageAction" -> "pageAction"
             "#browserAction" -> "browserAction"
             else -> throw IllegalArgumentException()
@@ -123,6 +139,10 @@ class ExtensionActionTest : BaseSessionTest() {
             extension!!.setActionDelegate(null)
             sessionRule.waitForResult(controller.uninstall(extension!!))
         }
+
+        if (otherExtension != null) {
+            sessionRule.waitForResult(controller.uninstall(otherExtension!!))
+        }
     }
 
     private fun testBackgroundActionApi(message: String, tester: (WebExtension.Action) -> Unit) {
@@ -134,33 +154,80 @@ class ExtensionActionTest : BaseSessionTest() {
         backgroundPort!!.postMessage(json)
 
         sessionRule.addExternalDelegateDuringNextWait(
-                WebExtension.ActionDelegate::class,
-                extension!!::setActionDelegate,
-                { extension!!.setActionDelegate(null) },
-                object : WebExtension.ActionDelegate {
-            override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                if (sessionRule.currentCall.counter == 1) {
-                    // When attaching the delegate, we will receive a default message, ignore it
-                    return
+            WebExtension.ActionDelegate::class,
+            extension!!::setActionDelegate,
+            { extension!!.setActionDelegate(null) },
+            object : WebExtension.ActionDelegate {
+                override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    if (sessionRule.currentCall.counter == 1) {
+                        // When attaching the delegate, we will receive a default message, ignore it
+                        return
+                    }
+                    assertEquals(id, "#browserAction")
+                    default = action
+                    tester(action)
+                    result.complete(null)
                 }
-                assertEquals(id, "#browserAction")
-                default = action
-                tester(action)
-                result.complete(null)
-            }
-            override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                if (sessionRule.currentCall.counter == 1) {
-                    // When attaching the delegate, we will receive a default message, ignore it
-                    return
+                override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    if (sessionRule.currentCall.counter == 1) {
+                        // When attaching the delegate, we will receive a default message, ignore it
+                        return
+                    }
+                    assertEquals(id, "#pageAction")
+                    default = action
+                    tester(action)
+                    result.complete(null)
+                }
+            },
+        )
+
+        sessionRule.waitForResult(result)
+    }
+
+    private fun testSetPopup(popupUrl: String, isUrlAllowed: Boolean) {
+        val setPopupResult = GeckoResult<Void>()
+
+        backgroundPort!!.setDelegate(object : WebExtension.PortDelegate {
+            override fun onPortMessage(message: Any, port: WebExtension.Port) {
+                val json = message as JSONObject
+                if (json.getString("resultFor") == "setPopup" &&
+                    json.getString("type") == type
+                ) {
+                    if (isUrlAllowed != json.getBoolean("success")) {
+                        val expectedResString = when (isUrlAllowed) {
+                            true -> "allowed"
+                            else -> "disallowed"
+                        }
+                        setPopupResult.completeExceptionally(
+                            IllegalArgumentException(
+                                "Expected \"${popupUrl}\" to be ${ expectedResString }",
+                            ),
+                        )
+                    } else {
+                        setPopupResult.complete(null)
+                    }
+                } else {
+                    // We should NOT receive the expected message result.
+                    setPopupResult.completeExceptionally(
+                        IllegalArgumentException(
+                            "Received unexpected result for: ${json.getString("type")} ${json.getString("resultFor")}",
+                        ),
+                    )
                 }
-                assertEquals(id, "#pageAction")
-                default = action
-                tester(action)
-                result.complete(null)
             }
         })
 
-        sessionRule.waitForResult(result)
+        var json = JSONObject(
+            """{
+           "action": "setPopupCheckRestrictions",
+           "popup": "$popupUrl"
+        }""",
+        )
+
+        json.put("type", type)
+        windowPort!!.postMessage(json)
+
+        sessionRule.waitForResult(setPopupResult)
     }
 
     private fun testActionApi(message: String, tester: (WebExtension.Action) -> Unit) {
@@ -172,24 +239,26 @@ class ExtensionActionTest : BaseSessionTest() {
         windowPort!!.postMessage(json)
 
         sessionRule.addExternalDelegateDuringNextWait(
-                WebExtension.ActionDelegate::class,
-                { delegate ->
-                    mainSession.webExtensionController.setActionDelegate(extension!!, delegate) },
-                { mainSession.webExtensionController.setActionDelegate(extension!!, null) },
-        object : WebExtension.ActionDelegate {
-            override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                assertEquals(id, "#browserAction")
-                val resolved = action.withDefault(default!!)
-                tester(resolved)
-                result.complete(null)
-            }
-            override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
-                assertEquals(id, "#pageAction")
-                val resolved = action.withDefault(default!!)
-                tester(resolved)
-                result.complete(null)
-            }
-        })
+            WebExtension.ActionDelegate::class,
+            { delegate ->
+                mainSession.webExtensionController.setActionDelegate(extension!!, delegate)
+            },
+            { mainSession.webExtensionController.setActionDelegate(extension!!, null) },
+            object : WebExtension.ActionDelegate {
+                override fun onBrowserAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    assertEquals(id, "#browserAction")
+                    val resolved = action.withDefault(default!!)
+                    tester(resolved)
+                    result.complete(null)
+                }
+                override fun onPageAction(extension: WebExtension, session: GeckoSession?, action: WebExtension.Action) {
+                    assertEquals(id, "#pageAction")
+                    val resolved = action.withDefault(default!!)
+                    tester(resolved)
+                    result.complete(null)
+                }
+            },
+        )
 
         sessionRule.waitForResult(result)
     }
@@ -210,8 +279,11 @@ class ExtensionActionTest : BaseSessionTest() {
         when (id) {
             "#browserAction" -> {
                 extension!!.setActionDelegate(object : WebExtension.ActionDelegate {
-                    override fun onBrowserAction(extension: WebExtension, session: GeckoSession?,
-                                                 action: WebExtension.Action) {
+                    override fun onBrowserAction(
+                        extension: WebExtension,
+                        session: GeckoSession?,
+                        action: WebExtension.Action,
+                    ) {
                         assertEquals(action.title, "Test action default")
                         result.complete(null)
                     }
@@ -219,8 +291,11 @@ class ExtensionActionTest : BaseSessionTest() {
             }
             "#pageAction" -> {
                 extension!!.setActionDelegate(object : WebExtension.ActionDelegate {
-                    override fun onPageAction(extension: WebExtension, session: GeckoSession?,
-                                              action: WebExtension.Action) {
+                    override fun onPageAction(
+                        extension: WebExtension,
+                        session: GeckoSession?,
+                        action: WebExtension.Action,
+                    ) {
                         assertEquals(action.title, "Test action default")
                         result.complete(null)
                     }
@@ -248,10 +323,12 @@ class ExtensionActionTest : BaseSessionTest() {
 
     @Test
     fun setOverridenTitle() {
-        testActionApi("""{
+        testActionApi(
+            """{
                "action": "setTitle",
                "title": "overridden title"
-            }""") { action ->
+            }""",
+        ) { action ->
             assertEquals(action.title, "overridden title")
             assertEquals(action.enabled, true)
         }
@@ -261,10 +338,12 @@ class ExtensionActionTest : BaseSessionTest() {
     fun setBadgeText() {
         assumeThat("Only browserAction supports this API.", id, equalTo("#browserAction"))
 
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setBadgeText",
            "text": "12"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.badgeText, "12")
             assertEquals(action.enabled, true)
@@ -288,10 +367,12 @@ class ExtensionActionTest : BaseSessionTest() {
     }
 
     private fun colorRawTest(actionName: String, color: String, expectedHex: String) {
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "$actionName",
            "color": $color
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.badgeText, "")
             assertEquals(action.enabled, true)
@@ -307,6 +388,7 @@ class ExtensionActionTest : BaseSessionTest() {
         }
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test
     fun setBadgeTextColor() {
         assumeThat("Only browserAction supports this API.", id, equalTo("#browserAction"))
@@ -324,40 +406,48 @@ class ExtensionActionTest : BaseSessionTest() {
         assumeThat("Only browserAction supports default properties.", id, equalTo("#browserAction"))
 
         // Setting a default value will trigger the default handler on the extension object
-        testBackgroundActionApi("""{
+        testBackgroundActionApi(
+            """{
             "action": "setTitle",
             "title": "new default title"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "new default title")
             assertEquals(action.badgeText, "")
             assertEquals(action.enabled, true)
         }
 
         // When an overridden title is set, the default has no effect
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setTitle",
            "title": "test override"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "test override")
             assertEquals(action.badgeText, "")
             assertEquals(action.enabled, true)
         }
 
         // When the override is null, the new default takes effect
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setTitle",
            "title": null
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "new default title")
             assertEquals(action.badgeText, "")
             assertEquals(action.enabled, true)
         }
 
         // When the default value is null, the manifest value is used
-        testBackgroundActionApi("""{
+        testBackgroundActionApi(
+            """{
            "action": "setTitle",
            "title": null
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.badgeText, "")
             assertEquals(action.enabled, true)
@@ -366,12 +456,12 @@ class ExtensionActionTest : BaseSessionTest() {
 
     private fun compareBitmap(expectedLocation: String, actual: Bitmap) {
         val stream = InstrumentationRegistry.getInstrumentation().targetContext.assets
-                .open(expectedLocation)
+            .open(expectedLocation)
 
         val expected = BitmapFactory.decodeStream(stream)
         for (x in 0 until actual.height) {
             for (y in 0 until actual.width) {
-                assertEquals(expected.getPixel(x, y), actual.getPixel(x, y))
+                assertEquals(expected[x, y], actual[x, y])
             }
         }
     }
@@ -380,10 +470,12 @@ class ExtensionActionTest : BaseSessionTest() {
     fun setIconSvg() {
         val svg = GeckoResult<Void>()
 
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setIcon",
            "path": "button/icon.svg"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
 
@@ -402,7 +494,7 @@ class ExtensionActionTest : BaseSessionTest() {
 
         val png32 = GeckoResult<Void>()
 
-        default!!.icon!!.getBitmap(32).accept ({ actual ->
+        default!!.icon!!.getBitmap(32).accept({ actual ->
             compareBitmap("web_extensions/actions/button/beasts-32.png", actual!!)
             png32.complete(null)
         }, { error ->
@@ -419,13 +511,15 @@ class ExtensionActionTest : BaseSessionTest() {
         val png19 = GeckoResult<Void>()
         val png10 = GeckoResult<Void>()
 
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setIcon",
            "path": {
              "19": "button/geo-19.png",
              "38": "button/geo-38.png"
            }
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
 
@@ -460,15 +554,17 @@ class ExtensionActionTest : BaseSessionTest() {
     fun setIconError() {
         val error = GeckoResult<Void>()
 
-        testActionApi("""{
+        testActionApi(
+            """{
             "action": "setIcon",
             "path": "invalid/path/image.png"
-        }""") { action ->
+        }""",
+        ) { action ->
             action.icon!!.getBitmap(38).accept({
                 error.completeExceptionally(RuntimeException("Should not succeed."))
             }, { exception ->
                 if (!(exception is ImageProcessingException)) {
-                    throw exception!!;
+                    throw exception!!
                 }
                 error.complete(null)
             })
@@ -478,45 +574,58 @@ class ExtensionActionTest : BaseSessionTest() {
     }
 
     @Test
-    @GeckoSessionTestRule.WithDisplay(width=100, height=100)
-    @Ignore("This test fails intermittently on try")
+    fun testSetPopupRestrictions() {
+        testSetPopup("https://example.com", false)
+        testSetPopup("${otherExtension!!.metaData.baseUrl}other-extension.html", false)
+        testSetPopup("${extension!!.metaData.baseUrl}same-extension.html", true)
+        testSetPopup("relative-url-01.html", true)
+        testSetPopup("/relative-url-02.html", true)
+    }
+
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
+    @Test
+    @GeckoSessionTestRule.WithDisplay(width = 100, height = 100)
     fun testOpenPopup() {
         // First, let's make sure we have a popup set
         val actionResult = GeckoResult<Void>()
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setPopup",
            "popup": "test-popup.html"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
 
             actionResult.complete(null)
         }
+        sessionRule.waitForResult(actionResult)
 
-        val url = when(id) {
-            "#browserAction" -> "/test-open-popup-browser-action.html"
-            "#pageAction" -> "/test-open-popup-page-action.html"
+        val url = when (id) {
+            "#browserAction" -> "test-open-popup-browser-action.html"
+            "#pageAction" -> "test-open-popup-page-action.html"
             else -> throw IllegalArgumentException()
         }
 
-        windowPort!!.postMessage(JSONObject("""{
-            "type": "load",
-            "url": "$url"
-        }"""))
+        var location = extension!!.metaData.baseUrl
+        mainSession.loadUri("$location$url")
+        sessionRule.waitForPageStop()
 
         val openPopup = GeckoResult<Void>()
-        mainSession.webExtensionController.setActionDelegate(extension!!,
-                object : WebExtension.ActionDelegate {
-            override fun onOpenPopup(extension: WebExtension,
-                                     popupAction: WebExtension.Action): GeckoResult<GeckoSession>? {
-                assertEquals(extension, this@ExtensionActionTest.extension)
-                // assertEquals(popupAction, this@ExtensionActionTest.default)
-                openPopup.complete(null)
-                return null
-            }
-        })
+        mainSession.webExtensionController.setActionDelegate(
+            extension!!,
+            object : WebExtension.ActionDelegate {
+                override fun onOpenPopup(
+                    extension: WebExtension,
+                    popupAction: WebExtension.Action,
+                ): GeckoResult<GeckoSession>? {
+                    assertEquals(extension, this@ExtensionActionTest.extension)
+                    openPopup.complete(null)
+                    return null
+                }
+            },
+        )
 
-        sessionRule.waitForPageStops(2)
         // openPopup needs user activation
         mainSession.synthesizeTap(50, 50)
 
@@ -534,18 +643,23 @@ class ExtensionActionTest : BaseSessionTest() {
                     pong.complete(null)
                 } else {
                     // We should NOT receive onClicked here
-                    pong.completeExceptionally(IllegalArgumentException(
-                            "Received unexpected: ${json.getString("method")}"))
+                    pong.completeExceptionally(
+                        IllegalArgumentException(
+                            "Received unexpected: ${json.getString("method")}",
+                        ),
+                    )
                 }
             }
         })
 
         val actionResult = GeckoResult<WebExtension.Action>()
 
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setPopup",
            "popup": "test-popup.html"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
 
@@ -556,8 +670,10 @@ class ExtensionActionTest : BaseSessionTest() {
         val action = sessionRule.waitForResult(actionResult)
 
         extension!!.setActionDelegate(object : WebExtension.ActionDelegate {
-            override fun onTogglePopup(extension: WebExtension,
-                                     popupAction: WebExtension.Action): GeckoResult<GeckoSession>? {
+            override fun onTogglePopup(
+                extension: WebExtension,
+                popupAction: WebExtension.Action,
+            ): GeckoResult<GeckoSession>? {
                 assertEquals(extension, this@ExtensionActionTest.extension)
                 assertEquals(popupAction, action)
                 togglePopup.complete(null)
@@ -572,9 +688,13 @@ class ExtensionActionTest : BaseSessionTest() {
         sessionRule.waitForResult(togglePopup)
 
         // If the response to ping reaches us before the onClicked we know onClicked wasn't called
-        backgroundPort!!.postMessage(JSONObject("""{
+        backgroundPort!!.postMessage(
+            JSONObject(
+                """{
             "type": "ping"
-        }"""))
+        }""",
+            ),
+        )
 
         sessionRule.waitForResult(pong)
     }
@@ -591,10 +711,12 @@ class ExtensionActionTest : BaseSessionTest() {
             }
         })
 
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setPopup",
            "popup": null
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
 
@@ -610,10 +732,12 @@ class ExtensionActionTest : BaseSessionTest() {
         val popupSession = sessionRule.createOpenSession()
 
         val actionResult = GeckoResult<WebExtension.Action>()
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setPopup",
            "popup": "test-popup-messaging.html"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
             actionResult.complete(action)
@@ -626,14 +750,18 @@ class ExtensionActionTest : BaseSessionTest() {
             override fun onMessage(
                 nativeApp: String,
                 message: Any,
-                sender: WebExtension.MessageSender
+                sender: WebExtension.MessageSender,
             ): GeckoResult<Any>? {
                 assertEquals(extension!!.id, sender.webExtension.id)
-                assertEquals(WebExtension.MessageSender.ENV_TYPE_EXTENSION,
-                    sender.environmentType)
+                assertEquals(
+                    WebExtension.MessageSender.ENV_TYPE_EXTENSION,
+                    sender.environmentType,
+                )
                 assertEquals(sender.isTopLevel, true)
-                assertEquals("${extension!!.metaData.baseUrl}test-popup-messaging.html",
-                    sender.url)
+                assertEquals(
+                    "${extension!!.metaData.baseUrl}test-popup-messaging.html",
+                    sender.url,
+                )
                 assertEquals(sender.session, popupSession)
                 messages.add(message as String)
                 if (messages.size == 2) {
@@ -646,23 +774,32 @@ class ExtensionActionTest : BaseSessionTest() {
 
             override fun onConnect(port: WebExtension.Port) {
                 assertEquals(extension!!.id, port.sender.webExtension.id)
-                assertEquals(WebExtension.MessageSender.ENV_TYPE_EXTENSION,
-                    port.sender.environmentType)
+                assertEquals(
+                    WebExtension.MessageSender.ENV_TYPE_EXTENSION,
+                    port.sender.environmentType,
+                )
                 assertEquals(true, port.sender.isTopLevel)
-                assertEquals("${extension!!.metaData.baseUrl}test-popup-messaging.html",
-                    port.sender.url)
+                assertEquals(
+                    "${extension!!.metaData.baseUrl}test-popup-messaging.html",
+                    port.sender.url,
+                )
                 assertEquals(port.sender.session, popupSession)
                 portResult.complete(port)
             }
         }
 
         popupSession.webExtensionController.setMessageDelegate(
-            extension!!, messageDelegate, "browser")
+            extension!!,
+            messageDelegate,
+            "browser",
+        )
 
         val action = sessionRule.waitForResult(actionResult)
         extension!!.setActionDelegate(object : WebExtension.ActionDelegate {
-            override fun onTogglePopup(extension: WebExtension,
-                                       popupAction: WebExtension.Action): GeckoResult<GeckoSession>? {
+            override fun onTogglePopup(
+                extension: WebExtension,
+                popupAction: WebExtension.Action,
+            ): GeckoResult<GeckoSession>? {
                 assertEquals(extension, this@ExtensionActionTest.extension)
                 assertEquals(popupAction, action)
                 return GeckoResult.fromValue(popupSession)
@@ -672,8 +809,16 @@ class ExtensionActionTest : BaseSessionTest() {
         action.click()
 
         val message = sessionRule.waitForResult(messageResult)
-        assertThat("Message should match", message, equalTo(listOf(
-            "testPopupMessage", "response: TEST_RESPONSE")))
+        assertThat(
+            "Message should match",
+            message,
+            equalTo(
+                listOf(
+                    "testPopupMessage",
+                    "response: TEST_RESPONSE",
+                ),
+            ),
+        )
 
         val port = sessionRule.waitForResult(portResult)
         val portMessageResult = GeckoResult<String>()
@@ -686,8 +831,11 @@ class ExtensionActionTest : BaseSessionTest() {
         })
 
         val portMessage = sessionRule.waitForResult(portMessageResult)
-        assertThat("Message should match", portMessage,
-            equalTo("testPopupPortMessage"))
+        assertThat(
+            "Message should match",
+            portMessage,
+            equalTo("testPopupPortMessage"),
+        )
     }
 
     @Test
@@ -702,10 +850,12 @@ class ExtensionActionTest : BaseSessionTest() {
         })
 
         val actionResult = GeckoResult<WebExtension.Action>()
-        testActionApi("""{
+        testActionApi(
+            """{
            "action": "setPopup",
            "popup": "test-popup.html"
-        }""") { action ->
+        }""",
+        ) { action ->
             assertEquals(action.title, "Test action default")
             assertEquals(action.enabled, true)
             actionResult.complete(action)
@@ -714,8 +864,10 @@ class ExtensionActionTest : BaseSessionTest() {
         val togglePopup = GeckoResult<Void>()
         val action = sessionRule.waitForResult(actionResult)
         extension!!.setActionDelegate(object : WebExtension.ActionDelegate {
-            override fun onTogglePopup(extension: WebExtension,
-                                     popupAction: WebExtension.Action): GeckoResult<GeckoSession>? {
+            override fun onTogglePopup(
+                extension: WebExtension,
+                popupAction: WebExtension.Action,
+            ): GeckoResult<GeckoSession>? {
                 assertEquals(extension, this@ExtensionActionTest.extension)
                 assertEquals(popupAction, action)
                 togglePopup.complete(null)
@@ -728,4 +880,3 @@ class ExtensionActionTest : BaseSessionTest() {
         sessionRule.waitForResult(onCloseRequestResult)
     }
 }
-
