@@ -14,6 +14,7 @@ import org.hamcrest.Matchers.notNullValue
 import org.junit.After
 import org.junit.Assume.assumeThat
 import org.junit.Before
+import org.junit.Ignore
 import org.junit.Test
 import org.junit.runner.RunWith
 import org.mozilla.geckoview.GeckoResult
@@ -80,6 +81,7 @@ class MediaSessionTest : BaseSessionTest() {
     fun teardown() {
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test
     fun domMetadataPlayback() {
         val onActivatedCalled = arrayOf(GeckoResult<Void>())
@@ -244,6 +246,13 @@ class MediaSessionTest : BaseSessionTest() {
                 mediaSession: MediaSession,
                 meta: MediaSession.Metadata,
             ) {
+                if (sessionRule.currentCall.counter == 7) {
+                    // Occasionally, a 7th call occurs from onStop with blank metadata.
+                    onMetadataCalled[sessionRule.currentCall.counter - 1]
+                        .complete(null)
+                    return
+                }
+
                 assertThat(
                     "Title should match",
                     meta.title,
@@ -349,32 +358,32 @@ class MediaSessionTest : BaseSessionTest() {
         mediaSession1!!.pause()
 
         sessionRule.waitForResult(completedStep3)
-        mediaSession1!!.play()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep4)
         sessionRule.waitForResult(completedStep5)
-        mediaSession1!!.pause()
-        mediaSession1!!.nextTrack()
-        mediaSession1!!.play()
+        mediaSession1.pause()
+        mediaSession1.nextTrack()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep6)
-        mediaSession1!!.pause()
-        mediaSession1!!.nextTrack()
-        mediaSession1!!.play()
+        mediaSession1.pause()
+        mediaSession1.nextTrack()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep7)
-        mediaSession1!!.pause()
+        mediaSession1.pause()
 
         sessionRule.waitForResult(completedStep8a)
-        mediaSession1!!.previousTrack()
-        mediaSession1!!.play()
+        mediaSession1.previousTrack()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep8b)
         sessionRule.waitForResult(completedStep9)
-        mediaSession1!!.play()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep10)
-        mediaSession1!!.stop()
+        mediaSession1.stop()
 
         sessionRule.waitForResult(completedStep11)
     }
@@ -467,12 +476,13 @@ class MediaSessionTest : BaseSessionTest() {
         mediaSession1!!.pause()
 
         sessionRule.waitForResult(completedStep3)
-        mediaSession1!!.play()
+        mediaSession1.play()
 
         sessionRule.waitForResult(completedStep4)
         sessionRule.waitForResult(completedStep5)
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test
     fun domMultiSessions() {
         val onActivatedCalled = arrayOf(
@@ -597,7 +607,7 @@ class MediaSessionTest : BaseSessionTest() {
 
                 assertThat(
                     "Should be active",
-                    mediaSession1?.isActive,
+                    mediaSession1.isActive,
                     equalTo(true),
                 )
             }
@@ -731,7 +741,7 @@ class MediaSessionTest : BaseSessionTest() {
                 )
                 assertThat(
                     "Should be active",
-                    mediaSession2!!.isActive,
+                    mediaSession2.isActive,
                     equalTo(true),
                 )
             }
@@ -808,13 +818,14 @@ class MediaSessionTest : BaseSessionTest() {
         mediaSession2!!.pause()
         sessionRule.waitForResult(completedStep6)
 
-        mediaSession1!!.pause()
-        mediaSession1!!.nextTrack()
-        mediaSession1!!.play()
+        mediaSession1.pause()
+        mediaSession1.nextTrack()
+        mediaSession1.play()
         sessionRule.waitForResult(completedStep7)
         sessionRule.waitForResult(completedStep8)
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test
     fun fullscreenVideoElementMetadata() {
         // Bug 1981579
@@ -984,10 +995,6 @@ class MediaSessionTest : BaseSessionTest() {
 
     @Test
     fun fullscreenVideoWithActivated() {
-        // Bug 1981579
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.UPSIDE_DOWN_CAKE) {
-            assumeThat(sessionRule.env.isIsolatedProcess, equalTo(false))
-        }
         sessionRule.setPrefsUntilTestEnd(
             mapOf(
                 "media.autoplay.default" to 0,
@@ -1026,6 +1033,7 @@ class MediaSessionTest : BaseSessionTest() {
         sessionRule.waitForResult(resultFullscreen)
     }
 
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test
     fun switchingProcess() {
         // Bug 1981579
