@@ -48,9 +48,6 @@ class PermissionDelegateTest : BaseSessionTest() {
         get() = InstrumentationRegistry.getInstrumentation().targetContext
 
     private fun hasPermission(permission: String): Boolean {
-        if (Build.VERSION.SDK_INT < 23) {
-            return true
-        }
         return PackageManager.PERMISSION_GRANTED ==
             InstrumentationRegistry.getInstrumentation().targetContext.checkSelfPermission(
                 permission,
@@ -550,6 +547,7 @@ class PermissionDelegateTest : BaseSessionTest() {
 
     // Tests that all pages have a PERMISSION_TRACKING permission,
     // except for pages that belong to Gecko like about:blank or about:config.
+    @Ignore("https://bugzilla.mozilla.org/show_bug.cgi?id=1988041")
     @Test fun trackingProtectionPermissionOnAllPages() {
         val settings = sessionRule.runtime.settings
         val aboutConfigEnabled = settings.aboutConfigEnabled
@@ -1197,16 +1195,18 @@ class PermissionDelegateTest : BaseSessionTest() {
         val importedPerm = ContentPermission.fromJson(jsonPerm!!)
         assertThat("JSON import should not be null", importedPerm, notNullValue())
 
-        assertThat("URIs should match", importedPerm?.uri, equalTo(notificationPerm?.uri))
-        assertThat("Types should match", importedPerm?.permission, equalTo(notificationPerm?.permission))
-        assertThat("Values should match", importedPerm?.value, equalTo(notificationPerm?.value))
-        assertThat("Context IDs should match", importedPerm?.contextId, equalTo(notificationPerm?.contextId))
-        assertThat("Private mode should match", importedPerm?.privateMode, equalTo(notificationPerm?.privateMode))
+        assertThat("URIs should match", importedPerm?.uri, equalTo(notificationPerm.uri))
+        assertThat("Types should match", importedPerm?.permission, equalTo(notificationPerm.permission))
+        assertThat("Values should match", importedPerm?.value, equalTo(notificationPerm.value))
+        assertThat("Context IDs should match", importedPerm?.contextId, equalTo(notificationPerm.contextId))
+        assertThat("Private mode should match", importedPerm?.privateMode, equalTo(notificationPerm.privateMode))
     }
 
     @Test
     fun localDeviceAccessPermission() {
         sessionRule.setPrefsUntilTestEnd(mapOf("network.lna.blocking" to true))
+        // enable LNA checks for local network to localhost checks
+        sessionRule.setPrefsUntilTestEnd(mapOf("network.lna.local-network-to-localhost.skip-checks" to false))
 
         mainSession.loadUri("https://example.com/")
         mainSession.waitForPageStop()
