@@ -6,9 +6,9 @@
 package org.mozilla.gecko;
 
 import android.annotation.SuppressLint;
-import android.annotation.TargetApi;
 import android.app.ActivityManager;
 import android.app.Service;
+import android.content.ComponentCallbacks;
 import android.content.Context;
 import android.content.Intent;
 import android.content.SharedPreferences;
@@ -16,6 +16,7 @@ import android.content.pm.ActivityInfo;
 import android.content.pm.ApplicationInfo;
 import android.content.pm.PackageManager;
 import android.content.pm.ResolveInfo;
+import android.content.res.Configuration;
 import android.content.res.TypedArray;
 import android.graphics.Bitmap;
 import android.graphics.Canvas;
@@ -44,8 +45,10 @@ import android.os.Debug;
 import android.os.LocaleList;
 import android.os.Looper;
 import android.os.PowerManager;
+import android.os.VibrationEffect;
 import android.os.Vibrator;
 import android.provider.Settings;
+import android.util.DisplayMetrics;
 import android.util.Log;
 import android.view.ContextThemeWrapper;
 import android.view.Display;
@@ -55,13 +58,13 @@ import android.view.inputmethod.InputMethodManager;
 import android.view.inputmethod.InputMethodSubtype;
 import android.webkit.MimeTypeMap;
 import androidx.annotation.Nullable;
+import androidx.annotation.RequiresApi;
 import androidx.collection.SimpleArrayMap;
 import androidx.core.content.res.ResourcesCompat;
 import java.net.InetSocketAddress;
 import java.net.Proxy;
 import java.nio.ByteBuffer;
 import java.util.List;
-import java.util.Locale;
 import java.util.StringTokenizer;
 import org.jetbrains.annotations.NotNull;
 import org.mozilla.gecko.annotation.RobocopTarget;
@@ -195,18 +198,16 @@ public class GeckoAppShell {
    */
   private static final int ADDITIONAL_SEARCH_HEADER_RAM_THRESHOLD_MEGABYTES = 1024;
 
-  private static int sDensityDpi;
-  private static Float sDensity;
   private static int sScreenDepth;
   private static boolean sUseMaxScreenDepth;
   private static Float sScreenRefreshRate;
 
-  /* Is the value in sVibrationEndTime valid? */
-  private static boolean sVibrationMaybePlaying;
-
-  /* Time (in System.nanoTime() units) when the currently-playing vibration
-   * is scheduled to end.  This value is valid only when
-   * sVibrationMaybePlaying is true. */
+  /*
+   * Time (in System.nanoTime() units) when the currently-playing vibration
+   * is scheduled to end. This value could be zero when the vibration is
+   * cancelled. `System.nanoTime() > sVibrationEndTime` means there is not a
+   * playing vibration now.
+   */
   private static long sVibrationEndTime;
 
   private static Sensor gAccelerometerSensor;
@@ -245,6 +246,8 @@ public class GeckoAppShell {
   /* package */ static native void reportJavaCrash(Throwable exc, String stackTrace);
 
   private static Rect sScreenSizeOverride;
+  private static int sDensityDpiOverride;
+  private static Float sDensityOverride;
 
   @WrapForJNI(stubName = "NotifyObservers", dispatchTo = "gecko")
   private static native void nativeNotifyObservers(String topic, String data);
@@ -509,9 +512,7 @@ public class GeckoAppShell {
       final float accuracy = location.hasAccuracy() ? location.getAccuracy() : Float.NaN;
 
       final float altitudeAccuracy =
-          Build.VERSION.SDK_INT >= 26 && location.hasVerticalAccuracy()
-              ? location.getVerticalAccuracyMeters()
-              : Float.NaN;
+          location.hasVerticalAccuracy() ? location.getVerticalAccuracyMeters() : Float.NaN;
 
       final float speed = location.hasSpeed() ? location.getSpeed() : Float.NaN;
 
@@ -785,6 +786,14 @@ public class GeckoAppShell {
   private static native void notifyAlertListener(
       String name, String topic, String action, String origin);
 
+  /** Called by the NotificationListener to notify Gecko that a notification has shown. */
+  public static void onNotificationShow(
+      final String name, final String cookie, @NotNull final String origin) {
+    if (GeckoThread.isRunning()) {
+      notifyAlertListener(name, "alertshow", cookie, origin);
+    }
+  }
+
   /**
    * Called by the NotificationListener to notify Gecko that a previously shown notification has
    * been closed.
@@ -820,39 +829,39 @@ public class GeckoAppShell {
     if (dpi == null) {
       return;
     }
-    if (sDensityDpi != 0) {
+    if (sDensityDpiOverride != 0) {
       Log.e(LOGTAG, "Tried to override screen DPI after it's already been set");
       return;
     }
-    sDensityDpi = dpi;
+    sDensityDpiOverride = dpi;
   }
 
   @WrapForJNI(calledFrom = "gecko")
-  public static synchronized int getDpi() {
-    if (sDensityDpi == 0) {
-      sDensityDpi = getApplicationContext().getResources().getDisplayMetrics().densityDpi;
+  private static synchronized int getDpi() {
+    if (sDensityDpiOverride != 0) {
+      return sDensityDpiOverride;
     }
-    return sDensityDpi;
+    return sScreenCompat.getDensityDpi();
   }
 
   public static synchronized void setDisplayDensityOverride(@Nullable final Float density) {
     if (density == null) {
       return;
     }
-    if (sDensity != null) {
+    if (sDensityOverride != null) {
       Log.e(LOGTAG, "Tried to override screen density after it's already been set");
       return;
     }
-    sDensity = density;
+    sDensityOverride = density;
   }
 
   @WrapForJNI(calledFrom = "gecko")
   private static synchronized float getDensity() {
-    if (sDensity == null) {
-      sDensity = Float.valueOf(getApplicationContext().getResources().getDisplayMetrics().density);
+    if (sDensityOverride != null) {
+      return sDensityOverride;
     }
 
-    return sDensity;
+    return sScreenCompat.getDensity();
   }
 
   private static int sTotalRam;
@@ -922,45 +931,34 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "gecko")
   private static boolean hasHDRScreen() {
-    if (Build.VERSION.SDK_INT < 24) {
-      return false;
-    }
-    final WindowManager wm =
-        (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
-    final Display display = wm.getDefaultDisplay();
-    if (Build.VERSION.SDK_INT >= 26) {
-      return display.isHdr();
-    }
-    final Display.HdrCapabilities hdrCapabilities = display.getHdrCapabilities();
-    if (hdrCapabilities == null) {
-      return false;
-    }
-    final int[] supportedHdrTypes = hdrCapabilities.getSupportedHdrTypes();
-    for (final int type : supportedHdrTypes) {
-      if (type == Display.HdrCapabilities.HDR_TYPE_HDR10
-          || type == Display.HdrCapabilities.HDR_TYPE_HDR10_PLUS
-          || type == Display.HdrCapabilities.HDR_TYPE_HLG
-          || type == Display.HdrCapabilities.HDR_TYPE_DOLBY_VISION) {
-        return true;
-      }
-    }
-    return false;
+    final Display display =
+        ((DisplayManager) getApplicationContext().getSystemService(Context.DISPLAY_SERVICE))
+            .getDisplay(Display.DEFAULT_DISPLAY);
+    return display != null && display.isHdr();
   }
 
   @WrapForJNI(calledFrom = "gecko")
   private static void performHapticFeedback(final boolean aIsLongPress) {
     // Don't perform haptic feedback if a vibration is currently playing,
     // because the haptic feedback will nuke the vibration.
-    if (!sVibrationMaybePlaying || System.nanoTime() >= sVibrationEndTime) {
-      final int[] pattern;
-      if (aIsLongPress) {
-        pattern = new int[] {0, 1, 20, 21};
+    if (System.nanoTime() >= sVibrationEndTime) {
+      final VibrationEffect effect;
+      if (Build.VERSION.SDK_INT >= 29) {
+        // API level 29 introduces pre-defined vibration effects for better
+        // haptic feedback, prefer to use them.
+        if (aIsLongPress) {
+          effect = VibrationEffect.createPredefined(VibrationEffect.EFFECT_HEAVY_CLICK);
+        } else {
+          effect = VibrationEffect.createPredefined(VibrationEffect.EFFECT_CLICK);
+        }
       } else {
-        pattern = new int[] {0, 10, 20, 30};
+        if (aIsLongPress) {
+          effect = VibrationEffect.createWaveform(new long[] {0, 1, 20, 21}, -1);
+        } else {
+          effect = VibrationEffect.createWaveform(new long[] {0, 10, 20, 30}, -1);
+        }
       }
-      vibrateOnHapticFeedbackEnabled(pattern);
-      sVibrationMaybePlaying = false;
-      sVibrationEndTime = 0;
+      vibrateOnHapticFeedbackEnabled(effect);
     }
   }
 
@@ -968,26 +966,21 @@ public class GeckoAppShell {
     return (Vibrator) getApplicationContext().getSystemService(Context.VIBRATOR_SERVICE);
   }
 
-  // Helper method to convert integer array to long array.
-  private static long[] convertIntToLongArray(final int[] input) {
-    final long[] output = new long[input.length];
-    for (int i = 0; i < input.length; i++) {
-      output[i] = input[i];
-    }
-    return output;
-  }
-
   // Vibrate only if haptic feedback is enabled.
-  private static void vibrateOnHapticFeedbackEnabled(final int[] milliseconds) {
+  @SuppressLint("MissingPermission")
+  private static void vibrateOnHapticFeedbackEnabled(final VibrationEffect effect) {
     if (Settings.System.getInt(
             getApplicationContext().getContentResolver(),
             Settings.System.HAPTIC_FEEDBACK_ENABLED,
             0)
         > 0) {
-      if (milliseconds.length == 1) {
-        vibrate(milliseconds[0]);
-      } else {
-        vibrate(convertIntToLongArray(milliseconds), -1);
+      // Here, sVibrationEndTime is not set. Compared to other kinds of
+      // vibration, haptic feedbacks are usually shorter and less important,
+      // which means it's ok to "nuke" them.
+      try {
+        vibrator().vibrate(effect);
+      } catch (final SecurityException ignore) {
+        Log.w(LOGTAG, "No VIBRATE permission");
       }
     }
   }
@@ -996,7 +989,6 @@ public class GeckoAppShell {
   @WrapForJNI(calledFrom = "gecko")
   private static void vibrate(final long milliseconds) {
     sVibrationEndTime = System.nanoTime() + milliseconds * 1000000;
-    sVibrationMaybePlaying = true;
     try {
       vibrator().vibrate(milliseconds);
     } catch (final SecurityException ignore) {
@@ -1016,7 +1008,6 @@ public class GeckoAppShell {
     }
 
     sVibrationEndTime = System.nanoTime() + vibrationDuration * 1000000;
-    sVibrationMaybePlaying = true;
     try {
       vibrator().vibrate(pattern, repeat);
     } catch (final SecurityException ignore) {
@@ -1027,7 +1018,6 @@ public class GeckoAppShell {
   @SuppressLint("MissingPermission")
   @WrapForJNI(calledFrom = "gecko")
   private static void cancelVibrate() {
-    sVibrationMaybePlaying = false;
     sVibrationEndTime = 0;
     try {
       vibrator().cancel();
@@ -1094,10 +1084,6 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "gecko", exceptionMode = "nsresult")
   private static String getDNSDomains() {
-    if (Build.VERSION.SDK_INT < 23) {
-      return "";
-    }
-
     ensureConnectivityManager();
     final Network net = sConnectivityManager.getActiveNetwork();
     if (net == null) {
@@ -1162,11 +1148,7 @@ public class GeckoAppShell {
     // TODO(m_kato):
     // Android 16 will have `layout related APIs such as setLayoutLabelNonLocalized
     // to get keyboard layout label.
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
-      return ims.getLanguageTag();
-    } else {
-      return ims.getLocale();
-    }
+    return ims.getLanguageTag();
   }
 
   @WrapForJNI(calledFrom = "gecko")
@@ -1427,9 +1409,8 @@ public class GeckoAppShell {
     return result;
   }
 
-  @WrapForJNI(calledFrom = "gecko")
   // For any-pointer and any-hover media queries features.
-  private static int getAllPointerCapabilities() {
+  /* package */ static int getAllPointerCapabilities() {
     int result = NO_POINTER;
 
     for (final int deviceId : InputDevice.getDeviceIds()) {
@@ -1471,17 +1452,15 @@ public class GeckoAppShell {
     if (hasInputDeviceSource(sources, InputDevice.SOURCE_STYLUS)) {
       result |= POINTING_DEVICE_PEN;
     }
-    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M
-        && hasInputDeviceSource(sources, InputDevice.SOURCE_BLUETOOTH_STYLUS)) {
+    if (hasInputDeviceSource(sources, InputDevice.SOURCE_BLUETOOTH_STYLUS)) {
       result |= POINTING_DEVICE_PEN;
     }
 
     return result;
   }
 
-  @WrapForJNI(calledFrom = "gecko")
   // For pointing devices telemetry.
-  private static int getPointingDeviceKinds() {
+  /* package */ static int getPointingDeviceKinds() {
     int result = POINTING_DEVICE_NONE;
 
     for (final int deviceId : InputDevice.getDeviceIds()) {
@@ -1510,9 +1489,21 @@ public class GeckoAppShell {
     Rect getScreenSize();
 
     int getRotation();
+
+    int getDensityDpi();
+
+    float getDensity();
   }
 
   private static class JellyBeanMR1ScreenCompat implements ScreenCompat {
+    private int mDensityDpi = 0;
+    private Float mDensity = null;
+
+    private static DisplayMetrics getDisplayMetrics() {
+      return getApplicationContext().getResources().getDisplayMetrics();
+    }
+
+    @Override
     public Rect getScreenSize() {
       final WindowManager wm =
           (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
@@ -1522,14 +1513,31 @@ public class GeckoAppShell {
       return new Rect(0, 0, size.x, size.y);
     }
 
+    @Override
     public int getRotation() {
       final WindowManager wm =
           (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
       return wm.getDefaultDisplay().getRotation();
     }
+
+    @Override
+    public int getDensityDpi() {
+      if (mDensityDpi == 0) {
+        mDensityDpi = getDisplayMetrics().densityDpi;
+      }
+      return mDensityDpi;
+    }
+
+    @Override
+    public float getDensity() {
+      if (mDensity == null) {
+        mDensity = getDisplayMetrics().density;
+      }
+      return mDensity;
+    }
   }
 
-  @TargetApi(Build.VERSION_CODES.S)
+  @RequiresApi(Build.VERSION_CODES.S)
   private static class AndroidSScreenCompat implements ScreenCompat {
     @SuppressLint("StaticFieldLeak")
     private static Context sWindowContext;
@@ -1542,19 +1550,49 @@ public class GeckoAppShell {
         sWindowContext =
             getApplicationContext()
                 .createWindowContext(display, WindowManager.LayoutParams.TYPE_APPLICATION, null);
+        sWindowContext.registerComponentCallbacks(
+            new ComponentCallbacks() {
+              @Override
+              public void onConfigurationChanged(final Configuration newConfig) {
+                if (GeckoScreenOrientation.getInstance().update()) {
+                  // refreshScreenInfo is already called.
+                  return;
+                }
+                ScreenManagerHelper.refreshScreenInfo();
+              }
+
+              @Override
+              public void onLowMemory() {}
+            });
       }
       return sWindowContext;
     }
 
+    private static DisplayMetrics getDisplayMetrics() {
+      return getWindowContext().getResources().getDisplayMetrics();
+    }
+
+    @Override
     public Rect getScreenSize() {
       final WindowManager windowManager = getWindowContext().getSystemService(WindowManager.class);
       return windowManager.getCurrentWindowMetrics().getBounds();
     }
 
+    @Override
     public int getRotation() {
       final WindowManager windowManager = getWindowContext().getSystemService(WindowManager.class);
       return windowManager.getDefaultDisplay().getRotation();
     }
+
+    @Override
+    public int getDensityDpi() {
+      return getDisplayMetrics().densityDpi;
+    }
+
+    @Override
+    public float getDensity() {
+      return getDisplayMetrics().density;
+    }
   }
 
   static {
@@ -1647,18 +1685,12 @@ public class GeckoAppShell {
 
   @WrapForJNI
   public static String[] getDefaultLocales() {
-    // XXX We may have to convert some language codes such as "id" vs "in".
-    if (Build.VERSION.SDK_INT >= 24) {
-      final LocaleList localeList = LocaleList.getDefault();
-      final String[] locales = new String[localeList.size()];
-      for (int i = 0; i < localeList.size(); i++) {
-        locales[i] = localeList.get(i).toLanguageTag();
-      }
-      return locales;
+    final LocaleList list = LocaleList.getDefault();
+    final int n = list.size();
+    final String[] locales = new String[n];
+    for (int i = 0; i < n; i++) {
+      locales[i] = list.get(i).toLanguageTag();
     }
-    final String[] locales = new String[1];
-    final Locale locale = Locale.getDefault();
-    locales[0] = locale.toLanguageTag();
     return locales;
   }
 
@@ -1681,11 +1713,6 @@ public class GeckoAppShell {
 
   @WrapForJNI(calledFrom = "gecko")
   private static int getMemoryUsage(final String stateName) {
-    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
-      // No API to get Java heap usages.
-      return -1;
-    }
-
     final Debug.MemoryInfo memInfo = new Debug.MemoryInfo();
     Debug.getMemoryInfo(memInfo);
     final String usage = memInfo.getMemoryStat(stateName);
@@ -1699,6 +1726,18 @@ public class GeckoAppShell {
     }
   }
 
+  @WrapForJNI
+  private static void crashByUncaughtException() {
+    final Thread crashThread =
+        new Thread("UncaughtExceptionThread") {
+          @Override
+          public void run() {
+            throw new IllegalStateException();
+          }
+        };
+    crashThread.start();
+  }
+
   @WrapForJNI
   public static native boolean isParentProcess();
 
@@ -1721,4 +1760,7 @@ public class GeckoAppShell {
 
   @WrapForJNI(dispatchTo = "gecko")
   public static native void onTimezoneChanged();
+
+  @WrapForJNI
+  public static native void logGpuProcessLaunchFailure(String aMessage);
 }
