@@ -45,6 +45,7 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
         final int repeatCount,
         final int flags,
         final boolean isSynthesizedImeKey,
+        final boolean waitingReply,
         final KeyEvent event) {
       GeckoEditableChild.this.onKeyEvent(
           action,
@@ -57,6 +58,7 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
           repeatCount,
           flags,
           isSynthesizedImeKey,
+          waitingReply,
           event);
     }
 
@@ -70,6 +72,11 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
       GeckoEditableChild.this.onImeReplaceText(start, end, text);
     }
 
+    @Override // IGeckoEditableChild
+    public void onImeInsertImage(final byte[] data, final String mimeType) {
+      GeckoEditableChild.this.onImeInsertImage(data, mimeType);
+    }
+
     @Override // IGeckoEditableChild
     public void onImeAddCompositionRange(
         final int start,
@@ -166,6 +173,7 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
       int repeatCount,
       int flags,
       boolean isSynthesizedImeKey,
+      boolean waitingReply,
       KeyEvent event);
 
   @WrapForJNI(dispatchTo = "proxy")
@@ -204,6 +212,10 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
   @Override // IGeckoEditableChild
   public native void onImeRequestCommit();
 
+  @WrapForJNI(dispatchTo = "proxy")
+  @Override // IGeckoEditableChild
+  public native void onImeInsertImage(byte[] data, String mimeType);
+
   @Override // JNIObject
   protected void disposeNative() {
     // Disposal happens in native code.
@@ -259,6 +271,7 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
       final String modeHint,
       final String actionHint,
       final String autocapitalize,
+      final boolean autocorrect,
       final int flags) {
     if (DEBUG) {
       ThreadUtils.assertOnGeckoThread();
@@ -272,7 +285,9 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
           .append(actionHint)
           .append("\", \"")
           .append(autocapitalize)
-          .append("\", 0x")
+          .append("\", ")
+          .append(autocorrect)
+          .append(", 0x")
           .append(Integer.toHexString(flags))
           .append(")");
       Log.d(LOGTAG, sb.toString());
@@ -283,7 +298,14 @@ public final class GeckoEditableChild extends JNIObject implements IGeckoEditabl
 
     try {
       mEditableParent.notifyIMEContext(
-          mEditableChild.asBinder(), state, typeHint, modeHint, actionHint, autocapitalize, flags);
+          mEditableChild.asBinder(),
+          state,
+          typeHint,
+          modeHint,
+          actionHint,
+          autocapitalize,
+          autocorrect,
+          flags);
     } catch (final RemoteException e) {
       Log.e(LOGTAG, "Remote call failed", e);
     }
