@@ -4,10 +4,8 @@
 
 package org.mozilla.gecko.media;
 
-import android.annotation.SuppressLint;
 import android.media.MediaCrypto;
 import android.media.MediaDrm;
-import android.os.Build;
 import android.util.Log;
 import java.util.ArrayList;
 import java.util.UUID;
@@ -37,22 +35,8 @@ public final class MediaDrmProxy {
   private GeckoMediaDrm mImpl;
   private String mDrmStubId;
 
-  private static boolean isSystemSupported() {
-    // Support versions >= Marshmallow
-    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
-      if (DEBUG)
-        Log.d(LOGTAG, "System Not supported !!, current SDK version is " + Build.VERSION.SDK_INT);
-      return false;
-    }
-    return true;
-  }
-
-  @SuppressLint("NewApi")
   @WrapForJNI
   public static boolean isSchemeSupported(final String keySystem) {
-    if (!isSystemSupported()) {
-      return false;
-    }
     if (keySystem.equals(WIDEVINE_KEY_SYSTEM)) {
       return MediaDrm.isCryptoSchemeSupported(WIDEVINE_SCHEME_UUID)
           && MediaCrypto.isCryptoSchemeSupported(WIDEVINE_SCHEME_UUID);
@@ -61,12 +45,8 @@ public final class MediaDrmProxy {
     return false;
   }
 
-  @SuppressLint("NewApi")
   @WrapForJNI
   public static boolean IsCryptoSchemeSupported(final String keySystem, final String container) {
-    if (!isSystemSupported()) {
-      return false;
-    }
     if (keySystem.equals(WIDEVINE_KEY_SYSTEM)) {
       return MediaDrm.isCryptoSchemeSupported(WIDEVINE_SCHEME_UUID, container);
     }
@@ -272,7 +252,7 @@ public final class MediaDrmProxy {
         return proxy.getMediaCryptoFromBridge();
       }
     }
-    if (DEBUG) Log.d(LOGTAG, " NULL crytpo ");
+    if (DEBUG) Log.d(LOGTAG, " NULL crypto ");
     return null;
   }
 
