@@ -9,8 +9,6 @@ package org.mozilla.geckoview;
 import static org.mozilla.geckoview.GeckoSession.GeckoPrintException.ERROR_NO_ACTIVITY_CONTEXT;
 import static org.mozilla.geckoview.GeckoSession.GeckoPrintException.ERROR_NO_ACTIVITY_CONTEXT_DELEGATE;
 
-import android.annotation.SuppressLint;
-import android.annotation.TargetApi;
 import android.app.Activity;
 import android.content.Context;
 import android.content.ContextWrapper;
@@ -56,10 +54,12 @@ import androidx.annotation.NonNull;
 import androidx.annotation.Nullable;
 import androidx.annotation.UiThread;
 import androidx.core.view.ViewCompat;
+import androidx.core.view.WindowInsetsCompat;
 import java.io.InputStream;
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.lang.ref.WeakReference;
+import java.util.HashMap;
 import java.util.Objects;
 import org.mozilla.gecko.AndroidGamepadManager;
 import org.mozilla.gecko.EventDispatcher;
@@ -67,15 +67,26 @@ import org.mozilla.gecko.InputMethods;
 import org.mozilla.gecko.SurfaceViewWrapper;
 import org.mozilla.gecko.util.ThreadUtils;
 
+/**
+ * A view container that hosts Gecko rendering, manages its surface, and dispatches input/events.
+ */
 @UiThread
 public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfaceProvider {
   private static final String LOGTAG = "GeckoView";
   private static final boolean DEBUG = false;
+  private static final String KEYBOARD_WINDOW_INSETS_LISTENER = "KEYBOARD_WINDOW_INSETS_LISTENER";
+  private final HashMap<String, androidx.core.view.OnApplyWindowInsetsListener>
+      mWindowInsetsListeners =
+          new HashMap<String, androidx.core.view.OnApplyWindowInsetsListener>();
 
+  /** Manages the underlying GeckoDisplay surface lifecycle and layout. */
   protected final @NonNull Display mDisplay = new Display();
 
   private Integer mLastCoverColor;
+
+  /** The currently attached GeckoSession, or null if none is set. */
   protected @Nullable GeckoSession mSession;
+
   WeakReference<Autofill.Session> mAutofillSession = new WeakReference<>(null);
 
   // Whether this GeckoView instance has a session that is no longer valid, e.g. because the session
@@ -95,7 +106,8 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
   private @Nullable ActivityContextDelegate mActivityDelegate;
   private GeckoSession.PrintDelegate mPrintDelegate;
 
-  private class Display implements SurfaceViewWrapper.Listener {
+  private class Display
+      implements SurfaceViewWrapper.Listener, androidx.core.view.OnApplyWindowInsetsListener {
     private final int[] mOrigin = new int[2];
 
     private GeckoDisplay mDisplay;
@@ -194,6 +206,15 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
       }
     }
 
+    @Override // OnApplyWindowInsetsListener
+    public WindowInsetsCompat onApplyWindowInsets(
+        @NonNull final View view, @NonNull final WindowInsetsCompat insets) {
+      if (mDisplay != null) {
+        mDisplay.windowInsetsChanged(insets);
+      }
+      return insets;
+    }
+
     public boolean shouldPinOnScreen() {
       return mDisplay != null && mDisplay.shouldPinOnScreen();
     }
@@ -239,13 +260,22 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     }
   }
 
-  @SuppressWarnings("checkstyle:javadocmethod")
+  /**
+   * Construct a new GeckoView instance.
+   *
+   * @param context The Context in which this view is running.
+   */
   public GeckoView(final Context context) {
     super(context);
     init();
   }
 
-  @SuppressWarnings("checkstyle:javadocmethod")
+  /**
+   * Construct a new GeckoView with layout attributes.
+   *
+   * @param context the Context in which this view is running
+   * @param attrs the set of view attributes to apply
+   */
   public GeckoView(final Context context, final AttributeSet attrs) {
     super(context, attrs);
     init();
@@ -298,12 +328,7 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
       mSelectionActionDelegate = new BasicSelectionActionDelegate(activity);
     }
 
-    if (Build.VERSION.SDK_INT >= 26) {
-      mAutofillDelegate = new AndroidAutofillDelegate();
-    } else {
-      // We don't support Autofill on SDK < 26
-      mAutofillDelegate = new Autofill.Delegate() {};
-    }
+    mAutofillDelegate = new AndroidAutofillDelegate();
     mPrintDelegate = new GeckoViewPrintDelegate();
   }
 
@@ -348,6 +373,7 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
    */
   public static final int BACKEND_TEXTURE_VIEW = 2;
 
+  /** View backend type definitions for GeckoView display backends. */
   @Retention(RetentionPolicy.SOURCE)
   @IntDef({BACKEND_SURFACE_VIEW, BACKEND_TEXTURE_VIEW})
   public @interface ViewBackend {}
@@ -594,8 +620,12 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     }
   }
 
+  /**
+   * Returns the current GeckoSession attached to this view.
+   *
+   * @return The GeckoSession instance, or null if none is set.
+   */
   @AnyThread
-  @SuppressWarnings("checkstyle:javadocmethod")
   public @Nullable GeckoSession getSession() {
     return mSession;
   }
@@ -606,12 +636,84 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     return mSession.getEventDispatcher();
   }
 
-  @SuppressWarnings("checkstyle:javadocmethod")
+  /**
+   * Retrieves the controller responsible for panning and zooming gestures.
+   *
+   * @return The non-null PanZoomController for this GeckoView.
+   */
   public @NonNull PanZoomController getPanZoomController() {
     ThreadUtils.assertOnUiThread();
     return mSession.getPanZoomController();
   }
 
+  /**
+   * Register an internal windowInsetsListener that will forward its calls to the listeners
+   * registered in {@link GeckoView#mWindowInsetsListeners}
+   *
+   * @param activity The target Activity to observe.
+   */
+  private void attachWindowInsetsListener(final @NonNull Activity activity) {
+    try {
+      final View rootView = activity.getWindow().getDecorView().getRootView();
+      ViewCompat.setOnApplyWindowInsetsListener(
+          rootView,
+          (view, insets) -> {
+            WindowInsetsCompat updatedInsets =
+                WindowInsetsCompat.toWindowInsetsCompat(
+                    view.onApplyWindowInsets(insets.toWindowInsets()));
+
+            for (final androidx.core.view.OnApplyWindowInsetsListener listener :
+                mWindowInsetsListeners.values()) {
+              updatedInsets = listener.onApplyWindowInsets(view, updatedInsets);
+            }
+            return updatedInsets;
+          });
+    } catch (final Exception e) {
+      Log.e(LOGTAG, "Failed to attach WindowInsetsListener: ", e);
+    }
+  }
+
+  /**
+   * Unregister the internal WindowInsetsListener attached to the Activity's root view and clear the
+   * listeners that were attached through {@link GeckoView#addWindowInsetsListener(String,
+   * androidx.core.view.OnApplyWindowInsetsListener) }
+   *
+   * @param activity The target Activity to stop observing.
+   */
+  private void detachWindowInsetsListener(final @NonNull Activity activity) {
+    try {
+      final View rootView = activity.getWindow().getDecorView().getRootView();
+      ViewCompat.setOnApplyWindowInsetsListener(rootView, null);
+    } catch (final Exception e) {
+      Log.e(LOGTAG, "Failed to detach WindowInsetsListener: ", e);
+    }
+  }
+
+  /**
+   * Add an OnApplyWindowInsetsListener to observe the root view WindowInsets changes.
+   *
+   * @param key The key associated to the listener.
+   * @param listener The OnApplyWindowInsetsListener to be invoked.
+   */
+  @UiThread
+  public void addWindowInsetsListener(
+      final @NonNull String key,
+      final @Nullable androidx.core.view.OnApplyWindowInsetsListener listener) {
+    ThreadUtils.assertOnUiThread();
+    mWindowInsetsListeners.put(key, listener);
+  }
+
+  /**
+   * Remove the OnApplyWindowInsetsListener to stop observing WindowInsets changed.
+   *
+   * @param key The key associated to the listener to remove.
+   */
+  @UiThread
+  public void removeWindowInsetsListener(final @NonNull String key) {
+    ThreadUtils.assertOnUiThread();
+    mWindowInsetsListeners.remove(key);
+  }
+
   @Override
   public void onAttachedToWindow() {
     if (mIsSessionPoisoned) {
@@ -629,12 +731,21 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     }
 
     super.onAttachedToWindow();
+
+    // This needs to be called after the `super.onAttachedToWindow()`.
+    addWindowInsetsListener(KEYBOARD_WINDOW_INSETS_LISTENER, mDisplay);
+    attachWindowInsetsListener(getActivityFromContext(getContext()));
   }
 
   @Override
   public void onDetachedFromWindow() {
     super.onDetachedFromWindow();
 
+    removeWindowInsetsListener(KEYBOARD_WINDOW_INSETS_LISTENER);
+    if (mWindowInsetsListeners.isEmpty()) {
+      detachWindowInsetsListener(getActivityFromContext(getContext()));
+    }
+
     if (mSession == null) {
       return;
     }
@@ -650,14 +761,6 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     if (mSession != null) {
       final GeckoRuntime runtime = mSession.getRuntime();
       if (runtime != null) {
-        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
-          // If API is 31+, DisplayManager API may report previous information.
-          // So we have to report it again. But since Configuration.orientation may still have
-          // previous information even if onConfigurationChanged is called, we have to calculate it
-          // from display data.
-          runtime.orientationChanged();
-        }
-
         runtime.configurationChanged(newConfig);
       }
     }
@@ -750,10 +853,7 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
 
   @Override
   public Handler getHandler() {
-    if (Build.VERSION.SDK_INT >= 24 || mSession == null) {
-      return super.getHandler();
-    }
-    return mSession.getTextInput().getHandler(super.getHandler());
+    return super.getHandler();
   }
 
   @Override
@@ -819,7 +919,6 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
     }
   }
 
-  @SuppressLint("ClickableViewAccessibility")
   @Override
   public boolean onTouchEvent(final MotionEvent event) {
     if (event.getActionMasked() == MotionEvent.ACTION_DOWN) {
@@ -897,7 +996,6 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
   }
 
   @Override
-  @TargetApi(26)
   public void autofill(@NonNull final SparseArray<AutofillValue> values) {
     // Note: we can't use mSession.getAutofillSession() because the app might have swapped
     // the session under us between the onProvideAutofillVirtualStructure and this call
@@ -946,7 +1044,6 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
    *
    * @param enabled Whether or not Android autofill is enabled for this view.
    */
-  @TargetApi(26)
   public void setAutofillEnabled(final boolean enabled) {
     mAutofillEnabled = enabled;
 
@@ -962,12 +1059,10 @@ public class GeckoView extends FrameLayout implements GeckoDisplay.NewSurfacePro
   /**
    * @return Whether or not Android autofill is enabled for this view.
    */
-  @TargetApi(26)
   public boolean getAutofillEnabled() {
     return mAutofillEnabled;
   }
 
-  @TargetApi(26)
   private class AndroidAutofillDelegate implements Autofill.Delegate {
     AutofillManager mAutofillManager;
     boolean mDisabled = false;
