@@ -33,6 +33,11 @@ public class MediaSession {
   private final GeckoSession mSession;
   private boolean mIsActive;
 
+  /**
+   * Create a new MediaSession for the given GeckoSession.
+   *
+   * @param session The GeckoSession this media session is associated with.
+   */
   protected MediaSession(final GeckoSession session) {
     mSession = session;
   }
@@ -415,7 +420,7 @@ public class MediaSession {
     public String toString() {
       final StringBuilder builder = new StringBuilder("Metadata {");
       builder
-          .append(", title=")
+          .append("title=")
           .append(title)
           .append(", artist=")
           .append(artist)
@@ -478,6 +483,7 @@ public class MediaSession {
     }
   }
 
+  /** Media session feature flag definitions for supported media operations. */
   @Retention(RetentionPolicy.SOURCE)
   @LongDef(
       flag = true,
@@ -498,6 +504,7 @@ public class MediaSession {
 
   /** Flags for supported media session features. */
   public static class Feature {
+    /** No media session features supported. */
     public static final long NONE = 0;
 
     /** Playback supported. */
@@ -537,19 +544,17 @@ public class MediaSession {
 
     /* package */ static long fromBundle(final GeckoBundle bundle) {
       // Sync with MediaController.webidl.
-      final long features =
-          NONE
-              | (bundle.getBoolean("play") ? PLAY : NONE)
-              | (bundle.getBoolean("pause") ? PAUSE : NONE)
-              | (bundle.getBoolean("stop") ? STOP : NONE)
-              | (bundle.getBoolean("seekto") ? SEEK_TO : NONE)
-              | (bundle.getBoolean("seekforward") ? SEEK_FORWARD : NONE)
-              | (bundle.getBoolean("seekbackward") ? SEEK_BACKWARD : NONE)
-              | (bundle.getBoolean("nexttrack") ? NEXT_TRACK : NONE)
-              | (bundle.getBoolean("previoustrack") ? PREVIOUS_TRACK : NONE)
-              | (bundle.getBoolean("skipad") ? SKIP_AD : NONE)
-              | (bundle.getBoolean("focus") ? FOCUS : NONE);
-      return features;
+      return NONE
+          | (bundle.getBoolean("play") ? PLAY : NONE)
+          | (bundle.getBoolean("pause") ? PAUSE : NONE)
+          | (bundle.getBoolean("stop") ? STOP : NONE)
+          | (bundle.getBoolean("seekto") ? SEEK_TO : NONE)
+          | (bundle.getBoolean("seekforward") ? SEEK_FORWARD : NONE)
+          | (bundle.getBoolean("seekbackward") ? SEEK_BACKWARD : NONE)
+          | (bundle.getBoolean("nexttrack") ? NEXT_TRACK : NONE)
+          | (bundle.getBoolean("previoustrack") ? PREVIOUS_TRACK : NONE)
+          | (bundle.getBoolean("skipad") ? SKIP_AD : NONE)
+          | (bundle.getBoolean("focus") ? FOCUS : NONE);
     }
   }
 
