@@ -29,7 +29,12 @@ import org.mozilla.gecko.util.BundleEventListener;
 import org.mozilla.gecko.util.EventCallback;
 import org.mozilla.gecko.util.GeckoBundle;
 import org.mozilla.geckoview.WebExtension.InstallException;
+import org.mozilla.geckoview.WebExtension.InvalidMetaDataException;
 
+/**
+ * Controller for managing WebExtensions within a GeckoRuntime instance. Provides functionality for
+ * installing, uninstalling, enabling, disabling, and managing delegates for WebExtensions.
+ */
 public class WebExtensionController {
   private static final String LOGTAG = "WebExtension";
 
@@ -271,31 +276,6 @@ public class WebExtensionController {
       return null;
     }
 
-    /**
-     * Called whenever an updated extension has new permissions. This is intended as an opportunity
-     * for the app to prompt the user for the new permissions required by this extension.
-     *
-     * @deprecated Use onUpdatePrompt(WebExtension, String[], String[], String[]) to account for
-     *     data collection permissions.
-     * @param currentlyInstalled The {@link WebExtension} that is currently installed.
-     * @param updatedExtension The {@link WebExtension} that will replace the previous extension.
-     * @param newPermissions The new permissions that are needed.
-     * @param newOrigins The new origins that are needed.
-     * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if
-     *     this extension should be update or {@link AllowOrDeny#DENY DENY} if this extension should
-     *     not be update. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
-     */
-    @Nullable
-    @Deprecated
-    @DeprecationSchedule(id = "web-extension-on-update-prompt", version = 144)
-    default GeckoResult<AllowOrDeny> onUpdatePrompt(
-        @NonNull final WebExtension currentlyInstalled,
-        @NonNull final WebExtension updatedExtension,
-        @NonNull final String[] newPermissions,
-        @NonNull final String[] newOrigins) {
-      return null;
-    }
-
     /**
      * Called whenever an updated extension has new permissions. This is intended as an opportunity
      * for the app to prompt the user for the new permissions required by this extension.
@@ -317,31 +297,6 @@ public class WebExtensionController {
       return null;
     }
 
-    /**
-     * Called whenever permissions are requested. This is intended as an opportunity for the app to
-     * prompt the user for the permissions required by this extension at runtime.
-     *
-     * @deprecated Use onOptionalPrompt(WebExtension, String[], String[], String[]) to account for
-     *     data collection permissions.
-     * @param extension The {@link WebExtension} that is about to be installed. You can use {@link
-     *     WebExtension#metaData} to gather information about this extension when building the user
-     *     prompt dialog.
-     * @param permissions The permissions that are requested.
-     * @param origins The requested host permissions.
-     * @return A {@link GeckoResult} that completes to either {@link AllowOrDeny#ALLOW ALLOW} if the
-     *     request should be approved or {@link AllowOrDeny#DENY DENY} if the request should be
-     *     denied. A null value will be interpreted as {@link AllowOrDeny#DENY DENY}.
-     */
-    @Nullable
-    @Deprecated
-    @DeprecationSchedule(id = "web-extension-on-optional-prompt", version = 144)
-    default GeckoResult<AllowOrDeny> onOptionalPrompt(
-        final @NonNull WebExtension extension,
-        final @NonNull String[] permissions,
-        final @NonNull String[] origins) {
-      return null;
-    }
-
     /**
      * Called whenever permissions are requested. This is intended as an opportunity for the app to
      * prompt the user for the permissions required by this extension at runtime.
@@ -366,6 +321,10 @@ public class WebExtensionController {
     }
   }
 
+  /**
+   * Delegate for receiving debugger-related WebExtension events. Used to notify about changes to
+   * extensions during development with tools like web-ext.
+   */
   public interface DebuggerDelegate {
     /**
      * Called whenever the list of installed extensions has been modified using the debugger with
@@ -930,6 +889,7 @@ public class WebExtensionController {
         .accept(result -> unregisterWebExtension(extension));
   }
 
+  /** Defines the possible sources that can enable or disable a WebExtension. */
   @Retention(RetentionPolicy.SOURCE)
   @IntDef({EnableSource.USER, EnableSource.APP})
   public @interface EnableSources {}
@@ -1092,52 +1052,60 @@ public class WebExtensionController {
 
     Log.d(LOGTAG, "handleMessage " + event);
 
-    if ("GeckoView:WebExtension:InstallPrompt".equals(event)) {
-      installPromptRequest(bundle, callback);
-      return;
-    } else if ("GeckoView:WebExtension:UpdatePrompt".equals(event)) {
-      updatePrompt(bundle, callback);
-      return;
-    } else if ("GeckoView:WebExtension:DebuggerListUpdated".equals(event)) {
-      if (mDebuggerDelegate != null) {
-        mDebuggerDelegate.onExtensionListUpdated();
+    try {
+      if ("GeckoView:WebExtension:InstallPrompt".equals(event)) {
+        installPromptRequest(bundle, callback);
+        return;
+      } else if ("GeckoView:WebExtension:UpdatePrompt".equals(event)) {
+        updatePrompt(bundle, callback);
+        return;
+      } else if ("GeckoView:WebExtension:DebuggerListUpdated".equals(event)) {
+        if (mDebuggerDelegate != null) {
+          mDebuggerDelegate.onExtensionListUpdated();
+        }
+        return;
+      } else if ("GeckoView:WebExtension:OnDisabling".equals(event)) {
+        onDisabling(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnOptionalPermissionsChanged".equals(event)) {
+        onOptionalPermissionsChanged(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnDisabled".equals(event)) {
+        onDisabled(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnEnabling".equals(event)) {
+        onEnabling(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnEnabled".equals(event)) {
+        onEnabled(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnUninstalling".equals(event)) {
+        onUninstalling(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnUninstalled".equals(event)) {
+        onUninstalled(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnInstalling".equals(event)) {
+        onInstalling(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnInstalled".equals(event)) {
+        onInstalled(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnDisabledProcessSpawning".equals(event)) {
+        onDisabledProcessSpawning();
+        return;
+      } else if ("GeckoView:WebExtension:OnInstallationFailed".equals(event)) {
+        onInstallationFailed(bundle);
+        return;
+      } else if ("GeckoView:WebExtension:OnReady".equals(event)) {
+        onReady(bundle);
+        return;
+      }
+    } catch (final InvalidMetaDataException e) {
+      Log.e(LOGTAG, "Unexpected invalid bundle data on event " + event, e);
+      if (callback != null) {
+        callback.sendError("Unexpected invalid WebExtensions metaData on event " + event);
       }
-      return;
-    } else if ("GeckoView:WebExtension:OnDisabling".equals(event)) {
-      onDisabling(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnOptionalPermissionsChanged".equals(event)) {
-      onOptionalPermissionsChanged(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnDisabled".equals(event)) {
-      onDisabled(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnEnabling".equals(event)) {
-      onEnabling(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnEnabled".equals(event)) {
-      onEnabled(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnUninstalling".equals(event)) {
-      onUninstalling(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnUninstalled".equals(event)) {
-      onUninstalled(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnInstalling".equals(event)) {
-      onInstalling(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnInstalled".equals(event)) {
-      onInstalled(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnDisabledProcessSpawning".equals(event)) {
-      onDisabledProcessSpawning();
-      return;
-    } else if ("GeckoView:WebExtension:OnInstallationFailed".equals(event)) {
-      onInstallationFailed(bundle);
-      return;
-    } else if ("GeckoView:WebExtension:OnReady".equals(event)) {
-      onReady(bundle);
       return;
     }
 
@@ -1264,34 +1232,21 @@ public class WebExtensionController {
   }
 
   private void updatePrompt(final GeckoBundle message, final EventCallback callback) {
-    final WebExtension currentExtension =
-        new WebExtension(mDelegateControllerProvider, message.getBundle("currentlyInstalled"));
-    final WebExtension updatedExtension =
-        new WebExtension(mDelegateControllerProvider, message.getBundle("updatedExtension"));
+    final WebExtension extension =
+        new WebExtension(mDelegateControllerProvider, message.getBundle("extension"));
     final String[] newPermissions = message.getStringArray("newPermissions");
     final String[] newOrigins = message.getStringArray("newOrigins");
+    final String[] newDataCollectionPermissions =
+        message.getStringArray("newDataCollectionPermissions");
 
     if (mPromptDelegate == null) {
-      Log.e(
-          LOGTAG,
-          "Tried to update extension " + currentExtension.id + " but no delegate is registered");
+      Log.e(LOGTAG, "Tried to update extension " + extension.id + " but no delegate is registered");
       return;
     }
 
-    // TODO - Bug 1974744: remove the first call when we remove the deprecated
-    // `onUpdatePrompt` method since this has been done to preserve backward
-    // compatibility.
-    GeckoResult<AllowOrDeny> promptResponse =
+    final GeckoResult<AllowOrDeny> promptResponse =
         mPromptDelegate.onUpdatePrompt(
-            currentExtension, updatedExtension, newPermissions, newOrigins);
-
-    if (promptResponse == null) {
-      final String[] newDataCollectionPermissions =
-          message.getStringArray("newDataCollectionPermissions");
-      promptResponse =
-          mPromptDelegate.onUpdatePrompt(
-              updatedExtension, newPermissions, newOrigins, newDataCollectionPermissions);
-    }
+            extension, newPermissions, newOrigins, newDataCollectionPermissions);
 
     if (promptResponse == null) {
       return;
@@ -1319,21 +1274,12 @@ public class WebExtensionController {
     final String[] permissions =
         message.bundle.getBundle("permissions").getStringArray("permissions");
     final String[] origins = message.bundle.getBundle("permissions").getStringArray("origins");
+    final String[] dataCollectionPermissions =
+        message.bundle.getBundle("permissions").getStringArray("data_collection");
 
-    // TODO - Bug 1972510: remove the first call when we remove the deprecated
-    // `onOptionalPrompt` method since this has been done to preserve backward
-    // compatibility.
-    GeckoResult<AllowOrDeny> promptResponse =
-        mPromptDelegate.onOptionalPrompt(extension, permissions, origins);
-
-    if (promptResponse == null) {
-      final String[] dataCollectionPermissions =
-          message.bundle.getBundle("permissions").getStringArray("data_collection");
-
-      promptResponse =
-          mPromptDelegate.onOptionalPrompt(
-              extension, permissions, origins, dataCollectionPermissions);
-    }
+    final GeckoResult<AllowOrDeny> promptResponse =
+        mPromptDelegate.onOptionalPrompt(
+            extension, permissions, origins, dataCollectionPermissions);
 
     if (promptResponse == null) {
       return;
@@ -1912,6 +1858,13 @@ public class WebExtensionController {
     return null;
   }
 
+  /**
+   * Creates a new Download object with the specified ID for tracking WebExtension downloads.
+   *
+   * @param id The unique identifier for the download
+   * @return A new Download object for the specified ID
+   * @throws IllegalArgumentException if a download with this ID already exists
+   */
   @Nullable
   @UiThread
   public WebExtension.Download createDownload(final int id) {
