@@ -19,22 +19,18 @@ def test(mod, path, entity=None):
         # keep this file list in sync with jar.mn
         if path in (
             "chrome/global/commonDialogs.properties",
-            "chrome/global/intl.properties",
             "chrome/global/intl.css",
+            "toolkit/branding/brandings.ftl",
+            "toolkit/global/processTypes.ftl",
+            "toolkit/global/resetProfile.ftl",
         ):
             return "error"
-        if re.match(r"crashreporter/[^/]*.ftl", path):
-            # error on crashreporter/*.ftl
-            return "error"
         if re.match(r"toolkit/about/[^/]*About.ftl", path):
             # error on toolkit/about/*About.ftl
             return "error"
         if re.match(r"toolkit/about/[^/]*Mozilla.ftl", path):
             # error on toolkit/about/*Mozilla.ftl
             return "error"
-        if re.match(r"toolkit/about/[^/]*Plugins.ftl", path):
-            # error on toolkit/about/*Plugins.ftl
-            return "error"
         if re.match(r"toolkit/about/[^/]*Rights.ftl", path):
             # error on toolkit/about/*Rights.ftl
             return "error"
@@ -47,6 +43,9 @@ def test(mod, path, entity=None):
         if re.match(r"toolkit/about/[^/]*Webrtc.ftl", path):
             # error on toolkit/about/*Webrtc.ftl
             return "error"
+        if re.match(r"toolkit/about/[^/]*Logging.ftl", path):
+            # error on toolkit/about/*Logging.ftl
+            return "error"
         return "ignore"
 
     if mod == "dom":
