@@ -5,12 +5,14 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsURILoader.h"
+#include "nsComponentManagerUtils.h"
+#include "nsContentSecurityUtils.h"
 #include "nsIURIContentListener.h"
 #include "nsIContentHandler.h"
 #include "nsILoadGroup.h"
 #include "nsIDocumentLoader.h"
 #include "nsIStreamListener.h"
-#include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsIChannel.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
@@ -38,14 +40,14 @@
 #include "nsMimeTypes.h"
 
 #include "nsDocLoader.h"
-#include "mozilla/Attributes.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/Preferences.h"
-#include "mozilla/Unused.h"
+#include "mozilla/Result.h"
 #include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StaticPrefs_general.h"
 #include "nsContentUtils.h"
+#include "imgLoader.h"
 
 mozilla::LazyLogModule nsURILoader::mLog("URILoader");
 
@@ -70,7 +72,8 @@ nsDocumentOpenInfo::nsDocumentOpenInfo(nsIInterfaceRequestor* aWindowContext,
       mFlags(aFlags),
       mURILoader(aURILoader),
       mDataConversionDepthLimit(
-          StaticPrefs::general_document_open_conversion_depth_limit()) {}
+          mozilla::StaticPrefs::
+              general_document_open_conversion_depth_limit()) {}
 
 nsDocumentOpenInfo::nsDocumentOpenInfo(uint32_t aFlags,
                                        bool aAllowListenerConversions)
@@ -78,7 +81,7 @@ nsDocumentOpenInfo::nsDocumentOpenInfo(uint32_t aFlags,
       mFlags(aFlags),
       mURILoader(nullptr),
       mDataConversionDepthLimit(
-          StaticPrefs::general_document_open_conversion_depth_limit()),
+          mozilla::StaticPrefs::general_document_open_conversion_depth_limit()),
       mAllowListenerConversions(aAllowListenerConversions) {}
 
 nsDocumentOpenInfo::~nsDocumentOpenInfo() {}
@@ -129,6 +132,28 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStartRequest(nsIRequest* request) {
     if (204 == responseCode || 205 == responseCode) {
       return NS_BINDING_ABORTED;
     }
+
+    if (!mozilla::StaticPrefs::
+            browser_http_blank_page_with_error_response_enabled()) {
+      // Bug 1325876: Show internal error page for HTTP responses with error
+      // codes (4xx, 5xx) and "Content-Length": 0 instead of blank page
+      int64_t contentLength = 0;
+      rv = httpChannel->GetContentLength(&contentLength);
+
+      if (NS_SUCCEEDED(rv) && contentLength == 0) {
+        nsCOMPtr<nsIURI> uri;
+        rv = httpChannel->GetURI(getter_AddRefs(uri));
+        // Bug 1944984: Avoid error page on view-source views of empty pages
+        if (NS_FAILED(rv) || !uri->SchemeIs("view-source")) {
+          if (responseCode >= 500) {
+            return NS_ERROR_NET_ERROR_RESPONSE;
+          }
+          if (responseCode >= 400) {
+            return NS_ERROR_NET_EMPTY_RESPONSE;
+          }
+        }
+      }
+    }
   }
 
   //
@@ -194,6 +219,7 @@ nsDocumentOpenInfo::OnDataAvailable(nsIRequest* request, nsIInputStream* inStr,
   // if we have retarged to the end stream listener, then forward the call....
   // otherwise, don't do anything
 
+  mReceivedData = true;
   nsresult rv = NS_OK;
 
   if (m_targetStreamListener)
@@ -202,10 +228,78 @@ nsDocumentOpenInfo::OnDataAvailable(nsIRequest* request, nsIInputStream* inStr,
   return rv;
 }
 
+NS_IMETHODIMP
+nsDocumentOpenInfo::OnDataFinished(nsresult aStatus) {
+  if (!m_targetStreamListener) {
+    return NS_ERROR_FAILURE;
+  }
+  nsCOMPtr<nsIThreadRetargetableStreamListener> retargetableListener =
+      do_QueryInterface(m_targetStreamListener);
+  if (retargetableListener) {
+    return retargetableListener->OnDataFinished(aStatus);
+  }
+
+  return NS_OK;
+}
+
+/* static */
+nsresult nsDocumentOpenInfo::CheckContentLengthDiscrepancy(
+    nsIRequest* request) {
+  // Blank page with content length discrepancy is allowed
+  if (mReceivedData ||
+      mozilla::StaticPrefs::
+          browser_http_blank_page_with_error_response_enabled()) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
+  if (!httpChannel) {
+    return NS_OK;
+  }
+
+  uint64_t decodedBodySize;
+  if (NS_FAILED(httpChannel->GetDecodedBodySize(&decodedBodySize)) ||
+      decodedBodySize != 0) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIURI> uri;
+  if (NS_SUCCEEDED(httpChannel->GetURI(getter_AddRefs(uri))) &&
+      uri->SchemeIs("view-source")) {
+    return NS_OK;
+  }
+
+  uint32_t responseCode = 0;
+  if (NS_SUCCEEDED(httpChannel->GetResponseStatus(&responseCode))) {
+    if (responseCode >= 500) {
+      LOG(
+          ("  Returning NS_ERROR_NET_ERROR_RESPONSE from "
+           "nsDocumentOpenInfo::CheckContentLengthDiscrepancy due to 5xx "
+           "responses with no content"));
+      return NS_ERROR_NET_ERROR_RESPONSE;
+    }
+    if (responseCode >= 400) {
+      LOG(
+          ("  Returning NS_ERROR_NET_EMPTY_RESPONSE from "
+           "nsDocumentOpenInfo::CheckContentLengthDiscrepancy due to 4xx "
+           "responses with no content"));
+      return NS_ERROR_NET_EMPTY_RESPONSE;
+    }
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest* request,
                                                 nsresult aStatus) {
   LOG(("[0x%p] nsDocumentOpenInfo::OnStopRequest", this));
 
+  // Bug 1945855 - Show error page for 4xx/5xx responses with no content,
+  // whether the Content-Length header is missing or lying
+  nsresult rv = CheckContentLengthDiscrepancy(request);
+  if (NS_FAILED(rv)) {
+    aStatus = rv;
+  }
+
   if (m_targetStreamListener) {
     nsCOMPtr<nsIStreamListener> listener(m_targetStreamListener);
 
@@ -224,6 +318,62 @@ NS_IMETHODIMP nsDocumentOpenInfo::OnStopRequest(nsIRequest* request,
   return NS_OK;
 }
 
+static bool IsContentPDF(nsIChannel* aChannel, const nsACString& aContentType) {
+  bool isPDF = aContentType.LowerCaseEqualsASCII(APPLICATION_PDF);
+  if (!isPDF && (aContentType.LowerCaseEqualsASCII(APPLICATION_OCTET_STREAM) ||
+                 aContentType.IsEmpty())) {
+    nsAutoString flname;
+    aChannel->GetContentDispositionFilename(flname);
+    isPDF = StringEndsWith(flname, u".pdf"_ns);
+    if (!isPDF) {
+      nsCOMPtr<nsIURI> uri;
+      aChannel->GetURI(getter_AddRefs(uri));
+      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+      if (url) {
+        nsAutoCString ext;
+        url->GetFileExtension(ext);
+        isPDF = ext.EqualsLiteral("pdf");
+      }
+    }
+  }
+
+  return isPDF;
+}
+
+static mozilla::Result<bool, nsresult> ShouldHandleExternally(
+    const nsACString& aMimeType) {
+  // For a PDF, check if the preference is set that forces attachments to be
+  // opened inline. If so, treat it as a non-attachment by clearing
+  // 'forceExternalHandling' again. This allows it open a PDF directly
+  // instead of downloading it first. It may still end up being handled by
+  // a helper app depending anyway on the later checks.
+  nsCOMPtr<nsIMIMEInfo> mimeInfo;
+
+  nsCOMPtr<nsIMIMEService> mimeSvc(do_GetService(NS_MIMESERVICE_CONTRACTID));
+  if (!mimeSvc) {
+    return mozilla::Err(NS_ERROR_FAILURE);
+  }
+
+  mimeSvc->GetFromTypeAndExtension(aMimeType, EmptyCString(),
+                                   getter_AddRefs(mimeInfo));
+
+  if (mimeInfo) {
+    int32_t action = nsIMIMEInfo::saveToDisk;
+    mimeInfo->GetPreferredAction(&action);
+
+    bool alwaysAsk = true;
+    mimeInfo->GetAlwaysAskBeforeHandling(&alwaysAsk);
+    return alwaysAsk || action != nsIMIMEInfo::handleInternally;
+  }
+
+  return false;
+}
+
+static bool IsSandboxed(nsIChannel* aChannel) {
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+  return loadInfo->GetSandboxFlags();
+}
+
 nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
   LOG(("[0x%p] nsDocumentOpenInfo::DispatchContent for type '%s'", this,
        mContentType.get()));
@@ -258,41 +408,53 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
   // could happen because the Content-Disposition header is set so, or, in the
   // future, because the user has specified external handling for the MIME
   // type.
-  bool forceExternalHandling = false;
   uint32_t disposition;
   rv = aChannel->GetContentDisposition(&disposition);
 
-  if (NS_SUCCEEDED(rv) && disposition == nsIChannel::DISPOSITION_ATTACHMENT) {
-    forceExternalHandling = true;
-  }
+  bool forceExternalHandling =
+      NS_SUCCEEDED(rv) && disposition == nsIChannel::DISPOSITION_ATTACHMENT;
 
   LOG(("  forceExternalHandling: %s", forceExternalHandling ? "yes" : "no"));
+  LOG(("  IsSandboxed: %s", IsSandboxed(aChannel) ? "yes" : "no"));
+  LOG(("  IsContentPDF: %s",
+       IsContentPDF(aChannel, mContentType) ? "yes" : "no"));
 
-  // For a PDF, check if it will be handled internally. If so, treat it as a
-  // non-attachment by clearing 'forceExternalHandling' again. This allows it
-  // open a PDF directly instead of downloading it first. It may still
-  // end up being handled by a helper app depending anyway on the later checks.
-  if (forceExternalHandling &&
-      mContentType.LowerCaseEqualsASCII(APPLICATION_PDF) &&
-      StaticPrefs::browser_download_improvements_to_download_panel()) {
-    nsCOMPtr<nsILoadInfo> loadInfo;
-    aChannel->GetLoadInfo(getter_AddRefs(loadInfo));
+  // Ignore the Content-Disposition header if we're loading a PDF or Image
+  // subresource within an object/embed element.
+  if (forceExternalHandling && (mFlags & nsIURILoader::IS_OBJECT_EMBED) &&
+      (imgLoader::SupportImageWithMimeType(mContentType) ||
+       IsContentPDF(aChannel, mContentType))) {
+    LOG(("Handling pdf/image MIME internally for object/embed element"));
+    forceExternalHandling = false;
+  }
 
-    nsCOMPtr<nsIMIMEInfo> mimeInfo;
+  bool maybeForceInternalHandling =
+      forceExternalHandling &&
+      mozilla::StaticPrefs::browser_download_open_pdf_attachments_inline();
 
-    nsCOMPtr<nsIMIMEService> mimeSvc(do_GetService(NS_MIMESERVICE_CONTRACTID));
-    NS_ENSURE_TRUE(mimeSvc, NS_ERROR_FAILURE);
-    mimeSvc->GetFromTypeAndExtension(nsLiteralCString(APPLICATION_PDF), ""_ns,
-                                     getter_AddRefs(mimeInfo));
+  // Check if this is a PDF which should be opened internally. We also handle
+  // octet-streams that look like they might be PDFs based on their extension.
+  if (maybeForceInternalHandling && IsContentPDF(aChannel, mContentType)) {
+    // For a PDF, check if the preference is set that forces attachments to be
+    // opened inline. If so, treat it as a non-attachment by clearing
+    // 'forceExternalHandling' again. This allows it open a PDF directly
+    // instead of downloading it first. It may still end up being handled by
+    // a helper app depending anyway on the later checks.
+    auto result = ShouldHandleExternally(nsLiteralCString(APPLICATION_PDF));
+    if (result.isErr()) {
+      return result.unwrapErr();
+    }
+    forceExternalHandling = result.unwrap();
 
-    if (mimeInfo) {
-      int32_t action = nsIMIMEInfo::saveToDisk;
-      mimeInfo->GetPreferredAction(&action);
-
-      bool alwaysAsk = true;
-      mimeInfo->GetAlwaysAskBeforeHandling(&alwaysAsk);
-      forceExternalHandling =
-          alwaysAsk || action != nsIMIMEInfo::handleInternally;
+    // If we're not opening the PDF externally we block it if it's sandboxed.
+    if (IsSandboxed(aChannel) && !forceExternalHandling) {
+      LOG(("Blocked sandboxed PDF"));
+      nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+      if (httpChannel) {
+        nsContentSecurityUtils::LogMessageToConsole(
+            httpChannel, "IframeSandboxBlockedDownload");
+      }
+      return NS_ERROR_CONTENT_BLOCKED;
     }
   }
 
@@ -331,32 +493,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
       }
 
       //
-      // Third step: Try to find a content listener that has not yet had
-      // the chance to register, as it is contained in a not-yet-loaded
-      // module, but which has registered a contract ID.
-      //
-      nsCOMPtr<nsICategoryManager> catman =
-          do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-      if (catman) {
-        nsCString contractidString;
-        rv = catman->GetCategoryEntry(NS_CONTENT_LISTENER_CATEGORYMANAGER_ENTRY,
-                                      mContentType, contractidString);
-        if (NS_SUCCEEDED(rv) && !contractidString.IsEmpty()) {
-          LOG(("  Listener contractid for '%s' is '%s'", mContentType.get(),
-               contractidString.get()));
-
-          listener = do_CreateInstance(contractidString.get());
-          LOG(("  Listener from category manager: 0x%p", listener.get()));
-
-          if (listener && TryContentListener(listener, aChannel)) {
-            LOG(("  Listener from category manager likes this type"));
-            return NS_OK;
-          }
-        }
-      }
-
-      //
-      // Fourth step: try to find an nsIContentHandler for our type.
+      // Third step: Try to find an nsIContentHandler for our type.
       //
       nsAutoCString handlerContractID(NS_CONTENT_HANDLER_CONTRACTID_PREFIX);
       handlerContractID += mContentType;
@@ -394,7 +531,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
     }
 
     //
-    // Fifth step:  If no listener prefers this type, see if any stream
+    // Fourth step: If no listener prefers this type, see if any stream
     //              converters exist to transform this content type into
     //              some other.
     //
@@ -413,6 +550,21 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
   NS_ASSERTION(!m_targetStreamListener,
                "If we found a listener, why are we not using it?");
 
+  // Before dispatching to the external helper app service, check for an HTTP
+  // error page.  If we got one, we don't want to handle it with a helper app,
+  // really.
+  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
+  if (httpChannel) {
+    bool requestSucceeded;
+    rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
+    if (NS_FAILED(rv) || !requestSucceeded) {
+      LOG(
+          ("  Returning NS_ERROR_NET_ERROR_RESPONSE from "
+           "nsDocumentOpenInfo::DispatchContent due to failed HTTP response"));
+      return NS_ERROR_NET_ERROR_RESPONSE;
+    }
+  }
+
   if (mFlags & nsIURILoader::DONT_RETARGET) {
     LOG(
         ("  External handling forced or (listener not interested and no "
@@ -420,22 +572,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
     return NS_ERROR_WONT_HANDLE_CONTENT;
   }
 
-  // Before dispatching to the external helper app service, check for an HTTP
-  // error page.  If we got one, we don't want to handle it with a helper app,
-  // really.
-  // The WPT a-download-click-404.html requires us to silently handle this
-  // without displaying an error page, so we just return early here.
-  // See bug 1604308 for discussion around what the ideal behaviour is.
-  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
-  if (httpChannel) {
-    bool requestSucceeded;
-    rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
-    if (NS_FAILED(rv) || !requestSucceeded) {
-      return NS_OK;
-    }
-  }
-
-  // Sixth step:
+  // Fifth step:
   //
   // All attempts to dispatch this content have failed.  Just pass it off to
   // the helper app service.
@@ -453,7 +590,7 @@ nsresult nsDocumentOpenInfo::DispatchContent(nsIRequest* request) {
     request->SetLoadFlags(loadFlags | nsIChannel::LOAD_RETARGETED_DOCUMENT_URI |
                           nsIChannel::LOAD_TARGETED);
 
-    if (isGuessFromExt) {
+    if (isGuessFromExt || mContentType.IsEmpty()) {
       mContentType = APPLICATION_GUESS_FROM_EXT;
       aChannel->SetContentType(nsLiteralCString(APPLICATION_GUESS_FROM_EXT));
     }
@@ -546,7 +683,15 @@ nsresult nsDocumentOpenInfo::ConvertData(nsIRequest* request,
 
 nsresult nsDocumentOpenInfo::TryStreamConversion(nsIChannel* aChannel) {
   constexpr auto anyType = "*/*"_ns;
-  nsresult rv = ConvertData(aChannel, m_contentListener, mContentType, anyType);
+
+  // A empty content type should be treated like the unknown content type.
+  nsCString srcContentType(mContentType);
+  if (srcContentType.IsEmpty()) {
+    srcContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
+  }
+
+  nsresult rv =
+      ConvertData(aChannel, m_contentListener, srcContentType, anyType);
   if (NS_FAILED(rv)) {
     m_targetStreamListener = nullptr;
   } else if (m_targetStreamListener) {
