@@ -7,19 +7,16 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "mozilla/net/NeckoCommon.h"
+#include "mozilla/StaticPrefs_browser.h"
+#include "nsComponentManagerUtils.h"
 #include "nsOSHelperAppService.h"
 #include "nsObjCExceptions.h"
 #include "nsISupports.h"
 #include "nsString.h"
 #include "nsTArray.h"
-#include "nsIURL.h"
 #include "nsIFile.h"
-#include "nsIHandlerService.h"
 #include "nsILocalFileMac.h"
 #include "nsMimeTypes.h"
-#include "nsIStringBundle.h"
-#include "nsIPromptService.h"
-#include "nsMemory.h"
 #include "nsCRT.h"
 #include "nsMIMEInfoMac.h"
 #include "nsEmbedCID.h"
@@ -27,9 +24,43 @@
 #import <CoreFoundation/CoreFoundation.h>
 #import <ApplicationServices/ApplicationServices.h>
 
-// chrome URL's
-#define HELPERAPPLAUNCHER_BUNDLE_URL "chrome://global/locale/helperAppLauncher.properties"
-#define BRAND_BUNDLE_URL "chrome://branding/locale/brand.properties"
+nsresult GetDefaultBundleURL(const nsACString& aScheme, CFURLRef* aBundleURL) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
+
+  nsresult rv = NS_ERROR_NOT_AVAILABLE;
+
+  CFStringRef schemeCFString = ::CFStringCreateWithBytes(
+      kCFAllocatorDefault, (const UInt8*)PromiseFlatCString(aScheme).get(),
+      aScheme.Length(), kCFStringEncodingUTF8, false);
+
+  if (schemeCFString) {
+    CFStringRef lookupCFString =
+        ::CFStringCreateWithFormat(NULL, NULL, CFSTR("%@:"), schemeCFString);
+
+    if (lookupCFString) {
+      CFURLRef lookupCFURL =
+          ::CFURLCreateWithString(NULL, lookupCFString, NULL);
+
+      if (lookupCFURL) {
+        *aBundleURL =
+            ::LSCopyDefaultApplicationURLForURL(lookupCFURL, kLSRolesAll, NULL);
+        if (*aBundleURL) {
+          rv = NS_OK;
+        }
+
+        ::CFRelease(lookupCFURL);
+      }
+
+      ::CFRelease(lookupCFString);
+    }
+
+    ::CFRelease(schemeCFString);
+  }
+
+  return rv;
+
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
+}
 
 using mozilla::LogLevel;
 
@@ -55,146 +86,159 @@ using mozilla::LogLevel;
 
 nsOSHelperAppService::~nsOSHelperAppService() {}
 
-nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char* aProtocolScheme,
-                                                       bool* aHandlerExists) {
+nsresult nsOSHelperAppService::OSProtocolHandlerExists(
+    const char* aProtocolScheme, bool* aHandlerExists) {
+  *aHandlerExists = false;
+
   // CFStringCreateWithBytes() can fail even if we're not out of memory --
   // for example if the 'bytes' parameter is something very weird (like
   // "\xFF\xFF~"), or possibly if it can't be interpreted as using what's
   // specified in the 'encoding' parameter.  See bug 548719.
-  CFStringRef schemeString =
-      ::CFStringCreateWithBytes(kCFAllocatorDefault, (const UInt8*)aProtocolScheme,
-                                strlen(aProtocolScheme), kCFStringEncodingUTF8, false);
-  if (schemeString) {
-    // LSCopyDefaultHandlerForURLScheme() can fail to find the default handler
-    // for aProtocolScheme when it's never been explicitly set (using
-    // LSSetDefaultHandlerForURLScheme()).  For example, Safari is the default
-    // handler for the "http" scheme on a newly installed copy of OS X.  But
-    // this (presumably) wasn't done using LSSetDefaultHandlerForURLScheme(),
-    // so LSCopyDefaultHandlerForURLScheme() will fail to find Safari.  To get
-    // around this we use LSCopyAllHandlersForURLScheme() instead -- which seems
-    // never to fail.
-    // http://lists.apple.com/archives/Carbon-dev/2007/May/msg00349.html
-    // http://www.realsoftware.com/listarchives/realbasic-nug/2008-02/msg00119.html
-    CFArrayRef handlerArray = ::LSCopyAllHandlersForURLScheme(schemeString);
-    *aHandlerExists = !!handlerArray;
-    if (handlerArray) ::CFRelease(handlerArray);
-    ::CFRelease(schemeString);
-  } else {
-    *aHandlerExists = false;
+  CFStringRef schemeString = ::CFStringCreateWithBytes(
+      kCFAllocatorDefault, (const UInt8*)aProtocolScheme,
+      strlen(aProtocolScheme), kCFStringEncodingUTF8, false);
+  if (!schemeString) {
+    return NS_ERROR_FAILURE;
   }
+
+  // LSCopyDefaultHandlerForURLScheme() can fail to find the default handler
+  // for aProtocolScheme when it's never been explicitly set (using
+  // LSSetDefaultHandlerForURLScheme()).  For example, Safari is the default
+  // handler for the "http" scheme on a newly installed copy of OS X.  But
+  // this (presumably) wasn't done using LSSetDefaultHandlerForURLScheme(),
+  // so LSCopyDefaultHandlerForURLScheme() will fail to find Safari.  To get
+  // around this we use LSCopyAllHandlersForURLScheme() instead -- which seems
+  // never to fail.
+  // http://lists.apple.com/archives/Carbon-dev/2007/May/msg00349.html
+  // http://www.realsoftware.com/listarchives/realbasic-nug/2008-02/msg00119.html
+  CFArrayRef handlerArray = ::LSCopyAllHandlersForURLScheme(schemeString);
+  ::CFRelease(schemeString);
+  if (handlerArray) {
+    *aHandlerExists = true;
+    ::CFRelease(handlerArray);
+  }
+
   return NS_OK;
 }
 
-NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(const nsACString& aScheme,
-                                                              nsAString& _retval) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(
+    const nsACString& aScheme, nsAString& _retval) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
+
+  CFURLRef handlerBundleURL;
+  nsresult rv = GetDefaultBundleURL(aScheme, &handlerBundleURL);
+  if (NS_FAILED(rv)) {
+    if (handlerBundleURL) {
+      ::CFRelease(handlerBundleURL);
+    }
+    return rv;
+  }
+
+  // Default to just using the application's name.
+  CFStringRef bundleName = ::CFURLCopyLastPathComponent(handlerBundleURL);
+  if (!bundleName) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // See if we can get the bundle display name from the plist.
+  CFBundleRef handlerBundle =
+      ::CFBundleCreate(kCFAllocatorDefault, handlerBundleURL);
+  ::CFRelease(handlerBundleURL);
+  if (handlerBundle) {
+    // Memory management for the value returned here is tricky. We don't have
+    // to release this value, but we can't retain it and we can't release the
+    // underlying bundle until we're done with this value. Just make a copy so
+    // we can ignore this and just assume release for all bundleName values.
+    CFStringRef tmpBundleName =
+        (CFStringRef)::CFBundleGetValueForInfoDictionaryKey(handlerBundle,
+                                                            kCFBundleNameKey);
+    if (tmpBundleName && (::CFStringGetLength(tmpBundleName) > 0)) {
+      ::CFRelease(bundleName);
+      bundleName = ::CFStringCreateCopy(kCFAllocatorDefault, tmpBundleName);
+    }
+    ::CFRelease(handlerBundle);
+  }
+
+  AutoTArray<UniChar, 255> buffer;
+  CFIndex bundleNameLength = ::CFStringGetLength(bundleName);
+  buffer.SetLength(bundleNameLength);
+  ::CFStringGetCharacters(bundleName, CFRangeMake(0, bundleNameLength),
+                          buffer.Elements());
+  _retval.Assign(reinterpret_cast<char16_t*>(buffer.Elements()),
+                 bundleNameLength);
+
+  ::CFRelease(bundleName);
+
+  return NS_OK;
+
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
+}
+
+NS_IMETHODIMP nsOSHelperAppService::IsCurrentAppOSDefaultForProtocol(
+    const nsACString& aScheme, bool* _retval) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   nsresult rv = NS_ERROR_NOT_AVAILABLE;
 
-  CFStringRef schemeCFString = ::CFStringCreateWithBytes(
-      kCFAllocatorDefault, (const UInt8*)PromiseFlatCString(aScheme).get(), aScheme.Length(),
-      kCFStringEncodingUTF8, false);
-
-  if (schemeCFString) {
-    CFStringRef lookupCFString =
-        ::CFStringCreateWithFormat(NULL, NULL, CFSTR("%@:"), schemeCFString);
-
-    if (lookupCFString) {
-      CFURLRef lookupCFURL = ::CFURLCreateWithString(NULL, lookupCFString, NULL);
-
-      if (lookupCFURL) {
-        CFURLRef appCFURL = NULL;
-        OSStatus theErr = ::LSGetApplicationForURL(lookupCFURL, kLSRolesAll, NULL, &appCFURL);
-
-        if (theErr == noErr) {
-          CFBundleRef handlerBundle = ::CFBundleCreate(NULL, appCFURL);
-
-          if (handlerBundle) {
-            // Get the human-readable name of the default handler bundle
-            CFStringRef bundleName = (CFStringRef)::CFBundleGetValueForInfoDictionaryKey(
-                handlerBundle, kCFBundleNameKey);
-
-            if (bundleName) {
-              AutoTArray<UniChar, 255> buffer;
-              CFIndex bundleNameLength = ::CFStringGetLength(bundleName);
-              buffer.SetLength(bundleNameLength);
-              ::CFStringGetCharacters(bundleName, CFRangeMake(0, bundleNameLength),
-                                      buffer.Elements());
-              _retval.Assign(reinterpret_cast<char16_t*>(buffer.Elements()), bundleNameLength);
-              rv = NS_OK;
-            }
-
-            ::CFRelease(handlerBundle);
-          }
-
-          ::CFRelease(appCFURL);
-        }
-
-        ::CFRelease(lookupCFURL);
-      }
-
-      ::CFRelease(lookupCFString);
+  CFURLRef handlerBundleURL;
+  rv = GetDefaultBundleURL(aScheme, &handlerBundleURL);
+  if (NS_SUCCEEDED(rv) && handlerBundleURL) {
+    // Ensure we don't accidentally return success if we can't get an app
+    // bundle.
+    rv = NS_ERROR_NOT_AVAILABLE;
+    CFBundleRef appBundle = ::CFBundleGetMainBundle();
+    if (appBundle) {
+      CFURLRef selfURL = ::CFBundleCopyBundleURL(appBundle);
+      *_retval = ::CFEqual(selfURL, handlerBundleURL);
+      rv = NS_OK;
+      ::CFRelease(selfURL);
     }
-
-    ::CFRelease(schemeCFString);
+    ::CFRelease(handlerBundleURL);
   }
 
   return rv;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
-nsresult nsOSHelperAppService::GetFileTokenForPath(const char16_t* aPlatformAppPath,
-                                                   nsIFile** aFile) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
+nsresult nsOSHelperAppService::GetFileTokenForPath(
+    const char16_t* aPlatformAppPath, nsIFile** aFile) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  nsresult rv;
-  nsCOMPtr<nsILocalFileMac> localFile(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  CFURLRef pathAsCFURL;
   CFStringRef pathAsCFString = ::CFStringCreateWithCharacters(
-      NULL, reinterpret_cast<const UniChar*>(aPlatformAppPath), NS_strlen(aPlatformAppPath));
-  if (!pathAsCFString) return NS_ERROR_OUT_OF_MEMORY;
-
-  if (::CFStringGetCharacterAtIndex(pathAsCFString, 0) == '/') {
-    // we have a Posix path
-    pathAsCFURL =
-        ::CFURLCreateWithFileSystemPath(nullptr, pathAsCFString, kCFURLPOSIXPathStyle, false);
-    if (!pathAsCFURL) {
-      ::CFRelease(pathAsCFString);
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-  } else {
-    // if it doesn't start with a / it's not an absolute Posix path
-    // let's check if it's a HFS path left over from old preferences
-
-    // If it starts with a ':' char, it's not an absolute HFS path
-    // so bail for that, and also if it's empty
-    if (::CFStringGetLength(pathAsCFString) == 0 ||
-        ::CFStringGetCharacterAtIndex(pathAsCFString, 0) == ':') {
-      ::CFRelease(pathAsCFString);
-      return NS_ERROR_FILE_UNRECOGNIZED_PATH;
-    }
-
-    pathAsCFURL =
-        ::CFURLCreateWithFileSystemPath(nullptr, pathAsCFString, kCFURLHFSPathStyle, false);
-    if (!pathAsCFURL) {
-      ::CFRelease(pathAsCFString);
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
+      NULL, reinterpret_cast<const UniChar*>(aPlatformAppPath),
+      NS_strlen(aPlatformAppPath));
+  if (!pathAsCFString) {
+    return NS_ERROR_FAILURE;
   }
 
-  rv = localFile->InitWithCFURL(pathAsCFURL);
+  // Require a posix path
+  if (::CFStringGetCharacterAtIndex(pathAsCFString, 0) != '/') {
+    ::CFRelease(pathAsCFString);
+    return NS_ERROR_FILE_UNRECOGNIZED_PATH;
+  }
+
+  CFURLRef pathAsCFURL = ::CFURLCreateWithFileSystemPath(
+      nullptr, pathAsCFString, kCFURLPOSIXPathStyle, false);
   ::CFRelease(pathAsCFString);
+  if (!pathAsCFURL) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsILocalFileMac> localFile;
+  nsresult rv =
+      NS_NewLocalFileWithCFURL(pathAsCFURL, getter_AddRefs(localFile));
   ::CFRelease(pathAsCFURL);
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
   *aFile = localFile;
   NS_IF_ADDREF(*aFile);
 
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 // Returns the MIME types an application bundle explicitly claims to handle.
@@ -222,7 +266,8 @@ static CFArrayRef GetMIMETypesHandledByApp(FSRef* aAppRef) {
   if (!infoDict) {
     return NULL;
   }
-  CFTypeRef cfObject = ::CFDictionaryGetValue(infoDict, CFSTR("CFBundleDocumentTypes"));
+  CFTypeRef cfObject =
+      ::CFDictionaryGetValue(infoDict, CFSTR("CFBundleDocumentTypes"));
   if (!cfObject || (::CFGetTypeID(cfObject) != ::CFArrayGetTypeID())) {
     ::CFRelease(infoDict);
     return NULL;
@@ -270,9 +315,10 @@ static CFArrayRef GetMIMETypesHandledByApp(FSRef* aAppRef) {
 }
 
 nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
-                                                 const nsACString& aFileExt, bool* aFound,
+                                                 const nsACString& aFileExt,
+                                                 bool* aFound,
                                                  nsIMIMEInfo** aMIMEInfo) {
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSNULL;
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
   MOZ_ASSERT(XRE_IsParentProcess());
 
   *aFound = false;
@@ -280,8 +326,9 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
   const nsCString& flatType = PromiseFlatCString(aMIMEType);
   const nsCString& flatExt = PromiseFlatCString(aFileExt);
 
-  MOZ_LOG(mLog, LogLevel::Debug,
-          ("Mac: HelperAppService lookup for type '%s' ext '%s'\n", flatType.get(), flatExt.get()));
+  MOZ_LOG(sLog, LogLevel::Debug,
+          ("Mac: HelperAppService lookup for type '%s' ext '%s'\n",
+           flatType.get(), flatExt.get()));
 
   // Create a Mac-specific MIME info so we can use Mac-specific members.
   RefPtr<nsMIMEInfoMac> mimeInfoMac = new nsMIMEInfoMac(aMIMEType);
@@ -300,18 +347,20 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
   CFStringRef cfMIMEType = NULL;
 
   if (!aMIMEType.IsEmpty()) {
-    typeIsOctetStream = aMIMEType.LowerCaseEqualsLiteral(APPLICATION_OCTET_STREAM);
+    typeIsOctetStream =
+        aMIMEType.LowerCaseEqualsLiteral(APPLICATION_OCTET_STREAM);
     CFURLRef appURL = NULL;
     // CFStringCreateWithCString() can fail even if we're not out of memory --
     // for example if the 'cStr' parameter is something very weird (like
     // "\xFF\xFF~"), or possibly if it can't be interpreted as using what's
     // specified in the 'encoding' parameter.  See bug 548719.
-    cfMIMEType = ::CFStringCreateWithCString(NULL, flatType.get(), kCFStringEncodingUTF8);
+    cfMIMEType = ::CFStringCreateWithCString(NULL, flatType.get(),
+                                             kCFStringEncodingUTF8);
     if (cfMIMEType) {
       err = ::LSCopyApplicationForMIMEType(cfMIMEType, kLSRolesAll, &appURL);
       if ((err == noErr) && appURL && ::CFURLGetFSRef(appURL, &typeAppFSRef)) {
         haveAppForType = true;
-        MOZ_LOG(mLog, LogLevel::Debug,
+        MOZ_LOG(sLog, LogLevel::Debug,
                 ("LSCopyApplicationForMIMEType found a default application\n"));
       }
       if (appURL) {
@@ -324,13 +373,15 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
     // for example if the 'cStr' parameter is something very weird (like
     // "\xFF\xFF~"), or possibly if it can't be interpreted as using what's
     // specified in the 'encoding' parameter.  See bug 548719.
-    CFStringRef cfExt = ::CFStringCreateWithCString(NULL, flatExt.get(), kCFStringEncodingUTF8);
+    CFStringRef cfExt =
+        ::CFStringCreateWithCString(NULL, flatExt.get(), kCFStringEncodingUTF8);
     if (cfExt) {
-      err = ::LSGetApplicationForInfo(kLSUnknownType, kLSUnknownCreator, cfExt, kLSRolesAll,
-                                      &extAppFSRef, nullptr);
+      err = ::LSGetApplicationForInfo(kLSUnknownType, kLSUnknownCreator, cfExt,
+                                      kLSRolesAll, &extAppFSRef, nullptr);
       if (err == noErr) {
         haveAppForExt = true;
-        MOZ_LOG(mLog, LogLevel::Debug, ("LSGetApplicationForInfo found a default application\n"));
+        MOZ_LOG(sLog, LogLevel::Debug,
+                ("LSGetApplicationForInfo found a default application\n"));
       }
       ::CFRelease(cfExt);
     }
@@ -338,7 +389,8 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
 
   if (haveAppForType && haveAppForExt) {
     // Do aMIMEType and aFileExt match?
-    if (::FSCompareFSRefs((const FSRef*)&typeAppFSRef, (const FSRef*)&extAppFSRef) == noErr) {
+    if (::FSCompareFSRefs((const FSRef*)&typeAppFSRef,
+                          (const FSRef*)&extAppFSRef) == noErr) {
       typeAppIsDefault = true;
       *aFound = true;
     }
@@ -381,9 +433,10 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
       CFArrayRef extAppMIMETypes = GetMIMETypesHandledByApp(&extAppFSRef);
       if (extAppMIMETypes) {
         if (cfMIMEType) {
-          if (::CFArrayContainsValue(extAppMIMETypes,
-                                     ::CFRangeMake(0, ::CFArrayGetCount(extAppMIMETypes)),
-                                     cfMIMEType)) {
+          if (::CFArrayContainsValue(
+                  extAppMIMETypes,
+                  ::CFRangeMake(0, ::CFArrayGetCount(extAppMIMETypes)),
+                  cfMIMEType)) {
             extAppIsDefault = true;
             *aFound = true;
           }
@@ -408,11 +461,13 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
       // the doc for this method says that if we have a MIME type (in
       // aMIMEType), we need to give it preference.)
       NSURLFileTypeMappings* map = [NSURLFileTypeMappings sharedMappings];
-      NSString* extStr = [NSString stringWithCString:flatExt.get() encoding:NSASCIIStringEncoding];
+      NSString* extStr = [NSString stringWithCString:flatExt.get()
+                                            encoding:NSASCIIStringEncoding];
       NSString* typeStr = map ? [map MIMETypeForExtension:extStr] : NULL;
       if (typeStr) {
         nsAutoCString mimeType;
-        mimeType.Assign((char*)[typeStr cStringUsingEncoding:NSASCIIStringEncoding]);
+        mimeType.Assign(
+            (char*)[typeStr cStringUsingEncoding:NSASCIIStringEncoding]);
         mimeInfoMac->SetMIMEType(mimeType);
         haveAppForType = true;
       } else {
@@ -431,100 +486,96 @@ nsresult nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
       }
     } else {
       // Otherwise set the MIME type to a reasonable fallback.
-      mimeInfoMac->SetMIMEType(NS_LITERAL_CSTRING(APPLICATION_OCTET_STREAM));
+      mimeInfoMac->SetMIMEType(nsLiteralCString(APPLICATION_OCTET_STREAM));
     }
   }
 
   if (typeAppIsDefault || extAppIsDefault) {
     if (haveAppForExt) mimeInfoMac->AppendExtension(aFileExt);
 
-    nsresult rv;
-    nsCOMPtr<nsILocalFileMac> app(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) {
-      [localPool release];
-      return rv;
-    }
+    const FSRef* appFSRef = typeAppIsDefault ? &typeAppFSRef : &extAppFSRef;
+    nsCOMPtr<nsILocalFileMac> app;
+    if (NS_SUCCEEDED(NS_NewLocalFileWithFSRef(appFSRef, getter_AddRefs(app)))) {
+      mimeInfoMac->SetDefaultApplication(app);
 
-    CFStringRef cfAppName = NULL;
-    if (typeAppIsDefault) {
-      app->InitWithFSRef(&typeAppFSRef);
-      ::LSCopyItemAttribute((const FSRef*)&typeAppFSRef, kLSRolesAll, kLSItemDisplayName,
-                            (CFTypeRef*)&cfAppName);
-    } else {
-      app->InitWithFSRef(&extAppFSRef);
-      ::LSCopyItemAttribute((const FSRef*)&extAppFSRef, kLSRolesAll, kLSItemDisplayName,
+      CFStringRef cfAppName = NULL;
+      ::LSCopyItemAttribute(appFSRef, kLSRolesAll, kLSItemDisplayName,
                             (CFTypeRef*)&cfAppName);
+      if (cfAppName) {
+        AutoTArray<UniChar, 255> buffer;
+        CFIndex appNameLength = ::CFStringGetLength(cfAppName);
+        buffer.SetLength(appNameLength);
+        ::CFStringGetCharacters(cfAppName, CFRangeMake(0, appNameLength),
+                                buffer.Elements());
+        nsAutoString appName;
+        appName.Assign(reinterpret_cast<char16_t*>(buffer.Elements()),
+                       appNameLength);
+        mimeInfoMac->SetDefaultDescription(appName);
+        ::CFRelease(cfAppName);
+      }
     }
-    if (cfAppName) {
-      AutoTArray<UniChar, 255> buffer;
-      CFIndex appNameLength = ::CFStringGetLength(cfAppName);
-      buffer.SetLength(appNameLength);
-      ::CFStringGetCharacters(cfAppName, CFRangeMake(0, appNameLength), buffer.Elements());
-      nsAutoString appName;
-      appName.Assign(reinterpret_cast<char16_t*>(buffer.Elements()), appNameLength);
-      mimeInfoMac->SetDefaultDescription(appName);
-      ::CFRelease(cfAppName);
-    }
-
-    mimeInfoMac->SetDefaultApplication(app);
-    mimeInfoMac->SetPreferredAction(nsIMIMEInfo::useSystemDefault);
-  } else {
-    mimeInfoMac->SetPreferredAction(nsIMIMEInfo::saveToDisk);
   }
 
+  mimeInfoMac->SetPreferredAction(nsIMIMEInfo::saveToDisk);
+
   nsAutoCString mimeType;
   mimeInfoMac->GetMIMEType(mimeType);
   if (*aFound && !mimeType.IsEmpty()) {
     // If we have a MIME type, make sure its extension list is included in our
     // list.
     NSURLFileTypeMappings* map = [NSURLFileTypeMappings sharedMappings];
-    NSString* typeStr = [NSString stringWithCString:mimeType.get() encoding:NSASCIIStringEncoding];
+    NSString* typeStr = [NSString stringWithCString:mimeType.get()
+                                           encoding:NSASCIIStringEncoding];
     NSArray* extensionsList = map ? [map extensionsForMIMEType:typeStr] : NULL;
     if (extensionsList) {
       for (NSString* extension in extensionsList) {
         nsAutoCString ext;
-        ext.Assign((char*)[extension cStringUsingEncoding:NSASCIIStringEncoding]);
+        ext.Assign(
+            (char*)[extension cStringUsingEncoding:NSASCIIStringEncoding]);
         mimeInfoMac->AppendExtension(ext);
       }
     }
 
-    CFStringRef cfType = ::CFStringCreateWithCString(NULL, mimeType.get(), kCFStringEncodingUTF8);
-    if (cfType) {
-      CFStringRef cfTypeDesc = NULL;
-      if (::LSCopyKindStringForMIMEType(cfType, &cfTypeDesc) == noErr) {
+    if (CFStringRef cfType = ::CFStringCreateWithCString(
+            NULL, mimeType.get(), kCFStringEncodingUTF8)) {
+      if (CFStringRef cfTypeDesc = ::UTTypeCopyDescription(cfType)) {
         AutoTArray<UniChar, 255> buffer;
         CFIndex typeDescLength = ::CFStringGetLength(cfTypeDesc);
         buffer.SetLength(typeDescLength);
-        ::CFStringGetCharacters(cfTypeDesc, CFRangeMake(0, typeDescLength), buffer.Elements());
+        ::CFStringGetCharacters(cfTypeDesc, CFRangeMake(0, typeDescLength),
+                                buffer.Elements());
         nsAutoString typeDesc;
-        typeDesc.Assign(reinterpret_cast<char16_t*>(buffer.Elements()), typeDescLength);
+        typeDesc.Assign(reinterpret_cast<char16_t*>(buffer.Elements()),
+                        typeDescLength);
         mimeInfoMac->SetDescription(typeDesc);
-      }
-      if (cfTypeDesc) {
         ::CFRelease(cfTypeDesc);
       }
       ::CFRelease(cfType);
     }
   }
 
-  MOZ_LOG(mLog, LogLevel::Debug, ("OS gave us: type '%s' found '%i'\n", mimeType.get(), *aFound));
+  MOZ_LOG(sLog, LogLevel::Debug,
+          ("OS gave us: type '%s' found '%i'\n", mimeType.get(), *aFound));
 
   [localPool release];
   mimeInfoMac.forget(aMIMEInfo);
   return NS_OK;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
+  NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
 NS_IMETHODIMP
-nsOSHelperAppService::GetProtocolHandlerInfoFromOS(const nsACString& aScheme, bool* found,
+nsOSHelperAppService::GetProtocolHandlerInfoFromOS(const nsACString& aScheme,
+                                                   bool* found,
                                                    nsIHandlerInfo** _retval) {
   NS_ASSERTION(!aScheme.IsEmpty(), "No scheme was specified!");
 
-  nsresult rv = OSProtocolHandlerExists(nsPromiseFlatCString(aScheme).get(), found);
+  nsresult rv =
+      OSProtocolHandlerExists(nsPromiseFlatCString(aScheme).get(), found);
   if (NS_FAILED(rv)) return rv;
 
-  nsMIMEInfoMac* handlerInfo = new nsMIMEInfoMac(aScheme, nsMIMEInfoBase::eProtocolInfo);
+  nsMIMEInfoMac* handlerInfo =
+      new nsMIMEInfoMac(aScheme, nsMIMEInfoBase::eProtocolInfo);
   NS_ENSURE_TRUE(handlerInfo, NS_ERROR_OUT_OF_MEMORY);
   NS_ADDREF(*_retval = handlerInfo);
 
