@@ -5,9 +5,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsArrayEnumerator.h"
+#include "nsComponentManagerUtils.h"
 #include "nsCOMArray.h"
 #include "nsLocalFile.h"
 #include "nsMIMEInfoWin.h"
+#include "nsLocalHandlerAppWin.h"
+#include "nsIMIMEService.h"
 #include "nsNetUtil.h"
 #include <windows.h>
 #include <shellapi.h>
@@ -97,13 +100,14 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
   }
 
   if (mPreferredAction == useSystemDefault) {
-    if (mDefaultApplication &&
-        StaticPrefs::browser_pdf_launchDefaultEdgeAsApp()) {
+    nsCOMPtr<nsIFile> defaultApp = GetDefaultApplication();
+    if (defaultApp &&
+        mozilla::StaticPrefs::browser_pdf_launchDefaultEdgeAsApp()) {
       // Since Edgium is the default handler for PDF and other kinds of files,
       // if we're using the OS default and it's Edgium prefer its app mode so it
       // operates as a viewer (without browser toolbars). Bug 1632277.
       nsAutoCString defaultAppExecutable;
-      rv = mDefaultApplication->GetNativeLeafName(defaultAppExecutable);
+      rv = defaultApp->GetNativeLeafName(defaultAppExecutable);
       if (NS_SUCCEEDED(rv) &&
           defaultAppExecutable.LowerCaseEqualsLiteral("msedge.exe")) {
         nsAutoString path;
@@ -117,8 +121,7 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
           appArg.Append(path);
           const wchar_t* argv[] = {appArg.get(), path.get()};
 
-          return ShellExecuteWithIFile(mDefaultApplication,
-                                       mozilla::ArrayLength(argv), argv);
+          return ShellExecuteWithIFile(defaultApp, std::size(argv), argv);
         }
       }
     }
@@ -203,7 +206,7 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
     nsAutoString path;
     aFile->GetPath(path);
     const wchar_t* argv[] = {path.get()};
-    return ShellExecuteWithIFile(executable, mozilla::ArrayLength(argv), argv);
+    return ShellExecuteWithIFile(executable, std::size(argv), argv);
   }
 
   return NS_ERROR_INVALID_ARG;
@@ -218,6 +221,22 @@ nsMIMEInfoWin::GetHasDefaultHandler(bool* _retval) {
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMIMEInfoWin::GetDefaultExecutable(nsIFile** aExecutable) {
+  nsCOMPtr<nsIFile> defaultApp = GetDefaultApplication();
+  if (defaultApp) {
+    defaultApp.forget(aExecutable);
+    return NS_OK;
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsMIMEInfoWin::GetDefaultDescription(nsAString& aDescription) {
+  UpdateDefaultInfoIfStale();
+  aDescription = mDefaultAppDescription;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMIMEInfoWin::GetEnumerator(nsISimpleEnumerator** _retval) {
   nsCOMArray<nsIVariant> properties;
@@ -247,9 +266,9 @@ static nsresult GetIconURLVariant(nsIFile* aApplication, nsIVariant** _retval) {
 NS_IMETHODIMP
 nsMIMEInfoWin::GetProperty(const nsAString& aName, nsIVariant** _retval) {
   nsresult rv;
-  if (mDefaultApplication &&
-      aName.EqualsLiteral(PROPERTY_DEFAULT_APP_ICON_URL)) {
-    rv = GetIconURLVariant(mDefaultApplication, _retval);
+  nsCOMPtr<nsIFile> defaultApp = GetDefaultApplication();
+  if (defaultApp && aName.EqualsLiteral(PROPERTY_DEFAULT_APP_ICON_URL)) {
+    rv = GetIconURLVariant(defaultApp, _retval);
     NS_ENSURE_SUCCESS(rv, rv);
   } else if (mPreferredApplication &&
              aName.EqualsLiteral(PROPERTY_CUSTOM_APP_ICON_URL)) {
@@ -295,7 +314,8 @@ nsresult nsMIMEInfoWin::LoadUriInternal(nsIURI* aURL) {
     // Ask the shell/urlmon to parse |utf16Spec| to avoid malformed URLs.
     // Failure is indicative of a potential security issue so we should
     // bail out if so.
-    LauncherResult<_bstr_t> validatedUri = UrlmonValidateUri(utf16Spec.get());
+    mozilla::LauncherResult<_bstr_t> validatedUri =
+        mozilla::UrlmonValidateUri(utf16Spec.get());
     if (validatedUri.isErr()) {
       return NS_ERROR_FAILURE;
     }
@@ -343,11 +363,21 @@ nsresult nsMIMEInfoWin::LoadUriInternal(nsIURI* aURL) {
   return rv;
 }
 
+void nsMIMEInfoWin::UpdateDefaultInfoIfStale() {
+  if (!mIsDefaultAppInfoFresh) {
+    nsCOMPtr<nsIMIMEService> mime = do_GetService("@mozilla.org/mime;1");
+    if (mime) {
+      mime->UpdateDefaultAppInfo(static_cast<nsIMIMEInfo*>(this));
+    }
+    mIsDefaultAppInfoFresh = true;
+  }
+}
+
 // Given a path to a local file, return its nsILocalHandlerApp instance.
 bool nsMIMEInfoWin::GetLocalHandlerApp(const nsAString& aCommandHandler,
                                        nsCOMPtr<nsILocalHandlerApp>& aApp) {
   nsCOMPtr<nsIFile> locfile;
-  nsresult rv = NS_NewLocalFile(aCommandHandler, true, getter_AddRefs(locfile));
+  nsresult rv = NS_NewLocalFile(aCommandHandler, getter_AddRefs(locfile));
   if (NS_FAILED(rv)) return false;
 
   aApp = do_CreateInstance("@mozilla.org/uriloader/local-handler-app;1");
@@ -540,6 +570,7 @@ bool nsMIMEInfoWin::GetProgIDVerbCommandHandler(const nsAString& appProgIDName,
 // entries to lower case and stores them in the trackList array.
 void nsMIMEInfoWin::ProcessPath(nsCOMPtr<nsIMutableArray>& appList,
                                 nsTArray<nsString>& trackList,
+                                const nsAutoString& appIdOrName,
                                 const nsAString& appFilesystemCommand) {
   nsAutoString lower(appFilesystemCommand);
   ToLowerCase(lower);
@@ -548,13 +579,16 @@ void nsMIMEInfoWin::ProcessPath(nsCOMPtr<nsIMutableArray>& appList,
   WCHAR exe[MAX_PATH + 1];
   uint32_t len = GetModuleFileNameW(nullptr, exe, MAX_PATH);
   if (len < MAX_PATH && len != 0) {
-    int32_t index = lower.Find(exe);
+    int32_t index = lower.Find(reinterpret_cast<const char16_t*>(exe));
     if (index != -1) return;
   }
 
   nsCOMPtr<nsILocalHandlerApp> aApp;
   if (!GetLocalHandlerApp(appFilesystemCommand, aApp)) return;
 
+  // Track the app id so that the pretty name can be determined later
+  (static_cast<nsLocalHandlerAppWin*>(aApp.get()))->SetAppIdOrName(appIdOrName);
+
   // Save in our main tracking arrays
   appList->AppendElement(aApp);
   trackList.AppendElement(lower);
@@ -659,7 +693,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
           if (GetProgIDVerbCommandHandler(appProgId, appFilesystemCommand,
                                           false) &&
               !IsPathInList(appFilesystemCommand, trackList)) {
-            ProcessPath(appList, trackList, appFilesystemCommand);
+            ProcessPath(appList, trackList, appProgId, appFilesystemCommand);
           }
         }
       }
@@ -687,7 +721,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                                          false) ||
               IsPathInList(appFilesystemCommand, trackList))
             continue;
-          ProcessPath(appList, trackList, appFilesystemCommand);
+          ProcessPath(appList, trackList, appName, appFilesystemCommand);
         }
       }
       regKey->Close();
@@ -715,7 +749,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                                            false) ||
               IsPathInList(appFilesystemCommand, trackList))
             continue;
-          ProcessPath(appList, trackList, appFilesystemCommand);
+          ProcessPath(appList, trackList, appProgId, appFilesystemCommand);
         }
       }
       regKey->Close();
@@ -748,7 +782,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                                          false) ||
               IsPathInList(appFilesystemCommand, trackList))
             continue;
-          ProcessPath(appList, trackList, appFilesystemCommand);
+          ProcessPath(appList, trackList, appValue, appFilesystemCommand);
         }
       }
     }
@@ -777,7 +811,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                                            false) ||
               IsPathInList(appFilesystemCommand, trackList))
             continue;
-          ProcessPath(appList, trackList, appFilesystemCommand);
+          ProcessPath(appList, trackList, appProgId, appFilesystemCommand);
         }
       }
       regKey->Close();
@@ -815,7 +849,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                                              false) ||
                   IsPathInList(appFilesystemCommand, trackList))
                 continue;
-              ProcessPath(appList, trackList, appFilesystemCommand);
+              ProcessPath(appList, trackList, appName, appFilesystemCommand);
             }
           }
         }
@@ -843,7 +877,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
         if (!GetAppsVerbCommandHandler(appName, appFilesystemCommand, false) ||
             IsPathInList(appFilesystemCommand, trackList))
           continue;
-        ProcessPath(appList, trackList, appFilesystemCommand);
+        ProcessPath(appList, trackList, appName, appFilesystemCommand);
       }
     }
     regKey->Close();
@@ -868,7 +902,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
         if (!GetAppsVerbCommandHandler(appName, appFilesystemCommand, false) ||
             IsPathInList(appFilesystemCommand, trackList))
           continue;
-        ProcessPath(appList, trackList, appFilesystemCommand);
+        ProcessPath(appList, trackList, appName, appFilesystemCommand);
       }
     }
   }
@@ -883,12 +917,13 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
 NS_IMETHODIMP
 nsMIMEInfoWin::IsCurrentAppOSDefault(bool* _retval) {
   *_retval = false;
-  if (mDefaultApplication) {
+  nsCOMPtr<nsIFile> defaultApp = GetDefaultApplication();
+  if (defaultApp) {
     // Determine if the default executable is our executable.
     nsCOMPtr<nsIFile> ourBinary;
     XRE_GetBinaryPath(getter_AddRefs(ourBinary));
     bool isSame = false;
-    nsresult rv = mDefaultApplication->Equals(ourBinary, &isSame);
+    nsresult rv = defaultApp->Equals(ourBinary, &isSame);
     if (NS_FAILED(rv)) {
       return rv;
     }
