@@ -12,20 +12,16 @@
 #include "nsIMIMEInfo.h"
 #include "nsMIMEInfoWin.h"
 #include "nsMimeTypes.h"
-#include "plstr.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsLocalFile.h"
 #include "nsIWindowsRegKey.h"
 #include "nsXULAppAPI.h"
-#include "mozilla/StaticPrefs_browser.h"
-#include "mozilla/UniquePtrExtensions.h"
-#include "mozilla/WindowsVersion.h"
 
 // shellapi.h is needed to build with WIN32_LEAN_AND_MEAN
 #include <shellapi.h>
 #include <shlwapi.h>
 
-#define LOG(args) MOZ_LOG(mLog, mozilla::LogLevel::Debug, args)
+#define LOG(...) MOZ_LOG(sLog, mozilla::LogLevel::Debug, (__VA_ARGS__))
 
 // helper methods: forward declarations...
 static nsresult GetExtensionFromWindowsMimeDatabase(const nsACString& aMimeType,
@@ -121,23 +117,21 @@ NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(
 
   NS_ConvertASCIItoUTF16 buf(aScheme);
 
-  if (mozilla::IsWin8OrLater()) {
-    wchar_t result[1024];
-    DWORD resultSize = 1024;
-    HRESULT hr = AssocQueryString(0x1000 /* ASSOCF_IS_PROTOCOL */,
-                                  ASSOCSTR_FRIENDLYAPPNAME, buf.get(), NULL,
-                                  result, &resultSize);
-    if (SUCCEEDED(hr)) {
-      _retval = result;
-      return NS_OK;
-    }
+  wchar_t result[1024];
+  DWORD resultSize = 1024;
+  HRESULT hr = AssocQueryString(0x1000 /* ASSOCF_IS_PROTOCOL */,
+                                ASSOCSTR_FRIENDLYAPPNAME, buf.get(), NULL,
+                                result, &resultSize);
+  if (SUCCEEDED(hr)) {
+    _retval = result;
+    return NS_OK;
   }
 
   NS_ENSURE_TRUE(mAppAssoc, NS_ERROR_NOT_AVAILABLE);
   wchar_t* pResult = nullptr;
   // We are responsible for freeing returned strings.
-  HRESULT hr = mAppAssoc->QueryCurrentDefault(buf.get(), AT_URLPROTOCOL,
-                                              AL_EFFECTIVE, &pResult);
+  hr = mAppAssoc->QueryCurrentDefault(buf.get(), AT_URLPROTOCOL, AL_EFFECTIVE,
+                                      &pResult);
   if (SUCCEEDED(hr)) {
     nsCOMPtr<nsIFile> app;
     nsAutoString appInfo(pResult);
@@ -327,6 +321,10 @@ nsresult nsOSHelperAppService::GetDefaultAppInfo(
         NS_ENSURE_SUCCESS(rv, rv);
       }
     }
+
+    LOG("AppInfo \"%s\" - Determined handler command: %s\n",
+        NS_ConvertUTF16toUTF8(aAppInfo).get(),
+        NS_ConvertUTF16toUTF8(handlerCommand).get());
   }
 
   // XXX FIXME: If this fails, the UI will display the full command
@@ -348,6 +346,18 @@ nsresult nsOSHelperAppService::GetDefaultAppInfo(
     aDefaultDescription.Assign(friendlyName, friendlyNameSize - 1);
   }
 
+  if (MOZ_LOG_TEST(sLog, mozilla::LogLevel::Debug)) {
+    nsAutoString path;
+    if (*aDefaultApplication) {
+      (*aDefaultApplication)->GetPath(path);
+    }
+
+    LOG("AppInfo \"%s\" - default application (name: %s ; path: %s)\n",
+        NS_ConvertUTF16toUTF8(aAppInfo).get(),
+        NS_ConvertUTF16toUTF8(aDefaultDescription).get(),
+        NS_ConvertUTF16toUTF8(path).get());
+  }
+
   return NS_OK;
 }
 
@@ -366,28 +376,44 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(
 
   RefPtr<nsMIMEInfoWin> mimeInfo = new nsMIMEInfoWin(typeToUse);
 
-  // windows registry assumes your file extension is going to include the '.',
-  // but our APIs expect it to not be there, so make sure we normalize that bit.
-  nsAutoString fileExtToUse;
-  if (aFileExt.First() != char16_t('.')) fileExtToUse = char16_t('.');
-
-  fileExtToUse.Append(aFileExt);
-
-  // don't append the '.' for our APIs.
+  // Our extension APIs expect extensions without the '.', so normalize:
+  uint32_t dotlessIndex = aFileExt.First() != char16_t('.') ? 0 : 1;
   nsAutoCString lowerFileExt =
-      NS_ConvertUTF16toUTF8(Substring(fileExtToUse, 1));
+      NS_ConvertUTF16toUTF8(Substring(aFileExt, dotlessIndex));
   ToLowerCase(lowerFileExt);
   mimeInfo->AppendExtension(lowerFileExt);
-  mozilla::StaticPrefs::browser_download_improvements_to_download_panel()
-      ? mimeInfo->SetPreferredAction(nsIMIMEInfo::saveToDisk)
-      : mimeInfo->SetPreferredAction(nsIMIMEInfo::useSystemDefault);
+  mimeInfo->SetPreferredAction(nsIMIMEInfo::saveToDisk);
+
+  if (NS_FAILED(InternalSetDefaultsOnMIME(mimeInfo))) {
+    return nullptr;
+  }
+
+  return mimeInfo.forget();
+}
+
+nsresult nsOSHelperAppService::InternalSetDefaultsOnMIME(
+    nsMIMEInfoWin* aMIMEInfo) {
+  NS_ENSURE_ARG(aMIMEInfo);
+
+  nsAutoCString primaryExt;
+  aMIMEInfo->GetPrimaryExtension(primaryExt);
+
+  if (primaryExt.IsEmpty()) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // windows registry assumes your file extension is going to include the '.',
+  // but our APIs don't have it, so add it:
+  nsAutoString assocType = NS_ConvertUTF8toUTF16(primaryExt);
+  if (assocType.First() != char16_t('.')) {
+    assocType.Insert(char16_t('.'), 0);
+  }
 
   nsAutoString appInfo;
   bool found;
 
   // Retrieve the default application for this extension
-  NS_ENSURE_TRUE(mAppAssoc, nullptr);
-  nsString assocType(fileExtToUse);
+  NS_ENSURE_TRUE(mAppAssoc, NS_ERROR_NOT_AVAILABLE);
   wchar_t* pResult = nullptr;
   HRESULT hr = mAppAssoc->QueryCurrentDefault(assocType.get(), AT_FILEEXTENSION,
                                               AL_EFFECTIVE, &pResult);
@@ -403,7 +429,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(
   if (appInfo.EqualsLiteral("XPSViewer.Document")) found = false;
 
   if (!found) {
-    return nullptr;
+    return NS_ERROR_NOT_AVAILABLE;
   }
 
   // Get other nsIMIMEInfo fields from registry, if possible.
@@ -412,16 +438,15 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(
 
   if (NS_FAILED(GetDefaultAppInfo(appInfo, defaultDescription,
                                   getter_AddRefs(defaultApplication)))) {
-    return nullptr;
+    return NS_ERROR_NOT_AVAILABLE;
   }
 
-  mimeInfo->SetDefaultDescription(defaultDescription);
-  mimeInfo->SetDefaultApplicationHandler(defaultApplication);
+  aMIMEInfo->SetDefaultDescription(defaultDescription);
+  aMIMEInfo->SetDefaultApplicationHandler(defaultApplication);
 
   // Grab the general description
-  GetMIMEInfoFromRegistry(appInfo, mimeInfo);
-
-  return mimeInfo.forget();
+  GetMIMEInfoFromRegistry(appInfo, aMIMEInfo);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -444,9 +469,8 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
   bool haveMeaningfulMimeType =
       !aMIMEType.IsEmpty() &&
       !aMIMEType.LowerCaseEqualsLiteral(APPLICATION_OCTET_STREAM);
-  LOG(("Extension lookup on '%S' with mimetype '%s'%s\n",
-       static_cast<const wchar_t*>(fileExtension.get()), flatType.get(),
-       haveMeaningfulMimeType ? " (treated as meaningful)" : ""));
+  LOG("Extension lookup on '%S' with mimetype '%s'%s\n", fileExtension.getW(),
+      flatType.get(), haveMeaningfulMimeType ? " (treated as meaningful)" : "");
 
   RefPtr<nsMIMEInfoWin> mi;
 
@@ -477,14 +501,13 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
        !typeFromExtEquals(fileExtension.get(), flatType.get()))) {
     usedMimeTypeExtensionForLookup = true;
     fileExtension = extensionFromMimeType;
-    LOG(("Now using '%s' mimetype's default file extension '%S' for lookup\n",
-         flatType.get(), static_cast<const wchar_t*>(fileExtension.get())));
+    LOG("Now using '%s' mimetype's default file extension '%S' for lookup\n",
+        flatType.get(), fileExtension.getW());
   }
 
   // If we have an extension, use it for lookup:
   mi = GetByExtension(fileExtension, flatType.get());
-  LOG(("Extension lookup on '%S' found: 0x%p\n",
-       static_cast<const wchar_t*>(fileExtension.get()), mi.get()));
+  LOG("Extension lookup on '%S' found: 0x%p\n", fileExtension.getW(), mi.get());
 
   if (mi) {
     bool hasDefault = false;
@@ -494,9 +517,8 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
     if (!hasDefault && !usedMimeTypeExtensionForLookup) {
       RefPtr<nsMIMEInfoWin> miFromMimeType =
           GetByExtension(extensionFromMimeType, flatType.get());
-      LOG(("Mime-based ext. lookup for '%S' found 0x%p\n",
-           static_cast<const wchar_t*>(extensionFromMimeType.get()),
-           miFromMimeType.get()));
+      LOG("Mime-based ext. lookup for '%S' found 0x%p\n",
+          extensionFromMimeType.getW(), miFromMimeType.get());
       if (miFromMimeType) {
         nsAutoString desc;
         miFromMimeType->GetDefaultDescription(desc);
@@ -511,8 +533,8 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
   // different:
   if (!extensionFromMimeType.IsEmpty() && !usedMimeTypeExtensionForLookup) {
     mi = GetByExtension(extensionFromMimeType, flatType.get());
-    LOG(("Mime-based ext. lookup for '%S' found 0x%p\n",
-         static_cast<const wchar_t*>(extensionFromMimeType.get()), mi.get()));
+    LOG("Mime-based ext. lookup for '%S' found 0x%p\n",
+        extensionFromMimeType.getW(), mi.get());
   }
   if (mi) {
     mi.forget(aMIMEInfo);
@@ -532,6 +554,12 @@ nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsOSHelperAppService::UpdateDefaultAppInfo(nsIMIMEInfo* aMIMEInfo) {
+  InternalSetDefaultsOnMIME(static_cast<nsMIMEInfoWin*>(aMIMEInfo));
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsOSHelperAppService::GetProtocolHandlerInfoFromOS(const nsACString& aScheme,
                                                    bool* found,
